jQuery.sap.declare("sas.hc.ui.core.util.ComponentServiceClassResolver");

jQuery.sap.require("sas.hc.ui.core.util.ServicesUtil");

/**
 * With HTML Commons client components being required to support both WIP and Petrichor services
 * the components are written in such a way as to support multiple service implementation classes.
 * For instance the Settings dialog uses WIPSettingsService.js or PetrichorSettingsAdapterService.js interchangeably.
 *
 * In order to avoid lots of if WIP then use this else if Petrichor use this type code in our service
 * based components we would rather they simply ask for a service implementation and we'll return the
 * one we think is appropriate for the environment being run in.
 *
 * Which environment being run in will be driven by a configuration property sas.servicePlatform which
 * will be defaulted appropriately by the sas-html:Application tag.  We can make a pretty good guess
 * at which service platform based on whether it is the Application tag from the WIP enabled
 * sas.commons.openui.midtier.jar or from the non WIP (Petrichor) jar.
 *
 * If our guess isn't good enough then a solution can override it in their config.js
 *
 *
 * This  means we need to be able to have a WIP service classname and a Petrichor service classname
 * mapping for each service needed by these components.
 * This file will serve as the location of this mapping for now along with methods for resolving service to
 * classname.
 *
 * There may be a desire at some point to externalize this mapping to a config file or to be generated at build
 * time based on annotations in the service classes themselves.  However I'm going to follow the KISS ideology
 * until there is a good reason to do otherwise.
 *
 * This centralized mapping and resolving will allow us to externalize the mappings at any point in the future with
 * relative ease.
 *
 * The only real downside to keeping the mapping in this file is that it must be maintained.  We should measure the
 * complexity of dynamically generating against whether this local mapping actually causes us any issues.
 *
 */
sas.hc.ui.core.util.ComponentServiceClassResolver =
{
    CONTENT_NAVIGATOR_DATA_DESCRIPTOR: "CONTENT_NAVIGATOR_DATA_DESCRIPTOR",
    CONTENT_NAVIGATOR_MODEL: "CONTENT_NAVIGATOR_MODEL",
    CONTENT_NAVIGATOR_OPERATION_RESULT_DESCRIPTOR: "CONTENT_NAVIGATOR_OPERATION_RESULT_DESCRIPTOR",
    CONTENT_NAVIGATOR_PERMISSIONS_DESCRIPTOR: "CONTENT_NAVIGATOR_PERMISSIONS_DESCRIPTOR",
    CONTENT_NAVIGATOR_TYPE_DESCRIPTOR: "CONTENT_NAVIGATOR_TYPE_DESCRIPTOR",
    CONTENT_NAVIGATOR_VALIDATION_RESULT_DESCRIPTOR: "CONTENT_NAVIGATOR_VALIDATION_RESULT_DESCRIPTOR",
    SEARCH_SERVICE: "SEARCH",
    SEARCH_APPLICATION_ACCESS_SERVICE : "SEARCH_APPLICATION_ACCESS",
    SEARCH_CONTENT_TYPE_SERVICE : "SEARCH_CONTENT_TYPE",
    SEARCH_SCHEMA_SERVICE : "SEARCH_SCHEMA",
    SETTINGS_SERVICE: "SETTINGS",
    SIDE_MENU_SERVICE: "SIDE_MENU",
    NOTIFICATIONS_SERVICE: "NOTIFICATIONS",
    APPLICATION_SWITCHER_MODEL: "APPLICATION_SWITCHER_MODEL",
    USERINFO_SERVICE: "USERINFO",
    SIGNOUT_SERVICE: "SIGNOUT",
    THEME_SERVICE: "THEME",
    THEME_UTIL_SERVICE: "THEME_UTIL",
    KEEPALIVE_SERVICE:"KEEP_ALIVE",
    HELP: "HELP",
    ABOUT_MODEL: "ABOUT_MODEL",
    CSRF_TASK: "CSRF_TASK",
    REGISTRY_TASK: "REGISTRY_TASK",
    VERSION_INFO_TASK: "VERSION_INFO_TASK",
    CACHE_MANAGER_TASK: "CACHE_MANAGER_TASK",
    LEFT_NAV_SERVICE: 'LEFT_NAV_SERVICE',
    RECENT_MODEL: "RECENT_MODEL",
    FONT: "FONT",

    serviceClassMappings:{

        "CONTENT_NAVIGATOR_DATA_DESCRIPTOR":
        {
            "WIP": "sas.hc.ui.services.navigation.ContentDataDescriptor",
            "PETRICHOR": "sas.hc.ui.services.navigation.descriptors.ContentPETDataDescriptor"
        },
        "CONTENT_NAVIGATOR_MODEL":
        {
            "WIP": "sas.hc.ui.services.navigation.ContentNavigationModel",
            "PETRICHOR": "sas.hc.ui.services.navigation.models.ContentNavigationPETModel"
        },
        "CONTENT_NAVIGATOR_OPERATION_RESULT_DESCRIPTOR":
        {
            "WIP": "sas.hc.ui.services.navigation.descriptors.OperationsResultDescriptor",
            "PETRICHOR": "sas.hc.ui.services.navigation.descriptors.PETOperationsResultDescriptor"
        },
        "CONTENT_NAVIGATOR_PERMISSIONS_DESCRIPTOR":
        {
            "WIP": "sas.hc.ui.services.navigation.descriptors.ContentPermissionsDescriptor",
            "PETRICHOR": "sas.hc.ui.services.navigation.descriptors.ContentPETPermissionsDescriptor"
        },
        "CONTENT_NAVIGATOR_TYPE_DESCRIPTOR":
        {
            "WIP": "sas.hc.ui.services.navigation.ContentTypeDescriptor",
            "PETRICHOR": "sas.hc.ui.services.navigation.descriptors.ContentPETTypeDescriptor"
        },
        "CONTENT_NAVIGATOR_VALIDATION_RESULT_DESCRIPTOR":
        {
            "WIP": "sas.hc.ui.services.navigation.descriptors.ContentValidationResultDescriptor",
            "PETRICHOR": "sas.hc.ui.services.navigation.descriptors.ContentPETValidationResultDescriptor"
        },
        "SEARCH":
        {
            "WIP": "sas.hc.ui.services.search.service.DefaultSearchServiceImpl",
            "PETRICHOR": "sas.hc.ui.services.search.service.DefaultPETSearchServiceImpl"
        },
        "SEARCH_APPLICATION_ACCESS":
        {
            "WIP": "sas.hc.ui.services.search.service.DefaultApplicationAndTypeAccessServiceImpl",
            "PETRICHOR": "sas.hc.ui.services.search.service.DefaultPETApplicationAndTypeAccessServiceImpl"
        },
        "SEARCH_CONTENT_TYPE":
        {
            "WIP": "sas.hc.ui.services.search.service.DefaultContentTypeServiceImpl",
            "PETRICHOR": "sas.hc.ui.services.search.service.DefaultPETContentTypeServiceImpl"
        },
        "SEARCH_SCHEMA":
        {
            "WIP": "sas.hc.ui.services.search.service.DefaultSchemaServiceImpl",
            "PETRICHOR": "sas.hc.ui.services.search.service.DefaultPETSchemaServiceImpl"
        },
        "ABOUT_MODEL":
        {
            "WIP": "sas.hc.ui.commons.banner.help.AboutContentProvider",
            "PETRICHOR": "sas.hc.ui.commons.banner.help.AboutContentPETProvider"
        },
        // Settings service (Preference Service Wrapper) used by Settings Dialog.  Any settings service
        // implementation defined here MUST adhere to the contract set forth by the Settings dialog.
        "SETTINGS":
        {
            "WIP": "sas.hc.ui.commons.settings.WIPSettingsService",
            "PETRICHOR": "sas.hc.ui.commons.settings.PetrichorSettingsAdapterService"
        },
        // Service which provide the list of available applications available to user.  I really dislike the
        // name "Side Menu" service.  It is meaningless and doesn't convey anything about what the service
        // actually does.
        "SIDE_MENU":
        {
            "WIP": "sas.hc.ui.commons.settings.DefaultSideMenuService"
            //                   "PETRICHOR": "NeedsImplementation"            // TODO once service exists provide new PET implementation
        },
        "NOTIFICATIONS":
        {
            "WIP": "sas.hc.ui.services.notifications.NotificationsService",
            "PETRICHOR": "sas.hc.ui.services.notifications.PetrichorNotificationsAdapterService"
        },
        "APPLICATION_SWITCHER_MODEL":
        {
            "WIP": "sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherModel",
            "PETRICHOR": "sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherPetrichorModel"
        },
        // user info service
        "USERINFO":{
            "WIP":"sas.hc.ui.core.util.WIPUserInfoService",
            "PETRICHOR":"sas.hc.ui.core.util.PetUserInfoService"
        },
        "SIGNOUT":{
            "PETRICHOR":"sas.hc.ui.core.util.PetSignoutService"
        },
        "THEME":
        {
            "WIP": "sas.hc.ui.core.theme.service.wip.WIPThemeService",
            "PETRICHOR": "sas.hc.ui.core.theme.service.petrichor.PetrichorThemeService"
        },
        "THEME_UTIL":
        {
            "PETRICHOR":"sas.hc.ui.services.petrichor.theme.ThemeService"
        },
        // keep alive service
        "KEEP_ALIVE":{
            "WIP": "sas.hc.ui.core.util.WIPKeepAliveService",
            "PETRICHOR": "sas.hc.ui.core.util.PetKeepAliveService"
        },
        // Help service
        "HELP": {
            "WIP": "sas.hc.ui.commons.banner.help.WebDocService",
            "PETRICHOR": "sas.hc.ui.commons.banner.help.HelpCenterService"
        },

        "REGISTRY_TASK":{
            "WIP": "sas.hc.ui.core.RegistryTask"
        },

        "VERSION_INFO_TASK":{
            "PETRICHOR": "sas.hc.ui.core.VersionInfoTask"
        },

        "CACHE_MANAGER_TASK":{
            "PETRICHOR": "sas.hc.ui.core.cache.CacheManagerTask"
        },

        "LEFT_NAV_SERVICE":{
            "PETRICHOR": "sas.hc.ui.core.LeftNavPetDataProvider"
        },
        "RECENT_MODEL": {
            "PETRICHOR": "sas.hc.ui.commons.recent.RecentDialogPETModel"
        },
        "FONT" : {
            "PETRICHOR": "sas.hc.ui.services.petrichor.fonts.FontService"
        }

    },

    /**
     * For the given serviceName map to an implementation class based on
     *
     * @param {string} serviceName name of service
     * @returns {object} class to go with service
     */
    getServiceClass:function(serviceName) {
        if(this.serviceClassMappings.hasOwnProperty(serviceName)) {
            var platform = sas.hc.ui.core.util.ServicesUtil.getServicePlatform();
            var servicePlatformMappings=this.serviceClassMappings[serviceName];
            if(servicePlatformMappings.hasOwnProperty(platform)) {
                return this.serviceClassMappings[serviceName][platform];
            }
            else
                sas.log.debug("ComponentServiceClassResolver: Unable to locate a service implementation for " + serviceName + " that handles " + platform + " platform.");
        }
        else
            sas.log.error("ComponentServiceClassResolver: Unable to locate any service implementation for " + serviceName);

        return null;
    },

    getServiceClassInstance:function(serviceName) {
        var serviceInstance=undefined,
            serviceClass = this._getConfiguredServiceClass(serviceName) || this.getServiceClass(serviceName);

        if(serviceClass) {
            try {
                // load it if needed
                jQuery.sap.require(serviceClass);
                var objService = jQuery.sap.getObject(serviceClass);
                serviceInstance = new objService();
            } catch (e) {
                sas.log.error('ComponentServiceClassResolver: Unable to instantiate ' + serviceName + " service implementation class : " + serviceClass);
            }
        }
        return serviceInstance;
    },

    /**
     * For the given serviceName map to a configured implementation class.
     * Configurations can be specified in say config.js  by consumers.
     *
     * @param {string} serviceName name of service
     * @returns {object} class to go with service
     * @private
     */
    _getConfiguredServiceClass:function(serviceName) {
        var serviceClassMappings = sas.config.get("serviceClassMappings");
        if(serviceClassMappings && serviceClassMappings.hasOwnProperty(serviceName)) {
            var platform = sas.hc.ui.core.util.ServicesUtil.getServicePlatform();
            var servicePlatformMappings=serviceClassMappings[serviceName];
            if(servicePlatformMappings.hasOwnProperty(platform)) {
                return serviceClassMappings[serviceName][platform];
            }
        }
        return null;
    }

};
