/************************************************************************************************
       Module:  dabt_get_lvl_wise_subset_dtl

     Function:  This macro gets subset table for a given level (in case of consolidated variable and main level).
				Also it gets the information of event date and avg calc date, and the subset in terms of SK
				if any of the source table is SK driven.

      Authors:  BIS Team
         Date:  11 February 2010
          SAS:  9.2

    Called-by:	Loop Job

        Calls:  1. dabt_create_lib macro
				2. dabt_err_chk macro
				3. dabt_param_update macro
				4. Event macro, in case the abt is event driven

Datasets used:  The dataset LIB_STG.SUBSET_ALL_INFO is used to obtain the 
				information of the subset to be applied on each dataset.
				LIB_STG.SOURCE_TABLE_INFO is used to obtain the information regarding if any of the source table is surrogate key based.

        Logic:  1. For the given base table, obtain the subset population table, and then
					subset the base table.
				2. After each proc or data step the macro dabt_err_chk is called by passing
					SQL or DATA respectively, which checks for any error in the proc or
					data step and assigns the appropriate error code.
				3. dabt_param_update macro is called to update the loop parameter list table
					with the highest return code (error code if any error occurred or 0, if
					there was no error).

   Parameters: INPUT:  
				1. trgt_tbl		: Target Table sk.
				2. subject_level_sk	: Subject level sk.
				3. output_table_cd	: Output Table code

			OUTPUT:
				1. Subset table for each level containing event information.
************************************************************************************************/

%macro dabt_get_lvl_wise_subset_dtl(trgt_tbl=, subject_level_sk=, output_table_cd=);
	
	/* Return code initialized to zero (this value changes if error occurs in any of the proc 
		or data step */
/*	%global job_rc;*/
	%let job_rc = 0;

	%let trgt_tbl = &trgt_tbl.;
	%let subject_level_sk = &subject_level_sk.;
	%let sub_tot = .;
	
	/* Since this code creates deployed file, hence there are lot of put statements for the same, 
	Also this code doesn't contain anything from localization perspective, hence making the entire code block as i18N OK */
	
	/* i18nOK:BEGIN */
	%if &m_deploy_abt_flg eq Y %then %do;
		/* Query to check whether any subset criteria exists for the given abt */
		proc sql noprint;
			select count(distinct subset_query_sk) into :sub_tot 
			from &lib_stg..subset_all_info;
		quit;
		/* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
		%dabt_err_chk(type=SQL);
		%let sub_tot = &sub_tot;

		/* Get the previous level_no */
		/* If more than one subset exists then query the 2nd level job, else query 1st level job */
		%if &sub_tot gt 1 %then %do;
			%let prv_level_no = 2;
		%end;
		%else %do;
			%let prv_level_no = 1;
		%end;

		%let sub_tbl_nm = ;
		proc sql noprint;
			select distinct output_table_cd into :sub_tbl_nm
			from &lib_stg..job_list_dabt
			where level_no = &prv_level_no. and run_no = &run_no.;
		quit;
		/* Obtain the error code for the previous sql executed,
			and retain the maximum error code */
		%dabt_err_chk(type=SQL);

/*		%let sub_tbl_nm = &sub_tbl_nm;*/

		/* Get information for the required level */
		%let level_mapping_tbl_nm = ;
		%let level_mapping_tbl_sk = ;
		%let event_data_type = ;
		%let event_dt = ;
		%let level_mapping_tbl_lib_sk = ;
		%let lvl_key_com = ;
		%let lvl_sk_com = ;
		%let m_valid_dts_where_clause =;
		%let m_valid_mstr_dts_where_clause = ;
		%let mstr_tbl_nm = ;
		%let mstr_lib_sk = ;
		%let level_cd = ;
		%let m_map_sel_col_nm = ;
		%let m_event_var_wrt_same_dt_flg = ;
		
		data _null_;
			set &lib_stg..level_details(where=(level_sk = &subject_level_sk.));
			call symput('level_mapping_tbl_nm', kstrip(MAPPING_SOURCE_TABLE_NM));
			call symput('level_mapping_tbl_sk', kstrip(MAPPING_SOURCE_TABLE_SK));
			call symput('event_data_type', kstrip(event_date_data_type_cd));
			call symput('event_dt', kstrip(event_date_nm));
			call symput('level_mapping_tbl_lib_sk', kstrip(MAPPING_SOURCE_TABLE_LIB_SK));
			call symput('lvl_key_com', kstrip(LEVEL_KEY_COLUMN_NM));
			call symput('lvl_sk_com', kstrip(level_sk_column_nm));
			call symput('m_valid_dts_where_clause', kstrip(valid_dts_where_clause));
			call symput('mstr_tbl_nm', kstrip(master_source_table_nm));
			call symput('mstr_lib_sk', kstrip(master_source_table_lib_sk));
			call symput('level_cd', kstrip(level_cd));
			call symput('m_valid_mstr_dts_where_clause', kstrip(valid_mstr_dts_where_clause));
			call symput('m_map_sel_col_nm', kstrip(select_map_lvl_col_nm));
			call symput('m_event_var_wrt_same_dt_flg', kstrip(event_var_wrt_same_dt_flg));
		run;
		%dabt_err_chk(type=DATA);
		
		%let level_mapping_tbl_nm = &level_mapping_tbl_nm;
		%let level_mapping_tbl_sk = &level_mapping_tbl_sk;
		%let event_data_type = &event_data_type;
		%let event_dt = &event_dt;
		%let level_mapping_tbl_lib_sk = &level_mapping_tbl_lib_sk;
		%let lvl_key_com = &lvl_key_com;
		%let lvl_sk_com = &lvl_sk_com;
		%let mstr_tbl_nm = &mstr_tbl_nm;
		%let mstr_lib_sk = &mstr_lib_sk;
		%let level_cd = &level_cd;
		%let m_map_sel_col_nm = &m_map_sel_col_nm;
		%let m_valid_dts_where_clause = %superq(m_valid_dts_where_clause);
		%let m_valid_mstr_dts_where_clause = %superq(m_valid_mstr_dts_where_clause);
		%let m_event_var_wrt_same_dt_flg = &m_event_var_wrt_same_dt_flg;

		/* Obtain the key of Primary ABT level */
		%let abt_lvl_key_column = ;
		%let abt_level_cd = ;
		proc sql noprint;
			select 	LEVEL_KEY_COLUMN_NM,level_cd
			into 	:abt_lvl_key_column, :abt_level_cd
			from 	&lib_stg..level_details
			where 	main_level_flg = 'Y';
		quit;
		%dabt_err_chk(type=SQL);
		%let abt_lvl_key_column = &abt_lvl_key_column.;
		%let abt_level_cd = &abt_level_cd.;

		%let avg_col_nm = ;
		%let avg_data_type = ;
		%let sk_driven_src = 0;
		%let rk_driven_src = 0;
		%let rk_driven_im = 0;
		proc sql noprint;
			select keep_column_name, keep_column_data_type into :avg_col_nm, :avg_data_type
			from &lib_stg..keep_required_columns
			where level_sk = &subject_level_sk. and upcase(strip(column_info_type)) = "AVG_USG";
			/* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
			%dabt_err_chk(type=SQL);
				
			
			select count(*) into :sk_driven_src
			from &lib_stg..source_table_info
			where level_sk = &subject_level_sk. and upcase(strip(has_retained_key_flg)) = "N";
			/* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
			%dabt_err_chk(type=SQL);
				

			select count(*) into :rk_driven_src
			from &lib_stg..source_table_info
			where level_sk = &subject_level_sk. and upcase(strip(has_retained_key_flg)) = "Y";
			/* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
			%dabt_err_chk(type=SQL);
			
		
			select count(*) into :rk_driven_im
			from &lib_stg..source_table_info
			where level_sk = &subject_level_sk. and upcase(strip(has_retained_key_flg)) = "Y"
					and upcase(strip(var_source_type_cd)) = "SRC_IM";
			/* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
			%dabt_err_chk(type=SQL);
				
		quit;
		%let avg_col_nm = &avg_col_nm;
		%let avg_data_type = &avg_data_type;
		%let sk_driven_src = &sk_driven_src;
		%let rk_driven_src = &rk_driven_src;
		%let rk_driven_im = &rk_driven_im;

		%let time_dttm_freq = ;
		%let time_date_freq = ;
		%let time_from = ;
		%let src_align = ;
		%let src_date_exp = ;
		%let src_dttm_exp = ;
		%let src_time_freq = ;
		%let m_abt_type_cd = ;
		/* Obtian the target period count and frequency */
		proc sql noprint;
			select target_period_int_dttm_exp, target_period_int_date_exp, coalesce(target_period_cnt,0), 
					source_data_time_align, source_data_time_int_date_exp, source_data_time_int_dttm_exp,
					source_data_time_grain_cd, abt_type_cd
					into :time_dttm_freq, :time_date_freq, :time_from, :src_align, :src_date_exp,
						 :src_dttm_exp, :src_time_freq, :m_abt_type_cd
			from &lib_stg..abt_info;
			/* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
			%dabt_err_chk(type=SQL);
		quit;
		

		%let time_dttm_freq = &time_dttm_freq;
		%let time_date_freq = &time_date_freq;
		%let src_align = &src_align;
		%let src_date_exp = &src_date_exp;
		%let src_dttm_exp = &src_dttm_exp;
		%let src_time_freq = %kupcase(&src_time_freq);
		%let m_abt_type_cd = &m_abt_type_cd;

		%let beh_cnt = 0;
		%let out_beh_cnt = 0;
		proc sql noprint;
			select sum(case when outcome_flg = 'Y' then 1 else 0 end),
					sum(case when outcome_flg = 'N' then 1 else 0 end)
					into :out_beh_cnt, :beh_cnt
			from &lib_stg..keep_required_columns
			where level_sk = &subject_level_sk. and upcase(strip(column_info_type)) = "BEH_USG";
		quit;
		/* Obtain the error code for the previous sql executed,
			and retain the maximum error code */
		%dabt_err_chk(type=SQL);
		
		%let beh_cnt = &beh_cnt;
		%let out_beh_cnt = &out_beh_cnt;

		%let rnt_cnt = 0;
		%let out_rnt_cnt = 0;
		proc sql noprint;
			select sum(case when outcome_flg = 'Y' then 1 else 0 end),
					sum(case when outcome_flg = 'N' then 1 else 0 end)
					into :out_rnt_cnt, :rnt_cnt
			from &lib_stg..keep_required_columns
			where level_sk = &subject_level_sk. and upcase(strip(column_info_type)) = "RNT_USG";
		quit;
		/* Obtain the error code for the previous sql executed,
			and retain the maximum error code */
		%dabt_err_chk(type=SQL);
		
		%let rnt_cnt = &rnt_cnt;
		%let out_rnt_cnt = &out_rnt_cnt;

		%let spm_cnt = 0;
		%let out_spm_cnt = 0;
		proc sql noprint;
			select sum(case when outcome_flg = 'Y' then 1 else 0 end),
					sum(case when outcome_flg = 'N' then 1 else 0 end)
					into :out_spm_cnt, :spm_cnt
			from &lib_stg..keep_required_columns
			where level_sk = &subject_level_sk. and upcase(strip(column_info_type)) = "SPM_USG";
		quit;
		/* Obtain the error code for the previous sql executed,
			and retain the maximum error code */
		%dabt_err_chk(type=SQL);
		
		%let spm_cnt = &spm_cnt;
		%let out_spm_cnt = &out_spm_cnt;

		/* In case of external variable, check if the variable is outcome variable, to get the count of external variable.
			This will be later used in case of event based ABT to get the event outcome date. */
		%let ext_cnt = 0;
		%let out_ext_cnt = 0;
		proc sql noprint;
			select count(1) into :ext_cnt
				from &lib_stg..variable_list_ext 
				where abt_lvl_sk = &subject_level_sk.;
		quit;
		%dabt_err_chk(type=SQL);

		proc sql noprint;
			select count(1) into :out_ext_cnt
				from &lib_stg..variable_list_out_ext 
				where abt_lvl_sk = &subject_level_sk.;
		quit;
		%dabt_err_chk(type=SQL);
		/* Obtain the error code for the previous sql executed,
			and retain the maximum error code */
		
		
		%let ext_cnt = &ext_cnt;
		%let out_ext_cnt = &out_ext_cnt;

		/* Check if consolidated var exists */
		%let cons_lvl_cnt = ;
		proc sql noprint;
			select count(1) into :cons_lvl_cnt
			from &lib_stg..level_details
			where main_level_flg ne 'Y';
			/* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
			%dabt_err_chk(type=SQL);
				
		quit;
		%let cons_lvl_cnt = &cons_lvl_cnt;
		
		/*Hot Fix: S1490546. Commented a line below. This line assumes that if there is external variable then it has to be surrogate key based subject of analysis */
		/*%let sk_driven_src = %sysfunc(max(&sk_driven_src,&ext_cnt,&out_ext_cnt));*/
		
		%let rk_driven_src = %sysfunc(max(&rk_driven_src,&ext_cnt,&out_ext_cnt,&cons_lvl_cnt));
		%let rk_driven_im = %sysfunc(max(&rk_driven_im,&ext_cnt,&out_ext_cnt,&cons_lvl_cnt));

		/* Flag to indicate whether the event date is to be adjusted. This is true if any BEH or RNT variable exists.) */
		%let adjust_event_dt_flg = N;
		%let adj_date_col_nm = dabt_event_date;
		%let adj_dttm_col_nm = dabt_event_dttm;
		%let in_time_list = ;
		%let in_date_int_exp = ;
		%let in_dttm_int_exp = ;
		%let in_avg_time_list = ;
		%let in_avg_date_int_exp = ;
		%let in_avg_dttm_int_exp = ;
		%let date_usg_exst = 0;
		%let dttm_usg_exst = 0;
		%if &beh_cnt gt 0 or &rnt_cnt gt 0 %then %do;
			proc sql noprint;
				select count(*) into :date_usg_exst
				from &lib_stg..keep_required_columns
				where level_sk = &subject_level_sk. and upcase(strip(column_info_type)) IN ("BEH_USG","RNT_USG")
						and upcase(strip(keep_column_data_type)) = 'DAT' and upcase(strip(outcome_flg)) = 'N';
				/* Obtain the error code for the previous sql executed,
					and retain the maximum error code */
				%dabt_err_chk(type=SQL);
					

				select count(*) into :dttm_usg_exst
				from &lib_stg..keep_required_columns
				where level_sk = &subject_level_sk. and upcase(strip(column_info_type)) IN ("BEH_USG","RNT_USG")
						and upcase(strip(keep_column_data_type)) = 'DTT' and upcase(strip(outcome_flg)) = 'N';
				/* Obtain the error code for the previous sql executed,
					and retain the maximum error code */
				%dabt_err_chk(type=SQL);
					
			quit;
			%let date_usg_exst = &date_usg_exst;
			%let dttm_usg_exst = &dttm_usg_exst;

			%if "&src_time_freq" ne "DAY" %then %do;
				%let adjust_event_dt_flg = Y;
				%let adj_date_col_nm = dabt_adj_event_date;
				%let adj_dttm_col_nm = dabt_adj_event_dttm;
			%end;
			proc sql noprint;
				select distinct catx('',time_period, time_frequency_cd), time_freq_int_date_exp, time_freq_int_dttm_exp 
						into :in_time_list separated by ',', :in_date_int_exp separated by ',', :in_dttm_int_exp separated by ','
				from 
				(
					%if &beh_cnt gt 0 %then %do;
						select distinct (time_period_from-1) as time_period, substr(time_frequency_cd,1,1) as time_frequency_cd,
							time_freq_int_date_exp, time_freq_int_dttm_exp
						from &lib_stg..variable_list_beh
						where abt_lvl_sk = &subject_level_sk.

						union

						select distinct time_period_to as time_period, substr(time_frequency_cd,1,1) as time_frequency_cd,
							time_freq_int_date_exp, time_freq_int_dttm_exp 
						from &lib_stg..variable_list_beh
						where abt_lvl_sk = &subject_level_sk.
					%end;

					%if &rnt_cnt gt 0 %then %do;
						%if &beh_cnt gt 0 %then %do;
						union
						%end;

						select distinct (time_period_from-1) as time_period, substr(time_frequency_cd,1,1) as time_frequency_cd,
							time_freq_int_date_exp, time_freq_int_dttm_exp 
						from &lib_stg..variable_list_rnt
						where abt_lvl_sk = &subject_level_sk.

						union

						select distinct time_period_to as time_period, substr(time_frequency_cd,1,1) as time_frequency_cd,
							time_freq_int_date_exp, time_freq_int_dttm_exp 
						from &lib_stg..variable_list_rnt
						where abt_lvl_sk = &subject_level_sk.
					%end;
				);
			quit;
			/* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
			%dabt_err_chk(type=SQL);
			

			%if &beh_cnt gt 0 %then %do;
				%if &avg_col_nm ne %then %do;
					proc sql noprint;
						select distinct catx('_',time_period_from-1, time_period_to, substr(time_frequency_cd,1,1)),
								time_freq_int_date_exp, time_freq_int_dttm_exp 
								into :in_avg_time_list separated by ',', :in_avg_date_int_exp separated by ',', :in_avg_dttm_int_exp separated by ','
						from &lib_stg..variable_list_beh
								where abt_lvl_sk = &subject_level_sk. and upcase(strip(agg_type_cd)) = "AVG";
					quit;
					/* Obtain the error code for the previous sql executed,
						and retain the maximum error code */
					%dabt_err_chk(type=SQL);
					
				%end;
			%end;
		%end;

		/* If any outcome variables is for beh or rnt then the outcome date is to be computed wrt to adjusted event date */
		%let adjust_event_out_dt_flg = N;
		%let adj_out_date_col_nm = dabt_event_out_date;
		%let adj_out_dttm_col_nm = dabt_event_out_dttm;
		%let out_time_list = ;
		%let out_date_int_exp = ;
		%let out_dttm_int_exp = ;
		%let out_avg_time_list = ;
		%let out_avg_date_int_exp = ;
		%let out_avg_dttm_int_exp = ;
		%let out_date_usg_exst = 0;
		%let out_dttm_usg_exst = 0;
		%if &out_beh_cnt gt 0 or &out_rnt_cnt gt 0 %then %do;
			proc sql noprint;
				select count(*) into :out_date_usg_exst
				from &lib_stg..keep_required_columns
				where level_sk = &subject_level_sk. and upcase(strip(column_info_type)) IN ("BEH_USG","RNT_USG")
						and upcase(strip(keep_column_data_type)) = 'DAT' and upcase(strip(outcome_flg)) = 'Y';
				/* Obtain the error code for the previous sql executed,
					and retain the maximum error code */
				%dabt_err_chk(type=SQL);
					

				select count(*) into :out_dttm_usg_exst
				from &lib_stg..keep_required_columns
				where level_sk = &subject_level_sk. and upcase(strip(column_info_type)) IN ("BEH_USG","RNT_USG")
						and upcase(strip(keep_column_data_type)) = 'DTT' and upcase(strip(outcome_flg)) = 'Y';
				/* Obtain the error code for the previous sql executed,
					and retain the maximum error code */
				%dabt_err_chk(type=SQL);
					
			quit;
			%let out_date_usg_exst = &out_date_usg_exst;
			%let out_dttm_usg_exst = &out_dttm_usg_exst;

			%if "&src_time_freq" ne "DAY" %then %do;
				%let adjust_event_out_dt_flg = Y;
				%let adj_out_date_col_nm = dabt_adj_event_out_date;
				%let adj_out_dttm_col_nm = dabt_adj_event_out_dttm;

				/* Setting the adjust event flag to Y, as it is used in case when build cap is applied */
				%let adjust_event_dt_flg = Y; 
				%let adj_date_col_nm = dabt_adj_event_date;
				%let adj_dttm_col_nm = dabt_adj_event_dttm;
				%if &out_date_usg_exst gt 0 %then %do;
					%if &date_usg_exst le 0 %then %do;
						%let date_usg_exst = 1;
					%end;
				%end;
				%if &out_dttm_usg_exst gt 0 %then %do;
					%if &dttm_usg_exst le 0 %then %do;
						%let dttm_usg_exst = 1;
					%end;
				%end;
			%end;
			proc sql noprint;
				select distinct catx('',time_period, time_frequency_cd), time_freq_int_date_exp, time_freq_int_dttm_exp 
						into :out_time_list separated by ',', :out_date_int_exp separated by ',', :out_dttm_int_exp separated by ','
				from 
				(
					%if &out_beh_cnt gt 0 %then %do;
						select distinct (time_period_from-1) as time_period, substr(time_frequency_cd,1,1) as time_frequency_cd,
							time_freq_int_date_exp, time_freq_int_dttm_exp 
						from &lib_stg..variable_list_out_beh
						where abt_lvl_sk = &subject_level_sk.

						union

						select distinct time_period_to as time_period, substr(time_frequency_cd,1,1) as time_frequency_cd,
							time_freq_int_date_exp, time_freq_int_dttm_exp 
						from &lib_stg..variable_list_out_beh
						where abt_lvl_sk = &subject_level_sk.
					%end;

					%if &out_rnt_cnt gt 0 %then %do;
						%if &out_beh_cnt gt 0 %then %do;
						union
						%end;

						select distinct (time_period_from-1) as time_period, substr(time_frequency_cd,1,1) as time_frequency_cd,
							time_freq_int_date_exp, time_freq_int_dttm_exp 
						from &lib_stg..variable_list_out_rnt
						where abt_lvl_sk = &subject_level_sk.

						union

						select distinct time_period_to as time_period, substr(time_frequency_cd,1,1) as time_frequency_cd,
							time_freq_int_date_exp, time_freq_int_dttm_exp 
						from &lib_stg..variable_list_out_rnt
						where abt_lvl_sk = &subject_level_sk.
					%end;
				);
			quit;
			/* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
			%dabt_err_chk(type=SQL);
			

			%if &out_beh_cnt gt 0 %then %do;
				%if &avg_col_nm ne %then %do;
					proc sql noprint;
						select distinct catx('_',time_period_from-1, time_period_to, substr(time_frequency_cd,1,1)),
								time_freq_int_date_exp, time_freq_int_dttm_exp 
								into :out_avg_time_list separated by ',', :out_avg_date_int_exp separated by ',', :out_avg_dttm_int_exp separated by ','
						from &lib_stg..variable_list_out_beh
								where abt_lvl_sk = &subject_level_sk. and upcase(strip(agg_type_cd)) = "AVG";
					quit;
					/* Obtain the error code for the previous sql executed,
						and retain the maximum error code */
					%dabt_err_chk(type=SQL);
					
				%end;
			%end;
		%end;

		%let out_spm_time_list = ;
		%let out_spm_dttm_int_exp = ;
		%let out_spm_date_int_exp = ;
		%let out_spm_date_usg_exst = 0;
		%let out_spm_dttm_usg_exst = 0;
		%if &out_spm_cnt gt 0 %then %do;
			proc sql noprint;
				select count(*) into :out_spm_date_usg_exst
				from &lib_stg..keep_required_columns
				where level_sk = &subject_level_sk. and upcase(strip(column_info_type)) = "SPM_USG"
						and upcase(strip(keep_column_data_type)) = 'DAT' and upcase(strip(outcome_flg)) = 'Y';
				/* Obtain the error code for the previous sql executed,
					and retain the maximum error code */
				%dabt_err_chk(type=SQL);
					

				select count(*) into :out_spm_dttm_usg_exst
				from &lib_stg..keep_required_columns
				where level_sk = &subject_level_sk. and upcase(strip(column_info_type)) = "SPM_USG"
						and upcase(strip(keep_column_data_type)) = 'DTT' and upcase(strip(outcome_flg)) = 'Y';
				/* Obtain the error code for the previous sql executed,
					and retain the maximum error code */
				%dabt_err_chk(type=SQL);
					
			quit;
			%let out_spm_date_usg_exst = &out_spm_date_usg_exst;
			%let out_spm_dttm_usg_exst = &out_spm_dttm_usg_exst;

			proc sql noprint;
				select distinct catx('',as_of_time_value, substr(time_frequency_cd,1,1)), 
							time_freq_int_dttm_exp, time_freq_int_date_exp  
						into :out_spm_time_list separated by ',', :out_spm_dttm_int_exp separated by ',',
								:out_spm_date_int_exp separated by ','
				from &lib_stg..variable_list_out_spm
					where abt_lvl_sk = &subject_level_sk.;
			quit;
			/* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
			%dabt_err_chk(type=SQL);
			
		%end;

		%let spm_time_list = ;
		%let spm_dttm_int_exp = ;
		%let spm_date_int_exp = ;
		%let spm_date_usg_exst = 0;
		%let spm_dttm_usg_exst = 0;
		%if &spm_cnt gt 0 %then %do;
			proc sql noprint;
				select count(*) into :spm_date_usg_exst
				from &lib_stg..keep_required_columns
				where level_sk = &subject_level_sk. and upcase(strip(column_info_type)) = "SPM_USG"
						and upcase(strip(keep_column_data_type)) = 'DAT' and upcase(strip(outcome_flg)) = 'N';
				/* Obtain the error code for the previous sql executed,
					and retain the maximum error code */
				%dabt_err_chk(type=SQL);
					

				select count(*) into :spm_dttm_usg_exst
				from &lib_stg..keep_required_columns
				where level_sk = &subject_level_sk. and upcase(strip(column_info_type)) = "SPM_USG"
						and upcase(strip(keep_column_data_type)) = 'DTT' and upcase(strip(outcome_flg)) = 'N';
				/* Obtain the error code for the previous sql executed,
					and retain the maximum error code */
				%dabt_err_chk(type=SQL);
					
			quit;
			%let spm_date_usg_exst = &spm_date_usg_exst;
			%let spm_dttm_usg_exst = &spm_dttm_usg_exst;

			proc sql noprint;
				select distinct catx('',as_of_time_value, substr(time_frequency_cd,1,1)),
							time_freq_int_dttm_exp,  time_freq_int_date_exp
						into :spm_time_list separated by ',', :spm_dttm_int_exp separated by ',',
								:spm_date_int_exp separated by ','
				from &lib_stg..variable_list_spm
					where abt_lvl_sk = &subject_level_sk.;
			quit;
			 /* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
			%dabt_err_chk(type=SQL);
			
		%end;

		/* Code to get date and datetime field for avg date */
		%if &avg_col_nm ne %then %do;
			%let select_avg_col = ;
			%dabt_dbms_date_processing(m_in_col_nm=&avg_col_nm, m_data_type=&avg_data_type,
									m_out_mcr_var_nm=select_avg_col, m_dt_col_nm=dabt_avg_date, m_dttm_col_nm=dabt_avg_dttm);
			/* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
			%dabt_err_chk(type=SQL);
			%dabt_err_chk(type=DATA);
				
		%end;

		/* Flag to indicate whether the subset table that is passed to event contains avg date */
		%let avg_in_sub_flg = N;

		%let main_lvl_flg = ;
		proc sql noprint;
			select kupcase(main_level_flg) into :main_lvl_flg
			from &lib_stg..level_details
			where level_sk = &subject_level_sk.;
			/* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
			%dabt_err_chk(type=SQL);
				
		quit;
		%let main_lvl_flg = &main_lvl_flg;

		/* Flag to indicate, whether the event macro is to be called. Default set as Y, as in case of main level, it is to be called */
		%let m_call_event_macro_flg = Y;
		/* In case of consolidated var if it is to be computed via macro then the flag remains Y, else if it is to be computed
			as per the event generated for the main level flag, then the event macro is not called for cons level
			and instead it is obtained from the main level event date. */
		%if &main_lvl_flg ne Y and &m_event_var_wrt_same_dt_flg = Y %then %do;
			%let m_call_event_macro_flg = N;
		%end;
				
		%let key_cnt = %eval(%sysfunc(countc(%quote(&lvl_key_com),","))+1);

		data _null_;
			file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000;/* i18nOK:Line */ 
			length stmt_str $32767.;
			put ;
			put '/* Drop output table before creating */';
			stmt_str = '%dabt_drop_table(m_table_nm=' || strip("%superq(output_table_cd)") || "_sk);";
			put stmt_str;
			put ;

			stmt_str = '%dabt_drop_table(m_table_nm=' || strip("%superq(output_table_cd)") || "_rk);";
			put stmt_str;
			put ;
		run;
		%dabt_err_chk(type=DATA);

		/* in case the level is abt level then perform following */
		%if &main_lvl_flg eq Y %then %do;
			%let map_tbl_nm = ; /* For passig to event macro */
			%if &sub_tot gt 0 %then %do;/* main level AND subset is present*/
				%if &event_dt ne %then %do;
					%let fnl_sub_evt_tbl_nm = %superq(sub_tbl_nm);
					%let avg_in_sub_flg = N;
				%end;
				%else %do;
					%if &sk_driven_src gt 0 %then %do; /* main level AND subset is present AND not event based AND atleast one _sk based source table */
					/* if we have atleast one abt level source table which is _sk driven  */

						data _null_;
							file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
							length stmt_str $32767.;
							put ;
							put '/* Creating subset sk table containing population satisfying all the specified subset criteria */';
							put ;
							put 'proc sql noprint &sql_option.;';
							stmt_str ="	create table " || strip("%superq(output_table_cd)") || "_sk as";/* i18nOK:Line */ 
							put stmt_str;
							put "	select &lvl_sk_com,";
							stmt_str = '';
							/* Obtain the number of columns which are part of the key */
							%do key_j = 1 %to &key_cnt;
								%let key_tkn =  %scan(%quote(&lvl_key_com),&key_j,%str(,));
								%if &key_j lt &key_cnt %then %do;
									stmt_str =  '	' || strip(stmt_str) || " t1.&key_tkn.,";
								%end;
								%else %do;
									stmt_str =  '	' || strip(stmt_str) || " t1.&key_tkn.";
								%end;
							%end;
							put stmt_str;
							%if &avg_col_nm ne %then %do;
								/* No table identifier, as subset table contains only key column, hence the date column only comes from master table. 
									Also the macro select_avg_col contains two column names avg_date and avg_dttm (with the appropriate expressions,
									hence cannot directly put the alias. */
								put "	, &select_avg_col ";

								%dabt_get_avg_calc_code(m_avg_col_avl_flg=N);
								
							%end;
							stmt_str = "	from " || strip("%superq(sub_tbl_nm)") || " as t1 inner join " || strip("&mstr_tbl_nm") || " as t2";
							put stmt_str;
							stmt_str =	"	on (";
							%do key_j = 1 %to &key_cnt;
								%let key_tkn =  %scan(%quote(&lvl_key_com),&key_j,%str(,));
								%if &key_j lt &key_cnt %then %do;
									stmt_str =	strip(stmt_str) || "t1.&key_tkn. = t2.&key_tkn. and ";
								%end;
								%else %do;
									stmt_str =	strip(stmt_str) || "t1.&key_tkn. = t2.&key_tkn.";
								%end;
							%end;
							stmt_str =	'	' || strip(stmt_str) ||	");" ;
							put stmt_str;
							put "quit;";
							put "/* Obtain the error code for the previous sql executed,and retain the maximum error code */";
							put '%dabt_err_chk(type=SQL);';
						run;
						%dabt_err_chk(type=DATA);
						
						/* Note: we create _sk table without any condition of valid as of abt build date. This is to get all _sk corresponding to a given _rk */
						/* data extraction has separate time based condition to take care of time based filter while extraction.*/
					%end;
					
					%if &rk_driven_src gt 0 %then %do; /* main level AND subset is present AND not event based AND atleast one _rk based source table */
						%if &avg_col_nm ne %then %do; /* main level AND subset is present AND not event based AND atleast one _rk based source table AND avg calc date applies to level*/
							data _null_;
								file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
								length stmt_str $32767.;
								put ;
								put '/* Creating subset rk table containing population satisfying all the specified subset criteria */';
								put ;
								put 'proc sql noprint &sql_option.;';
								stmt_str ="	create table %superq(output_table_cd)_rk as";/* i18nOK:Line */ 
								put stmt_str;
								stmt_str = "select  ";
								/* Obtain the number of columns which are part of the key */
								%do key_j = 1 %to &key_cnt;
									%let key_tkn =  %scan(%quote(&lvl_key_com),&key_j,%str(,));
									stmt_str = strip(stmt_str) || " t1.&key_tkn.,";
								%end;
								stmt_str = '	' || strip(stmt_str);
								put stmt_str;
									/* No table identifier, as subset table contains only key column, hence the date column only comes from master table. 
										Also the macro select_avg_col contains two column names avg_date and avg_dttm (with the appropriate expressions,
										hence cannot directly put the alias. */
								put	"	&select_avg_col";

								%dabt_get_avg_calc_code(m_avg_col_avl_flg=N);

								stmt_str = "	from " || strip("%superq(sub_tbl_nm)") || " as t1 inner join " || strip("&mstr_tbl_nm") || " as t2";
								put stmt_str;
								stmt_str =	"	on (";
								%do key_j = 1 %to &key_cnt;
									%let key_tkn =  %scan(%quote(&lvl_key_com),&key_j,%str(,));
									%if &key_j lt &key_cnt %then %do;
										stmt_str =	strip(stmt_str) || "t1.&key_tkn. = t2.&key_tkn. and ";
									%end;
									%else %do;
										stmt_str =	strip(stmt_str) || "t1.&key_tkn. = t2.&key_tkn.";
									%end;
								%end;
								stmt_str =	'	'|| strip(stmt_str) ||	")" ;
								put stmt_str;
									/* The columns in the where clause contain the table identifier */
								%if "%superq(m_valid_mstr_dts_where_clause)" ne "" %then %do;
									stmt_str = "	where " || strip("%superq(m_valid_mstr_dts_where_clause)");
									put stmt_str;
									/* to get value of avg calculation date as of abt build date*/
								%end;
								put "	;";
								put "quit;";
								put "/* Obtain the error code for the previous sql executed,and retain the maximum error code */";
								put '%dabt_err_chk(type=SQL);';
							run;
							%dabt_err_chk(type=DATA);
							
						%end;
						%else %do;/* main level AND subset is present AND not event based AND atleast one _rk based source table AND NO avg calc date applies to level*/
							data _null_;
								file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
								length stmt_str $32767.;
								put ;
								put '/* Creating subset rk table containing population satisfying all the specified subset criteria */';
								put ;
								put 'proc sql noprint &sql_option.;';
								stmt_str ="	create table " || strip("%superq(output_table_cd)") || "_rk as";/* i18nOK:Line */ 
								put stmt_str;
								put	"	select &lvl_key_com";
								stmt_str = 	"	from " || strip("%superq(sub_tbl_nm)") || ";";
								put stmt_str;
								put "quit;";
								put "/* Obtain the error code for the previous sql executed,and retain the maximum error code */";
								put '%dabt_err_chk(type=SQL);';
							run;
							%dabt_err_chk(type=DATA);
						%end;
					%end;
					

					%return; /* In case of main level, with subset and without event, the required tables are created, hence can return from this point. */
				%end;
			%end;
			%else %do;/* main level AND subset is not present*/
				/* If the abt is event driven and subset doesn't exist then the complete distinct list of _RK's is passed.
					Following code creates that table. */
				%if &event_dt ne %then %do;/* main level AND subset is not present AND event based*/
					%let fnl_sub_evt_tbl_nm = %superq(output_table_cd)_t1;
					%let avg_in_sub_flg = Y;
					data _null_;
						file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
						length stmt_str $32767.;
						put ;
						put '/* Drop output table before creating */';
						stmt_str = '%dabt_drop_table(m_table_nm=' || "%superq(output_table_cd)_t1);";
						put stmt_str;
						put ;
						put '/* Creating table containing population entire population valid as of build date, as subset is not specified */';
						put ;
						put 'proc sql noprint &sql_option.;';
						put "	create table %superq(output_table_cd)_t1 as";
						put	"	select &lvl_key_com";
						%if &avg_col_nm ne %then %do;
							put "	, &select_avg_col";

							%dabt_get_avg_calc_code(m_avg_col_avl_flg=N);

						%end;
						stmt_str =	"	from " || strip("&mstr_tbl_nm");
						put stmt_str;
						%if "%superq(m_valid_mstr_dts_where_clause)" ne "" %then %do;
							stmt_str = "	where " || strip("%superq(m_valid_mstr_dts_where_clause)");
							put stmt_str;
						%end;
						put "	;";
						put "quit;";
						put '%dabt_err_chk(type=SQL);';
					run;
					%dabt_err_chk(type=DATA);
				%end;
				%else %do;/* main level AND subset is not present AND not event based*/

					%if &sk_driven_src gt 0 %then %do; 
					/* main level AND subset is not present AND not event based AND avg cal date si applicable and AND has _sk based source table*/
						data _null_;
							file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
							length stmt_str $32767.;
							put ;
							put '/* Creating sk table containing entire population as subset is not specified */';
							put ;
							put 'proc sql noprint &sql_option.;';
							stmt_str ="	create table %superq(output_table_cd)_sk as";/* i18nOK:Line */ 
							put stmt_str;
							put	"	select &lvl_sk_com, &lvl_key_com";
							/* If average columns exists then get the column from master table */
							%if &avg_col_nm ne %then %do;
								put "	, &select_avg_col";

								%dabt_get_avg_calc_code(m_avg_col_avl_flg=N);
								
							%end;
							stmt_str = "	from " || strip("&mstr_tbl_nm") || ";";
							put stmt_str;
							put "quit;";
							put '%dabt_err_chk(type=SQL);';
						run;
						%dabt_err_chk(type=DATA);
						
					%end;

					%if &rk_driven_src gt 0 %then %do;
					/* main level AND subset is not present AND not event based AND avg cal date si applicable and AND has _rk based source table*/
						/* If average columns exists then get the column from master table, else no table will be created. */
						%if &avg_col_nm ne %then %do;
							data _null_;
								file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
								length stmt_str $32767.;
								put ;
								put '/* Creating rk table containing entire population as of build date, as subset is not specified */';
								put ;
								put 'proc sql noprint &sql_option.;';
								stmt_str ="	create table %superq(output_table_cd)_rk as";/* i18nOK:Line */ 
								put stmt_str;
								put "	select &lvl_key_com, &select_avg_col";

								%dabt_get_avg_calc_code(m_avg_col_avl_flg=N);
								
								stmt_str = "	from " || strip("&mstr_tbl_nm");
								put stmt_str;
								%if "%superq(m_valid_mstr_dts_where_clause)" ne "" %then %do;
									stmt_str = "	where " || strip("%superq(m_valid_mstr_dts_where_clause)");
									put stmt_str;
								%end;
								put "	;";
								put "quit;";
								put '%dabt_err_chk(type=SQL);';
							run;
							%dabt_err_chk(type=DATA);
							
						%end;
						%else %if &rk_driven_im gt 0 %then %do;/* main level AND subset is not present AND not event based AND atleast one _rk based
																	source table AND NO avg calc date applies to level and an IM source exists */
							data _null_;
								file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
								length stmt_str $32767.;
								put ;
								put '/* Creating rk table containing entire population as of build date, as subset is not specified */';
								put ;
								put 'proc sql noprint &sql_option.;';
								stmt_str ="	create table %superq(output_table_cd)_rk as";/* i18nOK:Line */ 
								put stmt_str;
								put "	select &lvl_key_com";
								stmt_str = "	from " || strip("&mstr_tbl_nm");
								put stmt_str;
								%if "%superq(m_valid_mstr_dts_where_clause)" ne "" %then %do;
									stmt_str = "	where " || strip("%superq(m_valid_mstr_dts_where_clause)");
									put stmt_str;
								%end;
								put "	;";
								put "quit;";
								put '%dabt_err_chk(type=SQL);';
							run;
							%dabt_err_chk(type=DATA);
							
						%end;
					%end;
					

					%return; /* In case of main level without event & subset, the required tables are created, hence can return from this point. */
				%end;
			%end;
		%end;/*in case the level is abt level */

		/* For consolidated level the following code is executed. */
		%else %do;

			%let map_key_cnt = %eval(%sysfunc(countc(%quote(&m_map_sel_col_nm),","))+1);
			%let abt_key_cnt = %eval(%sysfunc(countc(%quote(&abt_lvl_key_column),","))+1);
			
			%if &avg_col_nm ne  or &event_dt ne %then %do;
				%let map_tbl_nm = %superq(output_table_cd)_map; /* For passig to event macro */
			%end;
			%else %do;
				%let map_tbl_nm = %superq(output_table_cd)_rk;
			%end;
			 
			data _null_;
				file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
				length stmt_str $32767.;
				put ;
				/* Dropping the map table only avg/event is required, else the table is created with _rk, which is already dropped earlier */
				%if &avg_col_nm ne  or &event_dt ne %then %do;
					put '/* Drop output table before creating */';
					stmt_str = '%dabt_drop_table(m_table_nm=' || "%superq(map_tbl_nm));";
					put stmt_str;
					put ;
				%end;
				%if &sub_tot gt 0 %then %do;
					put '/* Creating mapping table containing population satisfying the specified subset criteria */';
				%end;
				%else %do;
					put '/* Creating mapping table containing entire population as of build date, as subset is not specified */';
				%end;
				put ;
				put 'proc sql noprint &sql_option.;';
				stmt_str ="	create table %superq(map_tbl_nm) as";/* i18nOK:Line */ 
				put stmt_str;
				put "	select &m_map_sel_col_nm";
				stmt_str = "	from " || strip("&level_mapping_tbl_nm") || " t1";
				put stmt_str;
				%if &sub_tot gt 0 %then %do;
					stmt_str = "	inner join " || strip("%superq(sub_tbl_nm)") || " t2";
					put stmt_str;
					stmt_str = "	on (";
					 %do abt_key_j = 1 %to &abt_key_cnt;
						%let abt_key_tkn =  %scan(%quote(&abt_lvl_key_column),&abt_key_j,%str(,));
						/* Get the column name for mapping table. This is done incase the level column name in mapping table
							is different from the level key column name */
						%do map_key_j = 1 %to &map_key_cnt;
							%let map_key_tkn =  %scan(%quote(&m_map_sel_col_nm),&map_key_j,%str(,));
							%let sub_col_len_tkn =  %sysfunc(find(%quote(&map_key_tkn),%str( as )));
							%let sub_col_key_tkn = %substr(%quote(&map_key_tkn),%eval(&sub_col_len_tkn + 4));
							/* Getting the appropriate mapping col name. As in the mapping table there are two 
								columns configured */
							%if %kupcase(&sub_col_key_tkn) eq %kupcase(&abt_key_tkn) %then %do;
								%let map_col_len_tkn =  %sysfunc(find(%quote(&map_key_tkn),%str( as )));
								%let map_col_key_tkn = %substr(%quote(&map_key_tkn),1,&map_col_len_tkn);

								%let map_key_j = %eval(&map_key_cnt + 1);
							%end;
						%end;
						stmt_str = strip(stmt_str) || "&map_col_key_tkn. = t2.&abt_key_tkn.";
						%if &abt_key_j lt &abt_key_cnt %then %do;
							stmt_str = strip(stmt_str) || " and ";
						%end;
					%end;
					stmt_str = '	' || strip(stmt_str) ||  ")";
					put stmt_str;
				%end;
				%if "%superq(m_valid_dts_where_clause)" ne "" %then %do;
					stmt_str = "	where " || strip("%superq(m_valid_dts_where_clause)");
					put stmt_str;
				%end;
				put	"	;";
				put "quit;";
				put '%dabt_err_chk(type=SQL);';
				put ;
			run;
			%dabt_err_chk(type=DATA);
			
			%if &event_dt ne %then %do;
				%let fnl_sub_evt_tbl_nm = %superq(output_table_cd)_t1;
				%let avg_in_sub_flg = N;
			%end;
			%else %do;
				%if &sk_driven_src gt 0 %then %do;
					data _null_;
						file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
						length stmt_str $32767.;
						put ;
						put '/* Creating sk table containing required population */';
						put ;
						put 'proc sql noprint &sql_option.;';
						stmt_str ="	create table %superq(output_table_cd)_sk as";/* i18nOK:Line */ 
						put stmt_str;
						stmt_str = 'select';
						%do abt_key_j = 1 %to &abt_key_cnt;
							%let abt_key_tkn =  %scan(%quote(&abt_lvl_key_column),&abt_key_j,%str(,));
							stmt_str = strip(stmt_str) || " t1.&abt_key_tkn.,";
						%end;
						stmt_str = '	' || strip(stmt_str);
						put stmt_str;
						/* Obtain the number of columns which are part of the key */
						stmt_str = '';
						%do key_j = 1 %to &key_cnt;
							%let key_tkn =  %scan(%quote(&lvl_key_com),&key_j,%str(,));
							stmt_str = strip(stmt_str) || "t1.&key_tkn.,";
						%end;
						stmt_str = '	' || strip(stmt_str);
						put stmt_str;
						put "	&lvl_sk_com";
						%if &avg_col_nm ne %then %do;
							put "	, &select_avg_col ";
							%dabt_get_avg_calc_code(m_avg_col_avl_flg=N);
						%end;
						stmt_str = "	from " || strip("%superq(map_tbl_nm)") || " t1 inner join " || strip("&mstr_tbl_nm") || " t2";
						put stmt_str;
						stmt_str = " on (";
						%do key_j = 1 %to &key_cnt;
							%let key_tkn =  %scan(%quote(&lvl_key_com),&key_j,%str(,));
							stmt_str = strip(stmt_str) || "t1.&key_tkn. = t2.&key_tkn.";
							%if &key_j lt &key_cnt %then %do;
								stmt_str = strip(stmt_str) || " and ";
							%end;
						%end;
						stmt_str = '	' || strip(stmt_str) || ') ;';
						put stmt_str;
						put 'quit;';
						put '%dabt_err_chk(type=SQL);';
					run;
					%dabt_err_chk(type=DATA);
					
				%end;
				
				%if &rk_driven_src gt 0 %then %do;
					%if &avg_col_nm ne %then %do;
						data _null_;
							file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
							length stmt_str $32767.;
							put ;
							put '/* Creating rk table containing required population */';
							put ;
							put 'proc sql noprint &sql_option.;';
							stmt_str ="	create table %superq(output_table_cd)_rk as";/* i18nOK:Line */ 
							put stmt_str;
							stmt_str = "select "; /* i18nOK:Line */ 
							%do abt_key_j = 1 %to &abt_key_cnt;
								%let abt_key_tkn =  %scan(%quote(&abt_lvl_key_column),&abt_key_j,%str(,));
								stmt_str = strip(stmt_str) || " t1.&abt_key_tkn.,"; /* i18nOK:Line */ 
							%end;
							stmt_str = '	' || strip(stmt_str); /* i18nOK:Line */ 
							put stmt_str;
							stmt_str = ""; /* i18nOK:Line */ 
							/* Obtain the number of columns which are part of the key */
							%do key_j = 1 %to &key_cnt;
								%let key_tkn =  %scan(%quote(&lvl_key_com),&key_j,%str(,));
								stmt_str = strip(stmt_str) || " t1.&key_tkn.,";
							%end;
							stmt_str = '	' || strip(stmt_str);
							put stmt_str;
						
							/* No table identifier, as subset table contains only key column, hence the date column only comes from master table. 
								Also the macro select_avg_col contains two column names avg_date and avg_dttm (with the appropriate expressions,
								hence cannot directly put the alias. */
							put "	&select_avg_col";

							%dabt_get_avg_calc_code(m_avg_col_avl_flg=N);

							stmt_str = "	from " || strip("%superq(map_tbl_nm)") || " t1 inner join " || strip("&mstr_tbl_nm") || " t2";
							put stmt_str;
							stmt_str =	" on (";
							%do key_j = 1 %to &key_cnt;
								%let key_tkn =  %scan(%quote(&lvl_key_com),&key_j,%str(,));
								stmt_str = strip(stmt_str) || "t1.&key_tkn. = t2.&key_tkn.";
								%if &key_j lt &key_cnt %then %do;
									stmt_str = strip(stmt_str) || " and ";
								%end;
							%end;
							stmt_str = '	' || strip(stmt_str) || ") ";
							put stmt_str;
								/* The columns in the where clause contain the table identifier */
							%if "%superq(m_valid_mstr_dts_where_clause)" ne "" %then %do;
								stmt_str = "	where " || strip("%superq(m_valid_mstr_dts_where_clause)");
								put stmt_str;
							%end;
							put '	;';
							put 'quit;'	;
							put '%dabt_err_chk(type=SQL);';
						run;
						%dabt_err_chk(type=DATA);
						
					%end;/* End of avg date exists check */
				%end;

				%if &avg_col_nm ne  or &event_dt ne %then %do;
					data _null_;
						file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
						length stmt_str $32767.;
						put ;
						put '	/* If the debug flag is set to N, then the temporary datasets and files are deleted */';
						stmt_str = '%dabt_clean_if_debug_on(m_table_nm=' || "%superq(map_tbl_nm), m_chk_exst=Y);";
						put stmt_str;
						put ;	
					run;
					%dabt_err_chk(type=DATA);
				%end;
									
				%return; /* In case of cons level, with subset and without event, the required tables are created, hence can return from this point. */
			%end;
		%end;

		/* if event date exists then perform the following steps */
		%if &event_dt ne %then %do;
			%let evt_ds_nm = %superq(output_table_cd)_evt;
			%let act_bld_dttm = DABT_LOAD_USER_INPUT_DTTM;

			/* If the event macro is not to be called, then get the table obtained in level 3 for the abt level event info*/
			%if &m_call_event_macro_flg eq N %then %do;
				%let m_main_evt_lvl_ds = ;
				proc sql noprint;
					select output_table_cd into :m_main_evt_lvl_ds
					from &lib_stg..job_list_dabt
					where level_no = 3 and run_no=&run_no.; /* Level 3 will contain only one job for a given run_no at main level */
				quit;
			%end;
			
			/* Obtain the key of Primary ABT level */
			%let abt_purpose_cd = ;
			proc sql noprint;
				select purpose_cd into :abt_purpose_cd
				from &lib_stg..abt_info;
			quit;
			%dabt_err_chk(type=SQL);
			%let abt_purpose_cd = &abt_purpose_cd.;

			data _null_;
				file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
				length stmt_str $32767.;
				put ;
				%if &m_call_event_macro_flg eq Y %then %do;
					%if &main_lvl_flg ne Y %then %do; /* In case of consolidated level create the distinct list table to be passed to the event macro */
						put '/* Drop output table before creating */';
						stmt_str = '%dabt_drop_table(m_table_nm=' || "%superq(fnl_sub_evt_tbl_nm));";
						put stmt_str;
						put ;
						put '/* Creating table containing consolidated level from the mapping table.  */';
						put ;
						put 'proc sql noprint &sql_option.;';
						put "	create table %superq(fnl_sub_evt_tbl_nm) as";
						put "	select distinct &lvl_key_com.";
						stmt_str = "	from " || strip("%superq(map_tbl_nm)") || ";";
						put stmt_str;
						put "quit;";
						put '%dabt_err_chk(type=SQL);';
						put ;
					%end;
					put '/* Drop output table before creating */';
					stmt_str = '%dabt_drop_table(m_table_nm=' || "%superq(evt_ds_nm));";
					put stmt_str;
					put ;
					put '/* The following macro creates a table containing the lvl_key_com column and event date for each key and stores it in the table &evt_ds_nm. */';
					put ;
					stmt_str = '%' || "&mcr_dabt_create_event_data.(abt_sk=&trgt_tbl., abt_purpose_cd=&abt_purpose_cd., abt_bld_dttm=" || '&' || "&act_bld_dttm.,";
					put stmt_str;
					put "					event_dt_column=&event_dt., event_key_column=%quote(&lvl_key_com.), event_level_cd=&level_cd.,";
					put "					abt_lvl_key_column = %quote(&abt_lvl_key_column.), abt_level_cd = &abt_level_cd., abt_type_cd=&m_abt_type_cd,";
					stmt_str = "					abt_and_cons_lvl_mapping_ds_nm=" || strip("%superq(map_tbl_nm)") || ", ";
					put stmt_str;
					stmt_str = "					subset_list_ds_nm = " || strip("%superq(fnl_sub_evt_tbl_nm)") || ", ";
					put stmt_str;
					stmt_str = "					out_ds_nm =  " || strip("%superq(evt_ds_nm)") || ");";
					put stmt_str;
					put ;
					put '%dabt_err_chk(type=DATA);';
					put '%dabt_err_chk(type=SQL);';
					put ;
					put '/* Drop output table before creating */';
					stmt_str = '%dabt_drop_table(m_table_nm=' || "%superq(output_table_cd)_et1);";
					put stmt_str;
					put ;
					put '/* Creating table containing the event date information for the required population. */';
					put ;
					put 'proc sql noprint &sql_option.;';
					put "	create table %superq(output_table_cd)_et1 as";
					%if %kupcase("&event_data_type") eq "DAT" %then %do;
						stmt_str = "	select t1.*, case when &event_dt ne . and &event_dt <= " || '&DABT_LOAD_USER_INPUT_DATE then ' || "&event_dt else " || '&DABT_LOAD_USER_INPUT_DATE end as event_date_col';
					%end;
					%else %do;
						stmt_str = "	select t1.*, case when &event_dt ne . and &event_dt <= " || '&DABT_LOAD_USER_INPUT_DTTM then ' || "&event_dt else " || '&DABT_LOAD_USER_INPUT_DTTM end as event_date_col';
					%end;
					put stmt_str;
					/* in case the level is abt level then perform following */
					%if &main_lvl_flg eq Y %then %do;
						stmt_str = "	from " || strip("%superq(fnl_sub_evt_tbl_nm)") || " t1 left join " || strip("%superq(evt_ds_nm)") || " t2";
					%end;
					%else %do; /* In case of consolidated var, consider the map table for join */
						stmt_str = "	from " || strip("%superq(map_tbl_nm)") || " t1 left join " || strip("%superq(evt_ds_nm)") || " t2";
					%end;
					put stmt_str;
					stmt_str = "on (";
						%do key_j = 1 %to &key_cnt;
							%let key_tkn =  %scan(%quote(&lvl_key_com),&key_j,%str(,));
							stmt_str = strip(stmt_str) || "t1.&key_tkn. = t2.&key_tkn.";
							%if &key_j lt &key_cnt %then %do;
								stmt_str = strip(stmt_str) || " and ";
							%end;
						%end;
					stmt_str = '	' || strip(stmt_str) || ");";
					put stmt_str;
					put 'quit';
					put '%dabt_err_chk(type=SQL);';
					put ;
				%end;
				%else %do; /* This block will be called only for cons var, if the event is to be computed as per main level*/
					put '/* Drop output table before creating */';
					stmt_str = '%dabt_drop_table(m_table_nm=' || "%superq(output_table_cd)_et1);";
					put stmt_str;
					put ;
					put '/* Creating table containing the event date information for the required population. */';
					put ;
					put 'proc sql noprint &sql_option.;';
					put "	create table %superq(output_table_cd)_et1 as";
					%if %kupcase("&event_data_type") eq "DAT" %then %do;
						stmt_str = "	select t1.*, dabt_event_date as event_date_col";
					%end;
					%else %do;
						stmt_str = "	select t1.*, dabt_event_dttm as event_date_col";
					%end;
					put stmt_str;
					stmt_str = "	from " || strip("%superq(map_tbl_nm)") || " t1 left join " || strip("%superq(m_main_evt_lvl_ds)") || "_rk t2";
					put stmt_str;
					stmt_str = "on (";
						%do abt_key_j = 1 %to &abt_key_cnt;
							%let abt_key_tkn =  %scan(%quote(&abt_lvl_key_column),&abt_key_j,%str(,));
							stmt_str = strip(stmt_str) || "t1.&abt_key_tkn. = t2.&abt_key_tkn.";
							%if &abt_key_j lt &abt_key_cnt %then %do;
								stmt_str = strip(stmt_str) || " and ";
							%end;
						%end;
					stmt_str = '	' || strip(stmt_str) || ");";
					put stmt_str;
					put 'quit';
					put '%dabt_err_chk(type=SQL);';
					put ;
				%end;
				put '/* Drop output table before creating */';
				stmt_str = '%dabt_drop_table(m_table_nm=' || "%superq(output_table_cd)_e1);";
				put stmt_str;
				put ;
			run;
			%dabt_err_chk(type=DATA);	

			/* Code to get date and datetime field for event date */
			%let select_event_col = ;
			%dabt_dbms_date_processing(m_in_col_nm=event_date_col, m_data_type=&event_data_type,
									m_out_mcr_var_nm=select_event_col, m_dt_col_nm=dabt_event_date, m_dttm_col_nm=dabt_event_dttm);
			/* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
			%dabt_err_chk(type=SQL);
			%dabt_err_chk(type=DATA);
				

			%let in_evt_fnl_ds_nm = %superq(output_table_cd)_et1;

			/* The rk table is created irrespective of whether any rk driven source is present.
				As this table will be used in case of extraction of source table, for taking the min and max event date. */
			%if &avg_col_nm ne %then %do;
				/* If mapping and master table are same, then get the infromation directly.
					Else for avg dates, join with master table */
				%if &avg_in_sub_flg eq Y %then %do;
					data _null_;
						file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
						length stmt_str $32767.;
						put ;
						put '/* Creating table containing event and average date and dttm column */';
						put ;
						put 'proc sql noprint &sql_option.;';
						put "	create table %superq(output_table_cd)_e1 as";
						/* in case the level is abt level then perform following */
						%if &main_lvl_flg eq Y %then %do;
							put "	select &lvl_key_com, dabt_avg_date, dabt_avg_dttm, &select_event_col";
						%end;
						%else %do; /* In case of consolidated variable, add the abt level key column as well */
							put "	select &abt_lvl_key_column, &lvl_key_com, dabt_avg_date, dabt_avg_dttm, &select_event_col";
						%end;
						stmt_str = "	from " || strip("%superq(in_evt_fnl_ds_nm)") || ";";
						put stmt_str;
						put 'quit;	';
						put '%dabt_err_chk(type=SQL);';
					run;
					%dabt_err_chk(type=DATA);
					
				%end;/* End of if source table contains the avg date */
				%else %do;
					data _null_;
						file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
						length stmt_str $32767.;
						put ;
						put '/* Creating table containing event and average date and dttm column */';
						put ;
						put 'proc sql noprint &sql_option.;';
						put "	create table %superq(output_table_cd)_e1 as";
						stmt_str = "select ";
						/* in case the level is consolidated level then add the abt level key column */
						%if &main_lvl_flg ne Y %then %do;
							%do abt_key_j = 1 %to &abt_key_cnt;
								%let abt_key_tkn =  %scan(%quote(&abt_lvl_key_column),&abt_key_j,%str(,));
								stmt_str = strip(stmt_str) || " t1.&abt_key_tkn.,"; /* i18nOK:Line */ 
							%end;
							stmt_str = '	' || strip(stmt_str); /* i18nOK:Line */ 
							put stmt_str;
							stmt_str = "";  /* i18nOK:Line */ 
						%end;
						/* Obtain the number of columns which are part of the key, with table identifier */
						%do key_j = 1 %to &key_cnt;
							%let key_tkn =  %scan(%quote(&lvl_key_com),&key_j,%str(,));
							stmt_str = strip(stmt_str) || " t1.&key_tkn.,";
						%end;
						stmt_str = '	' || strip(stmt_str);
						put stmt_str;	
						/* No table identifier, as subset table contains only key column, hence the date column only comes from master table. 
							Also the macro select_avg_col contains two column names avg_date and avg_dttm (with the appropriate expressions,
							hence cannot directly put the table identifier. */
						put "	&select_avg_col, &select_event_col";
						stmt_str = "	from " || strip("%superq(in_evt_fnl_ds_nm)") || " as t1 inner join " || strip("&mstr_tbl_nm") || " as t2";
						put stmt_str;
						stmt_str = "on (";
						%do key_j = 1 %to &key_cnt;
							%let key_tkn =  %scan(%quote(&lvl_key_com),&key_j,%str(,));
							stmt_str = strip(stmt_str) || "t1.&key_tkn. = t2.&key_tkn.";
							%if &key_j lt &key_cnt %then %do;
							stmt_str = strip(stmt_str) || " and ";
							%end;
						%end;
						stmt_str = '	' || strip(stmt_str) || ") ";
						put stmt_str;
							/* The columns in the where clause contain the table identifier */
						%if "%superq(m_valid_mstr_dts_where_clause)" ne "" %then %do;
							stmt_str = "	where " || strip("%superq(m_valid_mstr_dts_where_clause)");
							put stmt_str;
						%end;
						put '	;';
						put 'quit;';
						put '%dabt_err_chk(type=SQL);';
					run;
					%dabt_err_chk(type=DATA);				
				%end;
			%end;
			%else %do;
				data _null_;
					file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
					length stmt_str $32767.;
					put ;
					put '/* Creating table containing event date and dttm column */';
					put ;
					put 'proc sql noprint &sql_option.;';
					put "	create table %superq(output_table_cd)_e1 as";
					/* in case the level is abt level then perform following */
					%if &main_lvl_flg eq Y %then %do;
						put "	select &lvl_key_com, &select_event_col";
					%end;
					%else %do; /* In case of consolidated variable, add the abt level key column as well */
						put "	select &abt_lvl_key_column, &lvl_key_com, &select_event_col";
					%end;
					stmt_str = "	from " || strip("%superq(in_evt_fnl_ds_nm)") || ";";
					put stmt_str;
					put 'quit;';
					put '%dabt_err_chk(type=SQL);';
				run;
				%dabt_err_chk(type=DATA);			
			%end;

			%let in_tbl_nm = %superq(output_table_cd)_e1;
			%let out_tbl_nm = %superq(output_table_cd)_e2;

			%if &adjust_event_dt_flg eq Y or &out_beh_cnt gt 0 or &out_rnt_cnt gt 0 or &out_spm_cnt gt 0 or &out_ext_cnt gt 0 %then %do;
				data _null_;
					file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
					length stmt_str $32767.;
					put ;
					put '/* Drop output table before creating */';
					stmt_str = '%dabt_drop_table(m_table_nm=' || "%superq(out_tbl_nm));";
					put stmt_str;
					put ;
					put '/* Creating table containing the adjusted event date and dttm column and corresponding outcome date and dttm based on the event date column */';
					put '/* (dates are adjusted as of last month end date, if the source frequency is monthly.) */';
					put ;
					put 'proc sql noprint &sql_option.;';
					put "	create table %superq(out_tbl_nm) as";
					put "	select t1.*";
					%if &out_beh_cnt gt 0 or &out_rnt_cnt gt 0 or &out_spm_cnt gt 0 or &out_ext_cnt gt 0 %then %do;
						/* The dabt_event_date and dabt_event_out_date and correpsonding dttm field is always created as they are used in REFERENCE_DATE/DTTM as applicable. */
						stmt_str = '	, case when &DABT_OUTCOME_END_DATE <= %dabt_intnx("' || "&time_date_freq" || '",dabt_event_date,' || strip("&time_from,'same')");
						put stmt_str;
						put '	then &DABT_OUTCOME_END_DATE';
						stmt_str = '	else %dabt_intnx("' || "&time_date_freq" || '",dabt_event_date,' || strip("&time_from,'same')");
						put stmt_str;
						%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
							put	'	end as dabt_event_out_date format = nldate. informat = nldate.';
						%end;
						%else %do;
							put	'	end as dabt_event_out_date';
						%end;
						stmt_str = '	, case when &DABT_OUTCOME_END_DTTM <= %dabt_intnx("' || "&time_dttm_freq" || '",dabt_event_dttm,' || strip("&time_from,'same')");
						put stmt_str;
						put '	then &DABT_OUTCOME_END_DTTM';
						stmt_str = '	else %dabt_intnx("' || "&time_dttm_freq" || '",dabt_event_dttm,' || strip("&time_from,'same')");
						put stmt_str;
						%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
							put '	end as dabt_event_out_dttm format = nldatm. informat = nldatm.';
						%end;
						%else %do;
							put '	end as dabt_event_out_dttm';
						%end;
						
					%end;
				/* If any beh or rnt variable exists and the source grain is not day, then the event date is adjusted to the previos month end date,
							if the event date itself is not month end. */
					%if &adjust_event_dt_flg eq Y %then %do;
						%if &date_usg_exst gt 0 %then %do;
							stmt_str = '	, case when dabt_event_date ne %dabt_intnx("' || "&src_date_exp" || '",dabt_event_date,0,"' || "&src_align" || '")';
							put stmt_str; 
							stmt_str = '	then %dabt_intnx("' || "&src_date_exp" || '",dabt_event_date,-1,"' || "&src_align" || '")';
							put stmt_str;
							%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
								put	"	else dabt_event_date end as &adj_date_col_nm. format = nldate. informat = nldate.";
							%end;
							%else %do;
								put	"	else dabt_event_date end as &adj_date_col_nm.";
							%end;
						%end;
						%if &dttm_usg_exst gt 0 %then %do;
							stmt_str = '	, case when dabt_event_dttm ne %dabt_intnx("' || "&src_dttm_exp" || '",dabt_event_dttm,0,"' || "&src_align" || '")';
							put stmt_str; 
							stmt_str = '	then %dabt_intnx("' || "&src_dttm_exp" || '",dabt_event_dttm,-1,"' || "&src_align" || '") ';
							put stmt_str;
							%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
								put	"	else dabt_event_dttm end as &adj_dttm_col_nm. format = nldatm. informat = nldatm.";
							%end;
							%else %do;
								put	"	else dabt_event_dttm end as &adj_dttm_col_nm.";
							%end;
						%end;
					%end;
					stmt_str = "	from " || strip("%superq(in_tbl_nm)") || " as t1;";
					put stmt_str;
					put 'quit;';
					put '%dabt_err_chk(type=SQL);';
					put ;
					put '	/* If the debug flag is set to N, then the temporary datasets and files are deleted */';
					stmt_str = '%dabt_clean_if_debug_on(m_table_nm=' || strip("%superq(in_tbl_nm)") || ");";
					put stmt_str;
					put ;	
				run;
				%dabt_err_chk(type=DATA);
				
				%let in_tbl_nm = %superq(out_tbl_nm);
				%let out_tbl_nm = %superq(output_table_cd)_e3;

			%end;

			%if &adjust_event_out_dt_flg eq Y %then %do;
				data _null_;
					file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
					length stmt_str $32767.;
					put ;
					put '/* Drop output table before creating */';
					stmt_str = '%dabt_drop_table(m_table_nm=' || "%superq(out_tbl_nm));";
					put stmt_str;
					put ;
					put '/* Creating table containing adjusted outcome event date and dttm column (dates are adjusted as of last month end date, if the source frequency is monthly.) */';
					put ;
					put 'proc sql noprint &sql_option.;';
					put "	create table %superq(out_tbl_nm) as";
					put "	select t1.*";
					/* For any outocme variable: If any beh or rnt variable exists and the source grain is not day, 
						then the event date is adjusted to the previos month end date,
						if the event date itself is not month end. */
					/* Start: modified against track 7611446073. Outcome end date is obtained from the adjusted event date moving outcome period ahead */
					%if &date_usg_exst gt 0 %then %do;
						stmt_str = '	, case when &DABT_OUTCOME_END_DATE <= %dabt_intnx("' || "&time_date_freq" || '",' || "&adj_date_col_nm.," || strip("&time_from,'") || "&src_align" || "')";
						put stmt_str;
						put '	then &DABT_OUTCOME_END_DATE';
						stmt_str = '	else %dabt_intnx("' || "&time_date_freq" || '",' || "&adj_date_col_nm.," || strip("&time_from,'") || "&src_align" || "')";
						put stmt_str;
						%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
							put	"	end as &adj_out_date_col_nm format = nldate. informat = nldate.";
						%end;
						%else %do;
							put	"	end as &adj_out_date_col_nm";
						%end;
					%end;
					%if &out_dttm_usg_exst gt 0 %then %do;
						stmt_str = '	, case when &DABT_OUTCOME_END_DTTM <= %dabt_intnx("' || "&time_dttm_freq" || '",' || "&adj_dttm_col_nm.," || strip("&time_from,'") || "&src_align" || "')";
						put stmt_str;
						put '	then &DABT_OUTCOME_END_DTTM';
						stmt_str = '	else %dabt_intnx("' || "&time_dttm_freq" || '",' || "&adj_dttm_col_nm.," || strip("&time_from,'") || "&src_align" || "')";
						put stmt_str;
						%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
							put "	end as &adj_out_dttm_col_nm format = nldatm. informat = nldatm.";
						%end;
						%else %do;
							put "	end as &adj_out_dttm_col_nm";
						%end;
					%end;
					/* End: modified against track 7611446073*/
					stmt_str = "	from " || strip("%superq(in_tbl_nm)") || " as t1;";
					put stmt_str;
					put 'quit;';
					put '%dabt_err_chk(type=SQL);';
					put ;
					put '	/* If the debug flag is set to N, then the temporary datasets and files are deleted */';
					stmt_str = '%dabt_clean_if_debug_on(m_table_nm=' || strip("%superq(in_tbl_nm)") || ");";
					put stmt_str;
					put ;
				run;
				%dabt_err_chk(type=DATA);
				
				%let in_tbl_nm = %superq(out_tbl_nm);
				%let out_tbl_nm = %superq(output_table_cd)_e4;
			%end;

			data _null_;
				file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
				length stmt_str $32767.;
				put ;
				put '/* Drop output table before creating */';
				stmt_str = '%dabt_drop_table(m_table_nm=' || "%superq(out_tbl_nm));";
				put stmt_str;
				put ;
				put '/* Creating table containing the event date/dttm for distinct times based on the time period required for spm/beh/rnt variables */';
				put ;
				put 'proc sql noprint &sql_option.;';
				put "	create table %superq(out_tbl_nm) as";
				put "	select t1.*";
				%if &spm_cnt gt 0 %then %do;
					%let tot_time_list =  %eval(%sysfunc(countc(%quote(&spm_time_list),","))+1);
					%do time_cnt = 1 %to &tot_time_list;
						%let time_list_tkn = %scan(%quote(&spm_time_list),&time_cnt,%str(,));/* i18nOK:Line */
						%let get_time_tkn = %substr(&time_list_tkn,1,%length(&time_list_tkn)-1);
						%if &spm_dttm_usg_exst gt 0 %then %do;
							%let dttm_int_tkn = %scan(%quote(&spm_dttm_int_exp),&time_cnt,%str(,));/* i18nOK:Line */
							%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
								stmt_str = '	, %dabt_intnx("' || "&dttm_int_tkn" || '",dabt_event_dttm,' || "-&get_time_tkn,'same') as dabt_spm_event_dttm_&time_list_tkn. format = nldatm. informat = nldatm.";
							%end;
							%else %do;
								stmt_str = '	, %dabt_intnx("' || "&dttm_int_tkn" || '",dabt_event_dttm,' || "-&get_time_tkn,'same') as dabt_spm_event_dttm_&time_list_tkn.";
							%end;
							put stmt_str;
						%end;
						%if &spm_date_usg_exst gt 0 %then %do;
							%let date_int_tkn = %scan(%quote(&spm_date_int_exp),&time_cnt,%str(,));/* i18nOK:Line */
							%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
								stmt_str = '	, %dabt_intnx("' || "&date_int_tkn" || '", dabt_event_date,' || "-&get_time_tkn,'same') as dabt_spm_event_date_&time_list_tkn. format = nldate. informat = nldate.";
							%end;
							%else %do;
								stmt_str = '	, %dabt_intnx("' || "&date_int_tkn" || '", dabt_event_date,' || "-&get_time_tkn,'same') as dabt_spm_event_date_&time_list_tkn. ";
							%end;
							put stmt_str;
						%end;
					%end;
				%end;
				%if &out_spm_cnt gt 0 %then %do;
					%let tot_time_list =  %eval(%sysfunc(countc(%quote(&out_spm_time_list),","))+1);
					%do time_cnt = 1 %to &tot_time_list;
						%let time_list_tkn = %scan(%quote(&out_spm_time_list),&time_cnt,%str(,));/* i18nOK:Line */
						%let get_time_tkn = %substr(&time_list_tkn,1,%length(&time_list_tkn)-1);
						%if &out_spm_dttm_usg_exst gt 0 %then %do;
							%let dttm_int_tkn = %scan(%quote(&out_spm_dttm_int_exp),&time_cnt,%str(,));/* i18nOK:Line */
							%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
								stmt_str = '	, %dabt_intnx("' || "&dttm_int_tkn" || '",dabt_event_out_dttm,' || "-&get_time_tkn,'same') as dabt_spm_event_out_dttm_&time_list_tkn. format = nldatm. informat = nldatm.";
							%end;
							%else %do;
								stmt_str = '	, %dabt_intnx("' || "&dttm_int_tkn" || '",dabt_event_out_dttm,' || "-&get_time_tkn,'same') as dabt_spm_event_out_dttm_&time_list_tkn.";

							%end;
							put stmt_str;
						%end;
						%if &out_spm_date_usg_exst gt 0 %then %do;
							%let date_int_tkn = %scan(%quote(&out_spm_date_int_exp),&time_cnt,%str(,));/* i18nOK:Line */
							%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
								stmt_str = '	, %dabt_intnx("' || "&date_int_tkn" || '",dabt_event_out_date,' || "-&get_time_tkn,'same') as dabt_spm_event_out_date_&time_list_tkn.";
							%end;
							%else %do;
								stmt_str = '	, %dabt_intnx("' || "&date_int_tkn" || '",dabt_event_out_date,' || "-&get_time_tkn,'same') as dabt_spm_event_out_date_&time_list_tkn. format = nldate. informat = nldate.";
							%end;
							put stmt_str;
						%end;
					%end;
				%end;
				/* Calculate event date for each distinct time from and time to*/
				%if &beh_cnt gt 0 or &rnt_cnt gt 0 %then %do;
					%let tot_time_list =  %eval(%sysfunc(countc(%quote(&in_time_list),","))+1);
					%do time_cnt = 1 %to &tot_time_list;
						%let time_list_tkn = %scan(%quote(&in_time_list),&time_cnt,%str(,));/* i18nOK:Line */
						%let get_time_tkn = %substr(&time_list_tkn,1,%length(&time_list_tkn)-1);
						%if &dttm_usg_exst gt 0 %then %do;
							%let dttm_int_tkn = %scan(%quote(&in_dttm_int_exp),&time_cnt,%str(,));/* i18nOK:Line */
							%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
								stmt_str = '	, %dabt_intnx("' || "&dttm_int_tkn" || '",' || "&adj_dttm_col_nm.,-&get_time_tkn,'&src_align') as &adj_dttm_col_nm._&time_list_tkn. format = nldatm. informat = nldatm.";
							%end;
							%else %do;
								stmt_str = '	, %dabt_intnx("' || "&dttm_int_tkn" || '",' || "&adj_dttm_col_nm.,-&get_time_tkn,'&src_align') as &adj_dttm_col_nm._&time_list_tkn.";
							%end;
							put stmt_str;
						%end;
						%if &date_usg_exst gt 0 %then %do;
							%let date_int_tkn = %scan(%quote(&in_date_int_exp),&time_cnt,%str(,));/* i18nOK:Line */
							%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
								stmt_str = '	, %dabt_intnx("' || "&date_int_tkn" || '",' || "&adj_date_col_nm.,-&get_time_tkn,'&src_align') as &adj_date_col_nm._&time_list_tkn.";
							%end;
							%else %do;
								stmt_str = '	, %dabt_intnx("' || "&date_int_tkn" || '",' || "&adj_date_col_nm.,-&get_time_tkn,'&src_align') as &adj_date_col_nm._&time_list_tkn. format = nldate. informat = nldate.";
							%end;
							put stmt_str;
						%end;
					%end;
				%end;
				/* For outcome variable: Calculate event date for each distinct time from and time to*/
				%if &out_beh_cnt gt 0 or &out_rnt_cnt gt 0 %then %do;
					%let tot_time_list =  %eval(%sysfunc(countc(%quote(&out_time_list),","))+1);
					%do time_cnt = 1 %to &tot_time_list;
						%let time_list_tkn = %scan(%quote(&out_time_list),&time_cnt,%str(,));/* i18nOK:Line */
						%let get_time_tkn = %substr(&time_list_tkn,1,%length(&time_list_tkn)-1);
						%if &out_dttm_usg_exst gt 0 %then %do;
							%let dttm_int_tkn = %scan(%quote(&out_dttm_int_exp),&time_cnt,%str(,));/* i18nOK:Line */
							%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
								stmt_str = '	, %dabt_intnx("' || "&dttm_int_tkn" || '",' || "&adj_out_dttm_col_nm.,-&get_time_tkn,'&src_align') as &adj_out_dttm_col_nm._&time_list_tkn. format = nldatm. informat = nldatm.";
							%end;
							%else %do;
								stmt_str = '	, %dabt_intnx("' || "&dttm_int_tkn" || '",' || "&adj_out_dttm_col_nm.,-&get_time_tkn,'&src_align') as &adj_out_dttm_col_nm._&time_list_tkn.";
							%end;
							put stmt_str;
						%end;
						%if &out_date_usg_exst gt 0 %then %do;
							%let date_int_tkn = %scan(%quote(&out_date_int_exp),&time_cnt,%str(,));/* i18nOK:Line */
							%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
								stmt_str = '	, %dabt_intnx("' || "&date_int_tkn" || '",' || "&adj_out_date_col_nm.,-&get_time_tkn,'&src_align') as &adj_out_date_col_nm._&time_list_tkn. format = nldate. informat = nldate.";
							%end;
							%else %do;
								stmt_str = '	, %dabt_intnx("' || "&date_int_tkn" || '",' || "&adj_out_date_col_nm.,-&get_time_tkn,'&src_align') as &adj_out_date_col_nm._&time_list_tkn.";
							%end;
							put stmt_str;
						%end;
					%end;
				%end;
				put "	from %superq(in_tbl_nm) as t1;";
				put 'quit;';
				put '%dabt_err_chk(type=SQL);';
				put ;
				put ;
				put '	/* If the debug flag is set to N, then the temporary datasets and files are deleted */';
				stmt_str = '%dabt_clean_if_debug_on(m_table_nm=' || "%superq(in_tbl_nm));";
				put stmt_str;
				put ;
				put ;
				put '/* Creating rk table containing the required dates based on the variable */';
				put ;
				put 'proc sql noprint &sql_option.;';
				put "	create table %superq(output_table_cd)_rk as";
				put "	select t1.*";
				/* Calculate event date for each distinct time from and time to*/
				%if &beh_cnt gt 0 or &rnt_cnt gt 0 %then %do;
					%if &in_avg_time_list ne %then %do;
						%let tot_time_list =  %eval(%sysfunc(countc(%quote(&in_avg_time_list),","))+1);
						%do time_cnt = 1 %to &tot_time_list;
							%let time_list_tkn = %scan(%quote(&in_avg_time_list),&time_cnt,%str(,));/* i18nOK:Line */
							%let get_time_from_tkn = %scan(%quote(&time_list_tkn),1,%str(_));
							%let get_time_to_tkn = %scan(%quote(&time_list_tkn),2,%str(_));
							%let get_time_freq_tkn = %scan(%quote(&time_list_tkn),3,%str(_));
							%if &dttm_usg_exst gt 0 %then %do;
								%let dttm_int_tkn = %scan(%quote(&in_avg_dttm_int_exp),&time_cnt,%str(,));/* i18nOK:Line */
								put "	, case when dabt_avg_dttm <= &adj_dttm_col_nm._&get_time_to_tkn.&get_time_freq_tkn";
								put "	then &get_time_to_tkn - &get_time_from_tkn";
								stmt_str = '	else %dabt_intck("' || "&dttm_int_tkn" || '",dabt_avg_dttm,' || "&adj_dttm_col_nm._&get_time_from_tkn.&get_time_freq_tkn) + 1";
								put stmt_str;
								%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
									put "	end as dabt_avg_dttm_&time_list_tkn. format = nldatm. informat = nldatm.";
								%end;
								%else %do;
									put "	end as dabt_avg_dttm_&time_list_tkn.";
								%end;
							%end;
							%if &date_usg_exst gt 0 %then %do;
								%let date_int_tkn = %scan(%quote(&in_avg_date_int_exp),&time_cnt,%str(,));/* i18nOK:Line */
								put "	, case when dabt_avg_date <= &adj_date_col_nm._&get_time_to_tkn.&get_time_freq_tkn";
								put "	then &get_time_to_tkn - &get_time_from_tkn";
								stmt_str = '	else %dabt_intck("' || "&date_int_tkn" || '",dabt_avg_date,' || "&adj_date_col_nm._&get_time_from_tkn.&get_time_freq_tkn) + 1";
								put stmt_str;
								%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
									put "	end as dabt_avg_date_&time_list_tkn. format = nldate. informat = nldate.";
								%end;
								%else %do;
									put "	end as dabt_avg_date_&time_list_tkn.";
								%end;								
							%end;
						%end;
					%end;
				%end;
				/* For outcome variable: Calculate event date for each distinct time from and time to*/
				%if &out_beh_cnt gt 0 or &out_rnt_cnt gt 0 %then %do;
					%if &out_avg_time_list ne %then %do;
						%let tot_time_list =  %eval(%sysfunc(countc(%quote(&out_avg_time_list),","))+1);
						%do time_cnt = 1 %to &tot_time_list;
							%let time_list_tkn = %scan(%quote(&out_avg_time_list),&time_cnt,%str(,));/* i18nOK:Line */
							%let get_time_from_tkn = %scan(%quote(&time_list_tkn),1,%str(_));
							%let get_time_to_tkn = %scan(%quote(&time_list_tkn),2,%str(_));
							%let get_time_freq_tkn = %scan(%quote(&time_list_tkn),3,%str(_));
							%if &out_dttm_usg_exst gt 0 %then %do;
								%let dttm_int_tkn = %scan(%quote(&out_avg_dttm_int_exp),&time_cnt,%str(,));/* i18nOK:Line */
								put "	, case when dabt_avg_dttm <= &adj_out_dttm_col_nm._&get_time_to_tkn.&get_time_freq_tkn";
								put "	then &get_time_to_tkn - &get_time_from_tkn";
								stmt_str = '	else %dabt_intck("' || "&dttm_int_tkn" || '",dabt_avg_dttm,' || "&adj_out_dttm_col_nm._&get_time_from_tkn.&get_time_freq_tkn) + 1";
								put stmt_str;
								%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
									put "	end as dabt_avg_out_dttm_&time_list_tkn. format = nldatm. informat = nldatm.";
								%end;
								%else %do;
									put "	end as dabt_avg_out_dttm_&time_list_tkn.";
								%end;
								
							%end;
							%if &out_date_usg_exst gt 0 %then %do;
								%let date_int_tkn = %scan(%quote(&out_avg_date_int_exp),&time_cnt,%str(,));/* i18nOK:Line */
								put "	, case when dabt_avg_date <= &adj_out_date_col_nm._&get_time_to_tkn.&get_time_freq_tkn";
								put "	then &get_time_to_tkn - &get_time_from_tkn";
								stmt_str = '	else %dabt_intck("' || "&date_int_tkn" || '",dabt_avg_date,' || "&adj_out_date_col_nm._&get_time_from_tkn.&get_time_freq_tkn) + 1";
								put stmt_str;
								%if &dabt_db_eng.=HADOOP %then %do; /* i18nOK:Line */
									put "	end as dabt_avg_out_date_&time_list_tkn. format = nldate. informat = nldate.";
								%end;
								%else %do;
									put "	end as dabt_avg_out_date_&time_list_tkn.";
								%end;
							%end;
						%end;
					%end;
				%end;
				put "	from %superq(out_tbl_nm) as t1;";
				put 'quit;';
				put '%dabt_err_chk(type=SQL);';
				put ;
				put ;
				put '	/* If the debug flag is set to N, then the temporary datasets and files are deleted */';
				stmt_str = '%dabt_clean_if_debug_on(m_table_nm=' || "%superq(out_tbl_nm));";
				put stmt_str;
				%if &sk_driven_src gt 0 %then %do;
					put ;
					put ;
					put '/* Creating sk table containing the required dates based on the variable */';
					put ;
					put 'proc sql noprint &sql_option.;';
					put "	create table %superq(output_table_cd)_sk as";
					put "	select t1.*, &lvl_sk_com";
					put "	from %superq(output_table_cd)_rk as t1 inner join &mstr_tbl_nm as t2";
					stmt_str = "on (";
					%do key_j = 1 %to &key_cnt;
						%let key_tkn =  %scan(%quote(&lvl_key_com),&key_j,%str(,));
						stmt_str = strip(stmt_str) || " t1.&key_tkn. = t2.&key_tkn. ";
						%if &key_j lt &key_cnt %then %do;
							stmt_str = strip(stmt_str) || " and ";
						%end;
					%end;
					stmt_str = '	' || strip(stmt_str) || ") ;";
					put stmt_str;
					put 'quit;';
					put '%dabt_err_chk(type=SQL);';
				%end;
				put ;
				put '	/* If the debug flag is set to N, then the temporary datasets and files are deleted */';
				%if &m_call_event_macro_flg eq Y %then %do;
					stmt_str = '%dabt_clean_if_debug_on(m_table_nm=' || "%superq(evt_ds_nm));";
					put stmt_str;
					put ;
				%end;
				stmt_str = '%dabt_clean_if_debug_on(m_table_nm=' || "%superq(in_evt_fnl_ds_nm));";
				put stmt_str;
				put ;
				%if &main_lvl_flg ne Y %then %do; /* Drop the map table in case of consolidated var */
					stmt_str = '%dabt_clean_if_debug_on(m_table_nm=' || "%superq(map_tbl_nm), m_chk_exst=Y);";
					put stmt_str;
					put ;
				%end;
			run;
			%dabt_err_chk(type=DATA);
		%end;

		data _null_;
			file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
			put ;
			put '	/* If the debug flag is set to N, then the temporary datasets and files are deleted */';
			stmt_str = '%dabt_clean_if_debug_on(m_table_nm=' || "%superq(output_table_cd)_t1, m_chk_exst=Y);";
			put stmt_str;
			put ;
		run;
		%dabt_err_chk(type=DATA);
	%end;
	%if &m_build_abt_flg eq Y %then %do;
		/* If the file exists then include the file that contains sql query to be executed.
		   This will create the table containing the subset population. 
		   If the file does not exists then set the job_rc flag to 5.*/
		%if %sysfunc(fileexist("&m_deployed_code_path./&job_id..sas")) %then %do;
			%include "&m_deployed_code_path./&job_id..sas" /lrecl=64000;
		%end;
		%else %do;
			%let job_rc = 5;
		%end;
		
	%end;
	/* i18nOK:END */

%mend dabt_get_lvl_wise_subset_dtl;
