jQuery.sap.declare("sas.hc.ui.core.Configuration");

jQuery.sap.require("sap.ui.base.Object");

/**
 * Creates a new Configuration object for sas configurations.
 *
 * @class Collects and stores the configuration of the current environment.
 *
 * The Configuration is initialized once when the {@link sas.hc.core.Core#bootSasHC} is called.
 *
 * @extends sap.ui.base.Object
 * @author sinyas
 * @constructor
 * @public
 * @name sas.hc.ui.core.Configuration
 */

sap.ui.base.Object.extend("sas.hc.ui.core.Configuration", {

    constructor:function(){
        this.parentConf = sap.ui.getCore().getConfiguration();
        // read all configurations.
        // definition of supported settings
        var M_SETTINGS = {
            "appName"                         : { type : "string",   defaultValue : undefined},
            "appId"                           : { type : "string",   defaultValue : undefined},
            "contextPath"                     : { type : "string",   defaultValue : undefined},
            "crossDomainProxy"                : { type : "string",   defaultValue : "CASedCrossDomainProxy" },
            "wipRegistryUrl"                  : { type : "string",   defaultValue : undefined},
            "commonsRegistryUrl"              : { type : "string",   defaultValue : undefined},
            "webdocUrl"                       : { type : "string",   defaultValue : undefined},
            "sessionMaxInActiveInterval"      : { type : "int",      defaultValue : undefined},
            "sessionTimeoutUrl"               : { type : "string",   defaultValue : undefined},
            "sessionTimeoutWarningDuration"   : { type : "int",      defaultValue : undefined},
            "themeUrl"                        : { type : "string",   defaultValue : undefined},
            "iconNameOverrideLibrary"         : { type : "string" ,  defaultValue : undefined},
            "bootTask"                        : { type : "function", defaultValue : undefined, noUrl: true},
            "applicationSwitcherEnabled"      : { type : "boolean",  defaultValue : false, noUrl: true},
            "userCapabilities"                : { type : "string",   defaultValue : undefined},
            "autoMarking"                     : { type : "boolean",  defaultValue : false},
            "autoMarkingStackTraceLimit"      : { type : "int",  defaultValue : undefined},
            "autoMarkingStackTrace"           : { type : "string",  defaultValue : undefined},
            "measureStringLengthBy"         : { type : "string",  defaultValue : "Chars"}
        };

        /* Object that carries the real configuration data */
        var config = this;

        function setValue(sName, sValue) {
            if ( typeof sValue === "undefined" || sValue === null ) {
                return;
            }
            switch(M_SETTINGS[sName].type) {
                case "boolean":
                    if ( typeof sValue === "string" ) {
                        if (M_SETTINGS[sName].defaultValue) {
                            config[sName] = sValue.toLowerCase() !== "false";
                        } else {
                            config[sName] = sValue.toLowerCase() === "true" || sValue.toLowerCase() === "x";
                        }
                    } else {
                        // boolean etc.
                        config[sName] = !!sValue;
                    }
                    break;
                case "string":
                    config[sName] = "" + sValue; // enforce string
                    break;
                case "int":
                    config[sName] = parseInt(sValue); // enforce integer
                    break;
                case "code":
                    config[sName] = typeof sValue === "function" ? sValue : String(sValue);
                    break;
                case "function":
                    if ( typeof sValue !== "function" ) {
                        throw new Error("unsupported value");
                    }
                    config[sName] = sValue;
                    break;
                case "string[]":
                    if ( jQuery.isArray(sValue) ) {
                        config[sName] = sValue;
                    } else if ( typeof sValue === "string" ) {
                        config[sName] = jQuery.map(sValue.split(/[ ,;]/), function($) {
                            return jQuery.trim($);
                        });
                    } else {
                        throw new Error("unsupported value");
                    }
                    break;
                default:
                    throw new Error("illegal state");
            }
        }




        // collect the defaults
        for (var n in M_SETTINGS ) {
            if (M_SETTINGS.hasOwnProperty(n)) {
                config[n] = M_SETTINGS[n].defaultValue;
                // also add a getter function.
                // capitalize first letter.
                var sFunName = 'get'+n.charAt(0).toUpperCase()+ n.substring(1);
                this._delegator(sFunName,n);
            }
        }

        // apply settings from global config object (already merged with script tag attributes)
        var oGlbConfig = window["sap-ui-config"] || {};
        var oCfg = oGlbConfig.sas || {};
        for (var n in M_SETTINGS) {
            if (M_SETTINGS.hasOwnProperty(n)) {
                setValue(n, oCfg[n]);
            }
        }

        // apply the settings from the url (only if not blocked by app configuration)
        if ( !oGlbConfig.ignoreUrlParams ) {
            var sUrlPrefix = "sap-ui-sas-";
            var oUriParams = jQuery.sap.getUriParameters();

            // now analyze sap-ui parameters
            for (var n in M_SETTINGS) {
                if ( M_SETTINGS[n].noUrl ) {
                    continue;
                }
                var sValue = oUriParams.get(sUrlPrefix + n);
                // Allow passing of these system wide settings with simple request parm names
                // ?themeUrl=http://...
                // vs
                // ?sap-ui-sas-themeUrl=http://
                if(sValue===null)
                    sValue=oUriParams.get(n);

                if (sValue === "") {
                    //empty URL parameters set the parameter back to its system default
                    config[n] = M_SETTINGS[n].defaultValue;
                } else {
                    //sets the value (null or empty value ignored)
                    setValue(n, sValue);
                };
            };
        }

        this.oConf = oCfg;

        // add a generic method to return non common config.
        config.get = function(property){
            return config.oConf[property];
        };
    }  ,
    _delegator : function (name,prop) {
        this[name] = function() {
            return this[prop];
        };
    },
    /** Add ability to dynamically add configuration
     *
     * @param {string} name TODO-jsdoc
     * @param {*} val TODO-jsdoc
     */
    setValue: function(name, val) {
        this[name] = val;
        var sFunName = 'get'+name.charAt(0).toUpperCase()+ name.substring(1);
        this._delegator(sFunName,name);
    }
});



/*
* SAS extension of sas.services.RestService for HTML Commons.
* Copyright 2014 SAS Institute Inc.
* All Rights Reserved.
*/
jQuery.sap.declare("sas.hc.ui.services.RestService");

jQuery.sap.require("sap.ui.base.Object");
(function(){
    var urlPathFinderRegX = /{([^}]+)}/g;
    /**
     * @class
     * Creates an instance of RestService as wrapper class for communicating with REST services.
     * @author sinyas
     */
    sap.ui.base.Object.extend("sas.hc.ui.services.RestService", {
        name:null,
        url:null,
        crossDomain:undefined,
        authenticate:undefined,

        /**
         * Creates an instance of RestService as wrapper class for communicating with REST services.
         * @param {object} oOptions object to specify the target service url. If its cross domain call and authentication is required.
         * @param {string} oOptions.url url of the service.  Ex: 'http://foo'.
         * @param {string} oOptions.name name of the service
         * @param {boolean} oOptions.crossDomain if its a cross domain call
         * @param {boolean} oOptions.authenticate if cross domain then does it require authentication.
         * @param {string[]} oOptions.methods array of supported methods.
         * @param {string} oOptions.method supported method name E.g. GET,POST,DELETE or PUT.
         * @constructor
         * @public
         */
        constructor:function(oOptions){
            sap.ui.base.Object.apply(this,arguments);
            jQuery.extend(this,oOptions);

            // check for supported methods if any.
            if(this.methods || this.method){
                var that=this;
                // remove not supported HTTP REST methods.
                new jQuery.each(['get','post','delete','put'],function(index,method){
                    var supported = false;
                    if(that.methods){
                        if(jQuery.inArray(method,that.methods) > -1
                            || jQuery.inArray(method.toUpperCase(),that.methods) > -1
                            || jQuery.inArray(method.toLowerCase(),that.methods) > -1){
                            supported = true;
                        }
                    }
                    // check if method exist.
                    if(that.method){
                        if(that.method.toLowerCase() === method.toLowerCase()){
                            supported = true;
                        }
                    }
                    // remove if this method is not supported.
                    if(!supported){
                        if(index === 2){ // if this is delete function.
                            that['del'] = undefined;
                        }else{
                            that[method] = undefined;
                        }
                    }
                });
            }
        }
    });
    sas.hc.ui.services.RestService.prototype.get = function(oOptions){
        return this._send(oOptions, 'GET');
    };
    sas.hc.ui.services.RestService.prototype.post= function(oOptions){
        return this._send(oOptions, 'POST');
    };
    sas.hc.ui.services.RestService.prototype.del=function(oOptions){
        return this._send(oOptions, 'DELETE');
    };
    sas.hc.ui.services.RestService.prototype.put=function(oOptions){
        return this._send(oOptions, 'PUT');
    };
    sas.hc.ui.services.RestService.prototype._send=function(oOptions,sMethod){
        oOptions = oOptions || {};
        oOptions.type = sMethod;

        // If the url was overridden in options use that instead of the one provided in constructor
        var url = oOptions.hasOwnProperty("url")===true?oOptions.url:this.url;

        // if id is present in the option then tack it on to the end of url
        if(oOptions.id){
            url += "/" + encodeURI(oOptions.id);
        }
        // if the url contains any replacement parameters in the path, replace them with the named options and remove from query params
        var modifiedUrl = url;
        var pathParams = url.match(urlPathFinderRegX);
        if(pathParams !== null){
            for (var index in pathParams) {
                if (pathParams.hasOwnProperty(index)) {
                    var pathParam = pathParams[index];
                    pathParam = pathParam.substring(1,pathParam.length-1);
                    pathParams[index] = pathParam;
                    if (typeof oOptions[pathParam] !== "undefined") {
                        modifiedUrl = modifiedUrl.replace("{" + pathParam + "}", encodeURI(oOptions[pathParam]));
                    }
                }
            }
        };
        oOptions.url = modifiedUrl;
        if(typeof this.crossDomain !== "undefined" || typeof this.authenticate !== "undefined"){
            oOptions.sas = oOptions.sas || {};
            oOptions.sas.authenticate = this.authenticate;
            oOptions.sas.crossDomain = this.crossDomain;
        }
        return sas.ajax(oOptions);
    };
}());
jQuery.sap.declare("sas.hc.ui.core.util");
(function(){
    var keepAliveService=undefined;
    sas.hc.ui.core.util={
        APP_READY: "appReady",
        BEFORE_SIGN_OUT: 'before_sign_out',

        // I think isUrlCrossDomain should live in sas.hc.ui.core.util.Url.js instead of
        // of here.   For now I just put a function there as well which references back to this one.
        // This util.js seems like it could become a real dumping ground of unrelated things.
        isUrlCrossDomain:function(sUrl){
            if(sUrl === undefined ) {
                return false;
            }
            var oUri = URI.parse(sUrl);
            if(!oUri.protocol){
                return false; // relative url ?.
            }else{
                // check if commons is externalized?
                var hcExtLoc = sas && sas.config && sas.config.get('commonsUrl');
                // externalization of html commons is not possible without supporting CORS.
                if(typeof hcExtLoc === 'string' && hcExtLoc !== ''){
                    // accessing commons resource from url.
                    var commonsCntxUrl = sas.config.get('commonsContextUrl');
                    if(typeof commonsCntxUrl === 'string' && commonsCntxUrl !== '' && sUrl.indexOf(commonsCntxUrl+"/") === 0){
                        return false;
                    }
                }

                // check if sUrl is of same protocol
                if(oUri.protocol+":" !== location.protocol){
                    return true;
                }else{
                    // check if the host name are same.
                    if(oUri.hostname !== location.hostname){
                        return true;
                    }else{
                        // https://url.spec.whatwg.org/#concept-url-port
                        // https://url.spec.whatwg.org/#port-state
                        // If buffer is equal to url's scheme's default port, set buffer to the empty string.
                        // Hence set port to empty string if its a default port as per https://url.spec.whatwg.org/#default-port
                        // No need to check for the protocol again as it was taken care in previous checks.
                        var port = (oUri.port === '80' || oUri.port === '443') ? "" : oUri.port;
                        // check if the ports are same.
                        // ports are same even when uri port is undefined and location.port is empty string
                        if((port !== null && port !== location.port)
                                    || (port === null && location.port !== "")){
                            return true;
                        }
                        // https://developer.mozilla.org/en-US/docs/Web/Security/Same-origin_policy
                        // for now ignore if both apps are from different web context.
                    }
                }
            }
            return false; // case of relative url.
        },
        getStyle:function(sLibraryName,sClassName, sProp){
            var styleSheet = this.getStyleSheet(sLibraryName);
            if(!styleSheet) {
                return;
            }
            var classes = styleSheet.rules || styleSheet.cssRules;
            if(!classes) {
                return;
            }
            sClassName = "."+sClassName;
            for(var x=0; x<classes.length; x++) {
                if(classes[x].selectorText===sClassName) {
                    var oClass = classes[x];
                    if(sProp !== undefined){
                        return oClass.style[sProp];
                    }
                    return oClass.style;
                }
            }
        },
        getStyleSheet:function(sLibraryName){
            sLibraryName = "sap-ui-theme-"+sLibraryName;
            var allLinks = document.styleSheets;
            for(var i in allLinks){
                if (allLinks.hasOwnProperty(i)) {
                    var styleLink = allLinks[i];
                    if(styleLink.ownerNode.id === sLibraryName){
                        return styleLink;
                    }
                }
            }
        },
        disableBrowserContextMenu:function(){
            // Disable the browser's context menu on the page.  Other elements on
            // the page can provide their own context menu support but by default,
            // there won't be any menu shown.  The Ctrl key can re-enable the menu.
            jQuery(document.body).on("contextmenu", function(e){
                if(!e.ctrlKey){
                    e.preventDefault();
                }
            });
        },
        startTimeoutManager:function() {
            // start session timeout manager.
            if(sas.timeoutManager){
                // register logoff on timed out event.
                sas.eventBus.subscribe(sas.timeoutManager.events.TIMED_OUT,sas.hc.ui.core.util.timeout);
                sas.timeoutManager.start(true);
            }
        },
        doOnAppReady:function(){
            sas.eventBus.publish(sas.hc.ui.core.util.APP_READY, {appId: sas.config.getAppId()});
            // remove splash screen
            var splashScr = $('#sas-hc-splash');
            if(splashScr){
                splashScr.remove();
            }

            var transitionScr = $('#sas-hc-splashTransition');
            if(transitionScr){
                transitionScr.remove();
            }

            // Tell application switcher bootstrap (top frame) to remove its splash screen.
            if(sas.applicationSwitcher){
                sas.applicationSwitcher.removeSplash();
            }
            jQuery.sap.measure.end("sasBootstrapComplete");
            jQuery.sap.require("sas.hc.ui.commons.landmarks.Landmarks");
            sas.hc.ui.commons.landmarks.Landmarks.getSingleton().initWatch();
        },

        timeout:function(){
            // log off.
            var url = sas.config.getSessionTimeoutUrl();
            if(url){

                // hacky... we get timeout urls as flashvars from the flex jsp application tag.  They are ultimately retrieved
                // via WIP ServletUtil and end up retaining all request parameters  of the original request (??).
                // Typically this is good however the app switcher dynamically adds a request parameter to determine whether it has already
                // bootstrapped itself or not.  This request parm is called launchedFromAppSwitcher.  We NEVER want a return to application
                // from timeout to already ahve this value set or it will not bootstrap the app switcher code properly because it thinks
                // it has already been done.
                // If launchedFromAppSwitcher is present as a query parm on timeout URL just get rid of it.
                var timeoutUrl=decodeURIComponent(url);
                timeoutUrl=sas.hc.ui.core.util.Url.removeQueryStringParameter(timeoutUrl,"launchedFromAppSwitcher");
                if(sas.applicationSwitcher){
                    sas.applicationSwitcher.timeout(decodeURIComponent(timeoutUrl));
                }else{
                    // This needs to be on top window because of app switcher and iframes.
                    window.location.href=decodeURIComponent(timeoutUrl);
                }
            }
        },
        pingServer:function(successHandler,errorHandler){
            if(typeof keepAliveService === 'undefined'){
                //resolve keep alive service
                jQuery.sap.require("sas.hc.ui.core.util.ComponentServiceClassResolver");
                var serviceResolver = sas.hc.ui.core.util.ComponentServiceClassResolver;
                keepAliveService = serviceResolver.getServiceClassInstance(serviceResolver.KEEPALIVE_SERVICE);
            }
            if(typeof keepAliveService !== 'undefined' && keepAliveService){ // check for non null object
                keepAliveService.ping(successHandler,errorHandler);
            }
        }
    };
    /**
     * Utility function to get proxy Url for a given Url. Proxy Url is generally used to make cross domain calls.
     * @param {string} sUrl          :Original Url as string for which proxy Url is needed. The Url is not validated for any scheme.
     * @param {boolean} bAuthenticate :boolean, set to true if original Url requires authentication. False otherwise.
     * @returns {string}            :Proxy Url as string if original Url is cross domain. The proxy Url is relative to
     *                       the web application. Returns original Url otherwise.
     */
    sas.hc.ui.core.util.getProxyUrl=function(sUrl,bAuthenticate){
        if(sas.hc.ui.core.util.isUrlCrossDomain(sUrl)){
            // possibly a cross domain Url. Generate a proxy Url.
            var q = {};
            // check for authentication.
            if(bAuthenticate === true){
                q['sas-commons-auth'] = true;
            }
            q['sas-commons-destUrl'] = sUrl;
            var cdpsUrl = sas.hc.ui.core.util._getCrossDomainProxyUrl();
            if(cdpsUrl){
                return cdpsUrl+"?"+jQuery.param(q);
            }
        }
        return sUrl;
    };

    /**
     * return a 32 bit int hashcode for any string
     * @param {string} str TODO-jsdoc
     * @returns {int} TODO-jsdoc
     */
    sas.hc.ui.core.util.hash = function(str) {
        var hash = 0, char;
        if (typeof str !== 'string' || str.length === 0) {
            return hash;
        }
        for (var i = 0; i < str.length; i++) {
            char = str.charCodeAt(i);
            /*eslint-disable no-bitwise */
            hash = ((hash << 5) - hash) + char;
            hash = hash & hash; // Convert to 32bit integer
            /*eslint-enable no-bitwise */
        }
        return hash;
    };
    /**
     * Utility function to sing out from application. If application switcher is ON then application switcher log off mechanism is used.
     */
    sas.hc.ui.core.util.signout= function() {
        // fire before signout for apps to do some clean up
        sas.eventBus.publish(sas.hc.ui.core.util.BEFORE_SIGN_OUT);
        // use app switcher mechanism if available.
        if(sas.applicationSwitcher) {
            sas.applicationSwitcher.logoff();
        } else{
            jQuery.sap.require("sas.hc.ui.core.util.SignoutService");
            var signoutSrv = sas.hc.ui.core.util.SignoutService.instance;
            if(signoutSrv && signoutSrv.signout){
                signoutSrv.signout();
            }
        }
    };
    /**
     * Make properties of input object immutable preventing values of existing
     * properties from being modified and marking all existing properties as non-configurable.
     * @param {object} obj:Object of which properties need to be immutable. Must be an object.
     * @param {boolean} deep: True for recursive immutation. Defaults to false/undefined.
     * @returns {object} Same object with immutable properties. Same input argument is returned it is not an object.
     * @see Object.seal
     */
    sas.hc.ui.core.util.immutate=function(obj,deep){
        if(typeof obj !== 'object') {
            return obj;
        }
        for (var prop in obj){
            if (obj.hasOwnProperty(prop)) {
                try{
                    Object.defineProperty(obj,prop,{
                        enumerable: true,
                        writable:false,
                        configurable:false,
                        value:!!deep && typeof obj[prop] === 'object' ? sas.hc.ui.core.util.immutate(obj[prop],deep) : obj[prop]
                    });
                }catch (err){
                    // ignore if a property is not configurable.
                }
            }
        }
        return obj;
    };

    /**
     * Collection of all loaded Help Resource Bundles
     * @type {Array}
     */
    sas.hc.ui.core.util.mHelpResourceBundles = [];

    /**
     * Retrieves a HelpPopup resource bundle for the given library and locale.
     *
     * If only one argument is given, it is assumed to be the libraryName. The locale
     * then falls back to the current {@link sap.ui.core.Configuration.prototype.getLanguage session locale}.
     * If no argument is given, the library also falls back to a default: "sap.ui.core".
     *
     * @param {string} sLibraryName name of the library to retrieve the bundle for
     * @param {string} sLocale locale to retrieve the resource bundle for
     * @return {jQuery.sap.util.ResourceBundle} the best matching resource bundle for the given parameters or undefined
     */
    sas.hc.ui.core.util.getHelpResourceBundle = function(sLibraryName, sLocale) {
        jQuery.sap.assert((sLibraryName === undefined && sLocale === undefined) || typeof sLibraryName === "string", "sLibraryName must be a string or there is no argument given at all");
        jQuery.sap.assert(sLocale === undefined || typeof sLocale === "string", "sLocale must be a string or omitted");

        var core = sap.ui.getCore();
        sLibraryName = sLibraryName || "sap.ui.core";
        sLocale = sLocale || core.getConfiguration().getLanguage();
        var sKey = sLibraryName + "/" + sLocale;
        if (!sas.hc.ui.core.util.mHelpResourceBundles[sKey]) {
            var sURL = sap.ui.resource(sLibraryName, 'HelpPopup.properties');
            sas.hc.ui.core.util.mHelpResourceBundles[sKey] = jQuery.sap.resources({url: sURL, locale: sLocale});
        }
        return sas.hc.ui.core.util.mHelpResourceBundles[sKey];
    };

    /**
     * A promise that keeps track of whether or not the Help Center application is available.
     *
     * @type {object}
     */
    sas.hc.ui.core.util.HELP_SERVICE = null;

    /**
     * Check if the Help Center application is available
     *
     * @returns {object} A promise that is resolved if able to access Help Center and rejected otherwise
     */
    sas.hc.ui.core.util.getHelpService = function() {
        if (!sas.hc.ui.core.util.HELP_SERVICE) {
            var serviceResolver = sas.hc.ui.core.util.ComponentServiceClassResolver;
            sas.hc.ui.core.util.HELP_SERVICE = serviceResolver.getServiceClassInstance(serviceResolver.HELP);
        }

        return sas.hc.ui.core.util.HELP_SERVICE;
    };

    /**
     * Get version of specified library name asynchronously.
     * @param {string} strLibName: Name of the library to get version for.
     * @param {Array} arrAvailableLibs: Optional array of libraries. Generally its sap.ui.getVersionInfo().libraries.
     * @return {String} library version of specified library name.
     */
    sas.hc.ui.core.util.getLibraryVersion = function(strLibName,arrAvailableLibs){
        var version=undefined;
        jQuery.sap.assert((typeof strLibName === "string" && strLibName !== ''), "strLibName must be a non empty string");
        jQuery.sap.assert(Array.isArray(arrAvailableLibs), "arrAvailableLibs must be an Array of loaded libraries");

        if(typeof strLibName !== "string" || strLibName === '' || !Array.isArray(arrAvailableLibs)){
            return;
        }
        arrAvailableLibs.every(function(currentVal){
            if(currentVal.name === strLibName){
                version = currentVal.version;
                return false; // break
            }
            return true; // to continue traversing array.
        });
        return version;
    };

    /**
     * Get version of specified library name asynchronously.
     * @param {string} strLibName: Name of the library to get version for.
     * @return {Promise} Resolves to specified library version {String} information if found. Rejected otherwise.
     */
    sas.hc.ui.core.util.getLibraryVersionAsync = function(strLibName){
        jQuery.sap.assert((typeof strLibName === "string" && strLibName !== ''), "strLibName must be a non empty string");

        if(typeof strLibName !== "string" || strLibName === ''){
            return Promise.reject();
        }
        return new Promise(function(resolve, reject){
            sap.ui.getVersionInfo({async:true}).then(function(oVersionInfo){
                var version = sas.hc.ui.core.util.getLibraryVersion(strLibName, oVersionInfo.libraries);
                version ? resolve(version) : reject();
            },reject);
        });
    };


    /**
     * Get cross domain proxy path based on the configuration.
     * @private
     */
    sas.hc.ui.core.util._getCrossDomainProxyUrl = function(){
        var sasConf = sas && sas.config;
        if(sasConf){
            // if cross domain proxy config value doesn't start or contain '/' (a case with local cdps)
            // then treat it as running locally within application.
            var retUrl = (/\//.test(sasConf.getCrossDomainProxy()) ? "" : sasConf.getContextPath()+"/")+sasConf.getCrossDomainProxy();
            return retUrl;
        }
    };
    /**
     * Default bootHook registered. Don't invoke this API directly.
     * @private
     */
    sas.hc.ui.core.util._bootHook = function(){
        try{
            var _finalPromise = Promise.resolve();
            if(_bootHandlers.length > 0){
                jQuery.sap.measure.start("sasCoreBootHookComplete");
                var allPrmses = [], retPromise;

                while(_bootHandlers.length > 0){
                    allPrmses.push(_bootHandlers.shift().call());
                }

                retPromise = Promise.all(allPrmses);
                retPromise.then(function(){
                    jQuery.sap.measure.end("sasCoreBootHookComplete");
                });
                _finalPromise = retPromise;
            }
        }catch(err){
            sas && sas.log && sas.log.error("sas.hc.ui.core.util._bootHook: Error executing handlers",err);
        }
        var nonBlocking = sas && sas.config && sas.config.get('nonBlockingBootHooks');
        return nonBlocking=== true ? Promise.resolve(): _finalPromise;
    };

    var _bootHandlers = [];
    /**
     * API to register a boot hook that gets executed in boot strap mechanism after all openUI libraries are loaded.
     * @param {function} handler is a function and MUST return a Promise OR jQuery.Deferred#promise. Handler must resolve the promise
     * on completion. Important. Not resolving the promise would hold up boot process and application startup forever.
     * @private
     */
    sas.hc.ui.core.util.registerBootHook = function(handler){
        jQuery.sap.assert((typeof handler === "function"), "handler must be a function that returns promise.");
        _bootHandlers.push(handler);
    };

    /**
     * When performance matters, use the DOM API to update the text on the DOM element
     * in the SAP control.
     *
     * @param {object} oControl control that has the element
     * @param {string} sClass element's style class
     * @param {string} sValue the text context value
     * @private
     */
    sas.hc.ui.core.util.updateTextContent = function(oControl, sClass, sValue){
        var oDomRef,
            oElement;

        if (!oControl || !sClass){
            return false;
        }

        oDomRef = oControl.getDomRef();
        if (oDomRef) {
            oElement = oDomRef.querySelector("." + sClass);
            return this.updateTextContentElement(oElement, sValue);
        }
        return false;
    };

    /**
     * When performance matters, use the DOM API to update the text on the DOM element.
     *
     * @param {object} oElement the DOM element
     * @param {string} sValue the text context value
     * @private
     */
    sas.hc.ui.core.util.updateTextContentElement = function(oElement, sValue){
        jQuery.sap.assert( typeof sValue === "string", "sValue must be a string");

        if (oElement) {
            oElement.textContent = sValue;
            return true;
        }
        return false;
    };

    /**
     * Checks if the version on framework same as that of its parent if any.
     * @private
     * @returns {Promise} that resolves to true if self version matches with that of parent's. Rejects otherwise.
     */
    sas.hc.ui.core.util._hasSameVersionAsParent = function (){
        try{
            if(window !== window.top
                        && top.sap
                        && top.sap.ui
                        && top.sap.ui.getVersionInfo){
                return new Promise(function(resolve,reject){
                    var topRet,
                        libraryName= "sap.ui.core";

                    function completePromise(arr){
                        if(arr[0]){
                            var topVersion=sas.hc.ui.core.util.getLibraryVersion(libraryName,arr[0].libraries);
                            topVersion && topVersion === arr[1] ? resolve(true) : reject();
                        }else{
                            reject();
                        }
                    };

                    topRet =  top.sap.ui.getVersionInfo({async:true});
                    if(topRet instanceof top.Promise){
                        Promise.all([topRet,sas.hc.ui.core.util.getLibraryVersionAsync(libraryName)]).then(completePromise,reject);
                    }else{
                        sas.hc.ui.core.util.getLibraryVersionAsync(libraryName).then(function(selfVersion){
                            completePromise([topRet,selfVersion]);
                        },
                        reject);
                    }
                });
            }
        }catch(err){
            // security error will be thrown in case of
            // apps loaded from cross origin.
            // suppress the error.
        }
        return Promise.reject();
    };

    sas.hc.ui.core.util.deepFreeze = function (obj) {

        // Retrieve the property names defined on obj
        var propNames = Object.getOwnPropertyNames(obj);

        // Freeze properties before freezing self
        propNames.forEach(function(name) {
            var prop = obj[name];

            // Freeze prop if it is an object
            if (typeof prop == 'object' && prop !== null)
                sas.hc.ui.core.util.deepFreeze(prop);
        });

        // Freeze self (no-op if already frozen)
        return Object.freeze(obj);
    };

    sas.hc.ui.core.util.doOnInit = function (sViewName) {
        var initView = function(skipDetach){
            skipDetach !== true && sap.ui.getCore().detachThemeChanged(initView);

            jQuery("body").append('<div id="sas-welcome-fake-loading" class="sasWelcomeBackground sasWelcomeBackgroundDots" />');
            Promise.all([
                new Promise(function(fnFulfill, fnReject) {
                    // invoke Welcome Experience
                    sap.ui.require([
                        "sas/hc/m/welcome/WelcomeExperience"
                    ], function(WelcomeExperience) {
                        WelcomeExperience.start().then(fnFulfill);
                    });
                }),
                new Promise(function(fnFulfill, fnReject) {
                    // invoke application
                    var sas_hc_view = sap.ui.view({id:sas.config.getAppId(), viewName:sViewName, type:sap.ui.core.mvc.ViewType.JS, height:'100%'});
                    sas_hc_view.attachEventOnce('afterRendering',fnFulfill);
                    sas_hc_view.placeAt("sas_hc_content");
                })
            ]).then(function() {
                sas.hc.ui.core.util.doOnAppReady();
                jQuery("#sas-welcome-fake-loading").remove();
            });

        };
        if(sas.config && sas.config.get('nonBlockingBootHooks') === true){
            sap.ui.getCore().isThemeApplied() ? initView(true) : sap.ui.getCore().attachThemeChanged(initView);
        }else{
            initView(true);
        }
        
    };
})();

jQuery.sap.declare("sas.hc.ui.core.BootTask");
jQuery.sap.require("sap.ui.base.Object");

(function(){
    var BootTask = sap.ui.base.Object.extend("sas.hc.ui.core.BootTask",{

        name:"",
        // run : function() abstract. to be implemented by derived class.
        // runAsync : function() abstract. to be implemented by derived class if task runs async'ly.
        constructor: function(options){
            jQuery.extend(this,options);
        }
    });

    BootTask.prototype.execute = function(){
        if(this.run){
            try{
                this.run();
            }catch(error){
                sas.log.error(error.message,error);
                throw error;
            }
        }
    };

    BootTask.prototype.executeAsync = function(){
        if(this.runAsync){
            try{
                return this.runAsync();
            }catch(error){
                sas.log.error(error.message,error);
                throw error;
            }
        }
    };
}());
jQuery.sap.declare('sas.hc.ui.core.GlobalSettings');

/**
 * Enumeration of Global Settings constants.
 *
 * @namespace
 * @public
 */
sas.hc.ui.core.GlobalSettings = {

    /**
     * Name of the Visual Effects Field under Accessibility group.
     * @public
     */
    PLAY_VISUAL_EFFECTS: "HTML.Application.PlayVisualEffects",

    /**
     * Name of the Locale Field under General group.
     * @public
     */
    LOCALE_KEY: "Locale.DefaultLocale",

    /**
     * Name of the Offline Locale Field under General group.
     * @public
     */
    OFFLINE_LOCALE_KEY: "OfflineLocale.DefaultLocale",

    /**
     * Name of the Format Locale Field under General group.
     * @public
     */
    FORMAT_LOCALE_KEY: "FormatLocale.DefaultLocale",

    /**
     * Locales SAS supports
     * @public
     */
    SAS_SUPPORT_LOCALES: ['af','af_ZA','ar','ar_AE','ar_BH','ar_DZ','ar_EG','ar_IN',
        'ar_IQ','ar_JO','ar_KW','ar_LB','ar_LY','ar_MA','ar_OM','ar_QA','ar_SA',
        'ar_SD','ar_SY','ar_TN','ar_YE','be','be_BY','bg','bg_BG','bn','bn_IN',
        'bs','bs_BA','ca','ca_ES','cs','cs_CZ','da','da_DK','de','de_AT','de_CH',
        'de_DE','de_LI','de_LU','el','el_CY','el_GR','en','en_AU','en_BE','en_BW',
        'en_CA','en_CB','en_GB','en_HK','en_IE','en_IN','en_JM','en_MT','en_NZ',
        'en_PH','en_SG','en_US','en_ZA','en_ZW','es','es_AR','es_BO','es_CL',
        'es_CO','es_CR','es_DO','es_EC','es_ES','es_GT','es_HN','es_MX','es_NI',
        'es_PA','es_PE','es_PR','es_PY','es_SV','es_US','es_UY','es_VE','et',
        'et_EE','eu','eu_ES','fa','fa_IN','fa_IR','fi','fi_FI','fo','fo_FO','fr',
        'fr_BE','fr_CA','fr_CH','fr_FR','fr_LU','ga','ga_IE','he','he_IL','hi',
        'hi_IN','hr','hr_BA','hr_HR','hu','hu_HU','id','id_ID','is','is_IS','it',
        'it_CH','it_IT','ja','ja_JP','kk','kk_KZ','kl','kl_GL','ko','ko_KR','kw',
        'kw_GB','lt','lt_LT','lv','lv_LV','mk','mk_MK','mr','mr_IN','ms','ms_MY',
        'mt','mt_MT','nb','nb_NO','nl','nl_BE','nl_NL','nn','nn_NO','no','no_NO',
        'pl','pl_PL','pt','pt_BR','pt_PT','ro','ro_RO','ru','ru_RU','ru_UA',
        'sh','sh_BA','sh_ME','sh_RS','sk','sk_SK','sl','sl_SI','sq','sq_AL','sr',
        'sr_BA','sr_ME','sr_RS','sv','sv_SE','ta','ta_IN','te','te_IN','th','th_TH',
        'tr','tr_TR','uk','uk_UA','vi','vi_VN','zh_CN','zh_HK','zh_MO','zh_SG','zh_TW'],

    /**
     * Name of the Themes Field under General group.
     * @public
     */
    THEMES_KEY: "OpenUI.Theme.Default", /*Changing the value of this constant to match the mid-tier value*/

    /**
     * Name of the Profile picture chooser Field under General group.
     */
    PROFILE_PICTURE_CHOOSER_KEY: "SETTINGS_AVATAR",

    /**
     * Name of MessageHandler field under the General group.
     * @public
     */
    MESSAGE_HANDLER: "HTML.MessageHandler",

    /**
     * Name of the Locale Field under General group.
     * @public
     */
    DEFAULT: "Default",

    /**
     *
     * @public
     */
    USER_SUPPRESED_MESSAGES: "HTML.USER_SUPPRESED_MESSAGES",

    /**
     *
     * @public
     */
    SIDE_MENU: "HTML.APP_SWITCHER_AVLBL_APPS",

    /**
     * Name of the Play Earcons Field under Accessibility group.
     * @public
     */
    PLAY_EARCONS: "HTML.Application.PlayEarcons",

    /**
     * Name of the Table Reflow Field under Accessibility group.
     * @public
     */
    TABLE_REFLOW: "HTML.Application.TableReflow",

    /**
     * Name of the Invert Application Colors Field under Accessibility group.
     * @public
     */
    INVERT_APP_COLORS: "HTML.Application.InvertColors",

    /**
     * Name of the Focus Override field under Accessibility group.
     * @public
     */
    FOCUS_OVERRIDE: "HTML.Application.FocusOverride",

    /**
     * Name of the Focus Color field under Accessibility group.
     * @public
     */
    FOCUS_COLOR: "HTML.Application.FocusColor",

    /**
     * Name of the Focus Thickness Field under Accessibility group.
     * @public
     */
    FOCUS_THICKNESS: "HTML.Application.FocusThickness",

    /**
     * Name of the Focus Opacity field under Accessibility group.
     * @public
     */
    FOCUS_OPACITY: "HTML.Application.FocusAlpha",

    /**
     * Name of the Preview field under Accessibility group.
     * @public
     */
    FOCUS_PREVIEW: "HTML.Application.FocusSettingsPreview",
    
    /**
     * Name of the Setting meant for enabling tooltips for keyboard only users.
     * @public
     */
    ENABLE_KEYBOARD_TOOLTIPS: "HTML.Application.EnableKeyboardToolTips",


    /**
     * Name of the Setting meant to denote the location to display the tooltip for keyboard only users.
     * @public
     */
    KEYBOARD_TOOLTIP_LOCATION: "HTML.Application.KeyboardToolTipLocation",
    
    /**
     * Possible Tooltip locations for tooltips displayed for keyboard only users.
     */
    KEYBOARD_TOOLTIP_LOCATIONS: {
        BOTTOM_RIGHT: "BottomRight",
        BOTTOM_LEFT: "BottomLeft",
        TOP_RIGHT: "TopRight",
        TOP_LEFT: "TopLeft"
    },

    VALUE_CHANGE_ACTION: {
        /**
         * Value Confirmation Box Action, identifying Sign Out Now button.
         * @public
         */
        SIGN_OUT_NOW: "SignOutNow",

        /**
         * Value Confirmation Box Action, identifying Sign Out Later button.
         * @public
         */
        SIGN_OUT_LATER: "SignOutLater",

        /**
         * Value Confirmation Box Action, identifying Cancel button.
         * @public
         */
        CANCEL: "Cancel"
    },

    /**
     * Name of the Home field.
     * @public
     */
    HOME_KEY: "HOME",

    /**
     * Name of the Modern view under Home Settings.
     * @public
     */
    MODERN_VIEW: "Modern",

    /**
     * Name of the Classic view under Home Settings.
     * @public
     */
    CLASSIC_VIEW: "Classic",

    /**
     * constant to be used as key for the <code>customData</code> object to be added to the
     * <code>sas.hc.ui.commons.formControl.Field</code> for specifying lazy persistence.
     * @static
     * @public
     */
    LAZY_SAVE : "settings_persistence_lazy_save"
};

jQuery.sap.declare("sas.hc.ui.core.CSRFTask");

jQuery.sap.require("sas.hc.ui.core.BootTask");
jQuery.sap.require("sas.hc.ui.core.util");

(function(){
    sas.hc.ui.core.BootTask.extend('sas.hc.ui.core.CSRFTask',{
        name: 'sas.hc.ui.core.CSRFTask'
    });
    var registry = {};

    function toContextPath(path){
        if(typeof path !== 'string') return;
        var slashIndexAfterContextPath;
        if(path.charAt(0) === '/'){
            slashIndexAfterContextPath = path.indexOf("/",1);
            return slashIndexAfterContextPath > 0 ? path.substring(0,slashIndexAfterContextPath) : path;
        }
        return sas.config.getContextPath() || toContextPath(location.pathname);
    };

    function toContextPathUrl(sUrl){
        var oUri = URI.parse(sUrl),
            ctPath;
        if(!oUri.protocol && !oUri.hostname){
            // relative url.
            ctPath = toContextPath(sUrl);
        }else{
            ctPath = oUri.protocol ? oUri.protocol+"://" : "";
            ctPath += oUri.hostname ? oUri.hostname : "";
            ctPath += oUri.port ? ":"+oUri.port : "";
            ctPath += oUri.path ? toContextPath(oUri.path) : "";
        }
        return ctPath;
    };

    function register(url,token, expectedHeaderName){
        if(typeof url !== 'string' || url === "" || typeof token !== 'string'
                 || token === "" || typeof expectedHeaderName !== 'string'
                 || expectedHeaderName ===""){
            return;
        }
        registry[toContextPathUrl(url)] = {
            token: token,
            headerName: expectedHeaderName
        };
    };
    function csrfDataRecorder(jqXHR,options){
        try{
            if(typeof options !== 'object' || options === null || typeof jqXHR !== 'object' || jqXHR === null){
                return; // nothing to do
            }
            //https://gitlab.sas.com/petrichor/commons-web#cross-site-request-forgery
            register(options.url,jqXHR.getResponseHeader('X-CSRF-TOKEN'),jqXHR.getResponseHeader('X-CSRF-HEADER'));
        }catch(secError){ // might be thrown if the response is from cross domain endpoint.
            // ignore.
            sas.log.debug(secError);
        }
    };

    function getCSRFData (url){
        if(typeof url !== 'string' || url === "") return;
        return registry[toContextPathUrl(url)];
    };

    function csrfDataInserter (options, originalOptions, jqXHR){
        // Return if cache option already specified or if this is not a GET/HEAD call.
        if ( !options || typeof options !== "object" || typeof options.type !== 'string' || options.type === "") return;
        var upCaseType = options.type.toUpperCase();
        if( upCaseType === "GET" || upCaseType === "HEAD"){
            return; // CSRF data not required.
        }
        var csrfData = getCSRFData(options.url);
        if(csrfData && typeof csrfData === 'object'){
            options.headers = options.headers || {};
            options.headers[csrfData.headerName] = options.headers[csrfData.headerName] || csrfData.token;
        };
    };

    sas.hc.ui.core.CSRFTask.prototype.run=function(){
        try{
            // listen to ajaxcomplete event to read CSRF tokens in resp headers.
            jQuery(document).ajaxComplete(function(evt,jqXHR,options){
                csrfDataRecorder(jqXHR,options);
            });
            // need to register data recorder also on sas.evetBus to process ajaxComplete event.
            // generally fired by jqXHRRetryHandler.
            sas.eventBus.subscribe("ajaxComplete", function(channelId,evtId,oData){
                if(oData && typeof oData === 'object'){
                    csrfDataRecorder(oData.jqXHR,oData.options);
                }
            });

            //register a jquery ajax prefilter to add CSRF tokens in req headers.
            jQuery.ajaxPrefilter(csrfDataInserter);
            sas.log.debug("sas.hc.ui.core.CSRFTask","CSRF token handler has been activated.");
        }catch(err){
            sas.log.error("sas.hc.ui.core.CSRFTask","Unable to activate CSRF token handler.");
        }
    };
}());
(function() {
    jQuery.sap.declare("sas.hc.contributor.responsive.ResponsiveContributor");

    // Default clone logic for determining which responsive event logic to use.
    // By default, Tablet runs as Desktop
    var M_DEFAULT_RESPONSIVE_CLONE_LOGIC = {
            all: [
                ['Tablet', 'Desktop']
            ]
        },
        B_DEFAULT_INVALIDATEAFTERRESPONSIVECHANGE = false,
        fnNOOP = function() {};


    ///
    /// utility functions
    ///
    /**
     * Return associated controller function implementation
     * example:
     * If 'this' is an OpenUI5 view, and sTarget is 'onBeforeRendering',
     * then the controller of this view's onBeforeRendering function is returned.
     * @private
     * @param {string} sTarget name of function to return on "this" scopes controller
     * @returns {fn} controller implementation of sTarget method
     */
    function _getControllerFn( sTarget ) {
        var fn = null;
        if( sTarget && this.getController && this.getController() ) {
            fn = this.getController().onBeforeRendering;
        }
        return fn;
    }

    /**
     * Return the passed-in string with it's first letter upper-cased
     * example: string -> String
     * @private
     * @param {string} s name of function to return on "this" scopes controller
     * @returns {string} string with first letter upper-cased
     */
    function _upperCaseFirstLetter( s ) {
        return !s ? null : s[0].toUpperCase() + s.substring(1);
    }

    function isAssociation( sPotentialAssociationName ) {
        var key, mAssociations = this.mAssociations,
            bIsAssociation = false;
        for( key in mAssociations ) {
            if( key === sPotentialAssociationName ) {
                bIsAssociation = true;
                break;
            }
        }
        return bIsAssociation;
    }

    function isAggregation( sPotentialAggregationName ) {
        var key, mAggregations = this.mAggregations,
            bIsAggregation = false;
        for( key in mAggregations ) {
            if( key === sPotentialAggregationName ) {
                bIsAggregation = true;
                break;
            }
        }
        return bIsAggregation;
    }

    function invokeSetter( oControl, sProperty, oValue ) {
        var fnSetter;
        if( hasValue(oControl) === true ) {
            fnSetter = oControl['set' + sProperty];
            if( hasValue(oControl) === true && jQuery.isFunction(fnSetter) === true ) {
                fnSetter.apply(oControl, [oValue]);
            }
        }
    }

    function hasValue( oObject ) {
        return oObject !== undefined && oObject !== null;
    };

    ///
    /// _ResponsiveWorker
    ///
    /**
     * An object that contains functions for common responsive functionality.
     * These functions help implement the responsive lifecycle.
     * Invoke each function with proper scope.
     * Can be monkey-patched.
     *
     * Because the logic for invoking the steps of the responsive lifecycle is complicated,
     * the actual invoking order of these functions is purposely separate
     * (hence all the invoke* functions)
     */
    var _ResponsiveWorker = sas.hc.contributor._ResponsiveWorker = {
        // setup initialization property flags
        initResponsiveFlags: function() {
            // per-size initialization flags init stages
            this._initializedInitPhone = false;
            this._initializedInitTablet = false;
            this._initializedInitDesktop = false;
        },

        // init<Size> management
        // manage flag to indicate if that size has already had it's init callback invoked
        invokeInitCallbacks: function() {
            var sActingRangeName = $.proxy(_ResponsiveWorker._getActingRangeName, this)(),
                initFlag = this['_initializedInit' + sActingRangeName],
                fnCallback = this['init' + sActingRangeName];

            //if init<Size> exists then invoke
            if( initFlag === false && fnCallback && $.isFunction(fnCallback) === true ) {
                $.proxy(fnCallback, this)();

                // set size init flag
                this['_initializedInit' + sActingRangeName] = true;
            }
        },

        // onBeforeRendering<Size> management
        invokeOnBeforeRenderingCallbacks: function() {
            var sActingRangeName = $.proxy(_ResponsiveWorker._getActingRangeName, this)(),
                // TODO: should I use jQuery.Event?
                fnCallback = this['onBeforeRendering' + sActingRangeName];

            sap.ui.Device.media.detachHandler(_ResponsiveWorker.breakpointChangeHandler, this, sap.ui.Device.media.RANGESETS.SAP_STANDARD);

            if( fnCallback && $.isFunction(fnCallback) === true ) {
                $.proxy(fnCallback, this)();
            }
        },

        // onAfterRendering<Size> management
        invokeOnAfterRenderingCallbacks: function() {
            var sActingRangeName = $.proxy(_ResponsiveWorker._getActingRangeName, this)(),
                fnCallback = this['onAfterRendering' + sActingRangeName];

            if( fnCallback && $.isFunction(fnCallback) === true ) {
                $.proxy(fnCallback, this)();
            }
        },

        // exit<Size> management
        invokeExitCallbacks: function() {
            var sActingRangeName = $.proxy(_ResponsiveWorker._getActingRangeName, this)(),
                fnCallback = this['exit' + sActingRangeName];

            if( fnCallback && $.isFunction(fnCallback) === true ) {
                $.proxy(fnCallback, this)();
            }
            //S1231239-Uncaught Error :duplicate id 'Breadcrumb-interactive-verticalHistoryListSelect' is seen on browser
            sap.ui.Device.media.detachHandler(_ResponsiveWorker.breakpointChangeHandler, this, sap.ui.Device.media.RANGESETS.SAP_STANDARD);
        },

        // "properties" management
        invokePropertyChanges: function() {
            var self = this,
                sActingRangeName = $.proxy(_ResponsiveWorker._getActingRangeName, this)(),
                oMetadataActions = this.responsiveMetadata.actions,
                bHasProperties = hasValue(oMetadataActions) === true && hasValue(oMetadataActions.properties) === true;

            function handlePropertyAction( aPropertyAction ) {
                var sControl = aPropertyAction[0],
                    sProperty = _upperCaseFirstLetter(aPropertyAction[1]),
                    oValue = aPropertyAction[2],
                    oControl;

                if( hasValue(sControl) === true ) {
                    // The 4 property action situations:
                    // 1. the consumer can specify "this" if the target object is itself
                    // 2. The action target object is an aggregation
                    // 3. The action target object is an association
                    // 4. a non-managed object on the context of the target object

                    if( sControl === 'this' ) {
                        oControl = self;
                    } else if( jQuery.proxy(isAggregation, self)(sControl) === true ) {
                        oControl = self.getAggregation(sControl);
                    } else if( jQuery.proxy(isAssociation, self)(sControl) === true ) {
                        oControl = sap.ui.getCore().byId(self.getAssociation(sControl));
                    } else {
                        oControl = self[sControl];
                    }

                    jQuery.proxy(invokeSetter, self)(oControl, sProperty, oValue);
                }
            }

            if( bHasProperties === true && jQuery.isArray(oMetadataActions.properties[sActingRangeName]) === true ) {
                oMetadataActions.properties[sActingRangeName].forEach(handlePropertyAction);
            }
        },

        // event handler when responsive size change occurs
        breakpointChangeHandler: function( range ) {
            // init<Name> callbacks
            $.proxy(_ResponsiveWorker.invokeInitCallbacks, this)();

            // property changes ("properties")
            $.proxy(_ResponsiveWorker.invokePropertyChanges, this)();

            // onBeforeRendering<Size> callbacks
            $.proxy(_ResponsiveWorker.invokeOnBeforeRenderingCallbacks, this)();

            // onAfterRendering<Size> callbacks
            $.proxy(_ResponsiveWorker.invokeOnAfterRenderingCallbacks, this)();

            // check flag to see if we should force an invalidation (re-render)
            var invalidateAfterResponsiveChange = this.responsiveMetadata.invalidateAfterResponsiveChange || B_DEFAULT_INVALIDATEAFTERRESPONSIVECHANGE;
            if( invalidateAfterResponsiveChange === true ) {
                this.invalidate();
            }
        },

        // check responsive metadata ("clones") to determine
        // which responsive size should be considered to drive logic
        _getActingRangeName: function() {
            var sSize = _ResponsiveWorker._getMediaParams().name,
                oClones = this.responsiveMetadata.clone;

            if( !oClones ) {
                oClones = M_DEFAULT_RESPONSIVE_CLONE_LOGIC;
            }

            // check "all"
            var aAllClones = oClones.all;
            if( aAllClones ) {
                for( var i = 0; i < aAllClones.length; i++ ) {
                    if( aAllClones[i][0] === sSize ) {
                        sSize = aAllClones[i][1];
                        break;
                    }
                }
            }
            //TODO: "init" and "onBefore" and "onAfter"

            return sSize;
        },

        // get the current responsive state
        _getMediaParams: function() {
            return sap.ui.Device.media.getCurrentRange(sap.ui.Device.media.RANGESETS.SAP_STANDARD);
        }
    };

    ///
    /// sas.hc.contributor.responsive.ResponsiveContributor
    ///
    /**
     * Proxy-patch the scope context (a control or view) with functions to implement the responsive lifecycle.
     * This is a contributor class and is meant to be invoked with the context scope of an object that is to be decorated.
     * Invoked after the proxy-patched ManagedObject.applySettings (see MonkeyPatchApplySettingsTask)
     * @public
     * @name sas.hc.contributor.responsive.ResponsiveContributor
     * @param {object} mMetadata name of function to return on "this" scope's controller
     * @function
     */
    sas.hc.contributor.responsive.ResponsiveContributor = function( mMetadata ) {
        // check if there are any id restrictions (this is a debugging feature)
        var self = this;
        function shouldExecute() {
            var _aActiveIds = sas.hc.contributor.responsive.ResponsiveContributor._aActiveIds;
            if( _aActiveIds.length > 0 ) {
                var i = 0, sId = self.getId && self.getId();
                for(; i < _aActiveIds.length; i++ ) {
                    if( _aActiveIds[i] === sId ) {
                        return true;
                    }
                }
                return false;
            }
            return true;
        }
        if( shouldExecute() === false ) {
            sas.log.trace('A ResponsiveContributor was stopped from executing due to _aActiveIds');
            return;
        }

        var
            // is the client object a view or control?
            // there are differences in their life-cycles (such as createContent) that must be managed
            bIsView = !!this.createContent,

            // "super" function implementations
            fnPreviousOnBeforeRendering = this.onBeforeRendering || $.proxy(_getControllerFn, this)('onBeforeRendering') || fnNOOP,
            fnPreviousOnAfterRendering = this.onAfterRendering || $.proxy(_getControllerFn, this)('onAfterRendering') || fnNOOP,
            fnPreviousExit = this.exit || $.proxy(_getControllerFn, this)('exit') || fnNOOP;

        if( bIsView === true ) {
            sas.log.warning('Views not supported by ResponsiveContributor at this time');
            return;
        }

        // manage instance variable "responsiveMetadata"
        // It is already a consumer-defined "this" variable if "this" is a control (to mimic control metadata).
        // It is passed in to ResponsiveContributor if "this" is a view
        if( !this.responsiveMetadata ) {
            this.responsiveMetadata = mMetadata;
        }

        // right after init, and applySettings (in that order)
        $.proxy(_ResponsiveWorker.initResponsiveFlags, this)();
        $.proxy(_ResponsiveWorker.invokeInitCallbacks, this)();

        // decorate client object
        //
        // onBeforeRendering
        //
        this.onBeforeRendering = function() {
            // call super
            fnPreviousOnBeforeRendering.apply(this, arguments);

            // execute responsive lifecycle actions that take place before rendering
            $.proxy(_ResponsiveWorker.invokePropertyChanges, this)();
            $.proxy(_ResponsiveWorker.invokeOnBeforeRenderingCallbacks, this)();
        };

        //
        // onAfterRendering
        //
        this.onAfterRendering = function() {
            var self = this,
                // responsive metadata property for whether to listen to responsive size changes
                bListen = this.responsiveMetadata.listen;

            // call super
            fnPreviousOnAfterRendering.apply(this, arguments);

            // execute responsive lifecycle actions that take place after rendering
            $.proxy(_ResponsiveWorker.invokeOnAfterRenderingCallbacks, this)();

            // breakpoint listener
            if( bListen !== false ) {
                setTimeout(function() {
                    sap.ui.Device.media.detachHandler(_ResponsiveWorker.breakpointChangeHandler, self, sap.ui.Device.media.RANGESETS.SAP_STANDARD);
                    sap.ui.Device.media.attachHandler(_ResponsiveWorker.breakpointChangeHandler, self, sap.ui.Device.media.RANGESETS.SAP_STANDARD);
                }, 0);
            }
        };

        //
        // exit
        //
        this.exit = function() {
            // execute responsive lifecycle actions that take place after rendering
            $.proxy(_ResponsiveWorker.invokeExitCallbacks, this)();

            // call super
            if( fnPreviousExit ) {
                fnPreviousExit.apply(this, arguments);
            }
        };
    };

    // Only execute ResponsiveContributor functionality for controls with an id in this array.
    // Used for debugging. Can monkey-patch during boot so only certain target ResponsiveContributors execute.
    sas.hc.contributor.responsive.ResponsiveContributor._aActiveIds = [];
}());

jQuery.sap.declare("sas.hc.ui.core.util.ServicesUtil");

jQuery.sap.require("sas.hc.ui.core.util");

/**
 * Utility functions for manipulating or evaluating service results.
 */
sas.hc.ui.core.util.ServicesUtil =
{
    /**
     * Service platform identifiers.
     *
     * @type {{WIP: string, PETRICHOR: string}}
     */
    ServicePlatform : {
        /**
         * @public
         */
        WIP : "WIP",
        /**
         * @public
         */
        PETRICHOR : "PETRICHOR"
    },

    /**
     * Get link object from list of specified links based on link method
     * and link relationship.
     *
     * @param {object[]} links An array of links to search through
     * @param {function} method The link method
     * @param {*} relationship The link relationship
     * @returns {object} a link
     */
    getLink:function(links, method, relationship) {
        if (!links)
            return null;
        for ( var i=0; i<links.length; i++ )
            if ( (links[i].method === method) && (links[i].rel === relationship) )
                return links[i];
        return null;
    },

    /**
     * Return the current service platform.
     *
     * @returns {object} identifier of the current service platform.
     */
    getServicePlatform:function() {
        // servicePlatform should always be present because of Application jsp tag.
        // Can also be overridden by solution in config.js
        var platform=sas.config.get("servicePlatform");

        // We should always have a platform available to use either automatically via jsp tag
        // or by solution specific override.
        if (typeof(platform) === "undefined" || platform === null)
            platform = this.ServicePlatform.PETRICHOR;

        return platform;
    }

};

jQuery.sap.declare("sas.hc.ui.services.ServiceRegistry");

jQuery.sap.require("sap.ui.base.Object");
jQuery.sap.require("sas.hc.ui.services.RestService");
jQuery.sap.require("sas.hc.ui.core.util");

/**
 * @class
 * Creates a ServiceRegistry object.
 * @public
 * @constructor
 * @param {object} oConfig Registry config object that specifies the registry service url
 * @author sinyas
 *
 */
(function(){

    function _onRegistryLoad (data){
        this._serviceDefs = this._serviceDefs || {};
        if(!data || (!data.services && !data.links)) {
            return;
        }
        var that=this;

        if(data.services){ // if the response has service definitions.
            var addService = function(oServiceDef){
                // create a get method for this service in the registry
                // by the name of its interface if available or default to name of the service
                var serviceName = oServiceDef.serviceInterface || oServiceDef.name;
                if(serviceName){
                    that._serviceDefs[serviceName] = sas.hc.ui.core.util.immutate(oServiceDef);
                    var spaceCharIndex = serviceName.indexOf(" ");
                    var methodName = serviceName.substring(0,spaceCharIndex > 0 ? spaceCharIndex : serviceName.length);
                    methodName = "get"+methodName;
                    that[methodName] = function(){
                        var serviceDef = that._serviceDefs[serviceName];
                        if(!serviceDef.service){
                            Object.defineProperty(serviceDef,'service',{
                                enumerable:false,
                                value: sas.hc.ui.core.util.immutate(new sas.hc.ui.services.RestService(jQuery.extend({},serviceDef,
                                    {crossDomain:that.crossDomain,authenticate:that.authenticate})))
                            });

                        }
                        return serviceDef.service;
                    };
                }
            };

            jQuery.each(data.services,function(index,serviceDef){
                if(serviceDef.type.toUpperCase() === "REST"){ // handle only REST services for now.
                    //put this serviceDef in to the store.
                    addService(serviceDef);
                }
            });
        }

        if(data.links){ // if response is links
            var addServiceFromLink = function(oLink){
                // create a get method for this service in the registry
                // by the relation of the link.
                var serviceName = oLink.rel;
                if(serviceName){
                    // check if this link is already added.
                    var oldLink = that._serviceDefs[serviceName];
                    if(typeof oldLink === "undefined"){
                        if(typeof oLink.methods === "undefined"){
                            oLink.methods = [].concat(oLink.method);
                        }
                        that._serviceDefs[serviceName] = sas.hc.ui.core.util.immutate(oLink);
                    }else{
                        oldLink.methods.push(oLink.method);
                    }
                    var spaceCharIndex = serviceName.indexOf(" ");
                    var methodName = serviceName.substring(0,spaceCharIndex > 0 ? spaceCharIndex : serviceName.length);
                    if(methodName !== null){
                        methodName = methodName.charAt(0).toUpperCase() + methodName.substring(1);
                    }
                    methodName = "get"+methodName;
                    that[methodName] = function(){
                        var serviceDef = that._serviceDefs[serviceName];
                        if(!serviceDef.service){
                            Object.defineProperty(serviceDef,'service',{
                                enumerable:true,
                                value:sas.hc.ui.core.util.immutate(new sas.hc.ui.services.RestService({
                                    name:serviceDef.name,
                                    url:serviceDef.href,
                                    crossDomain:that.crossDomain,
                                    authenticate:that.authenticate,
                                    methods:serviceDef.methods,
                                    method:serviceDef.method
                                }))
                            });
                        }
                        return serviceDef.service;
                    };
                }
            };

            jQuery.each(data.links,function(index,link){
                addServiceFromLink(link);
            });

        }

    }

    sap.ui.base.Object.extend("sas.hc.ui.services.ServiceRegistry", {

        isInitialized:false,
        authenticate:undefined,
        crossDomain:undefined,
        constructor: function(oConfig){
            sap.ui.base.Object.apply(this,arguments);
            jQuery.extend(this,oConfig);
            if(typeof this.asyncInit !== 'boolean'){
                this.async = false;
            }
            if(!this.asyncInit){
                this.init();
            }
        },
        init:function(){
            if(this.isInitialized){
                return this;
            };
            this.isInitialized=true;
            sas.ajax({
                url: this.url,
                sas:{
                    crossDomain:this.crossDomain,
                    authenticate:this.authenticate,
                },
                context:this,
                async:false
            }).done(_onRegistryLoad.bind(this));
        },
        initAsync:function(){
            if(this.isInitialized){
                return Promise.resolve();
            };
            var that = this,
                prm;
            prm = new Promise(function(resolve,reject){
                sas.ajax({
                    url: that.url,
                    sas:{
                        crossDomain:that.crossDomain,
                        authenticate:that.authenticate,
                    }
                }).done(function(){
                    _onRegistryLoad.apply(that,arguments);
                    resolve(true);
                }).fail(function(){
                    resolve(false);
                }).always(function(){
                    that.isInitialized=true;
                });
            });
            return prm;
        },

    });
}());
jQuery.sap.declare("sas.hc.ui.core.util.StringUtil");
sap.ui.define('sas/hc/ui/core/util/StringUtil',[
    "sap/ui/base/BindingParser",
], function(BindingParser) {
    "use strict";

    var StringUtil = {
        /**
         * Prepare string to be used while wiring up fields when using "complex" data-binding
         * @param {string} sValue The string to escape
         * @returns {string} Escaped string
         */
        complexBindingEscape: BindingParser.complexParser.escape,

        /**
         * Determine if a string is either null or undefined
         * @param {string} v The value to check
         * @returns {boolean} true if either null or undefined, false otherwise
         */
        isNil: function(v) {
            return v === null || v === undefined;
        },

        /**
         * Determine if a string is empty
         * @param {string} v The value to check
         * @returns {boolean} true if either empty, null, or undefined; false otherwise
         */
        isEmpty: function(v) {
            return this.isNil(v) || v === "";
        },

        /**
         * Cause the end-user's browser to download a file
         * @param {string} sContent The content of the file to download
         * @param {string} sFilename The name of the file to download
         */
        download: function(sContent, sFilename) {
            var blob, oLink;

            // provide defaults to passed-in args
            if (StringUtil.isNil(sContent)) {
                sContent = "";
            }
            if (StringUtil.isNil(sFilename)) {
                sFilename = "content.txt";
            }

            blob = new Blob([sContent]);

            // use feature detection (most-likely IE)
            if (window.navigator.msSaveOrOpenBlob) {
                window.navigator.msSaveOrOpenBlob(blob, sFilename);
            } else {
                oLink = document.createElement("a");
                document.body.appendChild(oLink);
                oLink.download = sFilename;
                oLink.href = URL.createObjectURL(blob);
                oLink.click();
                document.body.removeChild(oLink);
            }
        },

        /**
         * Construct a string that contains a base string repeated a number of times
         * @param sString {string} the base string to be repeated
         * @param iCount {number} the number of times to repeat the base string
         * @return {string} new string with iCount copies of sString
         */
        repeatString: function(sString, iCount) {

            var i,
                sRetval = "",
                bValidString = !StringUtil.isEmpty(sString),
                bValidCount = typeof iCount === "number" && !isNaN(iCount);

            if (bValidString && bValidCount) {
                for (i = 0; i < iCount; i++) {
                    sRetval += sString;
                }
            }

            return sRetval;
        }
    };

    return StringUtil;
}, true);

jQuery.sap.declare("sas.hc.ui.core.util.Url");

jQuery.sap.require("sas.hc.ui.core.util");
jQuery.sap.require("sas.hc.ui.core.util.StringUtil");

var StringUtil = sas.hc.ui.core.util.StringUtil;

/**
 * Utility functions for manipulating or evaluating url characteristics.
 */
var Url;

Url = {
    /**
     * Determine whether the provided url is considered cross domain
     * @param {string} url The Url string to consider
     * @returns {boolean} Whether the url is cross-domain
     */
    isCrossDomain: sas.hc.ui.core.util.isUrlCrossDomain,

    /**
     * Retrieve query string from given url
     *
     * Examples:
     * * http://test.com? => ""
     * * http://test.com?foo=bar => "foo=bar"
     * * http://test.com?foo=bar&baz=zoo => "foo=bar&baz=zoo"
     *
     * @param {string} url The given url
     * @returns {string} The query params as a string
     */
    getQueryString: function(sUrl) {
        if (StringUtil.isNil(sUrl)) {
            return undefined;
        }
        // TODO: if url not defined use current browser url
        return new URI(sUrl).query();
    },

    /**
     * Get the value of a url parameter
     * @param {string} sUrl The url to query from
     * @param {string} sParam The desired param
     * @returns {string} The param value
     */
    getQueryStringParameter: function(sUrl, sParam) {
        var oUri;
        if (StringUtil.isNil(sUrl) || StringUtil.isNil(sParam)) {
            return undefined;
        }
        oUri = new URI(sUrl);
        return URI.parseQuery(oUri.query())[sParam];
    },

    /**
     * Update/set the value of a url parameter
     * @param {string} sUrl The url to update
     * @param {string} sParam The param name
     * @param {string} sValue The param value
     * @returns {string} The updated URL
     */
    updateQueryStringParameter: function(sUrl, sParam, sValue) {
        var oUri;
        if (StringUtil.isNil(sUrl) || StringUtil.isNil(sParam)) {
            return undefined;
        }
        if (StringUtil.isNil(sValue)) {
            sValue = "";
        }
        oUri = new URI(sUrl);
        oUri.setSearch(sParam, sValue);
        return oUri.toString();
    },

    /**
     * Remove a url parameter
     * @param {string} sUrl The url to update
     * @param {string} sParam The param to remove
     * @returns {string} The updated URL
     */
    removeQueryStringParameter: function(sUrl, sParam) {
        var oUri;
        if (StringUtil.isNil(sUrl)) {
            return undefined;
        }
        if (StringUtil.isNil(sParam)) {
            sParam = '';
        }
        oUri = new URI(sUrl);
        oUri.removeSearch(sParam);
        return oUri.toString();
    },

    /**
     * Given a url or a query string return a JavaScript object with all of the
     * request parameters available as properties
     * @param {string} sUrl The url to derive the param object from
     * @returns {object} An object with request params as name-value pairs
     */
    queryToObject: function(sUrl) {
        if (StringUtil.isNil(sUrl)) {
            return undefined;
        }
        return URI.parseQuery((new URI(sUrl)).query());
    },

    /**
     * For a given url return a url path up to and including just the webapp context
     * @param {string} url The given URL
     * @returns {string} The context path
     */
    getContextPath: function(url) {
        var a = $("<a>", { href: url })[0];

        var pathname = a.pathname,
            context;
            // For IE only pathname doesn't retain the leading slash.  For Firefox and Chrome it does.
            // Normalized to have leading slash if it isn't present (IE only);
        if (pathname.indexOf("/") !== 0) {
            pathname = "/" + pathname;
        }

        if (pathname.indexOf("/",1) === -1) {
            context = pathname;
        } else {
            context = pathname.substr(0, pathname.indexOf("/", 1));
        }

        // Use known Logoff servlet
        return a.protocol + "//" + a.host + context;
    }

};

sas.hc.ui.core.util.Url = Url;

/*
 * SAS extension of sas.services.RestService for HTML Commons.
 * Copyright 2015 SAS Institute Inc.
 * All Rights Reserved.
 */
jQuery.sap.declare("sas.hc.ui.services.petrichor.appRegistry.AppRegistryService");
jQuery.sap.require("sas.hc.ui.services.RestService");

/**
 *
 * @class
 * Provides functions to communicate with mid-tier Petrichor AppRegistry Service.
 *
 * @extends sas.hc.ui.services.RestService
 *
 * @author SAS
 * @version 1.0
 *
 * @constructor
 * @public
 * @name sas.hc.ui.services.petrichor.appRegistry.AppRegistryService
 */

(function(){

    sas.hc.ui.services.RestService.extend("sas.hc.ui.services.petrichor.appRegistry.AppRegistryService", {

        constructor:function(options) {
            var restOptions=
                {
                    url: "/appRegistry",
                    async: false
                };
            sas.hc.ui.services.RestService.apply(this, [restOptions]);
        }
    });

    /**
     * Gets the application tree from the appregistry service.
     *
     * @param {object} options for ajax call
     * @return {jQuery.Promise} TODO-jsdoc
     * @public
     */

    sas.hc.ui.services.petrichor.appRegistry.AppRegistryService.prototype.getApplicationTree=function(options) {

        var getApplicationTreeOptions = jQuery.extend( {
            headers:{
                'Accept-Item': 'application/vnd.sas.app.registry.application+json',
                'Accept': 'application/vnd.sas.collection+json'
            },
            dataType: 'json',
            url: this.url+"/applications/applicationTree"
        }, options );

        return this.get(getApplicationTreeOptions);
    };

    sas.hc.ui.services.petrichor.appRegistry.AppRegistryService.prototype.getSupportedTypes=function(options) {

        var getApplicationTypesOptions = jQuery.extend( {
            headers:{
                'Accept-Item': 'application/vnd.sas.app.registry.application+json',
                'Accept': 'application/vnd.sas.collection+json'
            },
            dataType: 'json',
            url: this.url+"/types"
        }, options );

        return this.get(getApplicationTypesOptions);
    };

}());

/*
* SAS extension of sas.services.RestService for HTML Commons.
* Copyright 2014 SAS Institute Inc.
* All Rights Reserved.
*/
jQuery.sap.declare("sas.hc.ui.services.petrichor.preferences.PreferencesService");
jQuery.sap.require("sas.hc.ui.services.RestService");

(function(){

    /**
     * @class
     * Creates an instance of PreferencesService for communicating in controlled fashion with
     * the Petrichor preferences service.
     *
     * The Petrichor preferences service is currently only documented in readme which you can see at
     * http://gitlab.sas.com/heliopolis/preferences-service/tree/master
     *
     * Swagger doc at http://weblax01.unx.sas.com/microservicesdoc/swagger-ui/?url=../apispecifications/PreferencesAPI.yaml#/
     *
     * @author saszjt
     * @constructor
     * @public
     */
    sas.hc.ui.services.RestService.extend("sas.hc.ui.services.petrichor.preferences.PreferencesService", {

        constructor:function(options) {
            var restOptions=
                {
                    url: "/preferences/preferences",
                    crossDomain: false,
                    authenticate:true
                };
            sas.hc.ui.services.RestService.apply(this, [restOptions]);
        }
    });

    /**
     * Fetch the user id to use in all preference user specific rest calls.  Typically this will be the current user
     * however centralizing lookup here will allow for user id override functionality if needed.
     *
     * @returns userid
     *
     * @private
     */
    sas.hc.ui.services.petrichor.preferences.PreferencesService.prototype.getUserId=function() {
    // @currentUser is a special token honored by Preference Service (V2) instead of having to
    // look up the current user id.
        return "@currentUser";
    };


    /**
     * Fetch the requested preference.
     *
     * @param {string} key Key of preference to fetch
     * @param {object} [options] Optional arguments such as applicationName
     *
     * @return {jQuery.Deferred} TODO-jsdoc
     * <pre>
     *         {
     *            application: "appName",
     *            id: "abc",
     *            value: "abc value"
     *          }
     * </pre>
     *
     * @public
     * @function
     */
    sas.hc.ui.services.petrichor.preferences.PreferencesService.prototype.getPreference=function(key, options) {
        var getPreferencesOptions = $.extend( { dataType: 'json',
            url: this.url+"/"+this.getUserId()+"/"+key}, options );

        if(getPreferencesOptions.hasOwnProperty("applicationName")===true)
            getPreferencesOptions.url=getPreferencesOptions.url+"?filter=eq(application,'"+getPreferencesOptions.applicationName+"')";

        return this.get(getPreferencesOptions);
    };


    /**
     * Fetch all preferences for the current user.
     *
     * @param {object} [options] Optional arguments such as applicationName
     *
     * @return {jQuery.Deferred} TODO-jsdoc
     *
     * <pre>
     *     {
     *         items:
     *         [
     *            {
     *               application: "appName",
     *               id: "abc",
     *               value: "abc value"
     *               links: ...
     *            },
     *            {
     *               application: "appName",
     *               id: "xyz",
     *               value: "xyz value",
     *               links: ...
     *            }
     *         ]
     *     }
     * </pre>
     *
     * @public
     * @function
     */
    sas.hc.ui.services.petrichor.preferences.PreferencesService.prototype.getPreferences=function(options) {
        var getPreferencesOptions = $.extend( { dataType: 'json',
            url: this.url+"/"+this.getUserId()}, options );

        if(getPreferencesOptions.hasOwnProperty("applicationName")===true)
            getPreferencesOptions.url=getPreferencesOptions.url+"?filter=eq(application,'"+getPreferencesOptions.applicationName+"')";

        return this.get(getPreferencesOptions);
    };


    /**
     * Remove specified preference
     *
     * @param {string} key preference key to remove
     * @param {object} [options] An optional object that allows for additional options as needed.  Not typical.
     *
     * @return {jQuery.Deferred} TODO-jsdoc
     *
     * @public
     * @function
     */
    sas.hc.ui.services.petrichor.preferences.PreferencesService.prototype.removePreference=function(key, options) {
        var removePreferencesOptions = $.extend( {
            url: this.url+"/"+this.getUserId() + "/" + key
        }, options );

        return this.del(removePreferencesOptions);
    };


    /**
     * Remove specified preferences
     *
     * @param {string[]|object[]} keys An array of preference keys
     * @param {object} [options] An optional object that allows for additional options as needed.  Not typical.
     *
     * @return {jQuery.Deferred} TODO-jsdoc
     *
     * @public
     * @function
     */
    sas.hc.ui.services.petrichor.preferences.PreferencesService.prototype.removePreferences = function(keys, options) {
    // If application is not explicitly defined on each preference then inject our default value
        var preferenceObjects = $.map(keys, function(preferenceKey) {
            return {
                "id": preferenceKey
            };
        });

        var deletePreferencesOptions = $.extend({
            dataType: 'json',
            contentType: 'application/vnd.sas.collection+json',
            processData: false,
            data: JSON.stringify({
                "items": preferenceObjects
            }),
            url: this.url + "/" + this.getUserId()
        }, options);

        return this.del(deletePreferencesOptions);
    };



    /**
     * Remove all preference settings for the current user.
     *
     * @param {object} [options] An optional object that allows for additional options such as application filter.
     *
     * @return {jQuery.Deferred} TODO-jsdoc
     *
     * @public
     * @function
     */
    sas.hc.ui.services.petrichor.preferences.PreferencesService.prototype.clearPreferences=function(options) {

        // There is no clear preferences so we need to get a list of all applicable preferences and pass those keys
        // in through removePreferences.

        var that=this;
        return this.getPreferences(options).then(  function(preferenceResults)
        {

            var preferencesToClear=[];
            // If application is not explicitly defined on each preference then inject our default value
            $.each(preferenceResults.items, function(index, preferenceObject) {
                preferencesToClear.push(preferenceObject.id);
            });

            return that.removePreferences(preferencesToClear, options);
        });
    };

    /**
     * Persist specified preference object for current user
     *
     * A preference object is of the form
     * <pre>
     * {
     *    key : "abc",
     *    value : "123"
     * }
     * </pre>
     *
     * @param {object} preferenceObject A single preference object to be persisted
     * @param {object} [options] An optional object that allows for additional options as needed.  Not typical.
     * @return {jQuery.Deferred} TODO-jsdoc
     *
     * @public
     * @function
     */
    sas.hc.ui.services.petrichor.preferences.PreferencesService.prototype.setPreference=function(preferenceObject, options) {
        return this.setPreferences([preferenceObject], options);
    };


    /**
     * Persist specified preference objects for current user
     *
     * A preference object is of the form
     * <pre>
     * {
     *    id : "abc",
     *    value : "123"
     * }
     * </pre>
     *
     * @param {Array} preferenceObjects An array of preference objects to be persisted
     * @param {object} [options] An optional object that allows for additional options as needed.  Not typical.
     *
     * @return {jQuery.Deferred} TODO-jsdoc
     *
     * @public
     * @function
     */
    sas.hc.ui.services.petrichor.preferences.PreferencesService.prototype.setPreferences=function(preferenceObjects, options) {

    // If application is not explicitly defined on each preference then inject our default value
        $.each(preferenceObjects, function(index, preferenceObject) {
            if(preferenceObject.hasOwnProperty("application")===false)
                preferenceObject.application=sas.config.appName;
        });

        var setPreferencesOptions = $.extend( { dataType: 'json',
            accepts: {
                json: "application/vnd.sas.collection+json"
            },
            contentType: 'application/vnd.sas.preference+json',
            processData: false,
            data: JSON.stringify(preferenceObjects),
            url: this.url+"/"+this.getUserId()
        }, options );

        return this.put(setPreferencesOptions);
    };



}());

jQuery.sap.declare("sas.hc.ui.core.util.ComponentServiceClassResolver");

jQuery.sap.require("sas.hc.ui.core.util.ServicesUtil");

/**
 * With HTML Commons client components being required to support both WIP and Petrichor services
 * the components are written in such a way as to support multiple service implementation classes.
 * For instance the Settings dialog uses WIPSettingsService.js or PetrichorSettingsAdapterService.js interchangeably.
 *
 * In order to avoid lots of if WIP then use this else if Petrichor use this type code in our service
 * based components we would rather they simply ask for a service implementation and we'll return the
 * one we think is appropriate for the environment being run in.
 *
 * Which environment being run in will be driven by a configuration property sas.servicePlatform which
 * will be defaulted appropriately by the sas-html:Application tag.  We can make a pretty good guess
 * at which service platform based on whether it is the Application tag from the WIP enabled
 * sas.commons.openui.midtier.jar or from the non WIP (Petrichor) jar.
 *
 * If our guess isn't good enough then a solution can override it in their config.js
 *
 *
 * This  means we need to be able to have a WIP service classname and a Petrichor service classname
 * mapping for each service needed by these components.
 * This file will serve as the location of this mapping for now along with methods for resolving service to
 * classname.
 *
 * There may be a desire at some point to externalize this mapping to a config file or to be generated at build
 * time based on annotations in the service classes themselves.  However I'm going to follow the KISS ideology
 * until there is a good reason to do otherwise.
 *
 * This centralized mapping and resolving will allow us to externalize the mappings at any point in the future with
 * relative ease.
 *
 * The only real downside to keeping the mapping in this file is that it must be maintained.  We should measure the
 * complexity of dynamically generating against whether this local mapping actually causes us any issues.
 *
 */
sas.hc.ui.core.util.ComponentServiceClassResolver =
{
    CONTENT_NAVIGATOR_DATA_DESCRIPTOR: "CONTENT_NAVIGATOR_DATA_DESCRIPTOR",
    CONTENT_NAVIGATOR_MODEL: "CONTENT_NAVIGATOR_MODEL",
    CONTENT_NAVIGATOR_OPERATION_RESULT_DESCRIPTOR: "CONTENT_NAVIGATOR_OPERATION_RESULT_DESCRIPTOR",
    CONTENT_NAVIGATOR_PERMISSIONS_DESCRIPTOR: "CONTENT_NAVIGATOR_PERMISSIONS_DESCRIPTOR",
    CONTENT_NAVIGATOR_TYPE_DESCRIPTOR: "CONTENT_NAVIGATOR_TYPE_DESCRIPTOR",
    CONTENT_NAVIGATOR_VALIDATION_RESULT_DESCRIPTOR: "CONTENT_NAVIGATOR_VALIDATION_RESULT_DESCRIPTOR",
    SEARCH_SERVICE: "SEARCH",
    SEARCH_APPLICATION_ACCESS_SERVICE : "SEARCH_APPLICATION_ACCESS",
    SEARCH_CONTENT_TYPE_SERVICE : "SEARCH_CONTENT_TYPE",
    SEARCH_SCHEMA_SERVICE : "SEARCH_SCHEMA",
    SETTINGS_SERVICE: "SETTINGS",
    SIDE_MENU_SERVICE: "SIDE_MENU",
    NOTIFICATIONS_SERVICE: "NOTIFICATIONS",
    APPLICATION_SWITCHER_MODEL: "APPLICATION_SWITCHER_MODEL",
    USERINFO_SERVICE: "USERINFO",
    SIGNOUT_SERVICE: "SIGNOUT",
    THEME_SERVICE: "THEME",
    THEME_UTIL_SERVICE: "THEME_UTIL",
    KEEPALIVE_SERVICE:"KEEP_ALIVE",
    HELP: "HELP",
    ABOUT_MODEL: "ABOUT_MODEL",
    CSRF_TASK: "CSRF_TASK",
    REGISTRY_TASK: "REGISTRY_TASK",
    VERSION_INFO_TASK: "VERSION_INFO_TASK",
    CACHE_MANAGER_TASK: "CACHE_MANAGER_TASK",
    LEFT_NAV_SERVICE: 'LEFT_NAV_SERVICE',
    RECENT_MODEL: "RECENT_MODEL",
    FONT: "FONT",

    serviceClassMappings:{

        "CONTENT_NAVIGATOR_DATA_DESCRIPTOR":
        {
            "WIP": "sas.hc.ui.services.navigation.ContentDataDescriptor",
            "PETRICHOR": "sas.hc.ui.services.navigation.descriptors.ContentPETDataDescriptor"
        },
        "CONTENT_NAVIGATOR_MODEL":
        {
            "WIP": "sas.hc.ui.services.navigation.ContentNavigationModel",
            "PETRICHOR": "sas.hc.ui.services.navigation.models.ContentNavigationPETModel"
        },
        "CONTENT_NAVIGATOR_OPERATION_RESULT_DESCRIPTOR":
        {
            "WIP": "sas.hc.ui.services.navigation.descriptors.OperationsResultDescriptor",
            "PETRICHOR": "sas.hc.ui.services.navigation.descriptors.PETOperationsResultDescriptor"
        },
        "CONTENT_NAVIGATOR_PERMISSIONS_DESCRIPTOR":
        {
            "WIP": "sas.hc.ui.services.navigation.descriptors.ContentPermissionsDescriptor",
            "PETRICHOR": "sas.hc.ui.services.navigation.descriptors.ContentPETPermissionsDescriptor"
        },
        "CONTENT_NAVIGATOR_TYPE_DESCRIPTOR":
        {
            "WIP": "sas.hc.ui.services.navigation.ContentTypeDescriptor",
            "PETRICHOR": "sas.hc.ui.services.navigation.descriptors.ContentPETTypeDescriptor"
        },
        "CONTENT_NAVIGATOR_VALIDATION_RESULT_DESCRIPTOR":
        {
            "WIP": "sas.hc.ui.services.navigation.descriptors.ContentValidationResultDescriptor",
            "PETRICHOR": "sas.hc.ui.services.navigation.descriptors.ContentPETValidationResultDescriptor"
        },
        "SEARCH":
        {
            "WIP": "sas.hc.ui.services.search.service.DefaultSearchServiceImpl",
            "PETRICHOR": "sas.hc.ui.services.search.service.DefaultPETSearchServiceImpl"
        },
        "SEARCH_APPLICATION_ACCESS":
        {
            "WIP": "sas.hc.ui.services.search.service.DefaultApplicationAndTypeAccessServiceImpl",
            "PETRICHOR": "sas.hc.ui.services.search.service.DefaultPETApplicationAndTypeAccessServiceImpl"
        },
        "SEARCH_CONTENT_TYPE":
        {
            "WIP": "sas.hc.ui.services.search.service.DefaultContentTypeServiceImpl",
            "PETRICHOR": "sas.hc.ui.services.search.service.DefaultPETContentTypeServiceImpl"
        },
        "SEARCH_SCHEMA":
        {
            "WIP": "sas.hc.ui.services.search.service.DefaultSchemaServiceImpl",
            "PETRICHOR": "sas.hc.ui.services.search.service.DefaultPETSchemaServiceImpl"
        },
        "ABOUT_MODEL":
        {
            "WIP": "sas.hc.ui.commons.banner.help.AboutContentProvider",
            "PETRICHOR": "sas.hc.ui.commons.banner.help.AboutContentPETProvider"
        },
        // Settings service (Preference Service Wrapper) used by Settings Dialog.  Any settings service
        // implementation defined here MUST adhere to the contract set forth by the Settings dialog.
        "SETTINGS":
        {
            "WIP": "sas.hc.ui.commons.settings.WIPSettingsService",
            "PETRICHOR": "sas.hc.ui.commons.settings.PetrichorSettingsAdapterService"
        },
        // Service which provide the list of available applications available to user.  I really dislike the
        // name "Side Menu" service.  It is meaningless and doesn't convey anything about what the service
        // actually does.
        "SIDE_MENU":
        {
            "WIP": "sas.hc.ui.commons.settings.DefaultSideMenuService"
            //                   "PETRICHOR": "NeedsImplementation"            // TODO once service exists provide new PET implementation
        },
        "NOTIFICATIONS":
        {
            "WIP": "sas.hc.ui.services.notifications.NotificationsService",
            "PETRICHOR": "sas.hc.ui.services.notifications.PetrichorNotificationsAdapterService"
        },
        "APPLICATION_SWITCHER_MODEL":
        {
            "WIP": "sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherModel",
            "PETRICHOR": "sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherPetrichorModel"
        },
        // user info service
        "USERINFO":{
            "WIP":"sas.hc.ui.core.util.WIPUserInfoService",
            "PETRICHOR":"sas.hc.ui.core.util.PetUserInfoService"
        },
        "SIGNOUT":{
            "PETRICHOR":"sas.hc.ui.core.util.PetSignoutService"
        },
        "THEME":
        {
            "WIP": "sas.hc.ui.core.theme.service.wip.WIPThemeService",
            "PETRICHOR": "sas.hc.ui.core.theme.service.petrichor.PetrichorThemeService"
        },
        "THEME_UTIL":
        {
            "PETRICHOR":"sas.hc.ui.services.petrichor.theme.ThemeService"
        },
        // keep alive service
        "KEEP_ALIVE":{
            "WIP": "sas.hc.ui.core.util.WIPKeepAliveService",
            "PETRICHOR": "sas.hc.ui.core.util.PetKeepAliveService"
        },
        // Help service
        "HELP": {
            "WIP": "sas.hc.ui.commons.banner.help.WebDocService",
            "PETRICHOR": "sas.hc.ui.commons.banner.help.HelpCenterService"
        },

        "REGISTRY_TASK":{
            "WIP": "sas.hc.ui.core.RegistryTask"
        },

        "VERSION_INFO_TASK":{
            "PETRICHOR": "sas.hc.ui.core.VersionInfoTask"
        },

        "CACHE_MANAGER_TASK":{
            "PETRICHOR": "sas.hc.ui.core.cache.CacheManagerTask"
        },

        "LEFT_NAV_SERVICE":{
            "PETRICHOR": "sas.hc.ui.core.LeftNavPetDataProvider"
        },
        "RECENT_MODEL": {
            "PETRICHOR": "sas.hc.ui.commons.recent.RecentDialogPETModel"
        },
        "FONT" : {
            "PETRICHOR": "sas.hc.ui.services.petrichor.fonts.FontService"
        }

    },

    /**
     * For the given serviceName map to an implementation class based on
     *
     * @param {string} serviceName name of service
     * @returns {object} class to go with service
     */
    getServiceClass:function(serviceName) {
        if(this.serviceClassMappings.hasOwnProperty(serviceName)) {
            var platform = sas.hc.ui.core.util.ServicesUtil.getServicePlatform();
            var servicePlatformMappings=this.serviceClassMappings[serviceName];
            if(servicePlatformMappings.hasOwnProperty(platform)) {
                return this.serviceClassMappings[serviceName][platform];
            }
            else
                sas.log.debug("ComponentServiceClassResolver: Unable to locate a service implementation for " + serviceName + " that handles " + platform + " platform.");
        }
        else
            sas.log.error("ComponentServiceClassResolver: Unable to locate any service implementation for " + serviceName);

        return null;
    },

    getServiceClassInstance:function(serviceName) {
        var serviceInstance=undefined,
            serviceClass = this._getConfiguredServiceClass(serviceName) || this.getServiceClass(serviceName);

        if(serviceClass) {
            try {
                // load it if needed
                jQuery.sap.require(serviceClass);
                var objService = jQuery.sap.getObject(serviceClass);
                serviceInstance = new objService();
            } catch (e) {
                sas.log.error('ComponentServiceClassResolver: Unable to instantiate ' + serviceName + " service implementation class : " + serviceClass);
            }
        }
        return serviceInstance;
    },

    /**
     * For the given serviceName map to a configured implementation class.
     * Configurations can be specified in say config.js  by consumers.
     *
     * @param {string} serviceName name of service
     * @returns {object} class to go with service
     * @private
     */
    _getConfiguredServiceClass:function(serviceName) {
        var serviceClassMappings = sas.config.get("serviceClassMappings");
        if(serviceClassMappings && serviceClassMappings.hasOwnProperty(serviceName)) {
            var platform = sas.hc.ui.core.util.ServicesUtil.getServicePlatform();
            var servicePlatformMappings=serviceClassMappings[serviceName];
            if(servicePlatformMappings.hasOwnProperty(platform)) {
                return serviceClassMappings[serviceName][platform];
            }
        }
        return null;
    }

};

// (c) 2015, SAS Institute Inc.
/**
 * The base abstract model for an application theme provider.
 */
jQuery.sap.declare('sas.hc.ui.core.theme.ApplicationThemeProvider');
jQuery.sap.require('sap.ui.base.Object');
//jQuery.sap.require('sas.hc.ui.core.theme.ApplicationTheme'); // only referenced in jsdoc, removing dependency to optimize ThemeBootTask from bringing in this dependency chain

(function() {
    /**
     * The base abstract model for application theme providers.
     * @public
     * @name sas.hc.ui.core.theme.ApplicationThemeProvider
     * @class The base abstract model for application theme providers.
     * @extends sap.ui.base.Object
     */
    sap.ui.base.Object.extend('sas.hc.ui.core.theme.ApplicationThemeProvider', {
        metadata: {
            abstract: true,
            publicMethods: [
                'getApplicationThemes'
            ]
        },
        constructor: function() {
            sap.ui.base.Object.apply(this, arguments);
            if (this.getMetadata().getName() === "sas.hc.ui.core.theme.ApplicationThemeProvider") {
                throw new Error("The class 'sas.hc.ui.core.theme.ApplicationThemeProvider' is abstract and must not be instantiated!");
            }
        }
    });

    /**
     * Reference to ApplicationThemeProvider class.
     *
     * @private
     */
    var ApplicationThemeProvider = sas.hc.ui.core.theme.ApplicationThemeProvider;

    /**
     * Returns an array of available application theme objects
     * @returns {sas.hc.ui.core.theme.ApplicationTheme[]} array of application theme objects
     * @public
     */
    ApplicationThemeProvider.prototype.getApplicationThemes = function () {
        return [];
    };

    /**
     * expose _themeProviders on the class for testability
     * @name sas.hc.ui.core.theme.ApplicationThemeProvider#_themeProviders
     * @private
     * @static
     */
    ApplicationThemeProvider._themeProviders = [];

    /**
     * Registers an application theme provider to be used for lazy loading of available themes using getApplicationThemes.
     * @name sas.hc.ui.core.theme.ApplicationThemeProvider#registerProvider
     * @public
     * @static
     * @param {String|sas.hc.ui.core.theme.ApplicationThemeProvider} provider Either the name of a theme provider class that will be instantiated lazily with no arguments or an instance of a subclass of sas.hc.ui.core.theme.ApplicationThemeProvider
     */
    ApplicationThemeProvider.registerProvider = function(provider) {
        if (ApplicationThemeProvider._themeProviders.indexOf(provider) === -1) {
            // no duplicates allowed
            ApplicationThemeProvider._themeProviders.push(provider);
        }
    };

    /**
     * Returns an array of available application theme objects
     * @returns {sas.hc.ui.core.theme.ApplicationTheme[]} array of application theme objects
     * @public
     * @static
     */
    ApplicationThemeProvider.getApplicationThemes = function() {
        // TODO modify this API to return a promise to allow the provider implementations to operate asynchronously, unable to do this currently because ThemeDescriptor.getApplicationThemes was written synchronously and it would be an API change to use a promise there
        // merge the application themes of all the registered providers
        var applicationThemes = [],
            providers = ApplicationThemeProvider._themeProviders;
        function addThemes(provider) {
            if (typeof provider !== 'undefined' && typeof provider.getApplicationThemes === 'function') {
                // this is an ApplicationThemeProvider instance
                // cannot use concat because of scoping
                var i, toPush = provider.getApplicationThemes(), len = toPush.length;
                for (i = 0; i < len; ++i) {
                    applicationThemes.push(toPush[i]);
                }
            }
        }
        if (providers.length === 0) {
            // servicePlatform should always be present because of Application jsp tag.
            // Can also be overridden by solution in config.js
            var platform=sas.config.get("servicePlatform");

            // We should always have a platform available to use either automatically via jsp tag
            // or by solution specific override.  In the last ditch case that we don't attempt some
            // best guesses if possible.
            if (typeof(platform) === "undefined" || platform === null) {
                sas.registry.wip?platform = "WIP":platform="PETRICHOR";
            }

            if(platform==="WIP"){
                // default to the embedded and custom providers with default settings if nothing was setup differently
                providers = [
                    "sas.hc.ui.core.theme.EmbeddedApplicationThemeProvider",
                    "sas.hc.ui.core.theme.CustomApplicationThemeProvider"
                ];
            }else{
                providers = [
                    "sas.hc.ui.core.theme.CustomApplicationThemeProvider"
                ];
            }
        }
        providers.forEach(function(provider) {
            if (typeof provider === 'string') {
                jQuery.sap.require(provider);
                var constructor = jQuery.sap.getObject(provider);
                addThemes(new constructor());
            } else {
                addThemes(provider);
            }
        });
        return applicationThemes;
    };
}());

jQuery.sap.declare("sas.hc.ui.core.theme.ThemeDescriptor");

jQuery.sap.require("sas.hc.ui.core.theme.ApplicationThemeProvider");
jQuery.sap.require("sas.hc.ui.core.util.ComponentServiceClassResolver");

/**
 * This class provides convenient methods for accessing information
 * from descriptor.json in themes.
 * @name sas.hc.ui.core.theme.ThemeDescriptor
 * @param {object} desc JSON object with descriptor data usually generated from an openuitheme build
 * @class
 * @public
 */
sas.hc.ui.core.theme.ThemeDescriptor = function(desc){
    this._desc = desc || {};

    var ApplicationThemeProvider = sas.hc.ui.core.theme.ApplicationThemeProvider;

    /**
     * Returns an object representing the application theme for the given application theme ID.
     * @param {string} themeId the theme to lookup
     * @returns {object} application theme object
     * @public
     */
    this.getApplicationTheme = function(themeId){
        var appThemeArray = this.getAvailableApplicationThemes().filter(function(item){
            return item.id === themeId;
        });
        if (appThemeArray.length > 0){
            return appThemeArray[0];
        }else{
            return null;
        }
    };

    /**
     * Returns an object representing the report theme for the given theme ID.
     * @param {string} reportThemeId the id of the report theme to get
     * @returns {object} report theme object
     * @public
     */
    this.getReportTheme = function(reportThemeId){
        var repThemeArray = this.getAvailableReportThemes().filter(function(item){
            return item.id === reportThemeId;
        });
        if (repThemeArray.length > 0){
            return repThemeArray[0];
        }else{
            return null;
        }
    };

    /**
     * Returns the array of embedded application theme descriptions
     * @returns {array} array of application theme descriptions
     * @private
     */
    this._getRawApplicationThemeDescriptions = function() {
        // this method should only ever be called by tests and the EmbeddedApplicationThemeProvider
        // should refactor someday to separate the loading of the descriptor data from the implementation of this class
        // (i.e. there should be no need for this class if concerns are properly separated)
        return this._desc.applicationThemes || [];
    };

    /**
     * Returns an array of available application theme objects
     * @returns {array} array of application theme objects
     * @public
     */
    this.getAvailableApplicationThemes = function(){
        // Don't cache this. Need to get an updated value in case someone published/unpublished/deleted a
        // theme since the last time this was accessed.
        return ApplicationThemeProvider.getApplicationThemes();
    };

    /**
     * Returns an array of available report theme objects
     * @returns {array} array of report theme objects
     * @public
     */
    this.getAvailableReportThemes = function(){
        var themeService = sas.hc.ui.core.util.ComponentServiceClassResolver.getServiceClassInstance(sas.hc.ui.core.util.ComponentServiceClassResolver.THEME_UTIL_SERVICE);
        if(themeService){
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.hc.ui.core");
            var reportThemes = [];
            var options = {
                async: false,
                data: {
                    published: true,
                    limit: 10000
                },
                headers: {
                    "Accept-Item": "application/vnd.sas.theme.report+json"
                },
                dataType: "json"
            };
            themeService.get(options)
                .done(function(result){
                    if(result && result.items) {
                        result.items.forEach(function(themeJSON){
                            if(!themeJSON.name){
                                var themeName = rb.getText("systemTheme.reportTheme."+themeJSON.id+".label.txt");
                                if(themeName){
                                    themeJSON.name = themeName;
                                }
                            }
                            reportThemes.push(themeJSON);
                        });
                    }
                })
                .fail(function(error){
                    sas.log.error("ThemeUtil.loadReportTheme(): Load failed for report themes: "+error);
                });

            return reportThemes;
        }else {
            return this._desc.reportThemes || [];
        }
    };

    /**
     * Returns a display label String for the given report theme ID
     * @returns {string} report theme name
     * @public
     */
    this.getReportThemeName = function(themeId){
        var themeService = sas.hc.ui.core.util.ComponentServiceClassResolver.getServiceClassInstance(sas.hc.ui.core.util.ComponentServiceClassResolver.THEME_UTIL_SERVICE);
        var themeName = themeId;
        if(themeService){
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.hc.ui.core");
            var options = {
                async: false,
                id: themeId,
                dataType: "json"
            };
            themeService.get(options)
                .done(function(result){
                    if(result && !result.name) {
                        themeName = rb.getText("systemTheme.reportTheme."+result.id+".label.txt");
                    }
                })
                .fail(function(error){
                    sas.log.error("ThemeDescriptor.getReportThemeName(): Error: "+error);
                });
        }
        return themeName;
    };

    /**
     * Returns the current theme version
     * @returns {string} value of version property in descriptor.json
     * @public
     */
    this.getVersion = function(){
        var version = this._desc.version;
        if (!version) {
            version = "unknown";
            sas.log.error("Theme descriptor is missing a version number. This will prevent custom themes from applying correctly.");
        }
        return version;
    };

    /**
     * Returns the icon collections defined in descriptor.json
     * @returns {array} array of icon collections
     * @public
     */
    this.getIconCollections = function(){
        return this._desc.iconCollections || [];
    };
};

/**
 * @public
 * @method
 * @static
 * @name sas.hc.ui.core.theme.ThemeDescriptor#loadDescriptor
 * @param {String} [sUrl] The url of the descriptor to load. If neither sUrl or oData are specified, data will be loaded from the descriptor at sas.config.getThemeUrl() + '/descriptor.json'.
 * @param {Object} [oData] The descriptor data to use. If both sUrl and oData are specified, oData will be used.
 * @return {sas.hc.ui.core.theme.ThemeDescriptor} The loaded descriptor.
 */
sas.hc.ui.core.theme.ThemeDescriptor.loadDescriptor = function(sUrl, oData) {
    var async;

    if (typeof sUrl === "undefined" && typeof oData === "undefined") {
        // if neither is set, use the themeUrl
        sUrl = sas.config.getThemeUrl();
        sUrl = sUrl + (sUrl.slice(-1) === "/" ? "" : "/") + "sas/hc/ui/core/themes/descriptor.json";
    }
    if (typeof sUrl !== "undefined" && typeof oData !== "undefined") {
        // if both are set, use the data
        sUrl = null;
    }
    if (typeof sUrl === "string") {
        async = {};
        // load the descriptor from the url
        sas.ajax({url: sUrl, async: false}).done(function (oFetchedData) {
            async.data = oFetchedData;
        }).fail(function (oObj, textStatus) {
            async.error = textStatus;
        });
        if (async.error) {
            throw new Error("Couldn't load theme descriptor" + async.error);
        }
        oData = async.data;
    }

    return new sas.hc.ui.core.theme.ThemeDescriptor(oData);
};

/**
 * @public
 * @method
 * @static
 * @name sas.hc.ui.core.theme.ThemeDescriptor#loadDescriptorAsync
 * @return {Promise} That resolves with loaded {sas.hc.ui.core.theme.ThemeDescriptor} object.
 *
 */

sas.hc.ui.core.theme.ThemeDescriptor.loadDescriptorAsync = function() {
    return new Promise(function(resolve,reject){
        var sUrl = sas.config.getThemeUrl();
        sUrl = sUrl + (sUrl.slice(-1) === "/" ? "" : "/") + "sas/hc/ui/core/themes/descriptor.json";
        // load the descriptor from the url
        sas.ajax({url: sUrl}).done(function (oFetchedData) {
            resolve(new sas.hc.ui.core.theme.ThemeDescriptor(oFetchedData));
        }).fail(function (oObj, textStatus) {
            sas.log.error("Couldn't load theme descriptor" + textStatus);
            resolve(new sas.hc.ui.core.theme.ThemeDescriptor(null));
        });
    });
};


jQuery.sap.declare("sas.hc.ui.core.theme.ThemeUtil");

jQuery.sap.require("sas.hc.ui.core.theme.ThemeDescriptor");
// only used in jsdoc, removed for optimization of ThemeBootTask jQuery.sap.declare('sas.hc.ui.core.theme.ApplicationTheme');

/**
 * This class provides functions, events and properties related to themes.
 * @name sas.hc.ui.core.theme.ThemeUtil
 * @class
 * @public
 */
sas.hc.ui.core.theme.ThemeUtil = function(){

    "use strict";

    /**
     * The name of the user theme capability.
     * @constant
     * @public
     * @type {string}
     */
    this.CUSTOMIZE_USER_THEME_CAPABILITY = "CustomizeUserThemes";

    /**
     * The current commons version.  Used to determine the correct path for theme content.
     * @constant
     * @public
     * @type {string}
     */
    this.CURRENT_COMMONS_VERSION = "8.0.0";

    /**
     * Message published by loadReportTheme on a successful load.
     * @constant
     * @public
     * @type {string}
     * @default
     */
    this.REPORT_THEME_LOADED = "reportThemeLoaded";

    /**
     * Message published by loadReportTheme when an error occurs.
     * @constant
     * @public
     * @type {string}
     * @default
     */
    this.REPORT_THEME_LOAD_ERROR = "reportThemeLoadError";

    /**
     * Message published by unloadReportTheme.
     * @constant
     * @public
     * @type {string}
     * @default
     */
    this.REPORT_THEME_UNLOADED = "reportThemeUnloaded";

    /**
     * Message published by addReportThemeStyles.
     * @constant
     * @public
     * @type {string}
     * @default
     */
    this.REPORT_THEME_ADDED = "reportThemeAdded";

    /**
     * Message published by removeReportThemeStyles.
     * @constant
     * @public
     * @type {string}
     * @default
     */
    this.REPORT_THEME_REMOVED = "reportThemeRemoved";

    /**
      * Relative path to the report themes.
      * @constant
      * @private
      * @type {string}
      * @default
      */
    this._REPORT_THEME_RELATIVE_PATH = "sas/hc/graph/reportThemes";

    /**
     * EventBus channel for Themes messages.
     * @constant
     * @public
     * @type {string}
     * @default
     */
    this.THEMES = "Themes";

    // report theme name constants

    /**
     * Theme id for the light report theme.
     * @constant
     * @public
     * @type {string}
     * @default
     */
    this.LIGHT = "light";         // REPORT THEME

    /**
     * Theme id for the dark report theme.
     * @constant
     * @public
     * @type {string}
     * @default
     */
    this.DARK = "dark";           // REPORT THEME

    /**
     * Theme id for the highcontrast report theme.
     * @constant
     * @public
     * @type {string}
     * @default
     */
    this.HIGHCONTRAST = "highcontrast";  // REPORT THEME

    /**
     * Theme id for the snow report theme.
     * @constant
     * @public
     * @type {string}
     * @default
     */
    this.SNOW = "snow";  // REPORT THEME

    /**
     * Theme id for the corporate report theme.
     * @constant
     * @public
     * @type {string}
     * @default
     * @deprecated Use SNOW instead.
     */
    this.CORPORATE = this.SNOW;  // REPORT THEME

    /**
     * Theme id for the default report theme.
     * @constant
     * @public
     * @type {string}
     * @default
     */
    this.DEFAULT_REPORT_THEME = this.SNOW; // REPORT THEME

    // application theme name constants

    /**
     * Theme id for the corporate application theme.
     * @constant
     * @public
     * @type {string}
     * @default
     * @deprecated Since version 2.2. Use constants in sas.hc.ui.core.theme.ApplicationTheme instead.
     */
    this.SAS_CORPORATE = "sas_corporate"; // repeated here as an optimization for the ThemeBootTask so the ApplicationTheme dependency chain is not loaded sas.hc.ui.core.theme.ApplicationTheme.SAS_CORPORATE;  // APP THEME

    // Currently loaded report theme IDs
    var _currentReportThemes = [];

    this.getVersionLibraryThemeVersion = function(libraryName, defaultVersion){
        var libraryVersion = defaultVersion;
        var versionInfo = sap.ui.getVersionInfo();
        if(versionInfo && versionInfo.libraries) {
            versionInfo.libraries.forEach(function (library) {
                if (library.name === libraryName) {
                    libraryVersion = library.version;
                }
            });
        }

        if(libraryVersion){
            if(libraryVersion.indexOf("-")!==-1){
                libraryVersion = libraryVersion.substring(0, libraryVersion.indexOf("-"));
            }

            if(libraryVersion.indexOf(".")!==-1 && libraryVersion.indexOf(".")!==libraryVersion.lastIndexOf(".")){
                libraryVersion = libraryVersion.substring(0, libraryVersion.lastIndexOf("."));
            }
        }

        return libraryVersion;
    };

    /**
     * Loads a report theme for a given report theme ID.
     * Publishes ThemeUtil.REPORT_THEME_LOADED on the ThemeUtil.THEMES channel of sas.eventBus for a successful load
     * Publishes ThemeUtil.REPORT_THEME_LOAD_ERROR on the ThemeUtil.THEMES channel of sas.eventBus if an error occurs during loading.
     * @param {string} reportThemeId the theme to load
     * @public
     */
    this.loadReportTheme = function(reportThemeId){
        // Don't attempt to load if theme already loaded
        if (jQuery.inArray(reportThemeId, this.getCurrentReportThemes()) !== -1){
            sas.eventBus.publish(this.THEMES, this.REPORT_THEME_LOADED, {themeId: reportThemeId});
            sas.log.info("ThemeUtil.loadReportTheme(): " + reportThemeId + " already loaded.");
            return;
        }

        var that = this;

        var themeService = sas.hc.ui.core.util.ComponentServiceClassResolver.getServiceClassInstance(sas.hc.ui.core.util.ComponentServiceClassResolver.THEME_UTIL_SERVICE);
        var repThemeUrl;

        if(themeService){
            var options = {
                async: false,
                id: reportThemeId,
                dataType: "json",
                headers: {Accept: "application/vnd.sas.theme.report+json"}
            };
            themeService.get(options)
                .done(function(result){
                    if(result && result.rootUrl){
                        repThemeUrl = result.rootUrl + sas.themes.getVersionLibraryThemeVersion("sas.hc.graph", sas.themes.CURRENT_COMMONS_VERSION) + result.htmlStyleSheetPath;
                    }
                })
                .fail(function(error){
                    sas.log.error("ThemeUtil.loadReportTheme(): Load failed for report theme " + reportThemeId + ": "+error);
                });
        }else{
            var repTheme = sas.themeDescriptor.getReportTheme(reportThemeId);
            if (repTheme !== null) {
                var repThemeUrl = sas.config.getThemeUrl();
                repThemeUrl = repThemeUrl + (repThemeUrl.slice(-1) === "/" ? "" : "/") + this._REPORT_THEME_RELATIVE_PATH + "/" + reportThemeId + "/" + repTheme.stylesheets[0].url;
            }
        }


        if(repThemeUrl){

            // Add a link element for this report theme's stylesheet
            sas.log.trace("ThemeUtil.loadReportTheme(): Calling includeStylesheet()");
            jQuery.sap.includeStyleSheet(repThemeUrl, reportThemeId + "ReportStylesheet",
                function(){
                    if (jQuery.inArray(reportThemeId, that.getCurrentReportThemes()) === -1){
                        that.getCurrentReportThemes().push(reportThemeId);
                    }
                    that._publishReportThemeLoaded(reportThemeId);
                },
                function(){
                    that._publishReportThemeLoadError(reportThemeId);
                }
            );
        } else {
            sas.eventBus.publish(this.THEMES, this.REPORT_THEME_LOAD_ERROR, {themeId: reportThemeId});
            sas.log.error("ThemeUtil.loadReportTheme(): Report theme " + reportThemeId + " is not defined.");
        }
    };

    /**
     * Publishes the REPORT_THEME_LOAD_ERROR event.  Used by ThemeUtil and LessController to signal when a report
     * theme has failed to load.
     * @param reportThemeId
     * @private
     */
    this._publishReportThemeLoadError = function(reportThemeId){
        sas.eventBus.publish(this.THEMES, this.REPORT_THEME_LOAD_ERROR, {themeId: reportThemeId});
        sas.log.error("ThemeUtil.loadReportTheme(): Load failed for report theme " + reportThemeId);
    };

    /**
     * Publishes the REPORT_THEME_LOADED event.  Used by ThemeUtil and LessController to signal when a report
     * theme has been successfully loaded.
     * @param reportThemeId
     * @private
     */
    this._publishReportThemeLoaded = function(reportThemeId){
        sas.eventBus.publish(this.THEMES, this.REPORT_THEME_LOADED, {themeId: reportThemeId});
        sas.log.info("ThemeUtil.loadReportTheme(): Report theme " + reportThemeId + " loaded.");
    };

    /**
     * Unloads a report theme for a given report theme ID.
     * Publishes ThemeUtil.REPORT_THEME_UNLOADED on the ThemeUtil.THEMES channel of sas.eventBus for a successful unload
     * @param {string} reportThemeId the report theme to unload
     * @public
     */
    this.unloadReportTheme = function(reportThemeId){
        var i = this.getCurrentReportThemes().indexOf(reportThemeId);
        if (i >= 0){
            // Remove the link element for this report theme's stylesheet
            jQuery("#" + reportThemeId + "ReportStylesheet").remove();
            this.getCurrentReportThemes().splice(i, 1);
            sas.log.info("ThemeUtil.unloadReportTheme(): Report theme " + reportThemeId + " unloaded.");
        } else {
            sas.log.info("ThemeUtil.unloadReportTheme(): report theme " + reportThemeId + " not currently loaded.");
        }
        sas.eventBus.publish(this.THEMES, this.REPORT_THEME_UNLOADED, {themeId: reportThemeId});
    };

    /**
     * Returns an array of loaded report theme IDs
     * @returns {string[]} array of report theme IDs
     * @public
     * @public
     */
    this.getCurrentReportThemes = function(){
        return _currentReportThemes;
    };

    /**
     * Adds the report theme ID to the given element's class attribute to allow
     * report theme styles to be applied.
     * Publishes ThemeUtil.REPORT_THEME_ADDED on the ThemeUtil.THEMES channel of sas.eventBus for a successful add.
     * @param {string} reportThemeId the report theme to apply
     * @param {string} elementId the element to modify
     * @public
     */
    this.addReportThemeStyles = function(reportThemeId, elementId){
        if (elementId){
            var element = jQuery("#" + elementId);
            if (!element.hasClass(reportThemeId)){
                element.addClass(reportThemeId);
            }
            sas.eventBus.publish(this.THEMES, this.REPORT_THEME_ADDED, {themeId: reportThemeId, elementId: elementId});
        }
    };

    /**
     * Removes the report theme ID from the given element's class attribute.
     * Publishes ThemeUtil.REPORT_THEME_REMOVED on the ThemeUtil.THEMES channel of sas.eventBus for a successful remove.
     * @param {string} reportThemeId the report theme to remove
     * @param {string} elementId the element to modify
     * @public
     */
    this.removeReportThemeStyles = function(reportThemeId, elementId){
        if (elementId){
            var element = jQuery("#" + elementId);
            if (element.hasClass(reportThemeId)){
                element.removeClass(reportThemeId);
            }
            sas.eventBus.publish(this.THEMES, this.REPORT_THEME_REMOVED, {themeId: reportThemeId, elementId: elementId});
        }
    };

    /**
     * Returns the base url containing the default graph styles for the current theme. The sas.hc.graph library must be loaded or this method will return <code>null</code>.
     * @public
     * @returns {string} Full path that can be used for loading the graph stylesheet or <code>null</code> if the sas.hc.graph library is not loaded.
     * TODO graph should use a sas.hc.graph library and allow the library to load the styles rather than this mechanism and this method should be deprecated.
     */
    this.getGraphStylesheetUrl = function(){
        return this.getThemeResourceUrl("sas.hc.graph", "graph.css");
    };

    /**
     * Returns the url to the graph stylesheet for the given report theme.
     * @param {string} reportThemeId the report theme to get the stylesheet for
     * @returns {string} Full path that can be used for loading the report theme graph stylesheet
     * @public
     */
    this.getReportGraphStylesheetUrl = function(reportThemeId){

        var themeService = sas.hc.ui.core.util.ComponentServiceClassResolver.getServiceClassInstance(sas.hc.ui.core.util.ComponentServiceClassResolver.THEME_UTIL_SERVICE);
        var repThemeUrl;
        if(themeService){
            var options = {
                async: false,
                id: reportThemeId,
                dataType: "json",
                headers: {Accept: "application/vnd.sas.theme.report+json"}
            };
            themeService.get(options)
                .done(function(result){
                    if(result && result.rootUrl){
                        repThemeUrl = result.rootUrl + sas.themes.getVersionLibraryThemeVersion("sas.hc.graph", sas.themes.CURRENT_COMMONS_VERSION) + result.reportStyleSheetPath;
                    }
                })
                .fail(function(error){
                    sas.log.error("ThemeUtil.loadReportTheme(): Load failed for report theme " + reportThemeId + ": "+error);
                });
        }else{
            repThemeUrl = sas.config.themeUrl + "/" + this._REPORT_THEME_RELATIVE_PATH + "/"+reportThemeId+"/graph.css";
        }

        return repThemeUrl;
    };

    /**
     * Given a relative filename such as MyAudioFile.mp3 compute the full path to
     * where it is located in the theme directory.
     *
     * In the current implementation audio files while stored with themes are not changed per theme but rather a theme independent resource and always loaded from embedded themes.
     *
     * @param {string} sAudioFileName Relative path to the desired file, starting at the audio directory
     * @returns {string} Full path that can be used for loading the audio file
     * @public
     */
    this.getAudioPath = function(sAudioFileName) {
        return sas.config.getThemeUrl() + "/" + sAudioFileName;
    };

    /**
     * Wrapper for private sap.ui.Core _getThemePath method that appends a resource path. This API insulates consumers from changes to the core API.
     * @param {string} library The name of the library to look the theme resource up for. Example sas.hc.ui.commons.
     * @param {string} resourcePath The relative path to the resource to be looked up.
     * @param {string} [theme] The theme to lookup the resource in. Defaults to the currently applied theme if not specified.
     * @returns {string} The url to the resource.
     * @public
     * @see {@link https://openui5.hana.ondemand.com/docs/api/symbols/sap.ui.html#.resource|sap.ui.resource}
     */
    this.getThemeResourceUrl = function(library, resourcePath, theme) {
        var core = sap.ui.getCore(),
            theme = theme || core.getConfiguration().getTheme(),
            relativePath;
        // remove leading / if present
        if (resourcePath.charAt(0) === '/') {
            if (resourcePath.length === 1) {
                relativePath = "";
            } else {
                relativePath = resourcePath.substring(1);
            }
        } else {
            relativePath = resourcePath;
        }
        // during library initialization and theme changes, themes for the library are loaded and the theme is
        // registered as a module using the theme root to redirect to the correct location
        // see https://openui5.hana.ondemand.com/docs/api/symbols/sap.ui.html
        relativePath = "themes/" + theme + "/" + relativePath;
        return sap.ui.resource(library, relativePath);
    };

    /**
     * SAS images used by the icon pool are stored above the themes directory in a library independent "images" directory. Fix up the urls for this fact.
     * @param {string} resourcePath The relative path to the resource to be looked up.
     * @param {string} [theme] The theme to lookup the resource in. Defaults to the currently applied theme if not specified.
     * @param {string} [library] The library this themed image is part of. Defaults to sas.hc.ui.core.
     * @returns {string} The url to the resource.
     * @public
     * @see {@link https://openui5.hana.ondemand.com/docs/api/symbols/sap.ui.html#.resource|sap.ui.resource}
     */
    this.getThemeImageResourceUrl = function(resourcePath, theme, library) {
        var core = sap.ui.getCore();
        var theme = theme || core.getConfiguration().getTheme();
        var library = library || "sas.hc.ui.core";
        var url = this.getThemeResourceUrl(library, resourcePath, theme);

        // replace theme name with base/images where images actually live
        url = url.replace(theme, "sas_base/images");

        return url;
    };

    this.getCurrentThemeBannerImageUrl = function(callback, currentTheme) {
        if (!currentTheme)
            currentTheme = sap.ui.getCore().getConfiguration().getTheme();
        if (currentTheme && currentTheme !== sas.hc.ui.core.theme.ApplicationTheme.DEFAULT) {
            var themeService = sas.hc.ui.core.util.ComponentServiceClassResolver.getServiceClassInstance(sas.hc.ui.core.util.ComponentServiceClassResolver.THEME_UTIL_SERVICE);
            if (themeService) {
                var options = {
                    async: false,
                    headers: {
                        "Accept": "application/vnd.sas.theme.application.complete+json"
                    },
                    dataType: "json",
                    id: currentTheme
                };
                themeService.get(options).done(function(theme) {
                    if (theme.styleData) {
                        var bannerImageId = theme.styleData["@bannerImageId"];
                        if (bannerImageId) {
                            callback(sas.hc.ui.core.theme.ThemeUtil.unquoteImageId(bannerImageId));
                        }
                    }
                });
            }
        }
    };

    this.getDefaultApplicationThemeId = function(){
        var themeService = sas.hc.ui.core.util.ComponentServiceClassResolver.getServiceClassInstance(sas.hc.ui.core.util.ComponentServiceClassResolver.THEME_UTIL_SERVICE);
        var defaultTheme = sas.hc.ui.core.theme.ApplicationTheme.DEFAULT;
        if(themeService) {
            var options = {
                async: false,
                dataType: "json",
                id: "@defaultApplicationTheme"
            };
            themeService.get(options).done(function(applicationTheme){
                defaultTheme = applicationTheme.id;
            });
        }
        return defaultTheme;
    };

    /**
     * Function that detects the presence of style class mentioned in the function parameter in the control
     * and its parent DOM nodes until the UIArea DOM node using jQuery.closest API.
     *
     * @param {sap.ui.core.Control} control
     * @param {string} styleClass - styleClass name preceded by a dot
     * @return {boolean}
     * @private
     */
    this.isGivenStylePresentInParentChain = function(control, styleClass) {
        var match = false,
            $control = jQuery.sap.byId(control.getId()),
            uiArea = control.getUIArea(),
            uiAreaDom;

        if(uiArea)
            uiAreaDom = jQuery.sap.domById(uiArea.getId());

        //jQuery API - https://api.jquery.com/closest/
        if(!!uiAreaDom && $control.closest(styleClass, uiAreaDom).length === 1) {
            match = true;
        }

        return match;
    };

    this.controlShouldInheritStyleClass = function(oControl, sStyleClass) {
        var oControlToCheck = oControl;
        var sStyleClassQuery = "."+sStyleClass;

        while (oControlToCheck) {
            // While walking up the parent hierarchy not every control will have
            // the hasStyleClass() or $() method.
            var bHasStyleClass = jQuery.isFunction(oControlToCheck.hasStyleClass)
                    && oControlToCheck.hasStyleClass(sStyleClass);
            var bAncestorHasStlyeClass = jQuery.isFunction(oControlToCheck.$)
                    && oControlToCheck.$().closest(sStyleClassQuery).length > 0;

            if (bHasStyleClass || bAncestorHasStlyeClass) {
                return true;
            }

            // In theory all of the controls in the chain should have a getParent method, even
            // though the root one will return null from it, but we want to be safe
            if (jQuery.isFunction(oControlToCheck.getParent)) {
                oControlToCheck = oControlToCheck.getParent();
            } else {
                oControlToCheck = null;
            }
        }

        return false;
    };
};

sas.hc.ui.core.theme.ThemeUtil.getImageIdFromURL = function(url){
    if(jQuery.sap.startsWith(url,"\"/files/files/")){
        url = url.substring("\"/files/files/".length);
    }
    if(jQuery.sap.endsWith(url,"/content\"")){
        url = url.substring(0, url.length-"/content\"".length);
    }

    return url;
};

sas.hc.ui.core.theme.ThemeUtil.unquoteImageId = function(url){
    if(jQuery.sap.startsWith(url,"\"")){
        url = url.substring(1);
    }
    if(jQuery.sap.endsWith(url,"\"")){
        url = url.substring(0, url.length-1);
    }

    return url;
};

sas.hc.ui.core.theme.ThemeUtil.getURLForImageId = function(imageId){
    return "\"/files/files/"+imageId+"/content\"";
};

/*
 * Object proxy.
 * @private
 */
jQuery.sap.declare("sas.hc.ui.core.util.ObjectProxy");
/**
 * The Proxy object is used to define custom behavior for fundamental operations (e.g. property lookup, assignment,function invocation, etc).
 * @param {Object} target - A target object (can be any sort of objects to wrap with Proxy.
 * @param {Object} handler - An object whose properties are functions which define the behavior of the proxy when an operation is performed on it.
 *         Handler if provided MUST have at least get and set traps/functions.
 *         handler.get() - A trap for getting property values.
 *        handler.set() - A trap for setting property values.
 *
 * @see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Proxy
 * @private
 */
if(self.Proxy){
    sas.hc.ui.core.util.ObjectProxy = Proxy;
}else{
    sas.hc.ui.core.util.ObjectProxy = function(target,handler) {
        jQuery.sap.assert(target && typeof target !== 'undefined' , "target must be a non null object");


        var getFn,setFn;
        if (handler && typeof handler['get'] === 'function' ) {
            getFn = function(prop){
                return handler.get(target,prop);
            };
        } else {
            getFn = function(prop){
                return target[prop];
            };
        };

        if ( handler && typeof handler['set'] === 'function' ) {
            setFn = function(prop,newVal){
                handler.set(target,prop,newVal);
                return this;
            };
        } else {
            setFn = function(prop,newVal){
                target[prop] = newVal;
                return this;
            };
        }


        // copy all properties of target into this object.
        function defineProperty(prop){
            Object.defineProperty(this,prop,{
                enumerable: true,
                get: function(){
                    return getFn.call(this,prop);
                },
                set: function(newVal){
                    return setFn.call(this,prop,newVal);
                }
            });
        }

        for(var _prop in target){
            if (target.hasOwnProperty(_prop)) {
                defineProperty.call(this,_prop);
            }
        }
        // add a special constructor property to make this a jQuery plain object
        this.constructor = target.constructor;
    };
}
/*
 * jqXHRRetryHandler handler.
 * @private
 */
jQuery.sap.declare("sas.hc.ui.core.util.jqXHRRetryHandler");
jQuery.sap.require("sas.hc.ui.core.util.ObjectProxy");

(function(){

    sas.hc.ui.core.util.jqXHRRetryHandler = function(jqXHR, options) {
        jQuery.sap.assert(jqXHR && typeof jqXHR !== 'undefined' , "jqXHR must be a non null object");
        jQuery.sap.assert(options && typeof options !== 'undefined' , "options must be a non null object");
        this._options = options;
        this._deferred = jQuery.Deferred();
        this._jqXHR = jqXHR;
        // register a fail handler on jqXHR.
        this.registerFailHandler(true);
        this.registerDoneHandler();
    };


    sas.hc.ui.core.util.jqXHRRetryHandler.prototype.update = function(jqXHR){
        this._jqXHR = jqXHR;
        // set request headers if any
        if (typeof this._requestHeaders === 'object'){
            for( var name in this._requestHeaders ) {
                if (this._requestHeaders.hasOwnProperty(name)) {
                    this.setRequestHeader(name, this._requestHeaders[name]);
                }
            }
        }
        this.registerDoneHandler();
        this.registerFailHandler(false);
    };

    sas.hc.ui.core.util.jqXHRRetryHandler.prototype.get = function(target,prop){

        if ( prop in this._deferred ) {
            return this._deferred[prop];
        }

        var that = this;
        if ( prop === 'setRequestHeader'){
            return function(name,value){
                that._requestHeaders = that._requestHeaders || {};
                that._requestHeaders[name] = value;
                that.setRequestHeader(name,value);
            };
        }

        if ( prop in this._jqXHR ) {
            return this._jqXHR[prop];
        }
    };

    sas.hc.ui.core.util.jqXHRRetryHandler.prototype.set = function(target,prop,newVal){
        // not supported.
    };

    function shouldRetry(jqXHR){
        switch(jqXHR.status){
            case 449: return true;
            case 403: return jqXHR.getResponseHeader('X-Forbidden-Reason') === 'CSRF';
            case 401: return jqXHR.getResponseHeader('X-Unauthorized-Reason') === 'POSSIBLE_CSRF';
            default: return false;
        }
        return false;
    };


    sas.hc.ui.core.util.jqXHRRetryHandler.prototype.registerFailHandler = function(retryOnAuthError){
        var that = this;
        this._jqXHR.fail(function( jqXHR, textStatus, errorThrown){
            if(retryOnAuthError === true && shouldRetry(jqXHR)){
                // handle authentication retry as well as CSRF token check.
                // process ajaxComplete handlers if any before proceeding.
                // required for processing CSRF tokens if any.
                sas.eventBus.publish("ajaxComplete", {jqXHR: jqXHR, options:that._options});
                // retry the request.
                var retObj = jQuery.ajax(that._options);
                that.update(new sas.hc.ui.core.util.ObjectProxy(retObj,
                    new sas.hc.ui.core.util.jqXHRRetryHandler(retObj,that._options)));
            } else {
                that._deferred.rejectWith( this, [ jqXHR, textStatus, errorThrown ] );
            }
        });
    };

    sas.hc.ui.core.util.jqXHRRetryHandler.prototype.registerDoneHandler = function(){
        var that = this;
        this._jqXHR.done(function( success, statusText, jqXHR){
            that._deferred.resolveWith( this, [ success, statusText, jqXHR ] );
        });
    };


    sas.hc.ui.core.util.jqXHRRetryHandler.prototype.setRequestHeader = function(name,value){
        this._jqXHR.setRequestHeader(name,value);
    };

}());

jQuery.sap.declare("sas.hc.ui.core.UtilTask");

jQuery.sap.require("sas.hc.ui.core.BootTask");
jQuery.sap.require("sas.hc.ui.core.GlobalSettings");
jQuery.sap.require("sas.hc.ui.core.util");
jQuery.sap.require("sas.hc.ui.core.util.ObjectProxy");
jQuery.sap.require("sas.hc.ui.core.util.jqXHRRetryHandler");
jQuery.sap.require("sap.ui.core.EventBus");
jQuery.sap.require("sas.hc.ui.core.theme.ThemeUtil");
jQuery.sap.require("sas.hc.ui.core.util.ServicesUtil");
(function(){
    sas.hc.ui.core.BootTask.extend('sas.hc.ui.core.UtilTask',{
        name: 'sas.hc.ui.core.UtilTask'
    });

    var usrPrefFormatLangs;

    function registerLanguageFilters(){
        var sapConf = sap.ui.getCore().getConfiguration();

        // set user preferred language on each http call.
        // do it asap.
        // http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.4
        // Using private API from jQuery.sap.resources to get appropriate user locale and fall back locale if
        // user locale happens to be not among supported locales by this application.
        // Expect private API to break when SDK is updated and this API gets dropped.
        // As per 15w33 image SupportedLanguages locales are populated in java.util.Locales format hence need to use this API.
        var usrPrefLangs = jQuery.sap.resources._getFallbackLocales(sapConf.getLanguage(),sapConf.getSupportedLanguages());
        if(Array.isArray(usrPrefLangs)){
            usrPrefLangs = usrPrefLangs.toString();
            if(usrPrefLangs.charAt(usrPrefLangs.length-1) === ','){
                // get rid of the last comma.
                usrPrefLangs = usrPrefLangs.substr(0,usrPrefLangs.length-1);
            }
            // SAP API returns locales in non BCP-47 syntax. Convert them.
            usrPrefLangs = usrPrefLangs.replace(/_/g,'-');
        }
        // defer 'Accept-Language' header value to browser if condition not met.
        if(typeof usrPrefLangs === 'string' && usrPrefLangs !== ''){
            var acceptLanguageAjaxFilter = function(options, originalOptions, jqXHR ) {
                if(typeof options !== 'object' || options === null){
                    // should never reach here. Someone trying to be innovative.
                    return;
                }
                options.headers = options.headers || {};
                if(typeof options.headers === 'object'){
                    var isAcceptLanguage = true;
                    var isAcceptFormatLanguage = true;
                    for(var prop in options.headers){
                        // better to do nothing if headers already has
                        // accept-language value be it in different case.
                        // jQuery.ajax#setRequestHeaders function normalizes header names.
                        if(prop.toLowerCase() === 'accept-language'){
                            isAcceptLanguage = false;
                        }
                        if(prop.toLowerCase() === 'accept-locale'){
                            isAcceptFormatLanguage = false;
                        }
                    }
                    if (isAcceptLanguage)
                        options.headers["Accept-Language"] = usrPrefLangs;
                    if (isAcceptFormatLanguage &&   // Accept-Format-Language request header
                        options.acceptLocale === true && // acceptLocale:true is explicitly specified in options?
                        usrPrefFormatLangs && typeof usrPrefFormatLangs === 'string' && usrPrefFormatLangs !== ''){
                        options.headers["Accept-Locale"] = usrPrefFormatLangs;
                    }
                }
            };

            jQuery.ajaxPrefilter(acceptLanguageAjaxFilter);
        }else{
            sas.log.error('Could not activate acceptLanguageAjaxFilter.');
        }
    };

    function registerFormatLocaleFilters(){
        var sapConf = sap.ui.getCore().getConfiguration();
        var formatLocale = sapConf.getFormatLocale();
        formatLocale = formatLocale.replace(/_/g,'-').toLowerCase(); // Convert to BCP 47 language tag.
        // Get preferred format locale and Set it in the OpenUI formatLocale
        sas.ajax({url:"/preferences/preferences/@currentUser/FormatLocale.DefaultLocale", async:true}).done(
            function(objUsrInfo){
                if(objUsrInfo && typeof objUsrInfo.value === 'string') {
                    formatLocale = objUsrInfo.value.replace(/_/g,'-').toLowerCase();// Convert to BCP 47 language tag.
                    sapConf.setFormatLocale(formatLocale);
                }
                updateFormatLocale(formatLocale);
            }).fail(function(oObj,txt){
                if(sas.log.info){
                    sas.log.info('sas.hc.ui.core.UtilTask','Could not fetch preferred format locale.');
                } else if(console && console.info){ // eslint-disable-line no-console
                    console.info('No preferred format locale found for current user. ' /* + oObj.statusText*/); // eslint-disable-line no-console
                }
                updateFormatLocale(formatLocale);
            });

        function updateFormatLocale(formatLocale) {
            // Update the usrPrefFormatLangs
            var aLocales = addOldJDKLocales(sas.hc.ui.core.GlobalSettings.SAS_SUPPORT_LOCALES);
            usrPrefFormatLangs = jQuery.sap.resources._getFallbackLocales(formatLocale, aLocales);
            if(Array.isArray(usrPrefFormatLangs)){
                usrPrefFormatLangs = usrPrefFormatLangs.toString();
                if(usrPrefFormatLangs.charAt(usrPrefFormatLangs.length-1) === ','){
                    // get rid of the last comma.
                    usrPrefFormatLangs = usrPrefFormatLangs.substr(0,usrPrefFormatLangs.length-1);
                }
                // SAP API returns locales in non BCP-47 syntax. Convert them.
                usrPrefFormatLangs = usrPrefFormatLangs.replace(/_/g,'-');
            }
        };
        // This regular expression came from jquery.sap.resources.js.
        var rLocale = /^((?:[A-Z]{2,3}(?:-[A-Z]{3}){0,3})|[A-Z]{4}|[A-Z]{5,8})(?:-([A-Z]{4}))?(?:-([A-Z]{2}|[0-9]{3}))?((?:-[0-9A-Z]{5,8}|-[0-9][0-9A-Z]{3})*)((?:-[0-9A-WYZ](?:-[0-9A-Z]{2,8})+)*)(?:-(X(?:-[0-9A-Z]{1,8})+))?$/i;
        function addOldJDKLocales(aLocales) {
            var M_ISO639_NEW_TO_OLD = {
                "he" : "iw",
                "yi" : "ji",
                "id" : "in",
                "sr" : "sh"
            };
            var aReturnLocales = [];
            aLocales.forEach(function(sLocale) {
                aReturnLocales.push(sLocale);
                var m = rLocale.exec(sLocale.replace(/_/g, '-'));
                var sLanguage = m[1].toLowerCase();
                sLanguage = M_ISO639_NEW_TO_OLD[sLanguage];
                if (sLanguage) {
                    var sScript = m[2] ? m[2].toLowerCase() : undefined;
                    var sRegion = m[3] ? m[3].toUpperCase() : undefined;
                    if ( sLanguage === "zh" && !sRegion ) {
                        if ( sScript === "hans" ) {
                            sRegion = "CN";
                        } else if ( sScript === "hant" ) {
                            sRegion = "TW";
                        }
                    }
                    aReturnLocales.push(sLanguage + ((sRegion) ? "-" + sRegion: ""));
                }
            });
            return aReturnLocales;
        };
    };

    // utility function used in sas.ajax'
    var shouldHandleAuthRetry;

    var logIfLocalAjaxEventsUsed = function(options){
        //http://api.jquery.com/Ajax_Events/
        if(options.success || options.error || options.complete){
            sas.log.error('[sas.ajax]','Use of local ajax events like success/error/complete is strongly discouraged as it hinders execution of built-in retry handlers.'
                        + ' Use promise callbacks like then/done/fail/always instead. See http://api.jquery.com/Types/#Promise','Url:'+options.url);
            // log error once and change the function to no-op to avoid
            // executing then onwards.
            logIfLocalAjaxEventsUsed = Function.prototype;
        }
    };

    function _invokeAjaxCall(options){
        var retObj = jQuery.ajax(options);
        // check for sas handleAuthentication option
        if ( shouldHandleAuthRetry === true) {
            logIfLocalAjaxEventsUsed(options);
            return new sas.hc.ui.core.util.ObjectProxy(
                retObj,
                new sas.hc.ui.core.util.jqXHRRetryHandler(retObj,options)
            );
        } else {
            return retObj;
        }
    }

    function defineSasAjax(){

        shouldHandleAuthRetry = sas.config.get('xx-sasAjaxHandleAuthenticationRetry') !== false;

        // add sas ajax function.
        Object.defineProperty(sas,'ajax', {
            enumerable:true,
            value: function(options){

                options = options || {};
                options.acceptLocale = options.acceptLocale !== false ? true : false;
                options.type = typeof options.type === 'string' ? options.type.toUpperCase() : 'GET';
                options.processData = typeof options.processData === 'boolean' ? options.processData : true;
                // default the contents to json.
                if(options.dataType === undefined){
                    options.dataType = 'json';
                }
                if(options.contentType === undefined){
                    options.contentType = 'application/json';
                    // jsonize data if JSON exist.
                    if(options.processData && window.JSON && options.type !== 'GET' && typeof options.data === 'object'){
                        options.data = JSON.stringify(options.data);
                    }

                }
                // check for sas specific options
                if((options.sas && options.sas.crossDomain) || sas.hc.ui.core.util.isUrlCrossDomain(options.url)){
                    // change the url
                    var q = {};
                    var destUrl = options.url;
                    if(options.data && options.type === 'GET' && options.processData){
                        var params = jQuery.param( options.data,options.traditional);
                        destUrl += ( /\?/.test( destUrl ) ? "&" : "?" ) + params;
                        delete options.data;
                    }
                    q['sas-commons-destUrl'] = destUrl;
                    // check for authentication.
                    if(options.sas && options.sas.authenticate){
                        q['sas-commons-auth'] = true;
                    }
                    // if cross domain proxy config value doesn't start or contain '/' (a case with local cdps)
                    // then treat it as running locally within application.
                    options.url = sas.hc.ui.core.util._getCrossDomainProxyUrl()+"?"+jQuery.param(q);
                }
                var retObj;
                // check if there is any cache registered to handle this request.
                if(sas.cacheManager){
                    var _cache = sas.cacheManager.findCache(options);
                    if(_cache){
                        retObj = _cache.handle(options,_invokeAjaxCall);
                    }
                }
                return retObj || _invokeAjaxCall(options);
            }
        });
    }

    function configureThemeRoot(){
        // patch up the theme roots, our version of the OpenUI SDK has a bug in sap.ui.core.Configuration that will mask the themeRoot property in the global configuration
        // we setup a global themeroots property on the configuration similar to the way it is handled in the 1.26 SDK
        // TODO when we upgrade to a more recent SDK, verify we didn't break anything and still need to wrap cross domain urls
        var config = window["sap-ui-config"] || {};
        var themeRoots = config.themeroots || {};
        var core = sap.ui.getCore();

        // setup the roots
        for (var theme in themeRoots) {
            if (themeRoots.hasOwnProperty(theme)) {
                var themeRoot = themeRoots[theme];
                if(typeof themeRoot === 'string'){

                    // determine value of unversionedThemes. GUIDESINF-1416
                    // should be true in non Petrichor platofrm unless specified otherwise.
                    var unversionedThemes = config.sas && config.sas.unversionedThemes;
                    if(typeof unversionedThemes === 'undefined'){
                        var servicePlatform = sas && sas.config && sas.config.get('servicePlatform');
                        if (servicePlatform === 'PETRICHOR'){
                            unversionedThemes = false;
                        } else {
                            unversionedThemes = true;
                        }
                    }

                    //allows applications to override the theme URL adding version information (currently used by QUnit and ThemeHost
                    if(unversionedThemes){
                        core.setThemeRoot(theme, themeRoot);
                    }else {
                        var themeUtil = new sas.hc.ui.core.theme.ThemeUtil();
                        var versionInfo = sap.ui.getVersionInfo();
                        if (versionInfo && versionInfo.libraries) {
                            versionInfo.libraries.forEach(function (library) {
                                core.setThemeRoot(theme, [library.name], themeRoot + themeUtil.getVersionLibraryThemeVersion(library.name, themeUtil.CURRENT_COMMONS_VERSION));
                            }, this);
                        }
                    }
                }else if(typeof themeRoot === 'object'){
                    for (var lib in themeRoot) {
                        if (lib.length > 0) {
                            core.setThemeRoot(theme, [lib], themeRoot[lib]);
                        } else {
                            core.setThemeRoot(theme, themeRoot[lib]);
                        }
                    }
                }
            }
        }
    }


    sas.hc.ui.core.UtilTask.prototype.run=function(){

        registerLanguageFilters();

        // add SAS application level EventBus.  Could return the one
        // from sap.ui.getCore() but create a dedicate SAS bus instead.
        Object.defineProperty(sas,'eventBus',{
            value : new sap.ui.core.EventBus(),
            enumerable:true,

        });

        defineSasAjax();

        if (sas.hc.ui.core.util.ServicesUtil.getServicePlatform() === sas.hc.ui.core.util.ServicesUtil.ServicePlatform.PETRICHOR)
            registerFormatLocaleFilters();

        configureThemeRoot();

        // register an even listener for ltjs loading if any
        sas.eventBus.subscribe("ltjs","loading_started",function(){
            // code to update splash screen
            var splashScreenHeaderElm = jQuery("#sas-hc-splash-headerText");
            if( splashScreenHeaderElm && splashScreenHeaderElm.length > 0){
                // splash screen found.
                var rb = sap.ui.getCore().getLibraryResourceBundle("sas.hc.ui.core");
                splashScreenHeaderElm.text(rb && rb.getText('splash.header.initializing.txt'));
            }
        });

        if (sap.ui.Device.browser.edge) {
            jQuery("html").attr("x-ms-format-detection", "none");
        }
    };
}());
/*
 * Default user info service in WIP world.
 *
 */
jQuery.sap.declare("sas.hc.ui.core.util.WIPUserInfoService");

(function(){
    "use strict";

    sap.ui.base.Object.extend("sas.hc.ui.core.util.WIPUserInfoService",{
        metadata:{
            publicMethods:["getUserInfo","getUserInfoAsync"]
        }
    });

    /**
     * Loads UserInfo object from the server.
     * @returns {UserInfo}
     */
    sas.hc.ui.core.util.WIPUserInfoService.prototype.getUserInfo = function(){
        var objUsrInfo = undefined;
        if(sas && sas.registry && typeof sas.registry.getUserDetailsService === 'function'){
            sas.registry.getUserDetailsService().get({async:false}).done(
                function(oData){
                    if(oData){
                        objUsrInfo=oData;
                    }
                }).fail(function(oObj,txt){
                    if(sas.log.error){
                        sas.log.error('sas.hc.ui.core.util.WIPUserInfoService','Could not fetch userinfo.');
                    }
                });
        }
        return objUsrInfo;
    };

    /**
     * Loads UserInfo object from server asynchronously.
     * @returns Promise that will be resolved with {UserInfo} valid instance if fetched successfully.
     * Otherwise Promise would be resolved with undefined.
     */
    sas.hc.ui.core.util.WIPUserInfoService.prototype.getUserInfoAsync = function(){

        var defrrd = jQuery.Deferred(),
            objUsrInfo = undefined;
        if(sas && sas.registry && typeof sas.registry.getUserDetailsService === 'function'){
            sas.registry.getUserDetailsService().get().done(
                function(oData){
                    if(oData){
                        objUsrInfo=oData;
                    }
                    defrrd.resolve(objUsrInfo);
                }).fail(function(oObj,txt){
                    if(sas.log.error){
                        sas.log.error('sas.hc.ui.core.util.WIPUserInfoService','Could not fetch userinfo.');
                    }
                    defrrd.reject();
                });
        }else{
            defrrd.resolve();
        }
        return defrrd.promise();
    };


})();
/*
 * Default user info service in Petrichor SAS 10 world.
 *
 */
jQuery.sap.declare("sas.hc.ui.core.util.PetUserInfoService");

(function(){
    "use strict";

    sap.ui.base.Object.extend("sas.hc.ui.core.util.PetUserInfoService",{
        metadata:{
            publicMethods:["getUserInfo","getUserInfoAsync"]
        }
    });

    /**
     * Loads UserInfo object from server.
     * @returns {UserInfo}
     */
    sas.hc.ui.core.util.PetUserInfoService.prototype.getUserInfo = function(){
        var objUsrInfo = undefined;
        sas.ajax({url:"/identities/users/@currentUser", async:false}).done(
            function(oData){
                if(oData){
                    objUsrInfo=oData;
                }
            }).fail(function(oObj,txt){
                if(sas.log.error){
                    sas.log.error('sas.hc.ui.core.util.PetUserInfoService','Could not fetch userinfo.');
                }
            });
        if(typeof objUsrInfo === "object" && objUsrInfo){
            // create a displayName property if not available
            if(!objUsrInfo.hasOwnProperty("displayName") && objUsrInfo.hasOwnProperty("name")){
                objUsrInfo.displayName = objUsrInfo.name;
            }
            // create a userid property if not available
            if(!objUsrInfo.hasOwnProperty("userid") && objUsrInfo.hasOwnProperty("id")){
                objUsrInfo.userid = objUsrInfo.id;
            }
        }
        return objUsrInfo;
    };

    /**
     * Loads UserInfo object from server asynchronously.
     * @returns Promise that will be resolved with {UserInfo} valid instance if fetched successfully.
     * Otherwise Promise would be resolved with undefined.
     */
    sas.hc.ui.core.util.PetUserInfoService.prototype.getUserInfoAsync = function(){

        var defrrd = jQuery.Deferred(),
            objUsrInfo = undefined;
        sas.ajax({url:"/identities/users/@currentUser", async:true})
            .done(function(oData){
                if(oData){
                    objUsrInfo=oData;
                    if(typeof objUsrInfo === "object" && objUsrInfo){
                        // create a displayName property if not available
                        if(!objUsrInfo.hasOwnProperty("displayName") && objUsrInfo.hasOwnProperty("name")){
                            objUsrInfo.displayName = objUsrInfo.name;
                        }
                        // create a userid property if not available
                        if(!objUsrInfo.hasOwnProperty("userid") && objUsrInfo.hasOwnProperty("id")){
                            objUsrInfo.userid = objUsrInfo.id;
                        }
                    }
                }
                defrrd.resolve(objUsrInfo);
            }).fail(function(oObj,txt){
                if(sas.log.error){
                    sas.log.error('sas.hc.ui.core.util.PetUserInfoService','Could not fetch userinfo.');
                }
                defrrd.reject();

            });
        return defrrd.promise();
    };
})();

/**
 * Class to provide information of logged in user.
 */
jQuery.sap.declare("sas.hc.ui.core.util.UserInfo");
jQuery.sap.require("sas.hc.ui.core.util.ComponentServiceClassResolver");

(function(){
    "use strict";

    sas.hc.ui.core.util.UserInfo={};
    var userInfo=undefined,
        asyncPromise=undefined;

    Object.defineProperty(sas.hc.ui.core.util.UserInfo,'instance',{
        get: function(){
            if(typeof userInfo === 'undefined'){
                try{
                    var serviceResolver=sas.hc.ui.core.util.ComponentServiceClassResolver,
                        service=serviceResolver.getServiceClassInstance(serviceResolver.USERINFO_SERVICE);
                    if(typeof service === 'object'){
                        userInfo = service.getUserInfo();
                    }
                }catch(err){
                    if(sas.log.error){
                        sas.log.error('sas.hc.ui.core.util.UserInfo','Could not load userinfo.',err);
                    }
                }
            }
            return userInfo;
        }
    });

    // try loading userinfo asap
    function loadUserInfoAsync(){
        if(typeof userInfo === 'undefined' && !asyncPromise){
            asyncPromise = new Promise(function(resolve,reject){
                try{
                    var serviceResolver=sas.hc.ui.core.util.ComponentServiceClassResolver,
                        service=serviceResolver.getServiceClassInstance(serviceResolver.USERINFO_SERVICE);
                    if(typeof service === 'object'){
                        service.getUserInfoAsync().done(function (obj){
                            if(sas.log.trace){
                                sas.log.trace('sas.hc.ui.core.util.UserInfo','Successfully loaded Userinfo asyncly.',obj);
                            }
                            if(typeof userInfo === 'undefined'){
                                userInfo = obj;
                            }
                            resolve(userInfo);
                        }).fail(function(){
                            reject();
                        });
                    }else{
                        reject();
                    }
                }catch(err){
                    if(sas.log.error){
                        sas.log.error('sas.hc.ui.core.util.UserInfo','Could not load userinfo asyncly.',err);
                    }
                    reject();
                }
            });
            asyncPromise.then(function(){},function(){
                asyncPromise = undefined;
            });
        }
        return asyncPromise;
    };

    //asyncInstance
    Object.defineProperty(sas.hc.ui.core.util.UserInfo,'instanceAsync',{
        get: function(){
            return userInfo ? Promise.resolve(userInfo) : loadUserInfoAsync();
        }
    });
    // create a boot task for loading user information
    jQuery.sap.declare("sas.hc.ui.core.util.UserInfoTask");
    jQuery.sap.require("sas.hc.ui.core.BootTask");

    sas.hc.ui.core.BootTask.extend('sas.hc.ui.core.util.UserInfoTask',{
        name:'sas.hc.ui.core.util.UserInfoTask',
        dependsOn:['sas.hc.ui.core.RegistryTask']
    });

    sas.hc.ui.core.util.UserInfoTask.prototype.run = function(){
        loadUserInfoAsync();
    };

})();
jQuery.sap.declare("sas.hc.ui.core.settings.SettingsInfo");

sap.ui.define('sas/hc/ui/core/settings/SettingsInfo',
    ['jquery.sap.global', 'sas/hc/ui/core/BootTask','sas/hc/ui/core/util/ServicesUtil'],
    function(jQuery, BootTask, ServicesUtil) {
        "use strict";

        var writeAccess = true;

        sas.hc.ui.core.settings.SettingsInfo = Object.create(Object.prototype,{
            hasWriteAccess:{
                get: function(){
                    return writeAccess;
                }
            }
        });
        
        /** boot task to determine write access */
        
        jQuery.sap.declare("sas.hc.ui.core.settings.SettingsInfoTask");
        
        var SettingsInfoTask = BootTask.extend('sas.hc.ui.core.settings.SettingsInfoTask',{
            name:'sas.hc.ui.core.settings.SettingsInfoTask'
        });
        
        SettingsInfoTask.prototype.runAsync = function(){
            if (ServicesUtil.getServicePlatform() === ServicesUtil.ServicePlatform.PETRICHOR) {
                // must return a Promise.
                return new Promise(function(resolve, reject){

                    //explicitly invoking POST /authorization/bulkDecision since Prefernces HATEOAS is not matching the authorization.
                    //In the below code if the toplevel authorization call fails, it will still result in un-necessary call to get the 
                    //Preferences authorization but we will ignore the impact since the Op is async and scenario is a rare one.
                    sas.ajax({
                        url: "/authorization/"
                    }).always(function(){
                        setTimeout(function(){
                            sas.ajax({
                                url: "/authorization/bulkDecision",
                                type: "POST",
                                headers: {
                                    Accept: "application/vnd.sas.authorization.links+json",
                                    ContentType: "application/json"
                                },
                                data: {
                                    "links": {
                                        "{\"uri\": \"/preferences/preferences/@currentUser/**\",\"method\": \"PUT\"}": "update",
                                        "{\"uri\": \"/preferences/preferences/@currentUser/**\",\"method\": \"DELETE\"}": "delete"
                                    },
                                    "matchParams": false
                                }
                            }).done(function(response) {
                                var updateAccess = false,
                                    deleteAccess = false;
                                jQuery.each(response.authorizedLinks, function(key, value) {
                                    var linkRep = JSON.parse(key);

                                    if (linkRep.method === "DELETE")
                                        deleteAccess = value;
                                    else if (linkRep.method === "PUT")
                                        updateAccess = value;
                                });
                                writeAccess = updateAccess && deleteAccess;
                                // resolve the promise.
                                resolve(); 
                            }).fail(function(e) {
                                //do nothing
                                // resolve the promise.
                                resolve(); 
                            });
                        },0);
                    });
                });
            }
            return Promise.resolve();
        };
        
        return sas.hc.ui.core.settings.SettingsInfo;
    });
/**
 * Created by sinshp on 7/3/14.
 */
jQuery.sap.declare("sas.hc.ui.commons.settings.AbstractSettingsService");
jQuery.sap.require("sas.hc.ui.core.util.UserInfo");
jQuery.sap.require("sap.ui.base.EventProvider");


(function() {
    //There is a defect at 2.1 to share the logged-in user details. S1097885
    //Then this block would no longer be needed.

    var userId;

    /**
     * Contract for a Settings Service.
     *
     *
     * @param {string} [sId] id for the new control, generated automatically if no id is given
     *
     * @class
     * This is a non-visual class representing a contract for writing Settings service.
     * All the concrete implementations are expected to override public methods.
     *
     * If a concrete implementation other than <code>sas.hc.ui.commons.WIPSettingsService</code>
     * is needed, it is required to register through <code>sas.hc.ui.commons.settings.SettingsManager.addService()</code>.
     *
     * @extends sap.ui.base.EventProvider
     *
     * @author SAS
     * @version 1.0
     *
     * @constructor
     *
     * @public
     * @name sas.hc.ui.commons.settings.AbstractSettingsService
     */
    sap.ui.base.EventProvider.extend("sas.hc.ui.commons.settings.AbstractSettingsService", {
        metadata: {
            library: "sas.hc.ui.commons",

            publicMethods: ["getSettings", "saveSettings", "removeSettings", "isAvailable", "get", "put", "delete"]

        },


        /**
         * returns the user-id for a logged-in user.
         * @public
         */
        "getUserId": function() {
            if (!userId) {
                if (sas.hc.ui.core.util.UserInfo.instance) {
                    userId = sas.hc.ui.core.util.UserInfo.instance.userid;
                }
            }
            return userId;
        },

        /**
         * inidcates the presence of the service.
         * @public
         */
        "isAvailable": function() {
            return true;
        },

        /**
         * fetches the settings.
         * @abstract
         */
        "getSettings": function(xhrSettings) {
            throw new Error("Please provide a concrete implementation for the AbstractSettingsService#getSettings function in the subclass");
        },

        /**
         * saves the settings
         * @abstract
         */
        "saveSettings": function(xhrSettings) {
            throw new Error("Please provide a concrete implementation for the AbstractSettingsService#saveSettings function in the subclass");
        },

        /**
         * remove the settings.
         * @abstract
         */
        "removeSettings": function(xhrSettings) {
            throw new Error("Please provide a concrete implementation for the AbstractSettingsService#removeSettings function in the subclass");
        },

        /**
         * is a final method which deals with the getSettings
         * concrete implementation returning a xhr object or not.
         * @final
         * @param {object} xhrSettings TODO-jsdoc
         * @returns {$.promise} TODO-jsdoc
         */
        "get": function(xhrSettings) {
            var getPromise, syncDeferred, retObject;

            if (this.isAvailable()) {
                //Adding the below config for IE remote requests to tackle S1143545
                if (sap.ui.Device.browser.internet_explorer) {
                    xhrSettings.cache = false;
                }
                retObject = getPromise = this.getSettings(xhrSettings);
            }

            if (!(typeof getPromise !== "undefined" && getPromise.promise)) {
                syncDeferred = $.Deferred();
                getPromise = syncDeferred.promise();

                if (typeof retObject === "undefined") {
                    var logger = sas.log.getLogger("sas.hc.ui.commons.settings.AbstractSettingsService");
                    logger.fatal("Possibly the data-access settings service is not available");
                    syncDeferred.reject();
                } else {
                    syncDeferred.resolve(retObject);
                }
            }

            return getPromise;
        },

        /**
         * is a final method which deals with the saveSettings
         * concrete implementation returning a xhr object or not.
         *
         * @param {object} xhrSettings TODO-jsdoc
         * @returns {$.promise} TODO-jsdoc
         */
        "put": function(xhrSettings) {
            var savePromise, syncDeferred, retObject;

            if (this.isAvailable()) {
                retObject = savePromise = this.saveSettings(xhrSettings);
            }

            if (!(typeof savePromise !== "undefined" && savePromise.promise)) {
                syncDeferred = $.Deferred();
                savePromise = syncDeferred.promise();

                if (typeof retObject === "undefined") {
                    var logger = sas.log.getLogger("sas.hc.ui.commons.settings.AbstractSettingsService");
                    logger.fatal("Possibly the data-access settings service is not available");
                    syncDeferred.reject();
                } else {
                    syncDeferred.resolve();
                }
            }
            return savePromise;
        },

        /**
         * is a final method which deals with the removeSettings
         * concrete implementation returning a xhr object or not.
         *
         * @param {object} xhrSettings jQuery ajax settings
         * @returns {$.promise} jQuery Promise handle
         */
        "delete": function(xhrSettings) {
            var deletePromise;
            //Not adding any support for the scenario Where there is no remote AJAX call being made.
            //It was added for GET and PUT but not sure if it of use to anybody.

            //backward compatibility fix incase a consumer has plugged in his own service
            //and did not implement the removeSettings contract introduced at 7.0
            if (this.removeSettings === sas.hc.ui.commons.settings.AbstractSettingsService.prototype.removeSettings) {
                deletePromise = this.put(xhrSettings);
            } else {
                if (this.isAvailable()) {
                    deletePromise = this.removeSettings(xhrSettings);
                }
            }
            return deletePromise;
        },
        
        /**
         * returns if the logged-in user has writeAccess to underlying service utilized for
         * persisting preferences.
         *
         * @returns {boolean}
         * @name sas.hc.ui.commons.settings.AbstractSettingsService#hasWriteAccess
         * @public
         */
        hasWriteAccess : function() {
            return true;
        }
    });
})();


/**
 *
 * fetches the values for the given setting keys.
 *
 *
 * The input param should obey the following format
 * <pre>
 *     {data : {
 *              "name": ["key1", "key2", ..]
 *           }
 *    };
 * </pre>
 *
 * The response from this call would be of the following format
 * <pre>
 *     {preferences : [
 *                                 {key: "keyName", value: "keyValue"}
 *                                 ...
 *                                 ..
 *                             ]};
 * </pre>
 *
 * @param {object} xhr settings
 * @return {$.promise}
 * @public
 * @name sas.hc.ui.commons.settings.AbstractSettingsService#getSettings
 * @function
 */

/**
 * Save the settings.
 *
 * The input would be of the following format
 *
 * The response from this call would be of the following format
 *
 * <pre>
 *     {preferences : [
 *                                 {key: "keyName", value: "keyValue"}
 *                                 ...
 *                                 ..
 *                             ]};
 * </pre>
 *
 * @param {object} xhr settings
 * @return {$.promise}
 * @public
 * @name sas.hc.ui.commons.settings.AbstractSettingsService#saveSettings
 * @function
 */

/**
 * indicates availability of the service.
 * The default implementation always indicates availability of the service positively.
 *
 *
 * @return {boolean}
 * @public
 * @name sas.hc.ui.commons.settings.AbstractSettingsService#isAvailable
 * @function
 */
/**
 * Created by sinshp on 7/3/14.
 */
jQuery.sap.declare("sas.hc.ui.commons.settings.TransientSettingsAdapterService");
jQuery.sap.require("sas.hc.ui.commons.settings.AbstractSettingsService");

(function() {

    /**
     * Called once during runtime evaluation of this module code.
     * Creates an internal store to store preferences for serviceProxies 
     * to utilize in Guest mode. The store is maintained as a sealed Object
     * against the top window to make it available across all applications
     * in an ApplicationSwitcher environment.
     * @private
     */
    function initialize() {
        if (!top.transientStore) {
            var _transientStore = new TransientPreferenceStore();
            Object.seal(_transientStore);
            Object.defineProperty(top, "transientStore", {
                configurable: false,
                enumerable: true,
                value: _transientStore
            });
        }
    };
    
    /**
     * Store for maintaining transient preferences.
     * supports getItems, putItems and deleteItems operations.
     *
     * @private
     */
    function TransientPreferenceStore() {
        var __transientStore = Object.create({});

        Object.defineProperties(this, {
            getItems: {
                value: function() {
                    return deepCopy(__transientStore);
                },
                configurable: false
            },

            putItems: {
                value: function(aItems) {
                    jQuery.each(aItems, function(index, itemToSave) {
                        var itemKey = itemToSave["key"],
                            itemValue = itemToSave["value"];

                        if (!__transientStore[itemKey]) {
                            __transientStore[itemKey] = itemValue;
                        } else {
                            if (itemValue === null) {
                                delete __transientStore[itemKey];
                            } else {
                                __transientStore[itemKey] = itemValue;
                            }
                        }
                    });
                },
                configurable: false
            },

            removeItems: {
                value: function(aItems) {
                    jQuery.each(aItems, function(index, itemToDelete) {
                        var itemKey = itemToDelete["key"];
                        if (__transientStore[itemKey]) {
                            delete __transientStore[itemKey];
                        }
                    });
                },
                configurable: false
            }
        });
    };
    
    /**
     * commons getter to get a reference of the store.
     * @private
     */
    function getStore() {
        return top.transientStore;
    };

    /**
     * deep clones an Array or an Object using jQuery.extend API.
     *
     * @param {Array| Object}
     * @returns {Array|Object}
     * @private
     */
    function deepCopy(obj) {
        if (Array.isArray(obj)) {
            return jQuery.extend(true, [], obj);
        }
        if (typeof obj === 'object') {
            return jQuery.extend(true, {}, obj);
        }
        return obj;
    };
    
    var TransientSettingsAdapterService,
        AbstractSettingsService = sas.hc.ui.commons.settings.AbstractSettingsService;

    /**
     * Contract for a TransientSettingsAdapterService. 
     * It adapts an AbstractSettingsService to adapt for storing
     * preferences in guest mode.
     *
     * @class
     *
     * @author SAS
     * @version 1.0
     * @constructor
     * @public
     * @name sas.hc.ui.commons.settings.TransientSettingsAdapterService
     */
    TransientSettingsAdapterService = sas.hc.ui.commons.settings.TransientSettingsAdapterService = function() {
        if (!this instanceof AbstractSettingsService || this._bIsAdapted) {
            return;
        }
        //bootup.
        initialize();

        // apply the methods of the adapters prototype to the AbstractSettingsService instance
        for (var fn in TransientSettingsAdapterService.prototype) {
            if (TransientSettingsAdapterService.prototype.hasOwnProperty(fn)) {
                this[fn] = TransientSettingsAdapterService.prototype[fn];
            }
        }

        this._bIsAdapted = true;
    };


    /**
     * fetches the preferences
     * @override
     * @public
     */
    TransientSettingsAdapterService.prototype.getSettings = function(xhrSettings) {
        var itemsToGet,
            store = getStore(),
            returnValues = [],
            localDeferred = jQuery.Deferred();

        if (xhrSettings.data && Array.isArray(xhrSettings.data.name) && xhrSettings.data.name.length > 0) {
            itemsToGet = xhrSettings.data.name.slice();

            jQuery.each(itemsToGet, function(index, matchKey) {
                jQuery.each(store.getItems(), function(key, value) {
                    if (matchKey === key) {
                        returnValues.push({
                            "key": key,
                            "value": value
                        });
                        return false;
                    }
                });
            });
        }
        localDeferred.resolve({
            "preferences": returnValues
        });
        return localDeferred.promise();
    };

    /**
     * saves the preferences
     * @override
     * @public
     */
    TransientSettingsAdapterService.prototype.saveSettings = function(xhrSettings) {
        var itemsToSave,
            store = getStore(),
            localDeferred = jQuery.Deferred();

        if (xhrSettings.data && Array.isArray(xhrSettings.data.preferences) && xhrSettings.data.preferences.length > 0) {
            itemsToSave = deepCopy(xhrSettings.data.preferences);
            store.putItems(itemsToSave);
        }

        //No reason to ever reject the save in this local store.
        localDeferred.resolve();
        return localDeferred.promise();
    };

    /**
     * removes the preferences
     * @override
     * @public
     */
    TransientSettingsAdapterService.prototype.removeSettings = function(xhrSettings) {
        var itemsToDelete,
            store = getStore(),
            localDeferred = jQuery.Deferred();

        if (xhrSettings.data && Array.isArray(xhrSettings.data.preferences) && xhrSettings.data.preferences.length > 0) {
            itemsToDelete = xhrSettings.data.preferences;
            store.removeItems(itemsToDelete);
        }
        //No reason to ever reject the delete in this local store.
        localDeferred.resolve();
        return localDeferred.promise();
    };

})();
jQuery.sap.declare("sas.hc.ui.commons.settings.PetrichorSettingsAdapterService");

jQuery.sap.require("sas.hc.ui.commons.settings.AbstractSettingsService");
jQuery.sap.require("sas.hc.ui.services.petrichor.preferences.PreferencesService");
jQuery.sap.require("sas.hc.ui.core.settings.SettingsInfo");
jQuery.sap.require("sas.hc.ui.commons.settings.TransientSettingsAdapterService");

(function() {

    //var logger = sas.log.getLogger("sas.hc.ui.commons.settings.PetrichorSettingsAdapterService", sas.log.Level.ERROR);
    // TODO saszjt - Good enough or look this up from some discovery registration similar to sas.registry.wip
    var prefService = new sas.hc.ui.services.petrichor.preferences.PreferencesService(),
        petrichorPreferenceServiceAvailability,
        SettingsInfo = sas.hc.ui.core.settings.SettingsInfo;

    /**
     * Constructor for a PetrichorSettingsAdapterService.
     *
     * Is a proxy for <code>sas.hc.ui.services.petrichor.preferences.PreferencesService</code>
     * that communicates with the mid-tier Petrichor Preference REST service.
     *
     * It also adapts the expected results and requests to be the the correct format for Petrichor
     *  preference service from the formats dictated by AbstractSettingsService which are based
     *  on WIP preference service.
     *
     * @class
     * Provides functions to communicate with mid-tier Petrichor Preference Service.
     *
     * @extends sas.hc.ui.commons.settings.AbstractSettingsService
     *
     * @author SAS
     * @version 2.0
     *
     * @constructor
     * @public
     * @name sas.hc.ui.commons.settings.PetrichorSettingsService
     */
    sas.hc.ui.commons.settings.AbstractSettingsService.extend("sas.hc.ui.commons.settings.PetrichorSettingsAdapterService", {
        metadata: {
            library: "sas.hc.ui.commons",
            publicMethods: ["getSettings", "saveSettings", "isAvailable", "hasWriteAccess"]
        },

        constructor: function() {
            sas.hc.ui.commons.settings.AbstractSettingsService.apply(this, arguments);

            if (!SettingsInfo.hasWriteAccess) {
                sas.hc.ui.commons.settings.TransientSettingsAdapterService.apply(this); //adapter
            }
        },

        /**
         * indicates whether the service is available.
         * @returns {boolean} TODO-jsdoc
         *
         * @public
         */
        isAvailable: function() {
            // only check this once
            if (petrichorPreferenceServiceAvailability === undefined) {
                // TODO saszjt do a lookup.......  Use something like sas.registry.wip???

                petrichorPreferenceServiceAvailability = true; // or maybe false...
            }

            return petrichorPreferenceServiceAvailability;
        },


        /**
         * Gets the value for the preference keys.
         *
         * By default, this method is executed asynchronously. The jQuery Promise returned by this method
         * can be used to bind the done and fail handlers.
         *
         * The input is expected as a Object with a property named data
         * and value as an Array of Strings. It can contain any other
         * properties supported by $.ajax().
         * <pre>
         *     {data : {
         *              "name": ["key1", "key2", ..]
         *           }
         *    };
         * </pre>
         *
         *
         * The response from the WIP rest service would be in the below format.
         * <pre>
         *     {preferences : [
         *                        {key: "keyName", value: "keyValue"}
         *                                 ...
         *                                 ..
         *                        ]
         *                    };
         * </pre>
         *
         *
         * @param {object} xhrSettings TODO-jsdoc
         * @return {jQuery.Promise} TODO-jsdoc
         * @public
         */
        getSettings: function(xhrSettings) {
            var settings = xhrSettings.data.name;
            //Petrichor getPreferences returns all preferences for the user.  so this list ends up being meaningless.
            // We could request additional functionality from Petricor to limit keys requested or we could do it ourselves
            // in PreferencesService but for now just return more than we need.

            // The results from Petrichor preference service need to be massaged to fit the desired format
            // of AbstractSettingsService which is the WIP format even though it does contain unnecesary levels.

            var getPreferencesOptions = $.extend({}, xhrSettings, {
                dataFilter: function(data, dataType) {
                    // Why is it not already parsed if dataType is json?   When it gets through to the completion it is already parsed.
                    // Just doesn't seem to be parsed at the filter step.
                    if (dataType === "json")
                        data = JSON.parse(data);

                    // This is pretty ridiculous.. Because we are supporting both WIP and Petrichor and WIP format is still what settings
                    // expects we need to map the Petrichor "id" field to the WIP "key" field.
                    // Sacrificing peformance and long term maintainability for short term "something"
                    if (data.hasOwnProperty("items")) {
                        data.items = $.map(data.items, function(preferenceObject) {
                            if ($.inArray(preferenceObject["id"], settings) !== -1) {
                                preferenceObject["key"] = preferenceObject["id"];
                                return preferenceObject;
                            }
                        });
                    } else
                        data.items = [];

                    // wrap preference objects in object containing "preferences" key to adapt Petrichor style to WIP style
                    return JSON.stringify({
                        "preferences": data.items
                    });
                }
            });
            // Don't want WIP data format cluttering preference options since we are passing the preferences to save directly.
            delete getPreferencesOptions["data"];

            //The getPreferences call does not get all the preferences as stated above. The Preference microservice
            //uses a page limit of 20 records by default. So as a fix to get the entire set of preferences, we are mimicking the
            //Preferences Cache behavior here by sending a very high limit.
            getPreferencesOptions["data"] = {
                limit: (Math.pow(2, 31) - 1)
            };

            //Should we set some specific applicatioName?

            return prefService.getPreferences(getPreferencesOptions);
        },


        /**
         * Sets the value for the preference keys.
         *
         * By default, this method is executed asynchronously. The jQuery Promise returned by this method
         * can be used to bind the done and fail handlers.
         *
         * The request param should be of the following format. It can contain any other
         * properties supported by $.ajax().
         *
         * <pre>
         *     {data: {
         *                preferences : [
         *                                 {key: "keyName", value: "keyValue"}
         *                                 ...
         *                                 ..
         *                             ]
         *            }
         *      };
         * </pre>
         *
         *
         * @param {object} xhrSettings TODO-jsdoc
         * @return {jQuery.Promise} TODO-jsdoc
         * @public
         */
        saveSettings: function(xhrSettings) {
            // This is pretty ridiculous.. Because we are supporting both WIP and Petrichor and WIP format is still what settings
            // expects we need to map the Petrichor "id" field to the WIP "key" field.
            // Sacrificing peformance and long term maintainability for short term "something"
            $.each(xhrSettings.data.preferences, function(index, preferenceObject) {
                preferenceObject["id"] = preferenceObject["key"];
            });


            return prefService.setPreferences(xhrSettings.data.preferences);
        },

        /**
         * Removes the specified preference keys from the Preference Service store.
         * By default, this method is executed asynchronously. The jQuery Promise returned by this method
         * can be used to bind the done and fail handlers.
         *
         * The request param should be of the following format. It can contain any other
         * properties supported by $.ajax().
         *
         * <pre>
         *     {data: {
         *                preferences : [
         *                                 {key: "keyName"}
         *                                 ...
         *                                 ..
         *                             ]
         *            }
         *      };
         * </pre>
         *
         *
         * @param {object} xhrSettings settings object for the remote ajax request
         * @return {jQuery.Promise} jQuery Promise handle to Which success and failure handlers can be bound.
         * @public
         */
        removeSettings: function(xhrSettings) {
            var preferenceKeys = $.map(xhrSettings.data.preferences, function(preferenceObject) {
                return preferenceObject["key"];
            });

            delete xhrSettings["data"];

            return prefService.removePreferences(preferenceKeys, xhrSettings);
        },

        /**
         * indicates if the logged-in user has write access to petrichor preference micro-service.
         * @override
         * @public 
         */
        hasWriteAccess: function() {
            return SettingsInfo.hasWriteAccess;
        }

    });
})();
jQuery.sap.declare("sas.hc.ui.core.applicationSwitcher.Application");

/**
 * For every application actively loaded into the Application Switcher there is
 * an instance of this class containing information about the current state of
 * the application.Represents an application loaded via Application Switcher
 *
 * @extends sap.ui.base.Object
 *
 * @author Jim Toering
 * @version 2.0
 *
 * @class
 * @private
 * @name sas.hc.ui.core.applicationSwitcher.Application
 */
sap.ui.base.Object.extend("sas.hc.ui.core.applicationSwitcher.Application",
    {
        constructor : function() {
            sap.ui.base.Object.apply(this, arguments);
        },

        // TODO In OpenUI could do these as metadata properties and have auto getters and setters. This would require
        // ApplicationSwitcher.js to use getters/setters and would also require that we extend off of ManagedObject.

        /**
         *  Application identifier - The unique id used for identifying this application.  Used for any
         *  cross application communication and switching between apps.
         *
         *  @private
         */
        appId: null,

        /**
         * URL which was used to launch this application.
         *
         * @private
         */
        url: null,

        /**
         * title represents the current state of the app such as an open
         * document name or "New Report" if nothing saved yet.
         *
         * @private
         */
        title: null,

        /**
         * appName represents the name of the application
         *
         * @private
         */
        appName: null,

        /**
         * The iframe in the DOM which represents this application
         *
         * @private
         */
        iframe: null,

        /**
         * Dirty flag indicating whether the application is currently in a "dirty" state
         *
         * @private
         */
        dirty: false,

        /**
         * Low level application switcher associated with application
         *
         * @private
         */
        getApplicationSwitcher: function() {
            if (this.iframe &&
                this.iframe[0] &&
                this.iframe[0].contentWindow &&
                this.iframe[0].contentWindow.sas &&
                this.iframe[0].contentWindow.sas.applicationSwitcher) {
                return this.iframe[0].contentWindow.sas.applicationSwitcher;
            }
            return null;
        }

    });


jQuery.sap.declare("sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherModel");

jQuery.sap.require("sap.ui.base.Object");
jQuery.sap.require("sas.hc.ui.core.util");

/**
 * @class
 * The Application Switcher Model that provides data to @{sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher}
 *
 * @extends sap.ui.base.Object
 *
 * @author Yateen Shaha
 * @version 2.0
 *
 * @public
 * @name sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherModel
 * @see sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher
 */
sap.ui.base.Object.extend("sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherModel", {

    /**
     * Obtain the "hierarchical" JSON list of applications THIS USER can access.  The result being based on
     * Applications' integration with VA Hub, with role based capability restrictions
     * per that integration.
     * @returns {object} <a href="http://api.jquery.com/deferred.promise/">deferred.promise().</a>
     * @see jQuery.Deferred
     *
     */
    getAvailableApps:function(){
        // switch to bail on hub populating of available apps.
        var applicationSwitcherConfig = sas && sas.config && sas.config.get("applicationSwitcher"),
            hubPopulate=true,
            _availableApps=[],
            defer=jQuery.Deferred();
        hubPopulate= applicationSwitcherConfig && applicationSwitcherConfig.hubPopulate===false ? false : true;


        // Load list of available apps for JSAppView to populate into the application level push menu.
        if (sas.registry && hubPopulate===true && sas.registry.getAvailableAppsService) {
            sas.registry.getAvailableAppsService().get().done(
                function (oData) {
                    if (oData) {
                        _availableApps = sas.hc.ui.core.util.immutate(oData,true);
                    }
                    defer.resolve(_availableApps);
                }).fail(function(){
                    defer.reject();
                });
        }

        // add available application from the configuration.
        if(applicationSwitcherConfig!==undefined){
            // inject any explicitly defined apps in config
            if(applicationSwitcherConfig.injectApps!==undefined){
                _availableApps=_availableApps.concat(sas.hc.ui.core.util.immutate(applicationSwitcherConfig.injectApps,true));
                defer.resolve(_availableApps);
            }

            // load apps from a url provided by config
            if(applicationSwitcherConfig.injectUrl!==undefined){
                sas.ajax({url:applicationSwitcherConfig.injectUrl}).done( function(oData){
                    _availableApps=_availableApps.concat(sas.hc.ui.core.util.immutate(oData,true));
                    defer.resolve(_availableApps);
                }).fail(function(oObj,textStatus){
                    // something wrong with url or content (valid json?)
                    defer.reject();
                });
            }
        }
        return defer.promise();
    },

    /**
     * Talk to the Hub app info service if present and get the list of public metadata types and
     * a list of app ids that have ability to support the given public type.
     * The resulting JSON is Map where key is public type and value is an array of application ids.
     * @returns {object} <a href="http://api.jquery.com/deferred.promise/">deferred.promise().</a>
     * @see jQuery.Deferred
     */
    getPublicTypeSupportingApps:function(){
        var defer = jQuery.Deferred();
        // Load map of known public types and for each type a list of apps that support it.
        if (sas.registry && sas.registry.getPublicTypeSupportingAppsService) {
            sas.registry.getPublicTypeSupportingAppsService().get().done(
                function (oData) {
                    if (oData)
                        defer.resolve(sas.hc.ui.core.util.immutate(oData,true));
                }).fail(function(oObj,textStatus){
                    defer.reject();
                });
        }
        return defer.promise();
    }
});
jQuery.sap.declare("sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherPetrichorModel");

jQuery.sap.require("sap.ui.base.Object");
jQuery.sap.require("sas.hc.ui.core.util");
jQuery.sap.require("sas.hc.ui.services.petrichor.appRegistry.AppRegistryService");

/**
 * @class
 * The Application Switcher Petrichor Model that provides data to @{sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher}
 *
 * @extends sap.ui.base.Object
 *
 * @author Rob Strader
 * @version 1.0
 *
 * @public
 * @name sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherPetrichorModel
 * @see sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher
 */
sap.ui.base.Object.extend("sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherPetrichorModel", {

    /**
     * Obtain the "hierarchical" JSON list of applications THIS USER can access.  The result being based on
     * Applications' integration with VA Hub, with role based capability restrictions
     * per that integration.
     * @returns {object} <a href="http://api.jquery.com/deferred.promise/">deferred.promise().</a>
     * @see jQuery.Deferred
     *
     */
    getAvailableApps:function(){
        if (!this.appRegistryService){
            this.appRegistryService = new sas.hc.ui.services.petrichor.appRegistry.AppRegistryService();
        }

        // switch to bail on hub populating of available apps.
        var _availableApps=[],
            defer=jQuery.Deferred();

        this.appRegistryService.getApplicationTree({timeout:60000}).done(
            function(applicationTreeResults) {

                function createAppSwitcherNode(oAppRegistryNode) {
                    if(oAppRegistryNode.branch === false) {
                        return mapLeafToAppSwitcherModel(oAppRegistryNode);
                    } else {
                        return mapBranchToAppSwitcherModel(oAppRegistryNode);
                    }
                    return null;
                }

                function mapLeafToAppSwitcherModel(oAppRegistryNode) {
                    if (oAppRegistryNode.appId === undefined ||
                        oAppRegistryNode.appSwitcherCompliance === undefined ||
                        oAppRegistryNode.hidden === undefined ||
                        oAppRegistryNode.label === undefined ||
                        oAppRegistryNode.id === undefined ||
                        oAppRegistryNode.uri === undefined) {
                        return null;
                    }

                    var oAppSwitcherLeafNode = {};
                    oAppSwitcherLeafNode.appId = oAppRegistryNode.appId;
                    oAppSwitcherLeafNode.appSwitcherCompliance = oAppRegistryNode.appSwitcherCompliance;
                    oAppSwitcherLeafNode.isHidden = oAppRegistryNode.hidden;

                    if (oAppRegistryNode.specialRole === "home") {
                        oAppSwitcherLeafNode.isHome = true;
                    } else {
                        oAppSwitcherLeafNode.isHome = false;
                    }

                    oAppSwitcherLeafNode.label = oAppRegistryNode.label;
                    oAppSwitcherLeafNode.nodeId = oAppRegistryNode.id;
                    oAppSwitcherLeafNode.url = oAppRegistryNode.uri;
                    oAppSwitcherLeafNode.mapType = "appInfo";

                    if (oAppRegistryNode.hasOwnProperty("appName")) {
                        oAppSwitcherLeafNode.appName = oAppRegistryNode.appName;
                    }

                    if (oAppRegistryNode.hasOwnProperty("applicationSuite")) {
                        oAppSwitcherLeafNode.applicationSuite = oAppRegistryNode.applicationSuite;
                    }

                    if (oAppRegistryNode.hasOwnProperty("branch")) {
                        oAppSwitcherLeafNode.branch = oAppRegistryNode.branch;
                    }

                    if (oAppRegistryNode.hasOwnProperty("createDefaultShortcut")) {
                        oAppSwitcherLeafNode.createDefaultShortcut = oAppRegistryNode.createDefaultShortcut;
                    }

                    if (oAppRegistryNode.hasOwnProperty("shortcutLabel")) {
                        oAppSwitcherLeafNode.shortcutLabel = oAppRegistryNode.shortcutLabel;
                    }

                    if (oAppRegistryNode.hasOwnProperty("version")) {
                        oAppSwitcherLeafNode.version = oAppRegistryNode.version;
                    }

                    return oAppSwitcherLeafNode;
                }

                function mapBranchToAppSwitcherModel(oAppRegistryNode) {
                    if (oAppRegistryNode.label === undefined || oAppRegistryNode.children === undefined) {
                        return null;
                    }
                    var oAppSwitcherBranchNode = {};
                    oAppSwitcherBranchNode.mapType = "appGroupInfo";
                    oAppSwitcherBranchNode.label = oAppRegistryNode.label;
                    oAppSwitcherBranchNode.children = [];
                    oAppRegistryNode.children.forEach(function(oChildNode){
                        var oChildAppSwitcherNode = createAppSwitcherNode(oChildNode);
                        if (oChildNode !== null) {
                            oAppSwitcherBranchNode.children.push(oChildAppSwitcherNode);
                        }
                    });
                    return oAppSwitcherBranchNode;
                }

                var aApplicationTree = applicationTreeResults.items;
                if (aApplicationTree) {
                    aApplicationTree.forEach(function(oAppRegistryNode){
                        var oAppSwitcherNode = createAppSwitcherNode(oAppRegistryNode);
                        if (oAppSwitcherNode !== null) {
                            _availableApps.push(oAppSwitcherNode);
                        }
                    });
                }

                _availableApps = sas.hc.ui.core.util.immutate(_availableApps,true);
                defer.resolve(_availableApps); // keep is synchronous for now.
            }
        ).fail(function(){
            defer.reject();
        });

        return defer.promise();
    },

    /**
     * Talk to the Application Registry service if present and get the list of public metadata types and
     * a list of app ids that have ability to support the given public type.
     * The resulting JSON is Map where key is public type and value is an array of application ids.
     * @returns {object} <a href="http://api.jquery.com/deferred.promise/">deferred.promise().</a>
     * @see jQuery.Deferred
     */
    getPublicTypeSupportingApps:function(){
        var defer = jQuery.Deferred();

        if (!this.appRegistryService){
            this.appRegistryService = new sas.hc.ui.services.petrichor.appRegistry.AppRegistryService();
        }

        this.appRegistryService.getSupportedTypes({timeout:60000}).done(
            function (supportTypesResults) {

                function createTypeNode(oSupportedType) {
                    var aTypeNode = [];
                    if (oSupportedType.apps) {
                        oSupportedType.apps.forEach(function(oSupportedApp) {
                            if (oSupportedApp.appId && oSupportedApp.targetVerbLabel) {
                                var oSupportedAppNode = {};
                                oSupportedAppNode.appId = oSupportedApp.appId;
                                oSupportedAppNode.iconName = "";
                                oSupportedAppNode.verbLabel = oSupportedApp.targetVerbLabel;
                                oSupportedAppNode.directive = oSupportedApp.directive;

                                if (oSupportedApp.hasOwnProperty('createArtifact')) {
                                    oSupportedAppNode.createArtifact = oSupportedApp.createArtifact;
                                }

                                aTypeNode.push(oSupportedAppNode);
                            }
                        });
                    }
                    return aTypeNode;
                }

                var oPublicTypeSupportingApplications = {};
                var aTypeResults = supportTypesResults.items;
                if (aTypeResults) {
                    aTypeResults.forEach(function(oSupportedType){
                        var oTypeNode = createTypeNode(oSupportedType);
                        if (oTypeNode !== null) {
                            Object.defineProperty(
                                oPublicTypeSupportingApplications,
                                oSupportedType.type, {
                                    writable: true,
                                    enumerable: true,
                                    configurable: false,
                                    value: oTypeNode
                                }
                            );
                        }
                    });
                }

                defer.resolve(sas.hc.ui.core.util.immutate(oPublicTypeSupportingApplications, true));
            }
        ).fail(function(oObj, textStatus) {
            defer.reject();
        });

        return defer.promise();
    }
});

jQuery.sap.declare("sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher");

jQuery.sap.require("sap.ui.base.Object");
jQuery.sap.require("sas.hc.ui.core.util.Url");
jQuery.sap.require("sas.hc.ui.core.applicationSwitcher.Application");
jQuery.sap.require("sas.hc.ui.core.util");
jQuery.sap.require("sas.hc.ui.core.util.ServicesUtil");
jQuery.sap.require("sas.hc.ui.core.util.ComponentServiceClassResolver");
jQuery.sap.require("sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherModel");
jQuery.sap.require("sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherPetrichorModel");

/**
 * @class
 * The Application Switcher is a mechanism for loading multiple applications into a single
 * browser page at one time to facilitate fast switching between applications without having to
 * reload apps every time.
 *
 * This also for keeping application state as you switch between apps and for communicating
 * between open apps as needed for integration.
 *
 * There is a single ApplicationSwitcher instance automatically available for all SAS HTML Commons applications.
 * It can be referenced via sas.applicationSwitcher as shown in remove app example:
 * <pre>
 *   sas.applicationSwitcher.remove('abcd');
 * </pre>
 *
 * There can be both HTML Commons applications and Flex Commons applications loaded at the same time
 * via ApplicationSwitcher.  Flex Commons has its own ApplicationSwitcher implementation and it is important
 * that the two implementations stay compatible for as long as Flex Applications are still being supported.
 *
 * @extends sap.ui.base.Object
 *
 * @author Jim Toering
 * @version 2.0
 *
 * @public
 * @name sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher
 * @deprecated
 */
(function(){

    function setAvailableTypes(val){
        Object.defineProperty(this,'publicTypeSupportingApplications',{
            enumerable:true,
            configurable:true,
            writable:false,
            value:val
        });
        Object.freeze(this.publicTypeSupportingApplications);
    };

    function setAvailableApps(val){
        Object.defineProperty(this,'availableApplications',{
            enumerable:true,
            configurable:true,
            writable:false,
            value:val
        });
        Object.freeze(this.availableApplications);
    };

    function setAvailableAppsFlattened(val){
        Object.defineProperty(this,'availableApplicationsFlattened',{
            enumerable:true,
            configurable:true,
            writable:false,
            value:val
        });
        Object.freeze(this.availableApplicationsFlattened);
    };

    // utility function to check if app switcher properties can be reused
    function canReuseAppSwitcher(){
        try{
            if(!this._isMainApplication()   // and this is not main frame.
            && top.sas
            && top.sas.applicationSwitcher
            && top.sas.applicationSwitcher.technology === this.technology
            && top.sas.applicationSwitcher._version === this._version){
                return sas.hc.ui.core.util._hasSameVersionAsParent();
            }
        }catch(err){
            // security error will be thrown in case of
            // apps loaded from cross origin.
            // suppress the error.
        }
        return Promise.reject();
    }


    function loadData(){
        "use strict";

        var that=this;

        return new Promise(function(resolvePrms,rejectPrms){

            var loading = true,
                canReusePromise;

            function fireAppSwLoadedEvent(){
                resolvePrms();
                sas.eventBus.publish("AppSwitcher_Data_Loaded");
            }

            function loadingCompleted(){
                if(loading){
                    loading = false;
                }else{
                    fireAppSwLoadedEvent();
                }
            }

            canReusePromise = canReuseAppSwitcher.call(that);

            canReusePromise.then(function(){// reuse from the top frame if available.
                setAvailableApps.call(that,top.sas.applicationSwitcher.availableApplications);
                setAvailableTypes.call(that,top.sas.applicationSwitcher.publicTypeSupportingApplications);
                setAvailableAppsFlattened.call(that,top.sas.applicationSwitcher.availableApplicationsFlattened);
                fireAppSwLoadedEvent();
            },
            function(){
                var model = that.model;
                // load available apps.
                model.getAvailableApps().then(function(val){

                    setAvailableApps.call(that,val);

                    // The list of available applications is hierarchical.  Sometimes it is handy to have flat list of applications without all the
                    // groups and suites.  Create it up front here.
                    var appFlatten=function(appsArray, flattenedApps){
                        return $.merge(flattenedApps,
                            $.grep(appsArray, function(item, index) {
                                if(item.mapType==="appGroupInfo")
                                    flattenedApps=appFlatten(item.children, flattenedApps);
                                if(item.mapType==="appInfo")
                                    flattenedApps.push(item);
                            }));
                    };

                    setAvailableAppsFlattened.call(that,appFlatten(that.availableApplications, []));

                    // For now lets use same model Flex used where the application switcher coordinated logoff
                    // is performed on the "home" app which is indicated in application info metadata coming back from
                    // available apps service.
                    if(that.availableApplicationsFlattened) {
                        for (var i = 0; i < that.availableApplicationsFlattened.length; i++) {
                            var app = that.availableApplicationsFlattened[i];

                            // Hub appInfo service returns overly complicated mechanisms for representing things
                            // like whether an app is the Home app or not.  Deal with it.
                            if (app.hasOwnProperty("isHome")) {
                                if (app.isHome === true) {
                                    if(typeof app.contextUrl === 'string'){
                                        var appUrl = app.contextUrl;
                                        if(appUrl.charAt(appUrl.length-1) === '/'){
                                            appUrl = appUrl.substr(0,appUrl.length-1);
                                        }
                                        that.setLogoffUrl(app.contextUrl+"/Logoff");
                                    }else{
                                        that.setLogoffUrl( sas.hc.ui.core.util.Url.getContextPath(app.url)+"/Logoff");
                                    }
                                }
                            }

                            // Traditionally the appId on an app switcher application has been required to be unique.
                            // We are not allowing multiple entries in application switcher menu to reuse the same
                            // iframe and represent different entry points in the same application.  This throws us
                            // off in that now these appIds are not unique and instead depend on additional information
                            // (request parms) to differentiate between apps.
                            // This puts us in a pickle anywhere we need to map what app we are in to the list of available apps
                            // as discussed in defect S1104768.
                            // To alleviate this create a hidden id which is unique.  It will be appId + a generate hashcode based
                            // on the request parameters in the url.
                            app.hashId=that.getAppHashId(app.appId, app.url);
                            //       console.log(app.appId + ":"+app.hashId);

                        };
                    };
                }).always(function(){
                    loadingCompleted();
                });
                // Load map of known public types and for each type a list of apps that support it.
                model.getPublicTypeSupportingApps().then(function(val){
                    setAvailableTypes.call(that,val);
                }).always(function(){
                    loadingCompleted();
                });
            });

            //        canReusePromise.catch(); // Chrome 53.0.2785.143 logs an error message "Uncaught (in promise) undefined"
            //        if only Promise.catch is used to handle rejection. Ridiculous.

        });
    };

    /**
     * Process given message type and invoke specified function in specified scope.
     *
     */
    function processTypeAndInvoke(type,fn,appSwitcher){

        // re-load the application info/data.
        if(type === appSwitcher.APP_INFO_CHANGED_MESSAGETYPE){
            loadData.apply(appSwitcher).then(fn.bind(appSwitcher));
        }else{
            fn.call(appSwitcher);
        }
    }

    sap.ui.base.Object.extend("sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher",{

        constructor : function(){
            sas.log.trace(sas.config.getAppId(),'ApplicationSwitcher:constructor');
            sap.ui.base.Object.apply(this, arguments);

            var serviceResolver = sas.hc.ui.core.util.ComponentServiceClassResolver;
            this.model = serviceResolver.getServiceClassInstance(serviceResolver.APPLICATION_SWITCHER_MODEL);
            this._processURLParameters = true;
            this._oAppSwitcherMessage = null;
            var platform = sas.hc.ui.core.util.ServicesUtil.getServicePlatform();
            if (platform === sas.hc.ui.core.util.ServicesUtil.ServicePlatform.PETRICHOR) {
                this._preferencesService = serviceResolver.getServiceClassInstance(serviceResolver.SETTINGS_SERVICE);
                this.updateTrackDirectiveProperty();
            }

            // unblocks passing messages and empties queue of deferred messages
            sas.eventBus.subscribe(sas.hc.ui.core.util.APP_READY, jQuery.proxy(this._appReadyHandler, this));

            var that = this;
            sas.eventBus.subscribe(
                this.APP_SWITCHER_CHANNEL,
                this.APP_UPDATE_TYPES_MESSAGETYPE,
                jQuery.proxy(function(type,data){
                    if (that.model) {
                        that.model.getPublicTypeSupportingApps().then(function (val) {
                            setAvailableTypes.call(that,val);
                        });
                    }
                },this)
            );


            // the main appSwitcher is the only one allowed to actually execute commands such as addApp, removeApp, etc
            // Any time a secondary appSwitcher is requested to perform one of these actions it relays that command
            // on (in cross domain safe fashion) via postMessage at which point this code int he main appSwitcher
            // catches the message and executes the appropriate command.
            //
            // This handler exists for all secondary appSwitcher's as well but never gets hit because we typically post all
            // messages to top (top window) and not to the iframes which represent secondary appSwitchers.
            // The exception is passing messages to the sas:Flex or SAS HTML applications which must be done from within the apps iframe space.
            //
            // S1240214: If the appswitcher is the top level app switcher, the command will be processed as normal. If it is a secondary
            // app switcher and it has not received the APP_READY event from the util class, it will queue the command.
            $(window).on('message', function(e){
                if (that._isMainApplication() || that._initialized) {
                    that._processMessage(e);
                }
            });
            try{
                if(top.window && typeof top.window.onpopstate !== undefined){
                    if(!top.window.onpopstate){
                        top.window.onpopstate = function(oPopEvt){
                            if(oPopEvt && oPopEvt.state && sas && sas.applicationSwitcher && sas.applicationSwitcher.gotoApp){
                                var state = oPopEvt.state;
                                sas.applicationSwitcher.gotoApp(state.appId, state.url, state.isCompliant, state.options);

                            }
                        };
                    }
                }
            }catch(err){
                // security error will be thrown in case of
                // apps loaded from cross origin.
                // suppress the error as this feature is not possible.
            }
            // register to app show to record currenly displayed application.
            this.subscribe(this.APP_SHOW_MESSAGETYPE, jQuery.proxy(this._appShowMessageHandler,this));

            // register app to check preferences to see if last directive needs tracking
            var that = this;
            if (platform === sas.hc.ui.core.util.ServicesUtil.ServicePlatform.PETRICHOR) {
                this.subscribe(this.APP_PREFENCE_CHANGED_MESSAGETYPE, jQuery.proxy(this._preferenceChangedHandler, this));
            }

            // load related data.
            loadData.apply(this);
        },

        // This metadata block isn't actually used since this doesn't extend from ManagedObject. I just liked the list of publicMethods at the top.
        //        metadata :
        //        {
        //            publicMethods: [ "setLogoffUrl",
        //                             "removeApp",
        //                             "gotoApp",
        //                             "updateTitle",
        //                             "logoff",
        //                             "updateDirtyFlag",
        //                             "setDirtyMessageEnabled",
        //                             "isDirtyMessageEnabled",
        //                             "sendMessageToApp",
        //                             "broadcastMessage",
        //                             "subscribe"
        //                           ]
        //
        //
        //
        //        },

    });

    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype._appReadyHandler = function() {
        this._initialized = true;
        sas.log.info("Application Switcher is ready to receive messages.");
        if(window._sas && window._sas.postMessageHandler) {
            while (window._sas.postMessageHandler.queue.length > 0) {
                var command = window._sas.postMessageHandler.queue.shift();
                this._processMessage(command);
            }
            window._sas.postMessageHandler.destroy();
        }

        if (sas.config && sas.config.oConf && sas.config.oConf.commons && sas.config.oConf.commons.preloadDrive === "true") {
            sas.applicationSwitcher.gotoApp("SASDrive", "/SASDrive/", true, {}, true);
        }
    };

    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype._preferenceChangedHandler = function(type, data) {
        if (data && data.settings && data.settings[0]) {
            if (data.settings[0].newValue === "lastAppDirective") {
                this.updateTrackDirectiveProperty(true);
                if (top.sas.applicationSwitcher && top.sas.applicationSwitcher._currentApp) {
                    this.updateTrackDirectivePreference(top.sas.applicationSwitcher._currentApp.appId);
                }
            } else if (data.settings[0].oldValue === "lastAppDirective") {
                this.updateTrackDirectiveProperty(false);
            }
        }
    };

    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype._appShowMessageHandler = function(type, data) {
        if(data){
            // record the hashId of currently displayed app.
            this._currentAppHashId=data.hashId;
            // HTMLCOMMONS-5381: Set preference for lastDirective if SASHome preference for last action is set
            // Sending URL instead of directive, per conversation with Home Developer
            this.updateTrackDirectivePreference(data.appId);
            this.updateThemeIfDefaultHasChanged();
        }
    };

    /**
     * Update application data of this application swticher instance.
     * Updating the data may have impact on the performance of application switcher. Invoke this API carefully.
     *
     * @public
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.updateApplicationData = function(){
        // broadcast the message to all open applications.
        this.broadcastMessage(this.APP_INFO_CHANGED_MESSAGETYPE);
    };

    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype._processMessage = function(e) {
        var event = e;
        if (e.originalEvent) {
            event = e.originalEvent;
        }
        var message = null;

        if (typeof event.data === 'string') {
            try {
                message = JSON.parse(event.data);
            } catch(e) {
                return;
            }
        } else {
            message = event.data;
        }


        // Technically there could be messages coming from places other than appSwitcher postmessage.
        // In that case we need to be able to ignore those unwanted messages since we only handle
        // appswitcher messages here.
        // An example of unwanted message in HTML is if you hit ctrl+shift+alt+S the SAP diagnostics
        // dialog will send post messages back and forth.
        //
        // We are limited in what we can use to identify our own messages since we expect messages from
        // Flex as well.  So key off of the message version which we have had on all of our own
        // messages from the very beginning.  If someone else has a message with a version on it then
        // it will sneak through and probably have some issues.
        if(message===null || message.version===undefined)
            return;


        // Are we communicating across compatible versions? It not?
        // Always keep AppSwitcher backwards compatible or only breaking changes at
        // major versions?
        // Code to try and keep them compatible? Try and map compatibility automatically?
        if(sas.applicationSwitcher._version!==message.version) {
            sas.log.info('AppSwitcher version ' + sas.applicationSwitcher._version + " is trying to communicate with AppSwitcher version " + message.version);
        }

        sas.log.trace(sas.config.getAppId(),':doOnMessage-',message.command);
        switch(message.command) {
            case 'setLogoffUrl':
                sas.applicationSwitcher.setLogoffUrl(message.logoffUrl);
                break;
            case 'logoff':
                sas.applicationSwitcher.logoff();
                break;
            case 'updateDirtyFlag':
                sas.applicationSwitcher.updateDirtyFlag(message.appId, message.dirtyFlag);
                break;
            case 'setDirtyMessageEnabled':
                sas.applicationSwitcher.setDirtyMessageEnabled(message.allowDirtyMessage);
                break;
            case 'removeApp':
                sas.applicationSwitcher.removeApp(message.appId);
                break;
            case 'gotoApp':
                sas.applicationSwitcher.gotoApp(message.appId, message.url, message.isCompliant, message.options, message.silentLoad);
                break;
            case 'updateTitle':
                sas.applicationSwitcher.updateTitle(message.appId, message.title);
                break;
            case 'updateAppName':
                sas.applicationSwitcher.updateAppName(message.appId, message.appName);
                break;

            case 'sendMessageToApp':
                sas.applicationSwitcher.sendMessageToApp(message.appId, message.type, message.data);
                break;
            case '_sendMessageToApp':

                // disabling eslint here so as to not have to make structural changes to vital code. Will revisit if deemed necessary
                function doSendMessages(){   // eslint-disable-line
                    // This is the only command which gets caught in the iframe appSwitcher rather
                    // than the main appSwitcher.  It has to be that way to handle
                    // cross domain iframe messaging between our sas:Flex applications

                    // Fixed Channel and Event...  ApplicationSwitcher subscribe will deal with pulling apart the bits and pieces
                    // and allowing one to discern message type and contents.
                    sas.eventBus.publish("ApplicationSwitcher", "ApplicationSwitcher",{type: message.type, data: message.data});

                    // Also send it to general ApplicationSwitcher channel but specific event type so that it can be caught directly
                    // instead of high level application switcher message catcher and then switch statement
                    sas.eventBus.publish("ApplicationSwitcher", message.type ,  {type: message.type, data: message.data});

                };
                //check for app data updated message type before passing it on.
                processTypeAndInvoke(message && message.type,doSendMessages,sas.applicationSwitcher);

                break;
            case 'broadcastMessage':
                sas.applicationSwitcher.broadcastMessage(message.type, message.data);
                break;
            case 'removeSplash':
                sas.applicationSwitcher.removeSplash();
                break;
            case 'timeout':
                sas.applicationSwitcher.timeout(message.url);
            default:
                sas.log.trace('unknown appSwitcher message');
        }
    };

    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.updateTrackDirectiveProperty = function(value) {
        var that = this;
        if (value === undefined) {
            this._preferencesService.get({"data": {"name": ["sas.drive.logon.action"]}, "timeout": 60000}).done(function (preferenceResults) {
                if(preferenceResults.preferences.length > 0) {
                    var preferenceObj = preferenceResults.preferences[0];
                    if (preferenceObj && preferenceObj.value === "lastAppDirective") {
                        that._trackLastDirective = true;
                    } else {
                        that._trackLastDirective = false;
                    }
                } else {
                    that._preferencesService.get({"data": {"name": ["sashome.logon.action"]}, "timeout": 60000}).done(function (preferenceResults) {
                        var preferenceObj;
                        if(preferenceResults.preferences.length > 0) {
                            preferenceObj = preferenceResults.preferences[0];
                        }

                        if (preferenceObj && preferenceObj.value === "lastAppDirective") {
                            that._trackLastDirective = true;
                        } else {
                            that._trackLastDirective = false;
                        }
                    });
                }
            });
        } else {
            that._trackLastDirective = value;
        }
    };

    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.updateTrackDirectivePreference = function(sAppId) {
        if (this._trackLastDirective === true && this._preferencesService !== null && sAppId) {
            this._preferencesService.put({"data": {"preferences": [{"key":"sas.drive.logon.lastAppDirective", "value":sAppId}]}});
        }
    };

    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype._getDefaultThemeId = function() {

        return new Promise(
            function(resolve, reject) {
                var themeService = sas.hc.ui.core.util.ComponentServiceClassResolver.getServiceClassInstance(sas.hc.ui.core.util.ComponentServiceClassResolver.THEME_UTIL_SERVICE);
                if(themeService) {
                    var options = {
                        async: true,
                        dataType: "json",
                        id: "@defaultApplicationTheme"
                    };
                    themeService.get(options).done(function(applicationTheme){
                        resolve(applicationTheme.id);

                    });
                }
                else reject();
            });
    };

    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.updateThemeIfDefaultHasChanged = function() {
        var defaultThemeId,
            currentThemeId,
            that = this;

        function getDefaultTheme() {
            that._getDefaultThemeId().then(function(value){
                defaultThemeId = value;
                currentThemeId = sap.ui.getCore().getConfiguration().getTheme();
                if (currentThemeId !== defaultThemeId) {
                    var theme = sas.themeDescriptor.getApplicationTheme(defaultThemeId);
                    if (theme !== null) {
                        theme.applyTheme();
                    }
                }
            });
        };

        var sThemeParameter = jQuery.sap.getUriParameters().get("sap-ui-theme");
        var sThemeUrlParameter = jQuery.sap.getUriParameters().get("sap-ui-sas-themeUrl");

        // Only load new default theme if there is no theme preference set
        if (this._preferencesService && (sThemeParameter === undefined || sThemeParameter === null) && (sThemeUrlParameter === undefined || sThemeUrlParameter === null)) {
            this._preferencesService.get({"data": {"name": [sas.hc.ui.core.GlobalSettings.THEMES_KEY]}}).done(function(response) {
                if(response.preferences.length === 0) {
                    getDefaultTheme();
                }
            }).fail(function() { 
                getDefaultTheme();
            });
        }
    };

    Object.defineProperties(sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype, {
        APP_SWITCHER_CHANNEL: {
            value: "AppSwitcher.Channel"
        },

        APP_UPDATE_TYPES_MESSAGETYPE: {
            value: "AppSwitcher.UpdateTypes"
        },

        // types of system level message received
        APP_LOGOFF_IMMINENT_MESSAGETYPE: {
            value:"AppSwitcher.LogoffImminent"
        },

        APP_ADDED_MESSAGETYPE:{
            value:"AppSwitcher.AppAdded"
        },

        APP_REMOVED_MESSAGETYPE:{
            value:"AppSwitcher.AppRemoved"
        },

        APP_SWITCH_MESSAGETYPE:{
            value:"AppSwitcher.AppSwitch"
        },

        APP_SHOW_MESSAGETYPE:{
            value:"AppSwitcher.AppShow"
        },

        APP_HIDE_MESSAGETYPE:{
            value:"AppSwitcher.AppHide"
        },

        APP_TITLE_UPDATED_MESSAGETYPE:{
            value:"AppSwitcher.UpdateAppTitle"
        },

        APP_DIRTY_FLAG_CHANGED_MESSAGETYPE:{
            value:"AppSwitcher.DirtyFlagChanged"
        },

        APP_PREFENCE_CHANGED_MESSAGETYPE:{
            value:"AppSwitcher.PreferenceChanged"
        },

        APP_INFO_CHANGED_MESSAGETYPE:{
            value:"AppSwitcher.AppInfoChanged"//Same value is also used by flex version.
        },

        APP_RETURN_MESSAGETYPE: {
            value:"AppSwitcher.AppReturn"
        },


        /**
         * AppSwitcher Version - This is in sync with the Flex Commons release it ships with.
         * Once we shift entirely away from Flex apps we can sync it up with HTML Commons instead.
         * Allows us to do something about version differences.
         *
         * @type {string}
         *
         * @private
         */
        _version:{
            enumerable:true,
            value:'4.2'
        },

        /** Since Flex AppSwitcher and HTML AppSwitcher interact with each other and one of them is elected
         * leader based on which type of app is launched first we might need to know which is which.
         *
         * @public
         * @type {string}
         *
         * @memberof sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher
         */
        technology:{
            enumerable:true,
            value:'HTML Commons'
        },

        _postMessageOrigin:{
            enumerable:true,
            value:'*'
        },

        /**
         * availableApplications is a list of the apps that can potentially be switched to via the application level push menu.
         * Technically the application switcher itself doesn't really care about this list of potential apps since it only deals
         * with opening up apps or communicating with already open apps.
         *
         * Adding it here to application switcher just as a central place to put the list of potential apps which the banner push menu
         * will use to render available apps.  This is populated during ApplicationSwitcherBootTask startup.
         *
         * @private
         * @type {Array}
         * @memberof sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher
         */
        availableApplications:{
            enumerable:true,
            writable:true,
            configurable:true,
            value:[]
        },

        /**
         * availableApplications is a list of the apps that can potentially be switched to via the application level push menu.
         * Technically the application switcher itself doesn't really care about this list of potential apps since it only deals
         * with opening up apps or communicating with already open apps.
         *
         * Adding it here to application switcher just as a central place to put the list of potential apps which the banner push menu
         * will use to render available apps.  This is populated during ApplicationSwitcherBootTask startup.
         *
         * @private
         * @type {Array}
         * @memberof sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher
         */
        availableApplicationsFlattened: {
            enumerable:true,
            writable:true,
            configurable:true,
            value:[]
        },

        /**
         * Similar to availableApplications above application switcher really doesn't care about these public type to application
         * mappings directly.  However application switcher ends up being a useful place to hang these mappings off of for things
         * like search, home, recent work to do a quick lookup of a given public type and determine which applications are capable of
         * opening an object of that type.
         *
         * This is populated during ApplicationSwitcherBootTask startup.
         *
         * @private
         * @type {Object}
         * @memberof sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher
         */
        publicTypeSupportingApplications: {
            enumerable:true,
            writable:true,
            configurable:true,
            value:{}
        },


        /**
         * This is used to store all the applications that have been added to the app switcher.  This means
         * apps that have already been opened and are available to fast switch to.
         *
         * @private
         */
        _applications: {
            enumerable:true,
            value:{}
        },

        /**
         * Model that provides application data to application switcher.
         *
         * @private
         * @type {sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherModel}
         */
        model: {
            enumerable:true,
            writable:true
        },


        /**
         * The currently visible application
         *
         * @private
         */
        _currentApp: {
            enumerable:true,
            value:null,
            writable:true
        },

        // the ABSOLUTE url to logoff to when any application added to app switcher
        // calls app switcher logoff method. It cannot be a relative url or else it
        // will always be attempting to be relative to whatever is in the browser url
        // bar which is the app that was initially started up.  We expect someone to instead
        // set an absolute logoff url to whatever app they feel you should go to on logoff but there can be only one logoffUrl.
        _logoffUrl:{
            enumerable:true,
            value:null,
            writable:true
        },

        // Controls whether the document unload event shows a dirty message or not.  In general we want
        // it to but there are cases like timeout dialog where we don't want secondary messages to show.
        _dirtyMessageEnabled:{
            writable:true,
            enumerable:true,
            value:true
        },

        // A bucket for localized strings to be stored.  This is populated by ApplicationSwitcherBootstrap.jsp
        localizedStrings: {
            enumerable:true,
            value:{}
        },

        urlUtil: {
            writable:true,
            enumerable:true,
            value:sas.hc.ui.core.util.Url
        },

        _trackLastDirective: {
            writable: true,
            enumerable: true,
            value: false
        },

        _preferencesService: {
            writable:true,
            enumerable: true,
            value:null
        },

        _hasShownUnsavedMessagesFlag: {
            writable:true,
            enumerable: true,
            value:false
        },

        _messageHandlerMap : {
            writable: true,
            enumerable: true,
            value: {}
        }

    });

    /**
     * Attempt to locate the url for a given application.
     *
     * With the advent of shared appIds it isn't as simple as looking up based solely on AppId.
     * To allow applications to call gotoApp with the right url this mechanism will attempt
     * lookup based on applicationId and will optionally take directiveName into account
     * for apps that have many leafs sharing a single iframe.
     *
     *
     * @param {string} appId application id we are locating url for.
     * @param {string} [directiveName] directive name coupled with appId to lookup url for.
     * @returns {string} TODO-jsdoc
     *
     * @public
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.getApplicationUrl = function(appId, directiveName) {
        var possibleUrl;
        var exactUrl;

        for (var i = 0, len = this.availableApplicationsFlattened.length; i < len; i++) {
            if (this.availableApplicationsFlattened[i]["appId"] === appId) {
                possibleUrl=this.availableApplicationsFlattened[i].url;

                if(this.availableApplicationsFlattened[i]["nodeId"]===directiveName) {
                    exactUrl = this.availableApplicationsFlattened[i].url;
                    break;
                }
            }
        }

        return exactUrl?exactUrl:possibleUrl;
    };

    /**
     * Request that an object be opened up in one of the apps which supports it.
     *
     * The public type of the object being opened will be used to determine what applications
     * the current user has access to which are capable of opening the object.  If the current
     * application supports the object type it will always be used.   If the current application
     * does not support the object type then the first application in the list of support apps
     * will be used.
     *
     *
     * @param {string} objectType the type of object being requested
     * @param {object} objectInfo object which must contain all the information the application will
     *                            needed to open the object.  This includes:
     *                            {
     *                              targetReposKey
     *                              targetNavPath
     *                              targetVerb
     *                            }
     * @param {string} preferredAppId [fallback] Application Id to open app in.  If not specified we will determine
     *                                the appropriate app automatically.
     *
     *
     * @public
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.openObject = function(objectType, objectInfo, preferredAppId) {
        // Unlike most methods in application switcher openObject always functions in the iframe application switcher
        // instance and not in the top level window application switcher.

        // Step 1 - Determine application that should be used to open object based on object type
        var launchInAppId;

        if(objectType in this.publicTypeSupportingApplications) {
            // Order in which to choose app for open (if supported)
            // 1.  If consumer gave us a preferredAppId and it can handle the type
            // 2.  If currently open app can handle the type
            // 3.  First app in list of supported types
            var supportingApps= this.publicTypeSupportingApplications[objectType];

            var preferredAppSearchResult = $.grep(supportingApps,
                function(app) {
                    if(app.appId===preferredAppId)
                        return app;
                });
            var oPreferredApp=null;
            if(preferredAppSearchResult.length>0)
                oPreferredApp=preferredAppSearchResult[0];


            if(oPreferredApp!==null) {
                launchInAppId=oPreferredApp.appId;
            } else {
                // If the current application supports the requested object type then use it. If not
                // find the first application in the list that does support it.
                var currentAppSearchResult = $.grep(supportingApps,
                    function(app) {
                        if(app.appId===sas.config.getAppId())
                            return app;
                    });
                if (currentAppSearchResult.length>0)
                    launchInAppId = currentAppSearchResult[0].appId;
                else
                    launchInAppId = supportingApps[0].appId;
            }
        } else {
            jQuery.sap.require("sas.hc.m.MessageDialog");
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.hc.ui.core");
            sas.hc.m.MessageDialog.error({
                text: rb.getText("sas.applicationSwitcher.unableToOpenObject.txt"),
                title: rb.getText("sas.applicationSwitcher.unableToOpenObject.title")
            });
            return;
        }


        sas.log.info("opening object with " + launchInAppId);


        // Step 2 - Launch app

        // lookup appid in list of available apps to get additional app info for launching.
        var launchInApp;
        for (var i = 0, len = this.availableApplicationsFlattened.length; i < len; i++) {
            if (this.availableApplicationsFlattened[i]["appId"] === launchInAppId) {
                // Should be as simple as appId=launchInAppId check but unfortunately a single appId doesn't
                // uniquely identify a single app and you can have multiple "views" inside each app.  So if
                // there is more than one instance of the launchInAppId in the available applications we need
                // to narrow it down further if possible.  This would be by directive if it is available in the
                // openObject data.
                if(objectInfo.hasOwnProperty("directive")) {
                    if(objectInfo["directive"]===this.availableApplicationsFlattened[i]["nodeId"]) {
                        launchInApp = this.availableApplicationsFlattened[i];
                        break;
                    } else {
                        // Just in case we don't find a directive match it can still go to something with
                        // the same appid... This should always get overriden by directive match above which
                        // breaks out immediately.
                        launchInApp=this.availableApplicationsFlattened[i];
                    }
                } else {
                    launchInApp = this.availableApplicationsFlattened[i];
                    break;
                }
            }
        }


        if(launchInApp.appSwitcherCompliance==="1" || launchInApp.appSwitcherCompliance===true) {
            this.gotoApp(launchInApp.appId, launchInApp.url, true);

            // I don't like this.  AppSwitcher should know NOTHING about directives.
            // But if directive isn't already in the App.Open message put it there based on
            // our knowledge of the Hub integration returned objects.
            if(objectInfo) {
                if (objectInfo.hasOwnProperty("directive") === false)
                    objectInfo["directive"] = launchInApp.nodeId;
            }
            else
                objectInfo={"directive":launchInApp.nodeId};

            objectInfo["targetPublicType"]=objectType;

            this.sendMessageToApp(launchInApp.appId, "App.Open", objectInfo);
        } else {
            // If launching into a non compliant app then sending an appswitcher message won't cut it
            // since it won't be listening.  In that case we are going to just mess with the url to
            // give it info about what object to open and let Hub deal with it.  Ultimately it will
            //  redirect out to app it thinks can do the open and tack the appropriate request parms on
            // for it.

            // There should only ever be one app that is the "Home" app.
            var homeApps = $.grep(this.availableApplicationsFlattened, function(item, index) {
                if(item.hasOwnProperty("isHome")===true && item.isHome===true)
                    return true;
                else
                    return false;
            });
            var app = homeApps[0];
            if(!app) return;
            var hubOpenerUrl = app.url;
            if(objectInfo.hasOwnProperty("targetReposKey")===true)
                hubOpenerUrl = this.urlUtil.updateQueryStringParameter(app.url,"saspfs_request_entity", objectInfo["targetReposKey"]);
            if(objectInfo.hasOwnProperty("targetNavPath")===true)
                hubOpenerUrl = this.urlUtil.updateQueryStringParameter(app.url,"saspfs_request_path_url", objectInfo["targetNavPath"]);
            this.gotoApp(launchInApp.appId, hubOpenerUrl, false);
        }

    };

    /**
     * Request that an object be created by creator app.
     *
     * We don't currently have information on which app that supports a type is the one capable of "creating".
     * As such if appId is passed in use it otherwise pick the first app in the list.  We probably want to get appregistry meta for this at some point.
     *
     * @param {string} objectType the type of object being requested
     * @param {object} objectInfo Any aditional info about the about you are requesting being created.  This would be app specific.
     *                            {
     *                            }
     * @param {string} preferredAppId [fallback] Application Id to create object in.  If not specified we will pick one of the supporting apps and hope
     *                                           it has create ability.
     *
     *
     * @public
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.createObject = function(objectType, objectInfo, preferredAppId) {
        // Unlike most methods in application switcher createObject always functions in the iframe application switcher
        // instance and not in the top level window application switcher.

        // Step 1 - Determine application that should be used to open object based on object type
        var createWithAppId;

        if(objectType in this.publicTypeSupportingApplications) {
            // Order in which to choose app for create (if supported)
            // 1.  If consumer gave us a preferredAppId and it can handle the type
            // 2.  First app in list of supported types
            var supportingApps= this.publicTypeSupportingApplications[objectType];

            var preferredAppSearchResult = $.grep(supportingApps,
                function(app) {
                    if(app.appId===preferredAppId)
                        return app;
                });
            var oPreferredApp=null;
            if(preferredAppSearchResult.length>0)
                oPreferredApp=preferredAppSearchResult[0];


            if(oPreferredApp!==null) {
                createWithAppId=oPreferredApp.appId;
            }
            else {
                createWithAppId = supportingApps[0].appId;
            }
        } else {
            // TODO : Change from alert to standardized error dialog
            jQuery.sap.require("sas.hc.m.MessageDialog");
            var rb = sap.ui.getCore().getLibraryResourceBundle("sas.hc.ui.core");
            sas.hc.m.MessageDialog.error({
                text: rb.getText("sas.applicationSwitcher.unableToCreateObject.txt"),
                title: rb.getText("sas.applicationSwitcher.unableToCreateObject.title")
            });
            return;
        }


        sas.log.info("creating object with " + createWithAppId);


        // Step 2 - Launch app

        // lookup appid in list of available apps to get additional app info for launching.
        var createWithApp;
        for (var i = 0, len = this.availableApplicationsFlattened.length; i < len; i++) {
            if (this.availableApplicationsFlattened[i]["appId"] === createWithAppId) {
                // Should be as simple as appId=createWithAppId check but unfortunately a single appId doesn't
                // uniquely identify a single app and you can have multiple "views" inside each app.  So if
                // there is more than one instance of the createWithAppId in the available applications we need
                // to narrow it down further if possible.  This would be by directive if it is available in the
                // openObject data.
                if(objectInfo.hasOwnProperty("directive")) {
                    if(objectInfo["directive"]===this.availableApplicationsFlattened[i]["nodeId"]) {
                        createWithApp = this.availableApplicationsFlattened[i];
                        break;
                    } else {
                        // Just in case we don't find a directive match it can still go to something with
                        // the same appid... This should always get overriden by directive match above which
                        // breaks out immediately.
                        createWithApp=this.availableApplicationsFlattened[i];
                    }
                } else {
                    createWithApp = this.availableApplicationsFlattened[i];
                    break;
                }
            }
        }

        // Only support create in app switcher compliant applications.
        if(createWithApp.appSwitcherCompliance==="1" || createWithApp.appSwitcherCompliance===true) {
            this.gotoApp(createWithApp.appId, createWithApp.url, true);

            if(!objectInfo)
                objectInfo={};

            objectInfo["targetPublicType"]=objectType;

            this.sendMessageToApp(createWithApp.appId, "App.Create", objectInfo);
        }

    };

    /**
     * Set the single URL that logging off via Application Switcher API will go to.   If multiple
     * areas attempt to set this the last one wins.
     *
     * This will be set automatically by HTML Commons or Flex Commons so typical consumers should
     * never need to use this.
     *
     * @param {string} logoffUrl TODO-jsdoc
     *
     * @public
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.setLogoffUrl = function(logoffUrl) {

        /*  If this is not the main application switcher app then this method is being called on one
         of the included apps.  We need to forward this request on to the main app to handle.

         Because of cross domain possibilities we need to use postMessage to do this.  Could possibly
         detect crossdomain or not and if not just make call on parent window directly but for now
         do it same way regardless.
         */
        if(this._isMainApplication()===false) {
            this._sendCommandToMainApplication({'command':'setLogoffUrl', 'logoffUrl':logoffUrl});
            return;
        }

        this._logoffUrl=logoffUrl;
    };

    /**
     * Remove application switcher top frame splash screen if present.
     *
     *
     * @private
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.removeSplash = function() {

        /*  If this is not the main application switcher app then this method is being called on one
         of the included apps.  We need to forward this request on to the main app to handle.

         Because of cross domain possibilities we need to use postMessage to do this.  Could possibly
         detect crossdomain or not and if not just make call on parent window directly but for now
         do it same way regardless.
         */
        if(this._isMainApplication()===false) {
            this._sendCommandToMainApplication({'command':'removeSplash'});
            return;
        }

        // remove splash screen
        var splashScr = $('#sas-hc-splash');
        if(splashScr){
            splashScr.remove();
            jQuery.sap.measure.end("sasBootstrapComplete");
        }
    };

    /**
     * Close the application associated with the given appId
     *
     * @param {string} appId TODO-jsdoc
     *
     * @public
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.removeApp = function(appId) {
        // summary:
        //      remove app from the app switcher
        // appId : String
        //      Id of app to remove
        // tags:
        //      public

        /*  If this is not the main application switcher app then this method is being called on one
         of the included apps.  We need to forward this request on to the main app to handle.

         Because of cross domain possibilities we need to use postMessage to do this.  Could possibly
         detect crossdomain or not and if not just make call on parent window directly but for now
         do it same way regardless.
         */
        if(this._isMainApplication()===false) {
            this._sendCommandToMainApplication({'command':'removeApp', 'appId':appId});
            return;
        }

        if(this._appExists(appId)) {
            var app=this._getApp(appId);

            app.iframe.remove();
            //      domConstruct.destroy(app.iframe);
            delete this._applications[appId];

            this.broadcastMessage(this.APP_REMOVED_MESSAGETYPE,{'appId':appId, 'availableApps':this._getAppsRenderingInfo()} );
            // If removing the current app then better go somewhere.  First app in list it is.
            if(this._currentApp.appId===appId) {
                this._currentApp=null;
                for (var appId in this._applications) {
                    if (this._applications.hasOwnProperty(appId)) {
                        var app=this._getApp(appId);
                        this.gotoApp(app.appId, app.url, true);
                        break;
                    }
                }
            }


        }
        else
            sas.log.warning('unable to remove application ' + appId);
    };

    /**
     * Generate a unique id based on appId and request parameters on url.
     *
     * @param {string} appId TODO-jsdoc
     * @param {string} url TODO-jsdoc
     * @returns {string} TODO-jsdoc
     *
     * @private
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.getAppHashId = function(appId, url){
        // Traditionally the appId on an app switcher application has been required to be unique.
        // We are not allowing multiple entries in application switcher menu to reuse the same
        // iframe and represent different entry points in the same application.  This throws us
        // off in that now these appIds are not unique and instead depend on additional information
        // (request parms) to differentiate between apps.
        // This puts us in a pickle anywhere we need to map what app we are in to the list of available apps
        // as discussed in defect S1104768.
        // To alleviate this create a hidden id which is unique.  It will be appId + a generate hashcode based
        // on the request parameter 'view' in the url.
        var oQuerytoObject = sas.hc.ui.core.util.Url.queryToObject(url);
        return appId+((oQuerytoObject && oQuerytoObject.view) ? sas.hc.ui.core.util.hash(oQuerytoObject.view) : "");
    };

    /**
     * Launch the specified application.  If it is already open then it will fast switch to it.
     * If not already open we will launch the new application.
     *
     * If application has already been launched then you can pass only the appId.
     *
     * @param {string} appId - Unique application identifier
     * @param {string} [url] - The url used for launching this application
     * @param {boolean} [isCompliant=false] - Whether the app is application switcher compliant.  if not compliant
     *                      then we overwrite the entire window so as not to load this app into an iframe
     *                      and leave all the other apps still loaded into inaccessbile iframes.
     * @param {object} [options] TODO-jsdoc
     *
     * @public
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.gotoApp = function(appId, url, isCompliant, options, bSilentLoad) {
        sas.log.trace(sas.config.getAppId(),':gotoApp-',appId);
        if(typeof options==="undefined")
            options={};


        /*  If this is not the main application switcher app then this method is being called on one
         of the included apps.  We need to forward this request on to the main app to handle.

         Because of cross domain possibilities we need to use postMessage to do this.  Could possibly
         detect crossdomain or not and if not just make call on parent window directly but for now
         do it same way regardless.
         */
        if(this._isMainApplication()===false) {
            this._sendCommandToMainApplication({'command':'gotoApp', 'appId':appId, 'url':url, 'isCompliant':isCompliant, 'options':options, 'silentLoad': bSilentLoad});
            return;
        }

        var appSwitchBroadcastMessageData = {'appId':appId};

        // Tag any request parms from the url along with the app show message.  This
        // allows solutions to have multiple entry points in a single iframe/appid.  So they
        // might have two "Apps" defined with the same appid but different request parms to identify
        // what operation is required.  They can then listen to appShow message and interpret the request
        // parms however they see fit.
        var appShowMessageData = this.urlUtil.queryToObject(url);
        // S1104768 - Need truly unique app id of appId+url request parm hashcode
        appShowMessageData['hashId']=this.getAppHashId(appId,url);
        appShowMessageData['appId']=appId;
        appShowMessageData['url']=url;

        // Early exit if we are already on the requested app.
        if(this._currentApp && this._appExists(appId)===true && this._currentApp.appId===appId) {
            // update the url if its not same. Will be the case when apps are using views.
            this._currentApp.url = url;
            // notify app that we are switching to it so it can do "stuff".  Show even if current app
            // in case someone has multiple entry points inside same appid/iframe.
            this.sendMessageToApp(appId, this.APP_SHOW_MESSAGETYPE, appShowMessageData);
            return;
        }

        // If it doesn't already exist then the iframe which will contain this app.
        if(this._appExists(appId)===false) {


            // If launching an non app switcher enabled application via the application switcher API then
            // just load the app directly into top level of browser wiping out existing app switcher and any
            // already loaded apps.   Otherwise we end up with the non compliant app(s) running in iframes
            // inside the app switcher but they have their own timeouts, logoffs, links to other apps which get
            // loaded inside the iframe even if they are already loaded in app switcher.  Too many problems with
            // this scenario so just load it like we used to.
            if(isCompliant===null || isCompliant===undefined)
                isCompliant=false;

            if(isCompliant===false) {
                window.location.href=url;
                return;
            }

            // If not already created create the apps container div
            this._createAppsContainer();


            // they are being launched via API.
            var app = new sas.hc.ui.core.applicationSwitcher.Application();
            app.appId=appId;
            app.url=url;


            // I have mixed feelings about defect S1007369.  We oftentimes in development and testing use request parms to
            // do short term overrides of things such as locale, theme, etc at runtime without changing the deployment or
            // user preferences.  With the app switcher these request parms now only make it to the first app being
            // loaded and not any apps being switched to.
            // We can grab all the request parms and pass them along to all subsequent apps but what if there are parm names
            // that are used differently by different apps?
            // Lets hope that doesn't happen and transfer them all.  If it does happen we could go to a much more targeted list like
            // theme, themesURI, locale, etc...
            //
            // It happened ^^^^ Defect S1012194.  Lets go to a targeted list of things that we use mainly in development and testing as request
            // parameters:
            // locale, resourceModulesURI, logoffURL, logging, themesURI, customThemesURI, theme,  localeIsRTL, timeout*
            if(this._currentApp) {
                var currentQueryObject = this.urlUtil.queryToObject(this._currentApp.url);
                var destinationQueryObject = this.urlUtil.queryToObject(app.url);

                // useful dev/test request parameters
                var validTransferRequestParameters=["locale",     // Flex  and HTML
                    "_locale",                                 // Flex and HTML
                    "sas-commons-url",                       // HTML
                    "sap-ui-debug",                              // HTML
                    "sap-ui-preload",                         // HTML
                    //"sas-commons-externalized",              // HTML
                    "sas-commons-configUrl",                 // HTML
                    "sap-ui-sas-themeUrl",                   // HTML
                    "themeUrl",                              // HTML
                    "sessionMaxInActiveInterval",            // HTML
                    "sap-ui-sas-sessionMaxInactiveInterval", // HTML
                    "sessionTimeoutWarningDuration",         // HTML
                    "sap-ui-sas-sessionTimeoutWarningDuration", // HTML
                    "sessionTimeoutUrl",                      // HTML
                    "sap-ui-sas-sessionTimeoutUrl",          // HTML
                    "sas-commons-configUrl",                  // HTML
                    "accessibilityActive",    // Flex
                    "resourceModulesURI",     // Flex
                    "logoffURL", "logging",   // Flex
                    "themesURI",              // Flex
                    "customThemesURI",        // Flex
                    "theme",                  // Flex
                    "localeIsRTL",            // Flex
                    "timeoutMaxInactiveInterval",   // Flex
                    "timeoutEnabled",               // Flex
                    "timeoutWarningDuration",       // Flex
                    "timeoutURL",                   // Flex
                    "timeoutPingOnInitialActivity", // Flex
                    "timeoutExternalFunctionName",  // Flex
                    "timeoutSessionTouchEnabled",   // Flex
                    "timeoutSessionTouchURL",       // Flex
                    "timeoutPingWindowInterval"];   // Flex

                for(var requestParm in currentQueryObject) {
                    if (currentQueryObject.hasOwnProperty(requestParm)) {
                        if(destinationQueryObject.hasOwnProperty(requestParm)===false) {
                            if(validTransferRequestParameters.indexOf(requestParm)>-1)
                                app.url=this.urlUtil.updateQueryStringParameter(app.url,requestParm,currentQueryObject[requestParm]);
                        }
                    }
                }
            }


            this.sendMessageToApp(appId,"Rob Message", {});
            app.iframe=$("<iframe>",{ id:appId+'_iframe', loaded: false, allowfullscreen: true});
            if(sap.ui.Device.support.touch){//S1149101
                app.iframe.css({display:'none',borderWidth: 0});
            }else{
                app.iframe.css({width: '0', height: '0', borderWidth: 0});
            }
            // If app.iframe.load is done here it gets fired twice.  Once when iframe is added to dom and again when src is set on it
            //        app.iframe.load(function()
            //        {
            //          //  app.iframe.loaded=true;
            //          //       alert('the iframe is done loading '+app.iframe.src);
            //        });

            app.iframe.appendTo($('#appsContainer'));

            this._applications[appId]=app;

            // worth broadcasting?
            this.broadcastMessage(this.APP_ADDED_MESSAGETYPE,{'appId':appId, 'availableApps':this._getAppsRenderingInfo()});
        }

        // Should always exist at this point...
        if(this._appExists(appId)) {

            if(this._currentApp) {


                // My brain is about to melt...   display:block is a better way of hiding/showing the various apps added
                // to the app switcher BUT....  apparently in IE when you use display:block and the flash player becomes
                // invisible it actually wipes out any actionscript functions registered on the flash object via our
                // ExternalInterface.addCallback calls on the Flex side which completely breaks our messaging.  It works fine
                // in Firefox and Chrome.
                // Workaround to hide the previous iframe by resetting its width and height to 100%
                var that=this;

                // Brain melt part deux...  The above comment about Flex callbacks getting wiped out was when everything was all Flex
                // and there was only one app switcher.  Now that there is an HTML and Flex appswitcher I'm running into the same issue
                // again.  This time all the registered Flex callbacks seem to get wiped out when doing a jquery fadeOut on the iframe and
                // it is specifically causing problems in Firefox and IE.  Works fine on Chrome.  Take out the fade until we figure out
                // why this is happening and how to avoid it (if possible).
                //            this._currentApp.iframe.fadeOut(500, function()
                //            {
                if (!bSilentLoad) {
                    if (sap.ui.Device.support.touch) {//S1149101
                        that._currentApp.iframe.css({display: "none"});
                    } else {
                        that._currentApp.iframe.css({"width": "0", "height": "0"});
                    }
                    $(that._currentApp.iframe).attr("tabindex", "-1");
                    appSwitchBroadcastMessageData.previousAppId=this._currentApp.appId;
                    // Notify app it is being hidden
                    this.sendMessageToApp(this._currentApp.appId, this.APP_HIDE_MESSAGETYPE, {} );
                    //                    domStyle.set(this._currentApp.iframe,"display","none");
                }
                //
                //            });
            }

            var app=this._getApp(appId);
            $(app.iframe).attr("tabindex", "0");
            app.iframe.focus();

            if (!bSilentLoad) {
                this._currentApp=app;
            }

            //            this._currentApp.url=url;
            if(app.iframe[0].src===null || app.iframe[0].src===undefined || app.iframe[0].src==="") {
                // Flex apps participating in AppSwitcher environment need to know whether they are being launched via appswitcher via
                // api or attempting to be launched directly (not via app switcher api) in which case they need to bootstrap app switcher
                // and go through API channels.    Because of this we are going to append a reuqest parameter onto the url so they know when
                var iframeUrl=app.url;
                iframeUrl = sas.hc.ui.core.util.Url.updateQueryStringParameter(app.url,"launchedFromAppSwitcher", true);

                // If we are not coming from the initial bootstrapping app (first application switcher app loaded) then force
                // the splash screen to be the transition rather than the main app SAS splash.
                if(options.hasOwnProperty("bootstrapping")===false)
                    iframeUrl=sas.hc.ui.core.util.Url.updateQueryStringParameter(iframeUrl, "useTransitionSplash", true);

                app.iframe[0].src=iframeUrl;
                var that=this;

                if (that._processURLParameters === true) {
                    var sAppSwitcherMessageParameter = jQuery.sap.getUriParameters().get("appSwitcherMessage");
                    if (sAppSwitcherMessageParameter !== undefined) {
                        that._oAppSwitcherMessage = $.parseJSON(sAppSwitcherMessageParameter);
                    }
                }

                $(app.iframe).on("load", function() {
                    app.iframe.loaded=true;
                    // notify app that we are switching to it so it can do "stuff"
                    if (!bSilentLoad) {
                        that._completeAppSwitch(appId, appShowMessageData, appSwitchBroadcastMessageData);
                    }
                });
            } else {
                // notify app that we are switching to it so it can do "stuff"
                this.sendMessageToApp(appId, this.APP_RETURN_MESSAGETYPE, appShowMessageData);
                this.sendMessageToApp(appId, this.APP_SHOW_MESSAGETYPE, appShowMessageData);

                // This broadcast of AppSwitch is very similar to AppShow and AppHide however those are
                // intentionally targeted messages meant for internal use while AppSwitch is broadcast
                // to all apps is generally usable.
                this.broadcastMessage(this.APP_SWITCH_MESSAGETYPE, appSwitchBroadcastMessageData);
            }

            // My brain is about to melt...   display:block is a better way of hiding/showing the various apps added
            // to the app switcher BUT....  apparently in IE when you use display:block and the flash player becomes
            // invisible it actually wipes out any actionscript functions registered on the flash object via our
            // ExternalInterface.addCallback calls on the Flex side which completely breaks our messaging.  It works fine
            // in Firefox and Chrome.
            // Workaround to display the correct iframe by resetting its width and height to 100%


            //  basefx.fadeIn({node: app.iframe, duration: 500}).play();
            //       app.iframe.fadeIn(500, function()
            //       {
            if (!bSilentLoad) {
                if (sap.ui.Device.support.touch) {//S1149101
                    app.iframe.css({"width": "100%", "height": "100%", display: "block"});
                } else {
                    app.iframe.css({"width": "100%", "height": "100%"});
                }
                this._updateBrowserTabTitle(app);
                this._updateHistory(appId, url, isCompliant);
            } else {
                if (sap.ui.Device.support.touch) {//S1149101
                    app.iframe.css({display: "none"});
                } else {
                    app.iframe.css({"width": "0", "height": "0"});
                }
                $(app.iframe).attr("tabindex", "-1");
            }
        } else {
            sas.log.warning('unable to switch to application ' + appId);
        }
    };

    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype._updateHistory = function(appId, url, isCompliant) {
        // If we want to enable back and forward buttons then we would need to use
        // pushState instead and pass in all the info (appid, url, isCompliant) in as state object
        // then we would need to listen for restore state type events and drive app switcher api
        // with the state object.
        // Not supported in all older browsers.  IE 10+, FF4+, Chromse 5+, No Safari
        // S0992329 - Update browser url bar as we are switching between apps without actually reloading page since
        // this is all accomplished through iframe switching
        try{ //S1152396
            if(top.window.history.pushState) //S1117007
                top.window.history.pushState({'appId':appId, 'url':url, 'isCompliant':isCompliant},top.document.title, url);
        }catch(err){
            //cross domain apps will throw security error.
            //            if(top.window.history.pushState) //S1117007
            //                top.window.history.pushState({'appId':appId, 'url':url, 'isCompliant':isCompliant},top.document.title, url);
            sas.log.info(err.name+":History can't be updated for application ",url);
        }
    };


    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype._completeAppSwitch = function(appId, appShowMessageData, appSwitchBroadcastMessageData) {
        this.sendMessageToApp(appId,this.APP_SHOW_MESSAGETYPE, appShowMessageData);
        // This broadcast of AppSwitch is very similar to AppShow and AppHide however those are
        // intentionally targeted messages meant for internal use while AppSwitch is broadcast
        // to all apps is generally usable.
        this.broadcastMessage(this.APP_SWITCH_MESSAGETYPE, appSwitchBroadcastMessageData);
        if (this._oAppSwitcherMessage !== null && this._oAppSwitcherMessage.data !== undefined && this._processURLParameters === true) {
            this.sendMessageToApp(appId, "App.Open", this._oAppSwitcherMessage.data);
            this._processURLParameters = false;
        }
    };

    /**
     * Set additional title text for describing the current state of this application.
     *
     * This additional text is used in conjunction with application name to further identify
     * where the user is inside the application.  It can be anything from a an application module
     * to a document name.
     *
     * @param {string} [appId] - app id to set title for
     * @param {string} title - title text
     *
     * @public
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.updateTitle = function(appId, title) {

        // If no appId was passed in then first arg is the title flag and we should use
        // current app as the app id.  This will ONLY happen on calls to the application specific
        // updateTitle.  By the time we forward this request on to the main application switcher (bootstrap)
        // we will always be passing a specific appId
        if ( title===undefined ) {
            title=appId;
            appId=sas.config.getAppId();
        }

        /*  If this is not the main application switcher app then this method is being called on one
         of the included apps.  We need to forward this request on to the main app to handle.

         Because of cross domain possibilities we need to use postMessage to do this.  Could possibly
         detect crossdomain or not and if not just make call on parent window directly but for now
         do it same way regardless.
         */
        if(this._isMainApplication()===false) {
            this._sendCommandToMainApplication({'command':'updateTitle', 'appId':appId, 'title':title});
            return;
        }

        if(this._appExists(appId)) {
            var app=this._getApp(appId);
            app.title=title;

            this._updateBrowserTabTitle(app);

            this.broadcastMessage(this.APP_TITLE_UPDATED_MESSAGETYPE,{'appId':appId, 'title':title});
        } else {
            sas.log.warning('unable to update title for application ' + appId);
        }
    };


    /**
     * Update the application name for the given app.
     *
     * This is set automatically by HTML Commons infrastructure.
     *
     * @param {string} [appId] - Id of app to set name for
     * @param {string} appName - The application name for the app
     *
     * @public
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.updateAppName = function(appId, appName) {

        // If no appId was passed in then first arg is the appName flag and we should use
        // current app as the app id.  This will ONLY happen on calls to the application specific
        // updateAppName.  By the time we forward this request on to the main application switcher (bootstrap)
        // we will always be passing a specific appId
        if ( appId === undefined ) {
            appName=appId;
            appId=sas.config.getAppId();
        }


        // Don't do anything on appName = null.  Including unsetting existing appName.
        // There is a case S1313035 where Flex versions of application switcher are incorrectly
        // calling this with null value and wiping out names.
        if(appName===null) {
            return;
        }
        // From Flex ApplicationSwitcher.as comments...
        // we already provide updateTitle which is essentially either a document name
        // or a verb describing the action (Hub oriented).
        // UI wants the browser tab to be title - application name.  For Hub integrated
        // apps the Banner already has application name from hub integration and can
        // give it to us automatically.


        /*  If this is not the main application switcher app then this method is being called on one
         of the included apps.  We need to forward this request on to the main app to handle.

         Because of cross domain possibilities we need to use postMessage to do this.  Could possibly
         detect crossdomain or not and if not just make call on parent window directly but for now
         do it same way regardless.
         */
        if(this._isMainApplication()===false) {
            this._sendCommandToMainApplication({'command':'updateAppName', 'appId':appId, 'appName':appName});
            return;
        }

        if(this._appExists(appId)) {
            var app=this._getApp(appId);
            app.appName=appName;
            if(this._currentApp.appId===appId) {
                this._updateBrowserTabTitle(app);
            }


        }
    };

    /**
     * Update the browser tab text based on current application name and application title.
     *
     * @param {Application} app - The application to udpate browser tab for
     *
     * @private
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype._updateBrowserTabTitle = function(app) {
        var browserTitle=null;
        if(app.title!==null && app.title!=='undefined')
            browserTitle=app.title;

        if(app.appName!==null && app.appName!=='undefined') {
            if(browserTitle!==null && browserTitle!==app.appName)  {
                if (sap.ui.getCore().getConfiguration().getRTL()) {
                    browserTitle = app.appName + " - " + browserTitle;
                } else {
                    browserTitle = browserTitle + " - " + app.appName;
                }
            }
            else
                browserTitle=app.appName;
        }

        if(browserTitle===null)
            browserTitle='';

        top.document.title=browserTitle;

    };


    /**
     * Logoff of all applications open in the application switcher.
     *
     * This will use the logoff url specified via setLogoffUrl which is set automatically.
     *
     * @public
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.logoff = function() {
        /*  If this is not the main application switcher app then this method is being called on one
         of the included apps.  We need to forward this request on to the main app to handle.

         Because of cross domain possibilities we need to use postMessage to do this.  Could possibly
         detect crossdomain or not and if not just make call on parent window directly but for now
         do it same way regardless.
         */
        if(this._isMainApplication()===false) {
            this._sendCommandToMainApplication({'command':'logoff'});
            return;
        }

        var unsavedChangesMessage = this.getUnsavedChangesMessage(true);

        if (unsavedChangesMessage === null) {
            top.sas.applicationSwitcher.completeLogoff();
        } else {
            //sas.applicationSwitcher.localizedStrings provided at top level AppSwitcher by AppSwitcherBootstrap.jsp
            unsavedChangesMessage = unsavedChangesMessage + "\n\n" + this.localizedStrings['discardUnsavedChanges.txt'];

            var oLowLevelApplicationSwitcher = this._currentApp.getApplicationSwitcher ? this._currentApp.getApplicationSwitcher() : null;
            if (oLowLevelApplicationSwitcher && oLowLevelApplicationSwitcher.showUnsavedChangesDialog) {
                oLowLevelApplicationSwitcher.showUnsavedChangesDialog(unsavedChangesMessage);
            } else {
                var answer = confirm(unsavedChangesMessage);  // eslint-disable-line
                if (answer) {
                    top.sas.applicationSwitcher.completeLogoff();
                } else {
                    top.sas.applicationSwitcher._hasShownUnsavedMessagesFlag = false;
                }
            }
        }
    };


    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.completeLogoff = function() {
        if (!this._isMainApplication()) {
            return;
        }

        this.broadcastMessage(this.APP_LOGOFF_IMMINENT_MESSAGETYPE);
        //S1105116 - Sketchy situation at best.  If the app supporting Logoff (Home) was available when we started but somehow
        // went down afterwards while user is still in app then try to detect that the Logoff app is no longer valid and in that
        // case try falling back to main app (window.context.location) WIP Logoff.   Of course that could be down as well but
        // this is just to make some folks happy.
        // It's odd because just doing the ajax call will actually perform the logoff.  The subsequent call will show the results of
        // logoff in the browser indow allowing sign in if available.
        var platform = sas.config.get("servicePlatform");

        // We should always have a platform available to use either automatically via jsp tag
        // or by solution specific override.  In the last ditch case that we don't attempt some
        // best guesses if possible.
        if (typeof(platform) === "undefined" || platform === null) {
            sas.registry.wip ? platform = "WIP" : platform = "PETRICHOR";
        }

        var logoffURL;
        if (platform === "WIP") {
            logoffURL = "/Logoff";
        } else {
            logoffURL = "/logout";
        }

        //For S1235707, set LogoffUrl according to the platform, to avoid the request error.
        sas.applicationSwitcher.setLogoffUrl(this.urlUtil.getContextPath(window.location.href) + logoffURL);
        sas.log.info("AppSwitcher is attempting to logoff.");
        setTimeout(jQuery.proxy(function() {
            window.location.href = this._logoffUrl;
        }, this), 500);
    };

    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.showUnsavedChangesDialog = function(unsavedChangesMessage) {
        jQuery.sap.require("sas.hc.m.MessageDialog");
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.hc.ui.core");
        sas.hc.m.MessageDialog.warning({
            text: unsavedChangesMessage,
            title: rb.getText("sas.applicationSwitcher.unsavedChangesDialog.title"),
            buttonActions: [
                rb.getText("sas.applicationSwitcher.unsavedChangesDialog.discardAndExit.txt"),
                {
                    text: sas.hc.m.MessageDialogAction.Cancel,
                    initialFocus: true,
                    defaultButton: true
                }
            ]
        }).attachButtonPress(function(e) {
            var oButton = e.getParameter("origin");
            if (oButton.getText() === rb.getText("sas.applicationSwitcher.unsavedChangesDialog.discardAndExit.txt")) {
                top.sas.applicationSwitcher.completeLogoff();
            } else {
                top.sas.applicationSwitcher._hasShownUnsavedMessagesFlag = false;
            }
        });
    };


    /**
     * Calculate message to show when user attempts to shut down application switcher while
     * one or many open applications are in a dirty state.
     *
     * @returns {string} TODO-jsdoc
     *
     * @private
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.getUnsavedChangesMessage = function(bLogoffDialog) {
        var unsavedChangesMessage="";
        /*  Check all the apps for dirty flag of true.  If so then warn user they might lose changes
         and give them chance to bail on the logoff.
         */
        for (var appId in this._applications) {
            if (this._applications.hasOwnProperty(appId)) {
                var app=this._getApp(appId);
                if(app.dirty===true) {
                    unsavedChangesMessage=unsavedChangesMessage+"\n"+app.appName;
                }
            }
        }

        if(unsavedChangesMessage==="" || top.sas.applicationSwitcher._hasShownUnsavedMessagesFlag === true) {
            return null;
        } else {
            if (bLogoffDialog) {
                top.sas.applicationSwitcher._hasShownUnsavedMessagesFlag = true;
            }
            return this.localizedStrings['unsavedChangesNotification.txt'] + " \n" + unsavedChangesMessage;
        }
    };

    /**
     * Update the dirty flag for specified application.  This is necessary so that logoffs, timeouts, browser close
     * actions are given opportunity to let user know that there are unsaved changes which will be lost if the
     * initiating action continues.
     *
     * @param {string} [appId]  - optional application id.  If not specified use current application id.
     * @param {boolean} dirtyFlag - true or false indicating whether the app is dirty or not
     *
     * @public
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.updateDirtyFlag = function(appId, dirtyFlag) {

        // If no appId was passed in then first arg is the dirty flag and we should use
        // current app as the app id.  This will ONLY happen on calls to the application specific
        // updateDirtyFlag.  By the time we forward this request on to the main application switcher (bootstrap)
        // we will always be passing a specific appId
        if ( typeof appId === "boolean" ) {
            dirtyFlag=appId;
            appId=sas.config.getAppId();
        }

        /*  If this is not the main application switcher app then this method is being called on one
         of the included apps.  We need to forward this request on to the main app to handle.

         Because of cross domain possibilities we need to use postMessage to do this.  Could possibly
         detect crossdomain or not and if not just make call on parent window directly but for now
         do it same way regardless.
         */
        if(this._isMainApplication()===false) {
            this._sendCommandToMainApplication({'command':'updateDirtyFlag', 'appId':appId, 'dirtyFlag':dirtyFlag});
            return;
        }

        this.setDirtyFlag(appId, dirtyFlag);
    };

    /**
     * Logic to actually update the specified dirty flag.
     *
     * @param {string} appId - application id  of application to update dirty flag.
     * @param {boolean} dirtyFlag - true or false indicating whether the app is dirty or not
     *
     * @private
     * @function
     */

    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.setDirtyFlag = function(appId, dirtyFlag) {
        if(this._appExists(appId)) {
            var app=this._getApp(appId);
            app.dirty=dirtyFlag;
            this.broadcastMessage(this.APP_DIRTY_FLAG_CHANGED_MESSAGETYPE,{'appId':appId, 'dirtyFlag':dirtyFlag});
        } else {
            sas.log.warning('unable to update dirty flag for application ' + appId);
        }
    };

    /**
     * Gets dirty flag for the specified application.
     *
     * @param {string} appId - application id  of application to update dirty flag.
     * @param {boolean} dirtyFlag - true or false indicating whether the app is dirty or not
     *
     * @private
     * @function
     */

    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.getDirtyFlag = function(appId) {
        if (appId === undefined) {
            appId = sas.config.getAppId();
        }

        if(top && top.sas && top.sas.applicationSwitcher && top.sas.applicationSwitcher._appExists(appId)) {
            var app=top.sas.applicationSwitcher._getApp(appId);
            if (app) {
                return app.dirty;
            }
        }
        sas.log.warning('unable to get dirty flag for application ' + appId);
    };


    /**
     *  Allows for temporary disabling of the document unload showing a dirty message.
     *
     *  This is really meant for use by the timeout mechanism so that it does not show dirty state
     *  message if timeout expires without user responding.
     *
     * @param {boolean} allowDirtyMessage TODO-jsdoc
     *
     * @public
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.setDirtyMessageEnabled = function(allowDirtyMessage) {

        /*  If this is not the main application switcher app then this method is being called on one
         of the included apps.  We need to forward this request on to the main app to handle.

         Because of cross domain possibilities we need to use postMessage to do this.  Could possibly
         detect crossdomain or not and if not just make call on parent window directly but for now
         do it same way regardless.
         */
        if(this._isMainApplication()===false) {
            this._sendCommandToMainApplication({'command':'setDirtyMessageEnabled', 'allowDirtyMessage':allowDirtyMessage});

            return;
        }


        this._dirtyMessageEnabled=allowDirtyMessage;
    };

    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.isDirtyMessageEnabled = function() {
        return this._dirtyMessageEnabled;
    };

    /**
     *  If this is not the main application switcher app then this method is being called on one
     * of the included apps.  We need to forward this request on to the main app to handle.
     *
     * Take command and pass it on to main application to be processed
     * In the process normalize it with anything we want to always be
     * on every commanhd (like AppSwitcher Version).
     *
     * Because of cross domain possibilities we need to use postMessage to do this.  Could possibly
     * detect crossdomain or not and if not just make call on parent window directly but for now
     * do it same way regardless.
     *
     * @param {object} commandObject command to pass on to main application switcher.
     * @private
     * @function
     **/
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype._sendCommandToMainApplication = function(commandObject) {
        sas.log.trace(sas.config.getAppId(), ':_sendCommandToMainApplication-', arguments);
        commandObject.version = this._version;
        top.postMessage(JSON.stringify(commandObject), this._postMessageOrigin);
    };

    /**
     * Target a message to a specific application.  This is the primary mechanism
     * use dto communicate between application switcher enabled applications.
     *
     * @param {string} appId - Application to send message to
     * @param {string} type - String identifying the message type so that receiving app can make sense of what it is.
     * @param {object} [data] - A javascript object which contains any data needed to pass along to receiving application
     *
     * @public
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.sendMessageToApp= function(appId, type, data) {
        sas.log.trace(sas.config.getAppId(), ':sendMessageToApp-', arguments);

        if (data === null)
            data = {};


        /*  If this is not the main application switcher app then this method is being called on one
         of the included apps.  We need to forward this request on to the main app to handle.

         Because of cross domain possibilities we need to use postMessage to do this.  Could possibly
         detect crossdomain or not and if not just make call on parent window directly but for now
         do it same way regardless.
         */
        if (this._isMainApplication() === false) {
            // Asked to send message to someone other than ourselves so forward on to main appswitcher
            this._sendCommandToMainApplication({'command':'sendMessageToApp', 'appId':appId, 'type':type, 'data':data});
            return;
        }

        // Main application asked to send message to one of our sas:Flex applications being managed
        // by this appSwitcher.   Send the message directly to the application being managed to handle.
        // Most of the things in appSwitcher need to be handled in main appSwitcher because they are operating
        // on list of contributed apps or on main dom.
        // However this has to be handled the appSwitcher for the contributed app itself inside of it's own
        // iframe space.
        if (this._appExists(appId)) {
            var appToSendTo = this._getApp(appId);
            if (appToSendTo.iframe.loaded === false || appToSendTo.iframe.loaded === undefined) {
                var that = this;
                $(appToSendTo.iframe).on( "load", function () {
                    appToSendTo.iframe[0].contentWindow.postMessage(JSON.stringify({'version': that._version, 'command':'_sendMessageToApp', 'appId':appId, 'type':type, 'data':data}), that._postMessageOrigin);
                });
            } else {
                appToSendTo.iframe[0].contentWindow.postMessage(JSON.stringify({'version': this._version, 'command':'_sendMessageToApp', 'appId':appId, 'type':type, 'data':data}), this._postMessageOrigin);
            }
        } else {
            sas.log.warning('unable to send message to application ' + appId + ' because the app has not been added.');
        }

    };

    // message: Object
    // type: String
    /**
     * Broadcast a message to ALL HTML or Flex applications that are open in the application switcher environment.
     *
     * There is no guaranteed order that the apps will receive this message in.
     *
     * @param {string} type - String identifying the message type so that receiving app can make sense of what it is.
     * @param {object} [data] - A javascript object which contains any data needed to pass along to receiving application
     *
     * @public
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.broadcastMessage = function(type,data) {
        sas.log.trace(sas.config.getAppId(),':broadcastMessage');
        if(data===undefined || data===null)
            data={};

        /*  If this is not the main application switcher app then this method is being called on one
         of the included apps.  We need to forward this request on to the main app to handle.

         Because of cross domain possibilities we need to use postMessage to do this.  Could possibly
         detect crossdomain or not and if not just make call on parent window directly but for now
         do it same way regardless.
         */
        if(this._isMainApplication()===false) {
            this._sendCommandToMainApplication({'command':'broadcastMessage', 'type':type, 'data':data});
            return;
        }

        function doSendMessageToApp(){
            for (var appId in this._applications) {
                if (this._applications.hasOwnProperty(appId)) {
                    this.sendMessageToApp(appId,type, data);
                }
            }
        }
        // invoke message processor before broadcasting message to chlid frames.
        processTypeAndInvoke(type,doSendMessageToApp,this);
    };

    /**
     * Subscribe to Application Switcher messages.
     *
     * You can specify the message type directly and have multiple subscriptions per type or you can skip the
     * message type and have one general subscription in which you need to check the message type to determine
     * what operation to perform.
     *
     *
     * @param {string} [type=ALL] - Optional message type to listen for.  If not specified it will listen for all messages
     *               on the ApplicationSwitcher channel
     * @param {function(string,*)} fn - Handler function.
     * @param (oListener) Object associated with handler.
     *
     * @public
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.subscribe = function(type, fn, oListener) {
        sas.log.trace(sas.config.getAppId(),':subscribe',type);
        // If no type was passed in then first arg is the function.
        if ( typeof type === "function" ) {
            fn=type;
            type=undefined;
        }

        this._messageHandlerMap[fn] = function(channel, type, content) {
            fn.apply(this, [content.type, content.data]);
        };

        sas.eventBus.subscribe(
            "ApplicationSwitcher",
            type===undefined?"ApplicationSwitcher":type,
            this._messageHandlerMap[fn],
            oListener
        );
    };

    /**
     * Unsubscribe to Application Switcher messages.
     *
     * @param {string} [type=ALL] - Optional message type to listen for.  If not specified it will listen for all messages
     *               on the ApplicationSwitcher channel
     * @param {function(string,*)} fn - Handler function.
     * @param (oListener) Object associated with handler.
     *
     * @public
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.unsubscribe = function(type, fn, oListener) {
        sas.log.trace(sas.config.getAppId(),':subscribe',type);
        // If no type was passed in then first arg is the function.
        if ( typeof type === "function" ) {
            fn=type;
            type=undefined;
        }

        sas.eventBus.unsubscribe(
            "ApplicationSwitcher",
            type===undefined?"ApplicationSwitcher":type,
            this._messageHandlerMap[fn],
            oListener
        );

        delete this._messageHandlerMap[fn];
    };


    /**
     * Fetch the application object representing the given application id.
     *
     * @param {string} appId TODO-jsdoc
     * @returns {Application} TODO-jsdoc
     *
     * @private
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype._getApp = function(appId) {
        if(this._appExists(appId))
            return this._applications[appId];
        else
            return null;
    };

    /**
     * Return array of apps with miniminal info needed for rendering an appswitcher visual control.
     * This should just be label and id.  Can't do full applications array because it isn't JSONable
     * due to Application instances having iframe reference.
     *
     * @returns {Application[]} TODO-jsdoc
     * @private
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype._getAppsRenderingInfo = function() {
        var appsArray=[];
        for (var _appId in this._applications) {
            if (this._applications.hasOwnProperty(_appId)) {
                var app = this._applications[_appId];
                var renderingInfo = {'appId': app.appId, 'title': app.title, 'url': app.url, 'isDirty':app.dirty};
                //                  if(this._currentApp.appId===_appId)
                //                  renderingInfo["isCurrentApp"]=true;

                appsArray.push(renderingInfo);
            }
        }

        return appsArray;
    };

    /**
     * Returns boolean indicating whether the application represented by appId has been loaded or not.
     *
     * @param {string} appId TODO-jsdoc
     * @returns {boolean} TODO-jsdoc
     * @private
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype._appExists = function(appId) {
        return this._applications.hasOwnProperty(appId);
    };


    /**
     * Determine whether this is the top level application switcher bootstrap or an actual
     * application application switcher.
     *
     * @returns {boolean} TODO-jsdoc
     *
     * @private
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype._isMainApplication = function() {
        return window===top;
    };

    /**
     *  If not already created build an appsContainer div to store the apps iframes inside.
     * This was needed to get the iframes to fill up the entire space in a cross browser fashion.
     *
     * @private
     * @function
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype._createAppsContainer = function() {
        var appsContainer = $("#appsContainer")[0];
        if(appsContainer) {
            return;
        } else {
            var appsContainer=$("<div>",{ id:'appsContainer'});
            appsContainer.css({position:'absolute', overflow: 'hidden', top:'0px', left:'0px', width: '100%', height:'100%'});
            appsContainer.appendTo($('body'));
        }
    };

    /**
     * Times out application to specified url.
     * @param {string} url TODO-jsdoc
     * @function
     * @private
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.timeout=function(url){
        if ( typeof url !== 'string'){
            return;
        }

        // timeout to the top window.
        if(this._isMainApplication()===false){
            this._sendCommandToMainApplication({'command':'timeout', 'url':url});
            // give a millisec to reach below code block and give chance top frame to handle timeout.
            // See S1190297
            // Check if top level appswitcher is of same technology. Need to handle timeout separately if they don't match.
            var that=this;
            setTimeout(function(){
                try{
                    if(!(top.sas
                        && top.sas.applicationSwitcher
                        && top.sas.applicationSwitcher.technology === that.technology)){

                        sas.log.trace(sas.config.getAppId(),'Top appswitcher in possibly different technology.');
                        top.location.href=url;
                    }
                }catch(err){
                    // security error will be thrown in case of
                    // apps loaded from cross origin.
                    sas.log.trace(sas.config.getAppId(),'Possible cross origin application with that of top app.');
                    // handle the error case by loging off.
                    that.logoff();
                }
            },0);
            return;
        }

        window.location.href=url;
    };

    /**
     * Find application information that matches conditions compared by input compare function.
     * @param {function(object)} compareFunction Function that returns true for matchig application and false otherwise. The function is invoked with
     * one argument i.e. oApplication.
     * <code>
     *  function compare(oApp){
     *    return oApp.id === 'foo';
     *  }
     * </code>
     * @function
     * @returns {object} Applicaton data: a javascript object. Undefined if no application found.
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.findApplication = function(compareFunction){
        if(!Array.isArray(this.availableApplicationsFlattened) && typeof compareFunction !== 'function') return;
        for (var i = 0; i < this.availableApplicationsFlattened.length; i++) {
            var elm = this.availableApplicationsFlattened[i];
            if(compareFunction.call(this,elm)){
                return elm;
            }
        }
    };

    /**
     * Get application information that is currently associated with the state of the application.
     * For e.g. application suites like CI has various views associated with it. If user moves to CI->Content->Message
     * then invoking this API will return the application data for message view of CI.
     * @function
     * @returns {object} Applicaton data: a javascript object.
     */
    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher.prototype.getCurrentApplication = function(){
        var that=this,
            res = this.findApplication(function(oApp){
                return oApp && typeof oApp === 'object' && oApp.hashId === that._currentAppHashId;
            });
        if(!res && sas && sas.config && typeof sas.config.getAppId === 'function'){
            // try searching by app id.
            res = this.findApplication(function(oApp){
                return oApp && typeof oApp === 'object' && oApp.appId === sas.config.getAppId();
            });
        }
        return res;
    };

}());

/**
 * Data Provider for LeftNav.
 */
jQuery.sap.declare('sas.hc.ui.core.LeftNavDataProvider');
sap.ui.define('sas/hc/ui/core/LeftNavDataProvider',['jquery.sap.global','sap/ui/base/Object','sas/hc/ui/core/util/ComponentServiceClassResolver'],
    function(jQuery, BaseObject, resolver) {

        var LeftNavDataProvider = BaseObject.extend('sas.hc.ui.core.LeftNavDataProvider',{
            getViews: function(){
                return;
            },
            getViewsAsync: function(){
                return Promise.resolve();
            }
        });
        var _instance;
        Object.defineProperty(LeftNavDataProvider,"instance",{
            get:function(){
                if(_instance === undefined){
                    // populate instance.
                    _instance = resolver.getServiceClassInstance(resolver.LEFT_NAV_SERVICE);
                }
                return _instance;
            }
        });
        return LeftNavDataProvider;
    });
jQuery.sap.declare("sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherBootTask");

jQuery.sap.require("sas.hc.ui.core.BootTask");
jQuery.sap.require("sas.hc.ui.core.util");
jQuery.sap.require("sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher");
jQuery.sap.require("sas.hc.ui.core.util.Url");
jQuery.sap.require('sas.hc.ui.core.LeftNavDataProvider');

(function(){

    sas.hc.ui.core.BootTask.extend('sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherBootTask',
        {
            name: 'sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherBootTask',
            dependsOn:['sas.hc.ui.core.RegistryTask','sas.hc.ui.core.TimeoutHandlerTask','sas.hc.ui.core.cache.CacheManagerTask','sas.hc.ui.core.settings.SettingsInfoTask'],

            constructor: function() {


            }
        });


    sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherBootTask.prototype.run=function() {

        // register a boot hook to hold splash screen if app switcher takes
        // more time than ltjs to load. should never be a case but cover the use case.
        var appSwitcherDataLoaded = false,
            handler = function (){
                appSwitcherDataLoaded = true;
                sas.eventBus.unsubscribe("AppSwitcher_Data_Loaded",handler);
            },
            // populate leftNav data.
            leftNavConfig = sas.config.get('enableLeftNav') === true && sas.config.get('leftNav');

        if(leftNavConfig && leftNavConfig.viewId){
            sas.hc.ui.core.LeftNavDataProvider.instance;
        }

        sas.eventBus.subscribe("AppSwitcher_Data_Loaded",handler);

        sas.hc.ui.core.util.registerBootHook(function(){

            if(appSwitcherDataLoaded){
                return Promise.resolve();
            }else{
                return new Promise(function(_resolve_,_reject_){
                    var resolver = function(){
                        _resolve_();
                        sas.eventBus.unsubscribe("AppSwitcher_Data_Loaded",resolver);
                    };
                    sas.eventBus.subscribe("AppSwitcher_Data_Loaded",resolver);
                });
            }
        });

        // Create an application switcher instance and hang it off of sas namespace
        Object.defineProperty(sas,"applicationSwitcher",{
            enumerable:true,
            value : new sas.hc.ui.core.applicationSwitcher.ApplicationSwitcher()
        });

        // Set app name automatically something other than url shows in browser tab.  Solutions can
        // also set a title on their app as state changes to give a better representation of where
        // user is in app.
        sas.applicationSwitcher.updateAppName(sas.config.getAppId(), sas.config.getAppName());


        // Do some high level logging of messages received for all applications.
        sas.applicationSwitcher.subscribe(function (type, data) {

            // Why can't I just do sas.log.info here?
            var logger = sas.log.getLogger("ApplicationSwitcher.js", sas.log.Level.INFO);
            logger.info(sas.config.getAppId() + " has received message from AppSwitcher : " + type + " : " + JSON.stringify(data));
            //     console.info(sas.config.getAppId());
        });

        // Inline with what Flex timeouts do under Application Switcher management disable timeouts on the app
        // when it is not currently the active app.  This prevents timeout dialogs happening when user can't
        // see the application in question and prevents it from timing out underneath them.
        //
        // The other route would be to broadcast user activity events to all apps. Don't remember why we
        // didn't do that in Flex other than that there was an issue with it.
        //
        // Only downside to disabling timeouts in in nonvisible apps is session timeouts potentially being out
        // of sync but I have yet to hear anyone encounter that in Flex so keep with it.
        sas.applicationSwitcher.subscribe(sas.applicationSwitcher.APP_HIDE_MESSAGETYPE, function(type,data){
            if(sas.timeoutManager)
                sas.timeoutManager.quiet();
        });


        sas.applicationSwitcher.subscribe(sas.applicationSwitcher.APP_RETURN_MESSAGETYPE, function(type,data){
            if(sas.timeoutManager){
                sas.timeoutManager.awake();
            }

            // Whenever a new HTML based app switcher app is shown grab its generated hashId (S1104768) and make it available on the application
            // similar to how appId is avaialble on the sas.config area.  appId comes from the jsp tag and more traditional channels.  Lets inject
            // this into that same space.
        });

        sas.applicationSwitcher.subscribe(sas.applicationSwitcher.APP_SHOW_MESSAGETYPE, function(type,data){
            // Whenever a new HTML based app switcher app is shown grab its generated hashId (S1104768) and make it available on the application
            // similar to how appId is avaialble on the sas.config area.  appId comes from the jsp tag and more traditional channels.  Lets inject
            // this into that same space.
            sas.config.setValue("hashId", data.hashId); //TODO Don't think a good idea to update configurations. Configurations should be read-only. This could be a property/API on sas.appswitcher instead.
        });

        if(typeof sas.eventBus === 'object' && typeof sas.timeoutManager === 'object'){

            sas.eventBus.subscribe(sas.timeoutManager.events.BEFORE_TIME_OUT,function(){
                // We were disabling dirty message whenever timeout warning dialog was displayed because we don't want dirty message
                // to show if timeout warning dialog gets to timeout duration without user hitting Logoff or Continue.
                // However this also has the side effect that if user closes browser or leaves page while timeout warning dialog
                // is up then they won't see dirty message which is incorrect.
                // Do it when timeout warning duration is about expired to minimize this possibility.
                sas.applicationSwitcher.setDirtyMessageEnabled(false);
            });

            sas.eventBus.subscribe(sas.timeoutManager.events.USER_ACTIVE,function(){
                // While Timeout warning dialog is up we disable application switcher unsaved changes
                // mechanism so that it doesn't prevent a timeout.  If user hits continue in timeout warning
                // then we have to make sure to turn the unsaved changes message back on.
                sas.applicationSwitcher.setDirtyMessageEnabled(true);
            });
        }

    };
}());

jQuery.sap.declare("sas.hc.ui.core.TouchDetectionTask");

jQuery.sap.require("sas.hc.ui.core.BootTask");
jQuery.sap.require("sas.hc.ui.core.util");

(function(){
    sas.hc.ui.core.BootTask.extend('sas.hc.ui.core.TouchDetectionTask',{
        name: 'sas.hc.ui.core.TouchDetectionTask'
    });
    sas.hc.ui.core.TouchDetectionTask.prototype.run=function(){
        var hasTouch = jQuery.support.touch;
        sas.hasTouch = function() {
            return hasTouch;
        };
        if( hasTouch === true ) {
            jQuery(document.documentElement).addClass("sas-touch");
        }
    };
}());


/**
 * Styles class to be used for specifying custom styles on Controls.
 * Copyright 2014 SAS Institute Inc.
 * All Rights Reserved.
 * @name sas.hc.ui.core.Styles
 * @extends sap.ui.base.Object
 * @author sinyas
 */
jQuery.sap.declare("sas.hc.ui.core.Styles"); // declare this because we aggregate it in sas-ui-core so the export doesn't work as expected
sap.ui.define('sas/hc/ui/core/Styles',['jquery.sap.global', 'sap/ui/base/Object'],
    function(jQuery, BaseObject) {
        "use strict";

        var CustomStyles = BaseObject.extend('sas.hc.ui.core.Styles',{
            constructor: function(){
                BaseObject.apply(this);
            }
        });

        function setter(val,sGetterName){
            if(typeof val === 'object' && val){
                var _val = val;
                delete this[sGetterName];
                this[sGetterName] = function(){
                    return _val;
                };
            }else{
                delete this[sGetterName];
            }
        };

        /**
         * Setter for styles.
         * @param {object} oObj: A JSON object that contain CSS style properties. For. e.g. {'background-color':'red', color:'green'}
         * @returns {sas.hc.ui.core.Styles} TODO-jsdoc
         */
        CustomStyles.prototype.setStyles=function(oObj){
            setter.call(this,oObj,'getStyles');
            return this;
        };

        /**
         * Setter for custom styles related to a Control styles when its selected . Need not be supported by all Controls.
         * @param {object} oObj A JSON object that contain CSS style properties. For. e.g. {'background-color':'red', color:'green'}
         * @returns {sas.hc.ui.core.Styles} TODO-jsdoc
         */
        CustomStyles.prototype.setSelectedStateStyles=function(oObj){
            setter.call(this,oObj,'getSelectedStateStyles');
            return this;
        };

        CustomStyles.prototype.getStyles=CustomStyles.prototype.getSelectedStateStyles=function(){};

        /**
         * Get value of the case insensitive input property sPropName from the input object oStyles.
         * In case of the input object oStyles having multiple properties of same name then the value of the last occurrence
         * of the property is returned.
         * For e.g.
         * <code>this.getPropertyValue('color',{color:'red', COLOR:'blue', coLoR:'green'})</code> returns 'green'.
         * @param {string} sPropName: Name of the property to fetch value of.
         * @param {object} oStyles: Non null object from which the property value is obtained.
         * @returns {string} Value of the input property if found. Undefined otherwise.
         */
        CustomStyles.prototype.getPropertyValue=function(sPropName,oStyles){
            if(typeof oStyles !== 'object' || !oStyles) return;
            var val = undefined;
            sPropName = sPropName.toLowerCase();
            Object.keys(oStyles).forEach(function(prop){
                if(prop.toLowerCase() === sPropName){
                    val = oStyles[prop];
                }
            });
            return val;
        };

        // add some utility functions.
        Object.defineProperties(CustomStyles,{
            // From http://stackoverflow.com/questions/5623838/rgb-to-hex-and-hex-to-rgb
            /**
             * Converts hex value to corresponding rgb string.
             * E.g. hexToRgb('#0033ff') would return '0,51,255'
             */
            hexToRgb:{
                value:function(sHex){
                    jQuery.sap.assert(typeof(sHex) === 'string' && sHex, "Styles.hexToRgb: sHex must be an non-empty String");

                    if (sHex.charAt(0) === '#'){
                        sHex=sHex.substr(1);
                    }
                    var bigint = parseInt(sHex, 16);

                    /*eslint-disable no-bitwise */
                    var r = (bigint >> 16) & 255;
                    var g = (bigint >> 8) & 255;
                    var b = bigint & 255;
                    /*eslint-enable no-bitwise */

                    return [r,g,b,].join();
                }
            },
            /**
             * Converts hex value to corresponding rgba string.
             * E.g. hexToRgba('#0033ff',1) would return '0,51,255,1'
             */
            hexToRgba:{
                value:function(sHex,nOpacity){
                    jQuery.sap.assert(typeof(nOpacity) === 'number' && nOpacity <= 1, "Styles.hexToRgba: nOpacity must be a number <= 1");
                    var rgb = CustomStyles.hexToRgb(sHex);
                    return [rgb, nOpacity].join();
                }
            },

            /**
             * Converts rgb value to corresponding hex string.
             * E.g. rgbToHex(0, 51, 255) would return '#0033ff'
             */
            rgbToHex:{
                value:function (r, g, b) {
                    /*eslint-disable no-bitwise */
                    return "#" + ((1 << 24) + (r << 16) + (g << 8) + b).toString(16).slice(1);
                    /*eslint-enable no-bitwise */
                }
            }

        });


        return CustomStyles;
    },true);
jQuery.sap.declare("sas.hc.ui.core.MonkeyPatchApplySettingsTask");

jQuery.sap.require("sas.hc.ui.core.BootTask");
jQuery.sap.require("sap.ui.core.Control");
jQuery.sap.require("sas.hc.ui.core.Styles");

(function(){

    sas.hc.ui.core.BootTask.extend('sas.hc.ui.core.MonkeyPatchApplySettingsTask',{
        name: 'sas.hc.ui.core.MonkeyPatchApplySettingsTask'
    });

    sas.hc.ui.core.MonkeyPatchApplySettingsTask.prototype.run=function(){

        // Apply custom css patch code to Control.
        // Could be done in a separate boot task but doesn't make
        // sense in chaining applySetting proxy function.
        /**
         * Invalidate custom styles.
         * @returns {sap.ui.core.Control} TODO-jsdoc
         */
        sap.ui.core.Control.prototype.invalidateCustomStyles=function(){
            if(typeof this.doSetCustomStyles === 'function'){
                this.doSetCustomStyles(this.getCustomStyles());
            }
            if(this.getUIArea()){
                sas.hc.ui.getCore().addInvalidateCustomStyles(this);
            }else {
                var oParent = this.getParent();
                // has parent but not rendered before
                if (oParent && !this.bOutput){
                    sas.hc.ui.getCore().addInvalidateCustomStyles(this);
                }
            }
            return this;
        };

        /**
         * Validate CustomStyles.
         * @param {object} oStyles TODO-jsdoc
         */
        sap.ui.core.Control.prototype.validateCustomStyles=function(oStyles){
            if(typeof oStyles === 'object' && !(oStyles instanceof sas.hc.ui.core.Styles)){
                throw new Error("Can not set custom styles on "+this.toString()+". CustomStyles must be an instance of sas.hc.ui.core.Styles");
            }
        };

        /**
         * Sets custom styles on the control. Extending controls should <b>never</b> overwrite this function.
         * Implement <code>doSetCustomStyles</code> function if needed.
         * @param {object} oStyles Object containing styles to be set on this control.
         * @returns {sap.ui.core.Control} TODO-jsdoc
         */
        sap.ui.core.Control.prototype.setCustomStyles=function(oStyles){
            // lalour - have to disable this check as only the Selected State styles may have changed in the original styles object
            //if(this._oCustomStyles === oStyles) return;
            this.validateCustomStyles(oStyles);
            this._oCustomStyles = oStyles;
            this.invalidateCustomStyles();
            this.addStyleClass("sasCustomStylesApplied");
            return this;
        };

        /**
         * Getter for custom styles set if any on this control.
         * @returns {object} TODO-jsdoc
         */
        sap.ui.core.Control.prototype.getCustomStyles=function(){
            return this._oCustomStyles;
        };

        // Proxy Control.destroy
        var fnOldDestroy = sap.ui.core.Control.prototype.destroy;
        sap.ui.core.Control.prototype.destroy=function(bSuppressInvalidate){
            fnOldDestroy.apply(this,arguments);
            // Invlidate custome style if this control has any so that
            // custom styles gets removed from DOM.
            if(this.getCustomStyles()){
                sas.hc.ui.getCore().addInvalidateCustomStyles(this);
            }
        };

        // Proxy ManagedObject.applySettings
        var fnOld = sap.ui.base.ManagedObject.prototype.applySettings;
        sap.ui.base.ManagedObject.prototype.applySettings = function(mSettings, oScope) {
            // invoke applySettings
            if( fnOld ) {
                fnOld.apply(this, arguments);
            }

            if( this.responsiveMetadata ) {
                jQuery.sap.require("sas.hc.contributor.responsive.ResponsiveContributor");
                sas.hc.contributor.responsive.ResponsiveContributor.call(this);
            }
            if(mSettings && mSettings.customStyles && this instanceof sap.ui.core.Control){
                this.setCustomStyles(mSettings.customStyles);
            }
        };

        // proxy ManagedObjects.setParent
        var fnOldSetParent = sap.ui.base.ManagedObject.prototype.setParent;
        sap.ui.base.ManagedObject.prototype.setParent = function(oParent, sAggregationName, bSuppressInvalidate) {
            // invoke original function.
            if(fnOldSetParent){
                fnOldSetParent.apply(this,arguments);
            }
            // invalidate custom styles if set already.
            if(    this._oCustomStyles){
                this.invalidateCustomStyles();
            }
            return this;
        };

    };
}());

// (c) 2014, SAS Institute Inc.
/**
 * SAS implementation of document-wide context menu handler for HTML Commons.
 * Copyright 2014 SAS Institute Inc.
 * All Rights Reserved.
 * @name sas.hc.ui.core.ContextMenuHandler
 */

jQuery.sap.declare("sas.hc.ui.core.ContextMenuHandler");
jQuery.sap.require("sap.ui.base.Object");

//see sap.ui.core.FocusHandler as example of handler template

(function() {
    // objects selected by mousedown and mouseup actions
    var oMouseDown, oMouseUp;
    //private activation count for debugging info
    var _activationCount = 0;

    //class logger
    var _logger;

    //return a reference to the class logger
    function getLogger() {
        return _logger || (_logger = sas.log.getLogger("sas.hc.ui.core.ContextMenuHandler"));
    }

    //given a DOM element, find the SAP control for that element or the nearest parent - if such a control exists
    function findControlForDom(element) {
        while (element !== null && element !== undefined) {
            if (!(element instanceof sap.ui.base.Object) && element.id !== null) {
                var oElement = sap.ui.getCore().byId(element.id);
                if (oElement !== null && oElement !== undefined) {
                    return oElement;
                }
            }

            //find the parent
            element = jQuery(element).parent().get(0);  //let jQuery find one
        }

        return null;
    }

    // determines if given control is child of menu (works for items, submenus, and items of submenus
    function isChildOfMenu(oControl, oMenu) {
        while (oControl) {
            if (oControl === oMenu) {
                return true;
            }
            oControl = oControl.getParent();
        }
        return false;
    }

    /**
     * Constructs an instance of the handler.
     * Watches for and reacts to context menu activation requests.
     * Ways to activate context menus:
     * - Right-Click of Mouse (Holding Ctrl while clicking will bypass this handler)
     * - Shift-F10 on keyboard
     * - Tap and Hold on touch devices
     *
     * @class Handles requests for context menu activation.
     * @param {DOMElement} oRootRef e.g. document.body
     */
    sap.ui.base.Object.extend("sas.hc.ui.core.ContextMenuHandler", {
        constructor : function(oRootRef) {
            sap.ui.base.Object.apply(this);

            //default to document.body if no root supplied
            oRootRef = oRootRef || document.documentElement || document.body;

            // always repress browser menu?
            this.bAlwaysOverrideBrowserMenu = true;

            //ref to last opened menu
            this.oLastMenu = null;

            // buffer the activation events for correct order
            this.aEventQueue = [];

            this.fEventHandler = jQuery.proxy(this.onEvent, this);
            this.fDestroyHandler = jQuery.proxy(this.destroy, this);

            // initialize event handling
            jQuery(oRootRef).bind("contextmenu", this.fEventHandler);
            jQuery(oRootRef).bind("keydown", this.fEventHandler);
            jQuery(oRootRef).bind("mousedown", this.fEventHandler);
            jQuery(oRootRef).bind("mouseup", this.fEventHandler);
            //TODO taphold is showing up on long held mousedown
            if (!jQuery.device.is.desktop) {
                jQuery(oRootRef).bind("taphold", this.fEventHandler);
                /* NOTES on buggy touch eventing:
                 - taphold listener is required for iPad to activate context menu - no native 'contextmenu' event is fired by long-touch on iPad
                 - during iPad emulation, contextmenu event is fired in response to mousedown but taphold is also generated by simulated long-touch
                 - emulation causes
                 */
            }

            this.oLogger = getLogger();
            this.oLogger.info("ContextMenuHandler setup on Root " + oRootRef.type + (oRootRef.id?": " + oRootRef.id:""));

            // TODO: Or should we be destroyed by the Core?
            jQuery(window).bind("unload", {"oRootRef": oRootRef}, this.fDestroyHandler);
        }
    });

    /**
     * Destroy method of the Handler.
     * It unregisters any associated event handlers.
     *
     * @param {jQuery.Event} event the event that initiated the destruction of the handler
     * @private
     */
    sas.hc.ui.core.ContextMenuHandler.prototype.destroy = function(event) {
        //unless the event gives us a specific element to work with, use our predetermined root element
        var oRootRef = (event && event.data && event.data.oRootRef) ? event.data.oRootRef : this.oRootRef;
        if(oRootRef){
            jQuery(oRootRef).unbind("contextmenu", this.fEventHandler);
            jQuery(oRootRef).unbind("keydown", this.fEventHandler);
            jQuery(oRootRef).unbind("mousedown", this.fEventHandler);
            jQuery(oRootRef).unbind("mouseup", this.fEventHandler);
            jQuery(oRootRef).unbind("taphold", this.fEventHandler);
        }
        jQuery(window).unbind("unload", this.fDestroyHandler);
    };

    /**
     * Watches events on given root and puts appropriate contextmenu events into an event queue for further processing.
     *
     * @param {object} oBrowserEvent the event to process and respond to
     * @private
     */
    sas.hc.ui.core.ContextMenuHandler.prototype.onEvent = function(oBrowserEvent){
        if (oBrowserEvent) {    //nothing to do if no event was given
            //fix events to normalize for convenience - is this necessary, though? could impact performance.
            var oEvent = jQuery.event.fix(oBrowserEvent),
                bSkipActivateContextMenu = false;
            if(oEvent.type === "mousedown") {
                oMouseDown = oBrowserEvent.target;
                oMouseUp = null;
            } else if(oEvent.type === "mouseup") {
                oMouseUp = oBrowserEvent.target;
            } else if(oEvent.type === "keydown") {
                // reset the mouse selection, it might also be an event fired by the Windows Menu key in the sequence: 'keydown', 'keyup', 'contextmenu'
                oMouseUp = null;
                oMouseDown = null;
            }

            if (oEvent.type === "keydown" && (oEvent.shiftKey === false || oEvent.keyCode !== jQuery.sap.KeyCodes.F10)) { //shift-F10 is only keydown event to pass
                return;
            }

            if(oEvent.type === "keydown" || oEvent.type === "contextmenu" || oEvent.type === "taphold") {
                this.oLogger.trace("Event "+oEvent.type+" reached ContextMenu Handler (target: "+oEvent.target+(oEvent.target ? oEvent.target.id : "")+")");
                this.aEventQueue.push(oEvent);
            }
            if(this.aEventQueue.length === 1) {
                // Do not activate the context menu if right click on a control and release mouse on a different control
                // For the "contextmenu" event triggered by the Windows Meny key we have oMouseDown=oMouseUp=null
                bSkipActivateContextMenu = oEvent.type === "contextmenu" && oMouseDown !== null && oMouseUp !== null && oMouseDown !== oMouseUp;
                this._processEvent(bSkipActivateContextMenu);
            }
        }
    };

    /**
     * Processes the context menu activation events in the event queue.
     *
     * @param {boolean} bSkipActivateContextMenu - true when mouse down and mouse up actions occur on different controls
     * @returns {boolean} TODO-jsdoc
     * @private
     */
    sas.hc.ui.core.ContextMenuHandler.prototype._processEvent = function(bSkipActivateContextMenu){
        var oEvent = this.aEventQueue[0];
        if(!oEvent) {
            return;
        }
        this.aEventQueue.shift();
        if(oEvent.type === "contextmenu" || oEvent.type === "taphold" || (oEvent.type === "keydown" && oEvent.shiftKey === true)){
            //control key overrides our handling of mouse and touch events
            if (oEvent.ctrlKey && oEvent.type !== "keydown") {
                this.oLogger.debug("Ignoring context menu activation due to control-key");
            } else {
                if(bSkipActivateContextMenu) {
                    if(typeof oEvent.preventDefault === "function") {
                        oEvent.preventDefault();
                    }
                    return false;
                }
                this.activateContextMenu(oEvent);
            }
        }
        if(this.aEventQueue.length > 0) {
            this._processEvent();
        }
    };

    /**
     * Handle an individual context menu activation event by  determining the control closest to the event that
     * has an associated context menu and calling its 'open' function.
     * @param {object} oEvent TODO-jsdoc
     * @returns {boolean} TODO-jsdoc
     * @private
     */
    sas.hc.ui.core.ContextMenuHandler.prototype.activateContextMenu = function(oEvent){
        var logger = getLogger();   //ref to class logger
        logger.debug("Body recognizes context menu activation #"+_activationCount++ + "! type="+oEvent.type);

        //get control that contains the event target
        var oTarget = findControlForDom(oEvent.target);

        //check of last opened menu
        if (this.oLastMenu !== null) {   //possible to get context menu activation on item of menu that is no longer consider active
            if (oTarget && isChildOfMenu(oTarget, this.oLastMenu)) {
                //right-clicked on menu or sub-menu
                logger.trace("Ignoring activation on child of menu.");
                if (typeof oEvent.preventDefault === "function") {
                    oEvent.preventDefault(); //prevent default
                }
                return false;
            }
        }

        //find the appropriate context menu to show based on target of event
        var oMenu = this._findContextMenu(oEvent, oTarget);
        //check if this menu is the one already up and open
        if (this.oLastMenu !== null && this.oLastMenu.isActive() && this.oLastMenu.bOpen && oMenu === this.oLastMenu) {
            //attempted to open exact same context menu that was already up and open
            logger.trace("Ignoring duplicate menu activation.");
            if (typeof oEvent.preventDefault === "function") {
                oEvent.preventDefault(); //prevent default
            }
            return false;
        }

        //close the last menu if it is still open and different from new menu
        if (this.oLastMenu !== null && this.oLastMenu.isActive() && oMenu !== this.oLastMenu) {
            if (typeof this.oLastMenu.close === "function") {
                //close the last menu if we know how
                this.oLastMenu.close();
            }
            this.oLastMenu = null;  //remove reference to last menu
        }

        //we may not find an associated menu
        if (oMenu instanceof sap.ui.unified.Menu) {
            this.oLastMenu = oMenu; //save off ref to menu

            //various 'open' parameters - do we need to tweak these?
            var eDock = sap.ui.core.Popup.Dock,
                openTargetRef = oEvent.target,
                moveFocusToFirstItem = true;    //always move focus for now

            // GUIDESINF-1643  Do not preselect or set focus to the first item for Shift+F10, Menu key and right click
            if (oEvent.type === "keydown" || oEvent.type === "contextmenu"/* || oEvent.type === "taphold" */) {
                moveFocusToFirstItem = false;
            }

            var offset = "none none",
                collision,
                $openTargetRef = $(openTargetRef),
                pos, iX, iY,
                bRTL = sap.ui.getCore().getConfiguration().getRTL(),
                iRtlOffset = 11;
            if (oEvent.type === 'keydown' || (oEvent.type === 'contextmenu' && oMouseDown === null && oMouseUp === null)) {
                // the event comes from Shift+F10 or Windows Menu key
                iX = bRTL ? $openTargetRef.outerWidth() - iRtlOffset : 0;
                iY = $openTargetRef.length > 0 ? $openTargetRef.outerHeight() : 0;
            } else {
                // the event comes from the right mouse button click or 'taphold'
                pos = $openTargetRef.offset();
                iX = bRTL ? (oEvent.pageX - pos.left - iRtlOffset) : (oEvent.pageX - pos.left);
                iY = (oEvent.pageY - pos.top);
            }
            offset = iX + " " + iY;
            collision = "fit";  //use fit instead of 'undefined' for better positioning around the lower and right edges of the window

            _handleTouchOuterEventAndClick(oMenu, oTarget);

            if (oMenu.getVisible()) {
                // call open on the menu - we know how because we know this is a sap.ui.unified.Menu instance
                if (bRTL) {
                    oMenu.open(moveFocusToFirstItem, oTarget.getFocusDomRef(), eDock.RightTop, eDock.LeftTop, openTargetRef, offset, collision);
                } else {
                    oMenu.open(moveFocusToFirstItem, oTarget.getFocusDomRef(), eDock.BeginTop, eDock.BeginTop, openTargetRef, offset, collision);
                }
            }

            this.aEventQueue = [];  //clear out the queue

            // if we handled this event, then prevent the default browser behavior
            if (typeof oEvent.preventDefault === "function") {
                oEvent.preventDefault();
            }
            return false;
        } else if (this.bAlwaysOverrideBrowserMenu === true) {   //if we choose to always block browser menu
            if (typeof oEvent.preventDefault === "function") {
                oEvent.preventDefault(); //prevent default
            }
            return false;   //return false - only needed for old browsers?
        }
    };

    //Used to suppress click event that comes along, this handler will prevent extra unwanted events fired from either jQuery itself,
    // SAP's controlling layer, or a native browser from prematurely closing our pop-up context menu.
    function nullClickHandlerFunction(oEvent) {
        if (oEvent.type !== "click"){
            return;
        }
        getLogger().trace("Ignoring event: ", arguments);
        if (oEvent) {
            if (typeof oEvent.stopImmediatePropagation === "function") {  //prevent other handlers on target
                oEvent.stopImmediatePropagation();
            }
            if (typeof oEvent.stopPropagation === "function") {   //prevent bubbling
                oEvent.stopPropagation();
            }
            if (typeof oEvent.preventDefault === "function") {    //prevent default
                oEvent.preventDefault();
            }
            if (typeof oEvent.cancelBubble === "function") {      //other ways to prevent bubbling?
                oEvent.cancelBubble();
            }
        }
        return false;
    }

    function _handleTouchOuterEventAndClick(oMenu, oTarget) {
        if (sap.ui.Device.system.desktop){
            return;
        }

        var fnOld_handleEvent  = oTarget._handleEvent,
            oMenuPopup = oMenu.getPopup(),
            fnRestore = function() {
                if (sap.ui.Device.browser.safari) {
                    oMenu.handleOuterEvent = fnOriginalHandleOuterEvent;
                }
                oTarget._handleEvent = fnOld_handleEvent;
                oMenuPopup.detachClosed(fnRestore, oTarget);
            };

        // just in case they have not been reset back to normal.
        oMenuPopup.attachClosed(fnRestore, oTarget);

        if (sap.ui.Device.browser.safari){
            // prevent the menu to close prematurely and ignore the click event(s)
            var fnOriginalHandleOuterEvent = sas.hc.ui.unified.Menu.prototype.handleOuterEvent,
                bPreventClosing = true,
                fnHandleOuterEvent = function (oMenuId, oEvent) {
                    // If bPreventClosing, prevent both undesired mousedown & sapfocusleave,
                    // which calls the handleOuterEvent to prematurely close the menu.
                    // sapfocusleave is the last undesired event
                    if (oEvent.type === "sapfocusleave" && bPreventClosing) {
                        // revert to the original function
                        oMenu.handleOuterEvent = fnOriginalHandleOuterEvent;
                        // revert to the original events
                        oTarget._handleEvent = fnOld_handleEvent;
                        bPreventClosing = false;
                    }
                };

            // Check whether to suspend closing or not.
            // During suspending, the original handleOuterEvent, which closes
            // the menu, will not be called.
            oMenu.handleOuterEvent = fnHandleOuterEvent;

            // Cancel subsequent click events
            oTarget._handleEvent = nullClickHandlerFunction;

        } else {
            // only cancel the click event
            var fnHandleClick = function (oEvent) {
                nullClickHandlerFunction(oEvent);

                if (oEvent.type === "click") {
                    // revert to the original events
                    oTarget._handleEvent = fnOld_handleEvent;
                }

            };
            oTarget._handleEvent = fnHandleClick;
        }
    }

    /**
     * Finds the control closest to the point of interaction that has an associated context menu.
     *
     * @param {jQuery.Event} oEvent the event that initiated the destruction of the handler
     * @param {Object} $trg a DOMElement or SAP Control
     * @returns {object} TODO-jsdoc
     * @private
     */
    sas.hc.ui.core.ContextMenuHandler.prototype._findContextMenu = function (oEvent, $trg) {
        var $menu = null;
        // look at getControlIdForDOM(event.target) to optimize finding sap control from target
        while (!!$trg && !$menu) {
            if (!($trg instanceof sap.ui.base.Object) && $trg.id !== null) {
                var oElement = sap.ui.getCore().byId($trg.id);
                if (!!oElement) {
                    $trg = oElement;
                }
            }

            if ($trg instanceof sap.ui.base.ManagedObject) {
                $menu = $trg.getAggregation("contextMenu");
                if ($menu) {
                    //we have component $trg and menu $menu now!
                    //  we should allow component to customize the menu here
                    if (typeof $trg.customizeContextMenu === "function") {
                        $menu = $trg.customizeContextMenu($menu, oEvent);   //target may customize or replace menu
                        if (!!$menu && $menu.getItems().length === 0) {
                            $menu = null;   //if customize did not supply any items, then ignore this menu
                        }

                        if (!!$menu) {   //if we have a non-trivial menu (after customization), then we're done!
                            return $menu;   //return it
                        }
                    }
                }

                if (!$menu) {
                    $trg = $trg.getParent();
                }
            } else if (!!$trg) {
                if (!!$trg.parentNode) {
                    $trg = $trg.parentNode;
                } else {
                    this.oLogger.debug ("Unable to walk parent tree for event target.");
                    return null;
                }
            }
        }
        return $menu;
    };

})();

jQuery.sap.declare("sas.hc.ui.core.ContextMenuTask");
jQuery.sap.require("sas.hc.ui.core.BootTask");
jQuery.sap.require("sas.hc.ui.core.ContextMenuHandler");

(function() {

    "use strict";

    sas.hc.ui.core.BootTask.extend("sas.hc.ui.core.ContextMenuTask", {
        name: 'sas.hc.ui.core.ContextMenuTask'
    });

    sas.hc.ui.core.ContextMenuTask.prototype.run = function() {
        //Create the context menu handler
        new sas.hc.ui.core.ContextMenuHandler(document.documentElement);
    };
}());

jQuery.sap.declare("sas.hc.ui.core.ClearFocusRingTask");
jQuery.sap.require("sas.hc.ui.core.BootTask");

(function() {

    "use strict";

    var sClearFocusRingStyleName = "clearFocusRing";
    var sKeyEvent= "keydown";
    var sPointerEvent = "mousedown";
    if (window.PointerEvent){
        sPointerEvent = "pointerdown";
    }

    sas.hc.ui.core.BootTask.extend("sas.hc.ui.core.ClearFocusRingTask", {
        name: 'sas.hc.ui.core.ClearFocusRingTask'
    });

    sas.hc.ui.core.ClearFocusRingTask.prototype.run = function() {
        var fnAddClearFocusRingClass = function (event){
            var oBody = $("body");
            oBody.addClass(sClearFocusRingStyleName);
            document.body.removeEventListener(sPointerEvent, fnAddClearFocusRingClass, true);
            document.body.addEventListener(sKeyEvent, fnRemoveClearFocusRingClass, true);
        };

        var fnRemoveClearFocusRingClass = function (event){
            //if keyCode is Shift, Control or Alt
            if (event.keyCode === jQuery.sap.KeyCodes.SHIFT || event.keyCode === jQuery.sap.KeyCodes.CONTROL || event.keyCode === jQuery.sap.KeyCodes.ALT){
                return;
            }

            var oBody = $("body");
            oBody.removeClass(sClearFocusRingStyleName);
            document.body.removeEventListener(sKeyEvent, fnRemoveClearFocusRingClass, true);
            document.body.addEventListener(sPointerEvent, fnAddClearFocusRingClass, true);
        };

        document.body.addEventListener(sPointerEvent, fnAddClearFocusRingClass, true);
    };
}());

/*
 * Default Keep alive service in WIP world.
 *
 */
jQuery.sap.declare("sas.hc.ui.core.util.WIPKeepAliveService");

(function(){
    "use strict";

    sap.ui.base.Object.extend("sas.hc.ui.core.util.WIPKeepAliveService",{
        metadata:{
            publicMethods:["ping"]
        }
    });

    /**
     * Pings server to keep the user session alive.
     */
    sas.hc.ui.core.util.WIPKeepAliveService.prototype.ping = function(successHandler,errorHandler){
        // ping server to keep session live.
        var hcReg = sas.registry;
        if(hcReg && typeof hcReg.getCommonsSessionKeepAliveService === 'function'){
            var pingService = hcReg.getCommonsSessionKeepAliveService();
            if(pingService){
                pingService.post().done(function(){
                    sas.log.trace('sas.hc.ui.core.util','Successfully invoked session keep alive service.');
                    if(typeof successHandler === 'function'){
                        successHandler.call(arguments);
                    }
                }).fail(function(jqXHR){
                    if(jqXHR.status !== 429){// too many requests may happen in web apps if app switcher is on.
                        sas.log.warning('sas.hc.ui.core.util','Could not invoke session keep alive service.');
                    }
                    if(typeof errorHandler === 'function'){
                        errorHandler.call(arguments);
                    }
                });
            }
        }
    };

})();
/*
 * Default Keep alive service in SAS 10 (Petrichor) world.
 *
 */
jQuery.sap.declare("sas.hc.ui.core.util.PetKeepAliveService");

(function(){
    "use strict";

    sap.ui.base.Object.extend("sas.hc.ui.core.util.PetKeepAliveService",{
        metadata:{
            publicMethods:["ping"]
        }
    });

    /**
     * Pings server to keep the user session alive.
     */
    sas.hc.ui.core.util.PetKeepAliveService.prototype.ping = function(successHandler,errorHandler){
        // ping server to keep session live.
        var contextPath = (sas && sas.config && sas.config.getContextPath()) || "";
        if(contextPath !== "" && contextPath.charAt(contextPath.length-1) !== '/'){
            contextPath += "/";
        }
        sas.ajax({url:contextPath+"keepalive"}).done(function(){
            sas.log.trace('sas.hc.ui.core.util.PetKeepAliveService','Successfully invoked session keep alive service.');
            if(typeof successHandler === 'function'){
                successHandler.call(arguments);
            }
        }).fail(function(jqXHR){
            sas.log.warning('sas.hc.ui.core.util.PetKeepAliveService','Could not invoke session keep alive service.');
            if(typeof errorHandler === 'function'){
                errorHandler.call(arguments);
            }
        });
    };

})();
jQuery.sap.declare("sas.hc.ui.core.TimeoutHandlerTask");

jQuery.sap.require("sas.hc.ui.core.BootTask");
jQuery.sap.require("sas.hc.ui.core.util");

(function(){
    sas.hc.ui.core.BootTask.extend('sas.hc.ui.core.TimeoutHandlerTask',{
        name: 'sas.hc.ui.core.TimeoutHandlerTask',
        dependsOn:['sas.hc.ui.core.RegistryTask'],

        constructor: function(){

            this.inactiveInterval= sas.config.getSessionMaxInActiveInterval();
            // if its undefined or negative or not a number then disable session time out.
            if(isNaN(this.inactiveInterval)
                || this.inactiveInterval < 0){
                // timeout feature not possible.
                this.disabled = true;
                return;
            }

            this.warningDuration=sas.config.getSessionTimeoutWarningDuration();
            // don't entertain negative values.
            this.warningDuration = this.warningDuration > 0 ? this.warningDuration : 0;
            this.appReady = false;

            this.actualInterval = this.inactiveInterval - this.warningDuration;
            // reset warning duration if inactive interval is less than warning duration.
            if(this.inactiveInterval < this.warningDuration){
                this.warningDuration = this.inactiveInterval;
            }
        }
    });
    sas.hc.ui.core.TimeoutHandlerTask.prototype.run=function(){
        if(this.disabled) return;
        var that = this;
        Object.defineProperty(sas,'timeoutManager',{
            enumerable:true,
            value:Object.freeze({
                events:Object.freeze({
                    TIMED_OUT:'hc.sessionTimedOut',
                    USER_ACTIVE:'hc.userSessionActive',
                    KEEP_ALIVE:'hc.keepSessionAlive',
                    BEFORE_TIME_OUT:'hc.beforeSessionTimeOut',
                }),
                start:function(silentStart){
                    that.start(silentStart);
                },
                quiet:function(){
                    that.quiet();
                },
                awake:function(){
                    that.awake();
                },
                keepAlive:function(bVal){
                    that.keepAlive(bVal);
                }
            })
        });
        sas.eventBus.subscribe(sas.timeoutManager.events.KEEP_ALIVE,function(){
            sas.hc.ui.core.util.pingServer();
            that.recordUserActivity();
        });

        sas.eventBus.subscribe(sas.timeoutManager.events.USER_ACTIVE,function(){
            that.recordUserActivity();
        });

        // make sure if app switcher is enabled that it is not the outer app switcher when starting timeout
        var isMainApp = window===top ? true : false;
        if ((sas.config.getApplicationSwitcherEnabled()===true && isMainApp===false) || sas.config.getApplicationSwitcherEnabled()===false) {
            sas.hc.ui.core.util.startTimeoutManager();
        }

        var callback = function(){
            that.appReady = true;
            sas.timeoutManager.start();
            sas.eventBus.unsubscribe(sas.hc.ui.core.util.APP_READY, callback);
        };

        sas.eventBus.subscribe(sas.hc.ui.core.util.APP_READY, callback);
    };

    sas.hc.ui.core.TimeoutHandlerTask.prototype.pause=function(){
        sas.log.trace('sas.hc.ui.core.TimeoutHandlerTask',"#pause", "Timeout manager paused.");
        clearInterval(this.sessionTimeoutRef);
        this.sessionTimeoutRef = undefined;
        this._paused = true;
    };
    /**
     * Resume user activity and timeout dialog from where it was paused.
     *
     */
    sas.hc.ui.core.TimeoutHandlerTask.prototype.resume=function(){
        if(typeof this.sessionTimeoutRef === 'undefined'){
            var that=this;
            this.sessionTimeoutRef = setInterval(function(){
                that.checkForTimeout();
            },
            this.actualInterval * 1000 /* convert into mili sec*/);
            sas.log.trace('sas.hc.ui.core.TimeoutHandlerTask',"#resume", "Timeout manager resumed with interval",this.actualInterval);

        }
        this._paused = false;
    };
    sas.hc.ui.core.TimeoutHandlerTask.prototype.isPaused=function(){
        return this._paused;
    };
    sas.hc.ui.core.TimeoutHandlerTask.prototype.recordUserActivity=function(){
        if(this.isPaused() && !this._keepAlive){
            return;
        }
        this.lastActTime = Math.floor(Date.now()/1000); // in seconds
        sas.log.trace("sas.hc.ui.core.TimeoutHandlerTask","#recordUserActivity", "User activity recorded.");
    };
    sas.hc.ui.core.TimeoutHandlerTask.prototype.logoff=function(){
        // log off.
        sas.hc.ui.core.util.signout(); //TODO should do it over an event?.
    };

    sas.hc.ui.core.TimeoutHandlerTask.prototype.timeout=function(){
        // before log off.
        sas.eventBus.publish(sas.timeoutManager.events.BEFORE_TIME_OUT);
        // log off.
        // publish an event on the event bus that session is timed out.
        sas.eventBus.publish(sas.timeoutManager.events.TIMED_OUT);
    };


    sas.hc.ui.core.TimeoutHandlerTask.prototype.ping=function(isSilent){
        sas.log.trace('sas.hc.ui.core.TimeoutHandlerTask',"#ping", "Ping server.");
        sas.hc.ui.core.util.pingServer();
        if(isSilent !== true){
            sas.eventBus.publish(sas.timeoutManager.events.USER_ACTIVE);
        }
    };

    sas.hc.ui.core.TimeoutHandlerTask.prototype.start=function(silentStart){
        if(!!this.started) return;

        if (silentStart === undefined) {
            silentStart = false;
        }

        var that = this;
        // ping server to touch session(s) before starting timeout manager.
        this.ping(true); //silent ping.
        if (silentStart === false) {
            var onUserActivity = function() {
                sas.log.trace("sas.hc.ui.core.TimeoutHandlerTask#onUserActivity", "User activity detected.", arguments);
                that.recordUserActivity();
            };

            // register listener for user activity
            if (!!window.addEventListener) {
                // list of events borrowed from jQuery.sap.act. Can't use it as sap decided to fire the activity handler
                // only after 10000 milliseconds of inactivity which is not helping suppress session timeout dialog
                // when user action continue to happen.
                var aEvents = ["resize", "orientationchange", "mousemove", "mousedown", "mouseup", //"mouseout", "mouseover",
                    "touchstart", "touchmove", "touchend", "touchcancel", "paste", "cut", "keydown", "keyup",
                    "DOMMouseScroll", "mousewheel"];
                for (var i = 0; i < aEvents.length; i++) {
                    window.addEventListener(aEvents[i], onUserActivity, true);
                }
                // For now not listen for mutation observers as we want to listen for user activity.
            }
            var supressAjaxCompleteHandler = false;
            sas.eventBus.subscribe(sas.hc.ui.core.util.BEFORE_SIGN_OUT,function(){
                supressAjaxCompleteHandler = true;
            });

            // register ajaxComplete event handler to intercept ajax calls to determine
            // if auth session has expired. Timeout if so. This case arise
            // when say cookies are deleted after signing in to app.... or machine is locked or put to sleep
            // for longer time and application is accessed after bringing it up.
            jQuery(document).ajaxComplete(function(evt,jqXHR,options){
                if(supressAjaxCompleteHandler === true) return;
                try{
                    if(jqXHR && jqXHR.status >= 300 && jqXHR.getResponseHeader('X-Unauthorized-Reason') === 'Authentication'){
                        that.logoff();
                    }
                }catch(secError){ // might be thrown if the response is from cross domain endpoint.
                    // ignore.
                    sas.log.trace(secError);
                }
            });
            // every time window receives focus, check for timeout
            window.addEventListener('focus',that.checkForTimeout.bind(that),false);
            that.recordUserActivity();
            this.started=true;
        }
        this.resume();
        sas.log.trace('sas.hc.ui.core.TimeoutHandlerTask',"#start","Timeout manager started.");
    };

    sas.hc.ui.core.TimeoutHandlerTask.prototype.checkForTimeout=function(){
        if(this.isPaused() && !this._keepAlive){
            // session dialog is displayed.
            return;
        }
        if(this.lastActTime){
            // get current time
            var curTime = Math.floor(Date.now()/1000); // in seconds
            var timeLapsed = curTime - this.lastActTime;
            sas.log.trace('sas.hc.ui.core.TimeoutHandlerTask',"#checkForTimeout","time lapsed:"+timeLapsed+",interval:"+this.actualInterval);
            if(timeLapsed >= this.inactiveInterval){
                // if time lapsed is greated than max inactive interval then no point is showing timeout dialog.
                // show timeout page.
                this.timeout();
            }else if( timeLapsed >= this.actualInterval){
                this.sessionTimedOut();
            }else{
                // user is active. ping the server and continue
                this.ping();
            }
        }else{
            this.sessionTimedOut();
        }
    };
    sas.hc.ui.core.TimeoutHandlerTask.prototype.sessionTimedOut=function(){
        sas.log.trace('sas.hc.ui.core.TimeoutHandlerTask',"#sessionTimedOut","session timed out");
        var that = this;
        if(this._keepAlive){
            this.ping(true);
            return;
        }
        if(this.warningDuration > 0 && this.appReady === true) {
            this.pause();
            jQuery.sap.require('sas.hc.ui.commons.SessionTimeoutDialog');
            sas.hc.ui.commons.SessionTimeoutDialog.show(this.warningDuration,function(userAction){
                if(sas.hc.ui.commons.SessionTimeoutDialog.Action.CONTINUE === userAction){
                    that.resume();
                    that.recordUserActivity();
                    that.ping();
                }else if(sas.hc.ui.commons.SessionTimeoutDialog.Action.LOGOFF === userAction){
                    that.logoff();
                }else{
                    that.timeout();
                }
            });
        } else {
            that.timeout();
        }
    };

    /**
     * If timeout manager has started then STOP user activity monitoring and KEEP publishing sas.timeoutManager.events.KEEP_ALIVE event on sas eventbus periodically
     * to help keep user session alive.
     */
    sas.hc.ui.core.TimeoutHandlerTask.prototype.quiet=function(){
        if(!this.started) return;
        sas.log.trace('sas.hc.ui.core.TimeoutHandlerTask',"#quiet","Timeout manager quite.");
        this.pause();
        if(typeof this.keeAliveTimerRef === 'undefined'){
            this.keeAliveTimerRef = setInterval(function(){
                sas.eventBus.publish(sas.timeoutManager.events.KEEP_ALIVE);
            },
            this.actualInterval * 1000 /* convert into mili sec*/);
        }
    };

    /**
     * If timeout manager has started then STOP firing sas.timeoutManager.events.KEEP_ALIVE event on sas eventbus and RESUME user activity monitoring.
     */
    sas.hc.ui.core.TimeoutHandlerTask.prototype.awake=function(){
        if(!this.started) return;
        sas.log.trace('sas.hc.ui.core.TimeoutHandlerTask',"#awake","Timeout manager wokeup.");
        clearInterval(this.keeAliveTimerRef);
        this.keeAliveTimerRef = undefined;
        this.resume();
        this.recordUserActivity();
    };

    /**
     * If timeout manager has started then suppress session timeout and KEEP user session ALIVE.
     * @param {boolean} bVal True to keep user session alive. False otherwise.
     */
    sas.hc.ui.core.TimeoutHandlerTask.prototype.keepAlive=function(bVal){
        if(!this.started) return;
        sas.log.trace('sas.hc.ui.core.TimeoutHandlerTask',"#keepAlive","session keep alive.");
        if(bVal===false){
            this._keepAlive = false;
        }else{
            this._keepAlive = true;
        }
    };


}());

jQuery.sap.declare("sas.hc.ui.core.theme.ThemeBootTask");

jQuery.sap.require("sas.hc.ui.core.BootTask");
jQuery.sap.require("sas.hc.ui.core.theme.ThemeDescriptor");
jQuery.sap.require("sas.hc.ui.core.theme.ThemeUtil");

(function(){

    sas.hc.ui.core.BootTask.extend('sas.hc.ui.core.theme.ThemeBootTask', {
        name: 'sas.hc.ui.core.theme.ThemeBootTask',
        dependsOn:['sas.hc.ui.core.RegistryTask'],
        constructor: function(){}
    });

    sas.hc.ui.core.theme.ThemeBootTask.prototype.runAsync = function() {

        return new Promise(function(resolve,reject){

            sas.hc.ui.core.theme.ThemeDescriptor.loadDescriptorAsync().then(function(obj){
                sas.themeDescriptor = obj;
                resolve();
            });

            // set locale on html tag for use by CSS
            var language = sap.ui.getCore().getConfiguration().getLanguage();
            document.querySelector("html").setAttribute("lang", language);

            // Create a ThemeUtil instance and add it to the sas namespace
            sas.themes = new sas.hc.ui.core.theme.ThemeUtil();

            // Create wrappers for theme parameter CSS classes.
            // See https://openui5.hana.ondemand.com/docs/guide/CSSClassesForThemeParameters.html
            sas.styles = {
                Font: "sapThemeFont",
                FontFamily: "sapThemeFontFamily",
                FontSize: "sapThemeFontSize",
                Text: "sapThemeText",
                TextAsColor: "sapThemeText-asColor",
                TextAsBackgroundColor: "sapThemeText-asBackgroundColor",
                TextAsBorderColor: "sapThemeText-asBorderColor",
                TextInverted: "sapThemeTextInverted",
                TextInvertedAsColor: "sapThemeTextInverted-asColor",
                BaseBG: "sapThemeBaseBG",
                BaseBGAsBackgroundColor: "sapBaseBG-asBackgroundColor",
                BaseBGAsBorderColor: "sapThemeBaseBG-asBorderColor",
                BaseBGAsColor: "sapThemeBaseBG-asColor",
                BrandAsColor: "sapThemeBrand-asColor",
                BrandAsBorderColor: "sapThemeBrand-asBorderColor",
                BrandAsBackgroundColor: "sapThemeBrand-asBackgroundColor",
                HighlightAsColor: "sapThemeHighlight-asColor",
                HighlightAsBorderColor: "sapThemeHighlight-asBorderColor",
                HighlightAsBackgroundColor: "sapThemeHighlight-asBackgroundColor",
                PageBG: "sapThemePageBG",
                PageBGAsColor: "sapThemePageBG-asColor",
                BarBG: "sapThemeBarBG",
                BarHeading: "sapThemeBarHeading",
                BarText: "sapThemeBarText",
                NegativeText: "sapThemeNegativeText",
                CriticalText: "sapThemeCriticalText",
                PositiveText: "sapThemePositiveText",
                LightText: "sapThemeLightText",
                MediumText: "sapThemeMediumText",
                DarkText: "sapThemeDarkText"
            };
        });
    };
}());

jQuery.sap.declare("sas.hc.ui.core.RegistryTask");

jQuery.sap.require("sas.hc.ui.core.BootTask");
jQuery.sap.require("sas.hc.ui.core.util");

(function(){

    function canReuseWipRegistry(){
        try{
            if(sas.config.getApplicationSwitcherEnabled()
                && (window !== window.top)
                && top.sas
                && top.sas.registry
                && top.sas.registry.wip
                && top.sas.config
                && top.sas.config.getWipRegistryUrl()
                && top.sas.config.getWipRegistryUrl() === sas.config.getWipRegistryUrl()){
                return sas.hc.ui.core.util._hasSameVersionAsParent();
            }
        }catch(err){
            // throws an error when apps from cross origin are loaded.
            // suppress the error.
        }
        return Promise.reject();
    };

    var RegistryTask = sas.hc.ui.core.BootTask.extend('sas.hc.ui.core.RegistryTask',{
        name: 'sas.hc.ui.core.RegistryTask'
    });

    RegistryTask.prototype.runAsync=function(){

        var sasConf = sas.config,
            sasReg,sasWipReg,
            allPromises = [],
            wipRegUrl = sasConf.getWipRegistryUrl(),
            wipRegReused,
            retPromise;

        // commons registry load task.
        if(sasConf.getCommonsRegistryUrl()){
            sasReg = new sas.hc.ui.services.ServiceRegistry({url:sasConf.getCommonsRegistryUrl(),
                crossDomain:false,
                asyncInit:true});
            allPromises.push(sasReg.initAsync());
        }

        // wip registry load task.
        if(wipRegUrl){
            // wip registry is a special case where authentication is needed when its not a cross doamin call. :-(
            // hence use proxied url for the registry if registry url is NOT cross domain.
            if(!sas.hc.ui.core.util.isUrlCrossDomain(wipRegUrl)){
                var q = {};
                q['sas-commons-auth'] = true;
                q['sas-commons-destUrl'] = wipRegUrl;
                wipRegUrl = sasConf.getContextPath()+"/"+sasConf.getCrossDomainProxy()+"?"+jQuery.param(q);
            }
            allPromises.push(canReuseWipRegistry().then(function(){
                sasWipReg = top.sas.registry.wip;
                wipRegReused = true;
            },function(){
                sasWipReg = new sas.hc.ui.services.ServiceRegistry({url:wipRegUrl,
                    authenticate:true,
                    asyncInit:true});
                return sasWipReg.initAsync();
            }));
        }

        if(allPromises.length > 0){
            retPromise = Promise.all(allPromises);
            retPromise.then(function(){

                if(sasReg){
                    // instantiate the registry.
                    Object.defineProperty(sas,'registry',{
                        enumerable:true,
                        value:sas.hc.ui.core.util.immutate(sasReg)
                    });
                }

                if(sasWipReg){
                    // instantiate the registry.
                    sas.registry = sas.registry || {};

                    Object.defineProperty(sas.registry,'wip',{
                        enumerable:true,
                        value:wipRegReused ? sasWipReg : sas.hc.ui.core.util.immutate(sasWipReg)
                    });
                }
            },
            function(){
                sas.log.error("RegistryTask","Error creating sas registries.");
            });
            return retPromise;
        }else{
            return Promise.resolve();
        }
    };
}());


jQuery.sap.declare("sas.hc.ui.core.VersionInfoTask");

(function(){
    // Copied from sap/ui/Global.js
    /**
     * Stores the loading Promise for "sap-ui-version.json".
     * @see sap.ui.getVersionInfo
     * @private
     */
    var oVersionInfoPromise = null;

    sap.ui.base.Object.extend("sas.hc.ui.core.VersionInfoTask",{
        metadata:{
            publicMethods:["getVersionInfo"]
        }
    });

    // Copied from sap/ui/Global.js
    sas.hc.ui.core.VersionInfoTask.prototype.getVersionInfo = function(mOptions) {

        // Check for no parameter / library name as string
        if (typeof mOptions !== "object") {
            mOptions = {
                library: mOptions
            };
        }

        // Cast "async" to boolean (defaults to false)
        mOptions.async = mOptions.async === true;

        // Cast "failOnError" to boolean (defaults to true)
        mOptions.failOnError = mOptions.failOnError !== false;

        if (!sap.ui.versioninfo) {
            // Load and cache the versioninfo

            // When async is enabled and the file is currently being loaded
            // return the promise and make sure the requested options are passed.
            // This is to prevent returning the full object as requested in a
            // first call (which created this promise) to the one requested just a
            // single lib in a second call (which re-uses this same promise) or vice versa.
            if (mOptions.async && oVersionInfoPromise instanceof Promise) {
                return oVersionInfoPromise.then(function() {
                    return sap.ui.getVersionInfo(mOptions);
                });
            }

            var fnHandleSuccess = function(combinedLibraries) {
                // Remove the stored Promise as the versioninfo is now cached.
                // This allows reloading the file by clearing "sap.ui.versioninfo"
                // (however this is not documented and therefore not supported).
                oVersionInfoPromise = null;

                // "jQuery.sap.loadResource" returns "null" in case of an error when
                // "failOnError" is set to "false". In this case the won't be persisted
                // and undefined will be returned.
                if (combinedLibraries === null) {
                    return undefined;
                }

                // Persist the info object to return it in subsequent calls
                oBaseVersionInfo.libraries = combinedLibraries;
                sap.ui.versioninfo = oBaseVersionInfo;


                // Calling the function again with the same arguments will return the
                // cached value from "sap.ui.versioninfo".
                return sap.ui.getVersionInfo(mOptions);
            };

            var fnHandleError = function(oError) {
                // Remove the stored Promise as the versioninfo couldn't be loaded
                // and should be requested again the next time.
                oVersionInfoPromise = null;

                // Re-throw the error to give it to the user
                throw oError;
            };

            // SAS Edit Start
            // Handle the possibility of an externalized commons.  Need to check each library to
            // see if it comes from the container or not.
            var oBaseVersionInfo;

            var processVersionInfo = function(versionInfo, resourceRoot, baseFile) {
                var libraries = versionInfo.libraries;

                if (baseFile) {
                    oBaseVersionInfo = versionInfo;
                }

                for (var i = 0, len = libraries.length; i < len; i++) {
                    // Check to see which copy of the library is used.  If it's this library, add it to the combined list.
                    if (jQuery.sap.getModulePath(libraries[i].name).indexOf(resourceRoot) === 0) {
                        combinedLibraries.push(libraries[i]);
                    }
                }
            };

            var loadVersionFile = function(fileName, resourceRoot, baseFile) {
                var vReturn = jQuery.sap.loadResource(fileName, {
                    async: mOptions.async,

                    // "failOnError" only applies for sync mode, async should always fail (reject)
                    failOnError: mOptions.async || mOptions.failOnError,
                    url: fileName
                });

                if (vReturn instanceof Promise) {
                    loadPromises.push(vReturn.then(function(versionInfo) {
                        processVersionInfo(versionInfo, resourceRoot, baseFile);
                    }, fnHandleError));
                } else {
                    processVersionInfo(vReturn, resourceRoot, baseFile);
                }
            };

            var combinedLibraries = [];
            var loadPromises = [];
            loadVersionFile(window["sap-ui-config"].resourceroots[""] + "/sap-ui-version.json",
                jQuery.sap.getModulePath(""), true);

            // Make sure a container environment is present first.
            if (window["sap-ui-config"] && window["sap-ui-config"].sas &&
                (typeof window["sap-ui-config"].sas.commonsUrl === "string") && (window["sap-ui-config"].sas.commonsUrl !== "")) {
                var uniqueResourceRoots = {};

                // Determine all possible sap-ui-version.json file locations by looking at unique resource roots.
                for (var rootId in window["sap-ui-config"].resourceroots) {
                    if (window["sap-ui-config"].resourceroots.hasOwnProperty(rootId)) {
                        if (rootId !== "") { // discard the application's resource registration (already loaded)
                            var root = window["sap-ui-config"].resourceroots[rootId];
                            var rootUrl = root;
                            if ((typeof root === 'object') && (root !== null) && (root.url)) {
                                rootUrl = root.url;
                            }
                            if (typeof rootUrl === 'string') {
                                if (rootUrl.indexOf("/resources") !== -1) {
                                    var candidate = rootUrl.substring(0, rootUrl.indexOf('/resources') + "/resources".length);
                                    uniqueResourceRoots[candidate] = {versionFile: candidate + "/sap-ui-version.json"};
                                }
                            }
                        }
                    }
                }

                // Try loading sap-ui-version.json from each of the unique resource roots.
                for (var uniqueResourceRoot in uniqueResourceRoots) {
                    if (uniqueResourceRoots.hasOwnProperty(uniqueResourceRoot)) {
                        var versionFileUrl = uniqueResourceRoots[uniqueResourceRoot].versionFile;
                        loadVersionFile(versionFileUrl, uniqueResourceRoot);
                    }
                }
            }

            // if async (promise available?)
            if (mOptions.async) {
                oVersionInfoPromise = Promise.all(loadPromises).then(function() {
                    return fnHandleSuccess(combinedLibraries);
                });
                return oVersionInfoPromise;
            } else {
                return fnHandleSuccess(combinedLibraries);
            }
            // SAS Edit End
        } else {
            // Return the cached versioninfo

            var oResult;
            if (typeof mOptions.library !== "undefined") {
                // Find the version of the individual library
                var aLibs = sap.ui.versioninfo.libraries;
                if (aLibs) {
                    for (var i = 0, l = aLibs.length; i < l; i++) {
                        if (aLibs[i].name === mOptions.library) {
                            oResult = aLibs[i];
                            break;
                        }
                    }
                }
            } else {
                // Return the full version info
                oResult = sap.ui.versioninfo;
            }

            return mOptions.async ? Promise.resolve(oResult) : oResult;
        }
    };

}());
jQuery.sap.declare("sas.hc.ui.core.cache.UserPreferencesCache");

sap.ui.define('sas/hc/ui/core/cache/UserPreferencesCache',
    ['jquery.sap.global', 'sas/hc/ui/core/util','sas/hc/ui/core/util/ObjectProxy','sas/hc/ui/core/util/UserInfo', 'sap/ui/thirdparty/URI'],
    function(jQuery, util, ObjectProxy, userInfo, URI) {
        "use strict";

        var UserPreferencesCache,
            URL_REGEX_UID_PART = '@currentUser',
            URL_REGEX_END_PART = '($|(/|\\?).*)',
            SERVICE_ENDPT = '/preferences/preferences/',
            urlRegEx = new RegExp('^'+SERVICE_ENDPT+URL_REGEX_UID_PART+URL_REGEX_END_PART),
            SERVICE_URL=SERVICE_ENDPT+'@currentUser',
            DEFAULT_LIMIT = 20,
            MAX_LIMIT = (Math.pow(2,31) -1), //max java integer
            ERRONEOUS_PREF_ID = '_hc_xx_',
            FILTER_REGEX_APP_EQ_FIRST_PART = /^eq\s*\(\s*(\"|\')?application(\"|\')?\s*,/,
            FILTER_REGEX_APP_EQ            = /^eq\s*\(\s*(\"|\')?application(\"|\')?\s*,.*\)$/,
            FILTER_REGEX_ID_IN_FIRST_PART  = /^in\s*\(\s*(\"|\')?id(\"|\')?\s*,/,
            FILTER_REGEX_ID_IN             = /^in\s*\(\s*(\"|\')?id(\"|\')?\s*,.*\)$/,
            UNSUPPORTED_FILTERFNS_REGEX    = /.*(and|or|not)\(.*/,
            CACHE_MUTABLE = jQuery.sap.getUriParameters().get('xx-cachemanager-immutable') === "false"
                            || window["sap-ui-config"] && window["sap-ui-config"].sas && window["sap-ui-config"].sas['xx-cachemanager-immutable'] === false,
            doProcess,
            collatorComFn,
            collatorComFnPromise,
            MAX_ATTEMPTS_CONFIGURE_UID = 5;

        function appendQuery(_uri,_start,_limit){
            var query = _uri.indexOf("?") === -1 ? "?":"&",
                appendingPref = "";
            if(typeof _start !== 'undefined') {appendingPref="&"; query+="start="+_start;};
            if(typeof _limit !== 'undefined') {query +=appendingPref+"limit="+_limit;};
            return _uri+query;
        };

        function deepFreeze(obj){
            if(typeof obj === 'object' && !Object.isFrozen(obj)){
                return util.deepFreeze(obj);
            }
            return obj;
        }

        function deepCopy(obj){
            if(Array.isArray(obj)){
                return jQuery.extend(true,[],obj);
            }
            if(typeof obj === 'object'){
                return jQuery.extend(true,{},obj);
            }
            return obj;
        }

        doProcess = CACHE_MUTABLE ? deepCopy : deepFreeze;


        /**
         * Object proxy handler for preferences cached data object.
         * The data object is used as first (data) argument of the jQuery.ajax done handler.
         *
         * Adds pagination support on data based on input.
         * @private
         */
        function PaginationDataProxyHandler(config){
            this.start = (config.start && Array.isArray(config.start) ? parseInt(config.start[0]) : parseInt(config.start)) || 0;
            this.limit = (config.limit && Array.isArray(config.limit) ? parseInt(config.limit[0]) : parseInt(config.limit)) || DEFAULT_LIMIT;
        };
        /**
         * Getter trap for the target.
         */
        PaginationDataProxyHandler.prototype.get = function(target,property,receiver){
            if(this.hasOwnProperty(property)){
                return this[property];
            }

            if(property in target){
                switch(property){
                    case "items":{
                        if(!this.items){
                            this.items = doProcess(target.items.slice(this.start,Math.min((this.start+this.limit),target.count)));
                        }
                        return this.items;
                    };
                    case "links":{
                        if(!this.links){
                            var _uri,
                                linksCp = target.links.slice(),
                                selfData=getSelf(linksCp),
                                self_link;
                            if(selfData){
                                self_link = selfData.self;
                                _uri = self_link.uri;
                                linksCp.splice(selfData.selfIndex,1);
                                var selfUri = appendQuery(_uri,this.start,this.limit);
                                linksCp.push(jQuery.extend({},self_link,{uri:selfUri,href:selfUri}));
                            }

                            // add next,prev,first and last links if applicable.
                            if(this.start > 0){
                                // populate link for first.
                                var firstUri = appendQuery(_uri,0,this.limit),
                                    prevUri = appendQuery(_uri,Math.max(0,this.start - this.limit),this.limit);
                                linksCp.push(jQuery.extend({},self_link,{rel:'first',uri:firstUri,href:firstUri}));
                                linksCp.push(jQuery.extend({},self_link,{rel:'prev',uri:prevUri,href:prevUri}));
                            }
                            if((this.start + this.limit < target.count) && this.limit < target.count){
                                // last
                                var lastUri = appendQuery(_uri,(target.count - (this.limit - Math.ceil(target.count/this.limit))),this.limit),
                                    nextUri = appendQuery(_uri,(this.start + this.limit),this.limit);
                                linksCp.push(jQuery.extend({},self_link,{rel:'next',uri:nextUri,href:nextUri}));
                                linksCp.push(jQuery.extend({},self_link,{rel:'last',uri:lastUri,href:lastUri}));
                            }
                            this.links = doProcess(linksCp);
                        }
                        return this.links;
                    };
                    default: {
                        return doProcess(target[property]);
                    }
                }
            }
        };

        /**
         * Setter trap.
         */
        PaginationDataProxyHandler.prototype.set = function(target,property,value,receiver){
            this[property] = value;
            return true;
        };

        /**
         * Object proxy handler for jqXhr instance returned by cache.
         * The jqXhr object is used as third (jqXhr) argument of the jQuery.ajax done handler
         * as well as first argument of the jQuery.ajax fail handler.
         * @private
         */
        function JqXhrProxyHandler(respJson){
            this.responseJSON = respJson;
        };

        /**
         * Getter trap.
         */
        JqXhrProxyHandler.prototype.get = function(target,property,receiver){
            if(this.hasOwnProperty(property)){
                return this[property];
            }

            if(property in target){
                switch(property){
                    case "responseText":{
                        if(!this.responseText){
                            this.responseText = JSON.stringify(this.responseJSON);
                        }
                        return this.responseText;
                    };
                    default: {
                        return target[property];
                    }
                }
            }
        };

        /**
         * Setter trap.
         */
        JqXhrProxyHandler.prototype.set = function(target,property,value,receiver){
            // do nothing. this is read only.
            return false;
        };

        /**
         * Object proxy handler for JqXhr.responseJSON instance returned by cache
         * on error to get a given user preference.
         * @private
         */
        function ErrorJsonProxyHandler(prefId){
            this._prefId = prefId;
        };

        /**
         * Getter trap.
         */
        ErrorJsonProxyHandler.prototype.get = function(target,property,receiver){
            if(this.hasOwnProperty(property)){
                return this[property];
            }

            if(property in target){
                if(property === 'message'){
                    this.message = target.message.replace(ERRONEOUS_PREF_ID,this._prefId);
                    return this.message;
                }
                return target[property];
            }
        };

        /**
         * Setter trap
         */
        ErrorJsonProxyHandler.prototype.set = function(target,property,value,receiver){
            // do nothing. this is read only.
            return false;
        };

        /**
         * Object proxy handler for filtered preferences cached data object.
         * The data object is used as target for returning a proxy when filter query
         * parameter is specified while fetching user preferences.
         *
         * Filters out data based on input.
         * @private
         */
        function FilteredDataProxyHandler(config){
            if(config.filter && config.filterFn){
                this.filter = Array.isArray(config.filter) ? config.filter[0] : config.filter;
                this.filterFn = config.filterFn;
            }
        };
        /**
         * Getter trap for the target.
         */
        FilteredDataProxyHandler.prototype.get = function(target,property,receiver){
            // filter data first.
            if(!this.items){
                this.items = doProcess(target.items.filter(this.filterFn));
            }

            if(this.hasOwnProperty(property)){
                return this[property];
            }

            if(property in target){
                switch(property){
                    case "count":{
                        return this.items.length;
                    };
                    case "links":{
                        if(!this.links){
                            var linksCp = target.links.slice(),
                                selfData = getSelf(linksCp),
                                selfUri;
                            if(selfData){
                                linksCp.splice(selfData.selfIndex,1);
                                selfUri = selfData.self.uri+"?filter="+this.filter;
                                linksCp.push(jQuery.extend({},selfData.self,{uri:selfUri,href:selfUri}));
                            }
                            this.links = linksCp;
                        }
                        return this.links;
                    };
                    default: {
                        return doProcess(target[property]);
                    }
                }
            }
        };

        /**
         * Setter trap.
         */
        FilteredDataProxyHandler.prototype.set = function(target,property,value,receiver){
            // read only
            return false;
        };

        /**
         * @function
         * @private
         */
        function getSelf(target,removeQueryParams){
            var res;
            target.every(function(link, index){
                if(link.rel === "self" && link.method.toUpperCase() === 'GET'){
                    res = {};
                    res.self = link;
                    res.selfIndex = index;
                    return false;
                }
                return true;
            });
            if(!!removeQueryParams){
                res.self.uri=URI.parse(res.self.uri).path;
                res.self.href=res.self.uri;
            }
            return res;
        }

        /**
         * Helper function to find pref by given id
         */
        function findById(id, prefData){
            var res;
            prefData.items && prefData.items.every(function(item){
                if(item.id === id){
                    res = item;
                    return false;
                }
                return true;
            });
            return res;
        };

        /**
         * @function
         * @private
         */
        function _getFilterFunction(filterRawVal){
            var arrFilterVals, eqFilter, inFilter;

            if(Array.isArray(filterRawVal)){
                filterRawVal = filterRawVal[0];
            }
            filterRawVal = decodeURI(filterRawVal);
            if((eqFilter = FILTER_REGEX_APP_EQ.test(filterRawVal)) !== true
                        && (inFilter = FILTER_REGEX_ID_IN.test(filterRawVal)) !== true){
                return;
            }
            function mapFilterVal(_item){
                if(_item.charAt(0) === "'" || _item.charAt(0) === '"'){
                    return _item.substring(1,_item.length-1);
                }
                return _item;
            }
            //eq(application,sdfsdd,sdfsdf) filter.
            if(eqFilter){
                filterRawVal = filterRawVal.replace(FILTER_REGEX_APP_EQ_FIRST_PART,"");
                filterRawVal = filterRawVal.substring(0,filterRawVal.length-1);
                if(filterRawVal.length < 1 || UNSUPPORTED_FILTERFNS_REGEX.test(filterRawVal)){
                    return;
                }
                arrFilterVals = filterRawVal.split(',').map(mapFilterVal);
                return function(item){
                    var match = false;
                    arrFilterVals.every(function(appName){
                        if(collatorComFn(appName,item.application) === 0){
                            match = true;
                            return false;
                        }
                        return true;
                    });
                    return match;
                };
            }
            // in(id,sdfsd,sdfsdf) filter.
            if(inFilter){
                filterRawVal = filterRawVal.replace(FILTER_REGEX_ID_IN_FIRST_PART,"");
                filterRawVal = filterRawVal.substring(0,filterRawVal.length-1);
                if(filterRawVal.length < 1 || UNSUPPORTED_FILTERFNS_REGEX.test(filterRawVal)){
                    return;
                }
                arrFilterVals = filterRawVal.split(',').map(mapFilterVal);
                return function(item){
                    var match = false;
                    arrFilterVals.every(function(id){
                        if(collatorComFn(id,item.id) === 0){
                            match = true;
                            return false;
                        }
                        return true;
                    });
                    return match;
                };
            }
        };
        
        function getFilterFunction(filterRawVal,async){
            if(!(sas && sas.ltjs && sas.ltjs.ltjsLoader && sas.ltjs.ltjsLoader.isInitialized())){
                return;
            }
            var ffn = _getFilterFunction(filterRawVal);
            if(ffn){
                if(collatorComFn){
                    return ffn;
                }
                if(!collatorComFnPromise){
                    collatorComFnPromise = new Promise(function(res,reject){
                        jQuery.sap.require("sas.ltjs.sasicu.util.SASCollatorUtil");
                        sas.ltjs.sasicu.util.SASCollatorUtil.getComparatorPromise().then(function (comparator) {
                            collatorComFn = comparator;
                            res(ffn);
                        }, function(){
                            collatorComFnPromise = undefined;
                            reject();
                        });
                    });
                }
                if(async === false){
                    // collatorComFn must exist to continue
                    return;
                }
                // return promise.
                return collatorComFnPromise;
            }
        };


        /**
         * @class
         * @name CacheHelper
         * User preference cache helper. Used by cache to deal with data.
         */
        function CacheHelper(){
            /* private variables */
            var cachedData = undefined,
                dataInitialized = false,
                errorDataInitialized=false,
                dataInitializing = false,
                errorDataInitializing=false,
                cachedJqXhr,
                cachedErrObjects,
                textStatus,
                uid,
                alternateServiceUrl=SERVICE_URL,
                configureUidAttempts = 0,
                initAttempts = 0,
                instanceUrlRegEx = urlRegEx;


            Object.defineProperties(this,{

                /**
                 * read only properties
                 *
                 */
                instanceUrlRegEx:{
                    get:function(){
                        return instanceUrlRegEx;
                    }
                },
                cachedData:{
                    get:function(){
                        return cachedData;
                    }
                },
                textStatus:{
                    get:function(){
                        return textStatus;
                    }
                },
                cachedJqXhr:{
                    get:function(){
                        return cachedJqXhr;
                    }
                },
                cachedErrObjects:{
                    get:function(){
                        return cachedErrObjects;
                    }
                },
                alternateServiceUrl:{
                    get:function(){
                        return alternateServiceUrl;
                    }
                },

                /**
                 * @function
                 *
                 */
                initialized:{
                    value: function(){
                        return dataInitialized && errorDataInitialized;
                    }
                },

                /**
                 * @function
                 *
                 */
                init:{
                    value: function(){
                        if(dataInitializing || errorDataInitializing || initAttempts > MAX_ATTEMPTS_CONFIGURE_UID) return;
                        initAttempts ++;
                        sas.log.debug("UserPreferencesCache#CacheHelper",sas.config.getAppId()+":init");
                        (function configureUserId(){
                            if(!uid && configureUidAttempts <= MAX_ATTEMPTS_CONFIGURE_UID){
                                configureUidAttempts ++;
                                userInfo.instanceAsync.then(function(uInfo){
                                    uid = uInfo.id;
                                    if(uid){
                                        URL_REGEX_UID_PART = '(@currentUser|'+uid+')';
                                        instanceUrlRegEx = new RegExp('^'+SERVICE_ENDPT+URL_REGEX_UID_PART+URL_REGEX_END_PART);
                                        alternateServiceUrl = SERVICE_ENDPT+uid;
                                    }
                                },configureUserId);
                            }
                        }());
                        dataInitialized = false;
                        dataInitializing = true;
                        sas.ajax({url:SERVICE_URL,
                            dataType:'json',
                            headers:{
                                'Accept': 'application/*+json'
                            },
                            data:{
                                'limit':MAX_LIMIT
                            }
                        }).then(
                            function (data, textStatus, jqXHR){
                                cachedData = Object.seal(data);
                                getSelf(cachedData.links,true);
                                cachedJqXhr = Object.seal(jqXHR);
                                dataInitializing = false;
                                dataInitialized = true;
                                textStatus = textStatus;
                            },
                            function failHandler(jqXHR, textStatus, errorThrown){
                                dataInitializing = false;
                                cachedJqXhr = undefined;
                                cachedData = undefined;
                                dataInitialized = false;
                                textStatus = textStatus;
                            }).always(function(){
                                if(!cachedErrObjects || !cachedErrObjects[0].responseJSON){
                                    errorDataInitialized = false;
                                    errorDataInitializing = true;
                                    sas.ajax({
                                        url:SERVICE_URL+"/"+ERRONEOUS_PREF_ID,
                                        dataType:'json',
                                        headers:{
                                            'Accept': 'application/*+json'
                                        }
                                    }).fail(function(jqXHR, textStatus, errorThrown){
                                        cachedErrObjects = [];
                                        cachedErrObjects.push(Object.seal(jqXHR));
                                        cachedErrObjects.push(textStatus);
                                        cachedErrObjects.push(errorThrown);
                                        errorDataInitialized = true;
                                        errorDataInitializing=false;
                                    });
                                }
                            });
                    }
                },
                /**
                 * @function
                 *
                 */
                extractPrefId:{
                    value:function (path){
                        return (path.indexOf(SERVICE_URL) === 0 && path.substring(SERVICE_URL.length+1))
                        || path.indexOf(alternateServiceUrl) === 0 &&  path.substring(alternateServiceUrl.length+1);
                    }
                },
                /**
                 * Add data to cache
                 * @function
                 *
                 */
                addToCache:{
                    value:function (data, textStatus, jqXHR){
                        var contentTypeHdr = jqXHR.getResponseHeader('Content-Type');
                        if(!data || !contentTypeHdr || contentTypeHdr.indexOf('json') === -1 || !dataInitialized) return;
                        // check if data is collection.
                        if(contentTypeHdr.indexOf("vnd.sas.collection") !== -1 && Array.isArray(data.items)){
                            var newItems = data.items.filter(function(newItem){
                                var isNew = true;
                                cachedData.items.every(function(cachedItem,cachedIndex){
                                    if(cachedItem.id === newItem.id){
                                        isNew = false;
                                        cachedData.items[cachedIndex] = newItem;
                                        return false;
                                    }
                                    return true;
                                });
                                return isNew;
                            });
                            Array.prototype.push.apply(cachedData.items, newItems);
                        }else if(contentTypeHdr.indexOf("vnd.sas.preference") !== -1){
                            var isNew = true;
                            cachedData.items.every(function(cachedItem,cachedIndex){
                                if(cachedItem.id === data.id){
                                    isNew = false;
                                    cachedData.items[cachedIndex] = data;
                                    return false;
                                }
                                return true;
                            });
                            isNew && cachedData.items.push(data);
                        }
                        cachedData.count = cachedData.items.length;
                    }
                },
                /**
                 * Remove data from cache.
                 * @function removeFromCache
                 *
                 */
                removeFromCache:{
                    value:function (options){
                        if(dataInitialized === false && ! options.url) return;
                        var parsedUri = URI.parse(options.url),
                            prefId = this.extractPrefId(parsedUri.path),
                            parsedData=!prefId && options.data && typeof options.data === 'string' ? JSON.parse(options.data): options.data;
                        if(parsedData && Array.isArray(parsedData.items)){
                            // was a bulk delete
                            for( var i=0; i<parsedData.items.length; i++){
                                var itm = parsedData.items[i];
                                for(var ci=cachedData.items.length; ci-- ;){
                                    if(cachedData.items[ci].id === itm.id){
                                        cachedData.items.splice(ci,1);
                                        cachedData.count = cachedData.items.length;
                                        break;
                                    }
                                }
                            }
                        } else {
                            var matchingIndex=-1;
                            cachedData.items.every(function(cachedItem,cachedIndex){
                                if(cachedItem.id === prefId){
                                    matchingIndex = cachedIndex;
                                    return false;
                                }
                                return true;
                            });
                            if(matchingIndex !== -1){
                                cachedData.items.splice(matchingIndex,1);
                                cachedData.count = cachedData.items.length;
                            }
                        }
                    }
                }
            });

            this.init();
        };







        /**
         * Check of cache helper can be reused in app switcher environment
         * for performance reasons.
         * @function
         * @private
         */
        function canReuseCacheHelper(){
            try{
                if(sas.config.getApplicationSwitcherEnabled()
                    && (window !== window.top)
                    && top.sas
                    && top.sas.cacheManager){
                    var topPrefCache = top.sas.cacheManager.getCache(this.name);
                    if(topPrefCache){
                        return util._hasSameVersionAsParent().then(
                            function(){return topPrefCache.__helper;},
                            function(){return Promise.reject();}
                        );
                    }
                }
            }catch(err){
                // throws an error when apps from cross origin are loaded.
                // suppress the error.
            }
            return Promise.reject();
        };

        /**
         * @class
         * UserPreference Cache class.
         * @name UserPreferencesCache
         */
        function UserPreferencesCache(){
            var helper;

            function init(){
                // create helper
                if(self === top){
                    helper = new CacheHelper();
                }else{
                    canReuseCacheHelper.call(this).then(function(_helper){
                        helper = _helper;
                    },function(){
                        helper = new CacheHelper();
                    });
                }
            }

            Object.defineProperties(this,{
                "name" : {
                    value:'sas.PreferencesServiceCache'
                },
                __helper: {
                    get: function(){
                        return helper;
                    }
                },
                /**
                 * @function
                 * @name canHandle
                 */
                "canHandle": {
                    value:function(options){
                        return typeof options === 'object'
                                && !options.success
                                && !options.error
                                && options.dataType && options.url
                                && options.cache !== false
                                && options.dataType.toUpperCase() === 'JSON'
                                && helper
                                && helper.instanceUrlRegEx.test(options.url);
                    }
                },
                /**
                 * @function
                 * @name handle
                 */
                "handle":{
                    value:function (options,_ajaxCallInvokerFn){
                        // if helper is not ready then return from here
                        if(helper === undefined) return _ajaxCallInvokerFn(options);
                        var def, parsedUri, uriParams,prefId,
                            prefObj, respObj,
                            callbackContext = options.context||options,
                            cachedObj=helper.cachedData;

                        function _resolveDeffered(respObj,_textStatus,jqXhr, mimicAsync){

                            function doResolveDef(){
                                // handle dataFilter specified if any.
                                // dataFilter doesn't appear to be performance friendly.
                                // may be a good idea to log a warning to not use it.
                                // For compatibility need to support it.
                                if(options.dataFilter){
                                    var res = options.dataFilter(JSON.stringify(respObj),options.dataType);
                                    def.resolveWith(callbackContext,[JSON.parse(res),_textStatus,jqXhr]);
                                }else{
                                    def.resolveWith(callbackContext,[respObj,_textStatus,jqXhr]);
                                }
                            }
                            // mimic async if needed.
                            if(options.async === false || mimicAsync === false){
                                doResolveDef();
                            }else{
                                new Promise(function(res,rej){
                                    setTimeout(function(){
                                        doResolveDef();
                                        res();
                                    });
                                });
                            }
                            return  def.promise();
                        };

                        function _rejectDeffered(jqXHR, _textStatus, errorThrown){
                            function doRejectDef(){
                                def.rejectWith(callbackContext,[jqXHR, _textStatus, errorThrown]);
                            }

                            // mimick async if needed.
                            if(options.async === false){
                                doRejectDef();
                            }else{
                                new Promise(function(res,rej){
                                    setTimeout(function(){
                                        doRejectDef();
                                        res();
                                    });
                                });
                            }
                            return  def.promise();
                        };

                        function _callService(){
                            var result = _ajaxCallInvokerFn(options);
                            if(options.type === 'PUT'){
                                result.done(helper.addToCache.bind(helper));
                            }
                            if(options.type === 'DELETE'){
                                result.done(function(d,t,j){helper.removeFromCache(options);});
                            }
                            return result;
                        }

                        if(!this.canHandle(options)){
                            return _callService(options);
                        }

                        if(options.type !== 'GET'){
                            return _callService(options);
                        }

                        if(!helper.initialized()){
                            var retObj = _callService(options);
                            retObj.always(helper.init.bind(helper));
                            return retObj;
                        }

                        // do this stuff in promise to deal with async nature of call.
                        parsedUri = URI.parse(options.url);
                        // check if the call matches exactly with service url
                        // all preferences
                        if(parsedUri.path === SERVICE_URL || parsedUri.path === helper.alternateServiceUrl){
                            // try reading all query params.
                            uriParams = jQuery.sap.getUriParameters(options.url);
                            uriParams = uriParams.mParams || {};
                            jQuery.extend(uriParams,options.data);

                            // see if data/query has sortBy param.
                            if("sortBy" in uriParams){
                                // don't bother handling sorting.
                                return _callService(options);
                            }
                            def = jQuery.Deferred();
                            if("filter" in uriParams){
                                // check if specified filter is supported
                                var filterFn = getFilterFunction(uriParams.filter,options.async);
                                if(filterFn){
                                    cachedObj = new ObjectProxy(helper.cachedData,new FilteredDataProxyHandler({"filter":uriParams.filter,"filterFn":filterFn}));
                                }else if(filterFn instanceof Promise){
                                    // hookup a handler on the promise and return
                                    filterFn.then(function(_actualfilterFn){ // success handler
                                        cachedObj = new ObjectProxy(helper.cachedData,new FilteredDataProxyHandler({"filter":uriParams.filter,"filterFn":_actualfilterFn}));
                                        respObj = new ObjectProxy(cachedObj,new PaginationDataProxyHandler(uriParams));
                                        // resolve deferred without mimicking async call.
                                        _resolveDeffered(respObj,helper.textStatus,new ObjectProxy(helper.cachedJqXhr,new JqXhrProxyHandler(respObj)), false);
                                    },function(){ // rejection handler
                                        //call actual service.
                                        _callService(options).then(_resolveDeffered,_rejectDeffered);
                                    });
                                    return def.promise();
                                }else{
                                    return _callService(options);
                                }
                            }
                            respObj = new ObjectProxy(cachedObj,new PaginationDataProxyHandler(uriParams));
                            return _resolveDeffered(respObj,helper.textStatus,new ObjectProxy(helper.cachedJqXhr,new JqXhrProxyHandler(respObj)));
                        }
                        //get specific preference if requested.
                        prefId = helper.extractPrefId(parsedUri.path);
                        def = jQuery.Deferred();
                        prefObj = findById(prefId,cachedObj);
                        if(prefObj){
                            //found
                            respObj = deepFreeze(prefObj);
                            return _resolveDeffered(respObj,helper.textStatus,new ObjectProxy(helper.cachedJqXhr,new JqXhrProxyHandler(respObj)));
                        }
                        //not found.
                        respObj = helper.cachedErrObjects[0].responseJSON
                            ? new ObjectProxy(helper.cachedErrObjects[0].responseJSON,new ErrorJsonProxyHandler(prefId))
                            : undefined;
                        return _rejectDeffered(new ObjectProxy(helper.cachedErrObjects[0],new JqXhrProxyHandler(respObj)),
                            helper.cachedErrObjects[1],
                            helper.cachedErrObjects[2]);
                    }
                }
            });
            init.call(this);
        };

        return UserPreferencesCache;
    },true);
jQuery.sap.declare("sas.hc.ui.core.cache.CacheManagerTask");

sap.ui.define('sas/hc/ui/core/cache/CacheManagerTask',
    ['jquery.sap.global', 'sas/hc/ui/core/BootTask', 'sas/hc/ui/core/cache/UserPreferencesCache'],
    function(jQuery, BootTask, UserPrefCache) {
        "use strict";

        var CacheManagerTaskk,
            caches={},
            CONFKEY_CM_DISABLE = 'xx-cachemanager-disable',
            CONFKEY_PREFCACHE_DISABLE = 'xx-preferencescache-disable',

            CacheManagerTaskk = BootTask.extend('sas.hc.ui.core.cache.CacheManagerTask',{
                name:'sas.hc.ui.core.cache.CacheManagerTask'
            });

        CacheManagerTaskk.prototype.run = function(){
            var isDisabled = sas.config.get(CONFKEY_CM_DISABLE),
                sasCommonsConf = sas.config.get('commons');
            isDisabled = isDisabled || (sasCommonsConf && sasCommonsConf[CONFKEY_CM_DISABLE] === 'true');
            if(!isDisabled){
                if(!sas.cacheManager){
                    Object.defineProperty(sas,'cacheManager', {
                        enumerable:true,
                        value:Object.freeze({
                            findCache : function(options){
                                for(var key in caches){
                                    if(caches[key].canHandle(options)){
                                        return caches[key];
                                    }
                                }
                            },
                            getCache:function(name){
                                return caches[name];
                            }
                            //                                    Not now may be later.
                            //                                    registerCache : function(cache){
                            //                                        jQuery.sap.assert(cache && cache.name && cache.canHandle && cache.handle, "cache must be an object and implement canHandle(options) and handle(options) APIs");
                            //                                        caches[cache.name] = cache;
                            //                                    }
                        })
                    });
                }
                if(!(sasCommonsConf && sasCommonsConf[CONFKEY_PREFCACHE_DISABLE])){
                    // create a user preferences cache
                    var prefCache = new UserPrefCache();
                    caches[prefCache.name] = prefCache;
                }
            };
        };

        return CacheManagerTaskk;
    });
jQuery.sap.declare("sas.hc.ui.core.Core");

jQuery.sap.require("sas.hc.ui.core.Configuration");
jQuery.sap.require("sas.hc.ui.services.ServiceRegistry");
jQuery.sap.require("sas.hc.ui.core.BootTask");
jQuery.sap.require("sas.hc.ui.core.util");
jQuery.sap.require("sas.hc.ui.core.settings.SettingsInfo");
jQuery.sap.require("sas.hc.ui.core.settings.SettingsInfoTask");
jQuery.sap.require("sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherBootTask");
jQuery.sap.require("sas.hc.ui.core.TouchDetectionTask");
jQuery.sap.require("sas.hc.ui.core.MonkeyPatchApplySettingsTask");
jQuery.sap.require("sas.hc.ui.core.ContextMenuTask");
jQuery.sap.require("sas.hc.ui.core.ClearFocusRingTask");
jQuery.sap.require("sas.hc.ui.core.TimeoutHandlerTask");
jQuery.sap.require("sas.hc.ui.core.theme.ThemeBootTask");
jQuery.sap.require("sap.ui.core.Core");
jQuery.sap.require("sap.ui.core.RenderManager");
jQuery.sap.require("sas.hc.ui.core.UtilTask");
jQuery.sap.require("sas.hc.ui.core.util.ComponentServiceClassResolver");
jQuery.sap.require("sap.ui.core.Locale");
jQuery.sap.require("sas.hc.ui.core.util.UserInfo");
jQuery.sap.require("sas.hc.ui.core.util.UserInfoTask");
jQuery.sap.require("sas.hc.ui.core.CSRFTask");

(function(){
    /**
     *
     *
     * Core class that boots sas commons infrastructure to be accessed using sas.hc.getCore() utility api.
     * @returns {sas.hc.ui.core.Core} TODO-jsdoc
     */
    sas.hc.ui.core.Core = function(){
        // sas-namespaced logging
        sas.log = jQuery.extend($.sap.log);

        if(sas.hc.ui.getCore && sas.hc.ui.getCore()){
            return sas.hc.ui.getCore();
        }
        var that = this;
        if(sas.hc.ui.getCore === undefined){
            // Support custom styles rendering.
            var _invalidatedStyleControls={},
                renderCustomStylesTimer=undefined,
                toCSS=function(oObj){
                    var cssString,finalCssString="";
                    for(var prop in oObj){
                        if (oObj.hasOwnProperty(prop)) {
                            cssString="";
                            if(typeof oObj[prop] === 'object' && oObj[prop]){
                                cssString=JSON.stringify(oObj[prop],
                                    function(key,val){
                                        if(typeof val === 'string'){
                                            return val.replace(/,/g,'_Q_').replace(/\"/g,'');
                                        }
                                        return val;
                                    }
                                ).replace(/\"/g,'');
                                cssString=cssString.replace(/,/g,';').replace(/_Q_/g,',');
                                finalCssString += prop + cssString;
                            }
                        }
                    }
                    return finalCssString;
                },
                createStyleElm = function(id,content){
                    var styleEl = document.createElement('style');
                    styleEl.id=id;
                    styleEl.type="text/css";
                    if(typeof content === 'object'){
                        styleEl.textContent = toCSS(content);
                    }else{
                        styleEl.textContent=content;
                    }
                    return styleEl;
                },
                renderInvalidatedCustomStyles = function(){
                    sas.log.trace('Render invalidated custom styles');
                    var headElm = document.head,
                        fragment=document.createDocumentFragment(); //TODO more than 1 head elements. Issue.
                    do{
                        if(typeof renderCustomStylesTimer !== 'undefined'){
                        // clear timer.
                            jQuery.sap.clearDelayedCall(renderCustomStylesTimer);
                            renderCustomStylesTimer = undefined;
                        }

                        for(var sId in _invalidatedStyleControls){
                            if (_invalidatedStyleControls.hasOwnProperty(sId)) {
                                var oControl = _invalidatedStyleControls[sId];
                                delete _invalidatedStyleControls[sId];
                                // remove existing rules for this control if any.
                                // style tag for a control will have its id-style
                                var styleId = oControl.getId()+"-style";
                                var oldStyleElm = document.getElementById(styleId);
                                if(oldStyleElm){
                                    headElm.removeChild(oldStyleElm);
                                }
                                if(!oControl._bIsBeingDestroyed && oControl.getCustomStyles && oControl.getCustomStyles() && typeof oControl.getCustomStyles() !== 'undefined'){
                                    var renderer = oControl.getRenderer();
                                    if(typeof renderer === 'object' && typeof renderer.getCustomCSSRules === 'function'){
                                        fragment.appendChild(createStyleElm(styleId,renderer.getCustomCSSRules(oControl)));
                                    }else{
                                        sas.log.warning(oControl.toString()+' does not fully support setCustomStyles() API');
                                    }
                                }
                            }
                        }
                    }while(typeof renderCustomStylesTimer !== 'undefined');
                    // add all style elements to head.
                    headElm.appendChild(fragment);
                };

            // Proxy sap.ui.core.RenderManager.flush
            // Render custom styles before 'onAfterRendering' is called on controls.
            var fnOldFlush = sap.ui.core.RenderManager.prototype.flush;
            sap.ui.core.RenderManager.prototype.flush = function(){
                // go ahead and render custom styles.
                renderInvalidatedCustomStyles();
                fnOldFlush.apply(this,arguments);
            };

            // Proxy Control.renderControl function in Control class.
            var fnOldRenderControl = sap.ui.core.RenderManager.prototype.renderControl;
            sap.ui.core.RenderManager.prototype.renderControl = function(oCntrl){
                fnOldRenderControl.apply(this,arguments);
                if(oCntrl && oCntrl.getCustomStyles && oCntrl.getCustomStyles()){
                    sas.hc.ui.getCore().addInvalidateCustomStyles(oCntrl,true);
                }
            };

            Object.defineProperty(sas.hc.ui,'getCore',{
                enumerable:true,
                value:function(){
                    return Object.freeze({
                        registerTask:function(oTask){
                            that.registerTask.apply(that, arguments);
                        },
                        boot:function(fnCallBack){
                            that.boot.apply(that, arguments);
                        },
                        addInvalidateCustomStyles:function(oControl,bSuppressRendering){
                            jQuery.sap.assert(oControl && oControl instanceof sap.ui.core.Control, "oControl must be a sap.ui.core.Control");
                            _invalidatedStyleControls[oControl.getId()]=oControl;
                            bSuppressRendering = typeof bSuppressRendering === 'boolean' && bSuppressRendering;
                            if(!bSuppressRendering && typeof renderCustomStylesTimer === 'undefined'){
                                renderCustomStylesTimer = jQuery.sap.delayedCall(0,this,renderInvalidatedCustomStyles);
                            }
                        }
                    });
                }
            });
        }
    };

    function TasksChain(parent){
        this.tasksMap = new Map();
        this.next = undefined;
        this.parent = parent;
    };

    TasksChain.prototype.destroy = function(){
        this.tasksMap.clear();
        this.tasksMap = undefined;
        if(this.next){
            this.next.destroy();
            this.next = undefined;
        }
        this.parent = undefined;
    };

    TasksChain.prototype._isAncestor = function(node){
        while(node && node.parent){
            if (node.parent === this){
                return true;
            }
            node = node.parent;
        }
        return false;
    };

    TasksChain.prototype._findByName = function(name){
        if(this.tasksMap.get(name)){
            return this;
        }else if(this.next){
            return this.next._findByName(name);
        }
    };


    TasksChain.prototype._findParent = function(deps){
        var parentChain,
            potentialParent,
            iTaskDepName;
        while(deps.length > 0){
            iTaskDepName = deps.shift();
            potentialParent = this._findByName(iTaskDepName);
            if(potentialParent && parentChain){
                // check if potential parent is ancestor of this
                if(!potentialParent._isAncestor(parentChain)){
                    parentChain = potentialParent;
                }
            }else if(potentialParent){
                parentChain = potentialParent;
            }
        }
        return parentChain;
    };

    /**
     * Adds given task to chain depending upon its dependencies specified if any.
     * Expects specified dependency to have registered before the input task itself otherwise
     * adds the task to the default chain.
     */
    TasksChain.prototype.add = function(oTask){
        var parentChain;
        jQuery.sap.assert(oTask && oTask instanceof sas.hc.ui.core.BootTask, "oTask must be a sas.hc.ui.core.BootTask");
        jQuery.sap.assert(typeof oTask.name === "string" && oTask.name !== "", "oTask.name must be a non empty string");
        if(oTask.dependsOn){
            jQuery.sap.assert(Array.isArray(oTask.dependsOn), "oTask.dependsOn must be an array of strings");
            parentChain = this._findParent(oTask.dependsOn);
            if(parentChain){
                parentChain.next = parentChain.next || new TasksChain(parentChain);
                parentChain.next.tasksMap.set(oTask.name,oTask);
                return;
            }
        }
        this.tasksMap.set(oTask.name,oTask);
    };

    TasksChain.prototype.execute = function(){
        var that = this;

        return new Promise(function(resolve,reject){

            var asyncTasks;

            function executeNext(){
                if(that.next){
                    that.next.execute().then(resolve);
                }else{
                    resolve();
                }
            }

            that.tasksMap.forEach(function(oTask){
                sas.log.info('Executing task',oTask.name);
                if(oTask['runAsync']){
                    asyncTasks = asyncTasks || [];
                    asyncTasks.push(oTask.executeAsync.apply(oTask));
                }else{
                    oTask.execute.apply(oTask);
                }
            });

            if(asyncTasks && asyncTasks.length > 0){
                Promise.all(asyncTasks).then(executeNext);
            }else{
                executeNext();
            }
        });
    };

    sas.hc.ui.core.Core.prototype.tasks = new TasksChain();

    function registerCacheFilter(){
        //S1187387: Make sure IE does not used cached results from REST GET/HEAD calls.
        if( sap.ui.Device.browser.name === sap.ui.Device.browser.BROWSER.INTERNET_EXPLORER ) {
            //S1297772 exclude library preloads from IE bypass cache mechanism.
            var excludesPattern = sas.config.get("xx-bypassIECacheExcludes") || /.*\/library-preload\.json$/;

            jQuery.ajaxPrefilter(function( options, originalOptions, jqXHR ) {
                // Return if cache option already specified or if this is not a GET/HEAD call.
                if ( !options ||
                    (typeof options!=="object") ||
                    (options.cache!==undefined) ||
                    (options.type &&
                        (options.type.toUpperCase()!=="GET") &&
                        (options.type.toUpperCase()!=="HEAD")) ||
                    (typeof options.url === 'string' && options.url.match(excludesPattern))){
                    return;
                }

                // Set cache:false if Accept header value specified and its value contains JSON or XML.
                if( options.headers &&
                    (typeof options.headers==='object') ) {
                    for(var propertyName in options.headers) {
                        if (options.headers.hasOwnProperty(propertyName)) {
                            if ( propertyName.toUpperCase() === "ACCEPT" ) {
                                if( (typeof options.headers[propertyName]==="string") &&
                                    (options.headers[propertyName].toUpperCase().indexOf("JSON")>-1) ||
                                    (options.headers[propertyName].toUpperCase().indexOf("XML")>-1)) {
                                    options.cache=false;
                                    return;
                                }
                            }
                        }
                    }
                }
                // Return if dataType is not JSON and not XML.
                if ( options.dataType &&
                    (options.dataType.toUpperCase().indexOf("JSON")<0) &&
                    (options.dataType.toUpperCase().indexOf("XML")<0) )
                    return;

                // Set the cache option to false.
                options.cache=false;
            });
        }
    };

    // private function to register commons
    // boot task.
    sas.hc.ui.core.Core.prototype._registerCoreTask = function(oTask){
        this.tasks.add(oTask);
    };

    function configureLanguage(){
        var sapConf = sap.ui.getCore().getConfiguration(),
            confLang = sapConf.getLanguage();

        confLang = jQuery.sap.resources._normalizeLocale(confLang);
        // check if configured language is supported
        if(sapConf.getSupportedLanguages().indexOf(confLang) !== -1){
            sapConf.setLanguage(jQuery.sap.resources._convertLocaleToBCP47(confLang));
        } else {
            // check if only lang without sub script is supported.
            // like "fr_FR" might be configured language but not supported by app
            // but app supports "fr". Better using "fr" in such cases instead of "en-US"
            var lcl = new sap.ui.core.Locale(confLang);
            if(sapConf.getSupportedLanguages().indexOf(lcl.getLanguage()) !== -1){
                sapConf.setLanguage(lcl.getLanguage());
            } else { // fall back
                sapConf.setLanguage("en");
            }
        }
    };

    /**
     * Boots the sas commons infrastructure.
     * Creates a sas configuration object that can be accessed from sap.ui.getCore().getConfiguration() api.
     * Adds an ajax function to sas name space. A convenience api for Json Rest communication that supports cross domain calls using cross domain proxy.
     * Instantiates the WIP and Commons services registry if available. Exposed as sas.registry & sas.registry.wip respectively.
     * Provides hooks for consumers to register custom tasks to commons boot strap.
     * @param {function()} fnCallBack TODO-jsdoc
     */
    sas.hc.ui.core.Core.prototype.boot=function(fnCallBack){

        if(window.sas.hc.ui._bBooted){
            fnCallBack();
            return;
        }
        jQuery.sap.measure.start("sasCoreBootComplete");
        sas.hc.ui._bBooted = true;

        // populate sas configuration.
        var sasConf = new sas.hc.ui.core.Configuration();
        Object.defineProperty(sas,'config',{
            value : sasConf,
            enumerable:true,
        });

        registerCacheFilter();

        // create a sync point for tasks registration.
        var oHcSyncPoint = jQuery.sap.syncPoint("Sas commons bootstrap tasks",function(iOpenTasks, iFailures) {
                sas.log.trace("Sas commons bootstrap tasks registered: Total=" + iOpenTasks);
                jQuery.sap.measure.start("sasBootTaskExecution", "Core.js - boot");
                that.tasks.execute().then(function(){
                    that.tasks.destroy();
                    that.tasks = undefined;
                    jQuery.sap.measure.end("sasCoreBootComplete");
                    fnCallBack();
                });
            }),
            iCommonsBootTasks = oHcSyncPoint.startTask("sas commons boot tasks registration"),
            resolver=sas.hc.ui.core.util.ComponentServiceClassResolver,
            that=this,
            _isMainApplication = function (){
                return window === window.top;
            },
            regTask = resolver.getServiceClassInstance(resolver.REGISTRY_TASK),
            versionInfoTask = resolver.getServiceClassInstance(resolver.VERSION_INFO_TASK),
            cacheManagerTask = resolver.getServiceClassInstance(resolver.CACHE_MANAGER_TASK);


        // re-configure OpenUI5 language
        configureLanguage();

        // sas core util task.
        this._registerCoreTask(new sas.hc.ui.core.UtilTask());

        // CSRF token handler task
        this._registerCoreTask(new sas.hc.ui.core.CSRFTask());

        // setting info task
        this._registerCoreTask(new sas.hc.ui.core.settings.SettingsInfoTask());
        
        if(regTask){
            // Registry task if available
            this._registerCoreTask(regTask);
        }

        if (versionInfoTask) {
            sap.ui.getVersionInfo = versionInfoTask.getVersionInfo;
        }

        // user info
        this._registerCoreTask(new sas.hc.ui.core.util.UserInfoTask());

        if(cacheManagerTask){
            // register cache manager task.
            this._registerCoreTask(cacheManagerTask);
        }

        // check if app switcher is enabled and its a main window
        if(!sas.config.getApplicationSwitcherEnabled() ||  !_isMainApplication()){
            // don't execute these tasks in app switcher main window.
            // Talk to Yateen if you are not sure where should your task go.

            // theme root loader task.
            if(sasConf.getThemeUrl() !== undefined){
                this._registerCoreTask(new sas.hc.ui.core.theme.ThemeBootTask());
            }
            this._registerCoreTask(new sas.hc.ui.core.TouchDetectionTask());
            this._registerCoreTask(new sas.hc.ui.core.MonkeyPatchApplySettingsTask());
            this._registerCoreTask(new sas.hc.ui.core.ContextMenuTask());
        }

        // hook to register custom tasks to this sync point.
        var fnCustomBootTask = sasConf.getBootTask();
        if ( fnCustomBootTask ) {
            var iCustomBootTask = oHcSyncPoint.startTask("custom sas boot task");
            fnCustomBootTask( function(bSuccess) {
                oHcSyncPoint.finishTask(iCustomBootTask, typeof bSuccess === "undefined" || bSuccess === true );
            });
        }

        //register clearFocusRingTask for mouse and keyboard focus handling
        this._registerCoreTask(new sas.hc.ui.core.ClearFocusRingTask());

        // let session time out be the last task.
        this._registerCoreTask(new sas.hc.ui.core.TimeoutHandlerTask());

        // This comes after timeouts since app switcher needs to coordinate timeouts across apps and is dependent
        // on single app timeout already being in place
        if(sas.config.getApplicationSwitcherEnabled()===true)
            this._registerCoreTask(new sas.hc.ui.core.applicationSwitcher.ApplicationSwitcherBootTask());

        oHcSyncPoint.finishTask(iCommonsBootTasks);
    };
    /**
     *
     * Registers a task to the core. The task needs to have run function on to it.
     * @param {object} oTask TODO-jsdoc
     */
    sas.hc.ui.core.Core.prototype.registerTask=function(oTask){
        if(!Array.isArray(oTask.dependsOn)){
            oTask.dependsOn = ["sas.hc.ui.core.RegistryTask","sas.hc.ui.core.theme.ThemeBootTask"];
        }
        this.tasks.add(oTask);
    };
}());

(function(){
    new sas.hc.ui.core.Core();
}());
